unit Calcul;

interface

uses FicheTpo, Classes, Dialogs, Controls, Forms, UDate, Lacune, SysUtils;

const
  deBranche    = 1;
  deBoucle     = 2;
  deCul_de_sac = 3;
  Silencieux = false;
  Bruyant = true;
  moBoucle = 1;
  moExpert = 2;
  Max_Point = 10000;
  Max_Carrefour = 1000;

type

  TLigne = array[0..Max_Carrefour-1] of double;
  PLigne = ^TLigne;
  TCreux = array[0..Max_Carrefour-1] of boolean;
  PCreux = ^TCreux;
  TMatrice = class
    Gauche : array[0..Max_Carrefour-1] of PLigne;
    Creux : array[0..Max_Carrefour-1] of PCreux;
    Droite : PLigne;
    Taille : integer;
    constructor Create(N : integer);
    destructor Destroy; override;
    procedure Rempli(P,Q : integer; Poids, K : double);
    procedure Inverse;
  end;

  TAncienne = array[1..Max_Point] of longint;
  PAncienne = ^TAncienne;
  TSens = array[1..Max_Point] of boolean;
  PSens = ^TSens;
  TDistance = array[1..Max_Point] of double;
  PDistance = ^TDistance;

  TConcentre = class
    Depart, Arrivee : TPointTopo;
    DX, DY, DZ, L, LP, L2, Contrainte_Plan, Contrainte_Z : double;
    Nombre_Fiche, N_Depart, N_Arrivee : integer;
    Approximation, Concerne : boolean;
    Anciennes_Fiche, Anciennes_Galerie : PAncienne;
    Inverse : PSens;
    constructor Create;
    destructor Destroy; override;
    procedure Resume(Une_Galerie : TGalerie;AF, AG : longint);
  end;

  TCalculateur = class
    Fiches, Reference, Noeud, Boucle, Cul_de_sac, Source : TList;
    Vue, Approx : PSens;
    Distance : PDistance;
    Numero : PAncienne;
    Utilise, Imparfait, Tout_Faux : boolean;
    constructor Create;
    destructor Destroy; override;
    procedure Initialise(var Les_Fiches, Les_Points : TList;Mode : integer);
    function Ajouter_Point(Le_Point : TPointTopo) : integer;
    function Atteint(Echo : boolean) : boolean;
    procedure Fusion(N_Point : integer);
    procedure Reduction(Affichage : boolean);
    procedure Approxime;
    procedure La_Totale(Affichage : boolean);
    procedure Fission;
    procedure Decompresse(La_Fiche : TConcentre; Nature : shortInt);
    procedure Retour_Info;
    procedure Resoud(Echo : boolean); 
  end;

var

  Calcul_En_Cours, Calcul_Arrete : boolean;

implementation

(********************** TMatrice *********************)
constructor TMatrice.Create(N : integer);
var
  I, J : integer;
begin
  inherited Create;
  if N>Max_Carrefour then ShowMessage('Trop de carrefours'#13#10'Demandez une version spciale de CyberTopo');
  Taille:=N;
  GetMem(Droite,sizeof(double)*Taille);
  for I:=0 to Taille-1 do begin
    GetMem(Gauche[I],sizeof(double)*Taille);
    GetMem(Creux[I],sizeof(boolean)*Taille);
    for J:=0 to Taille-1 do begin
      Gauche[I]^[J]:=0;
      Creux[I]^[J]:=true;
    end;
    Droite^[I]:=0;
  end;
end;

destructor TMatrice.Destroy;
var
  I : integer;
begin
  FreeMem(Droite,sizeof(double)*Taille);
  for I:=0 to Taille-1 do begin
    FreeMem(Gauche[I],sizeof(double)*Taille);
    FreeMem(Creux[I],sizeof(boolean)*Taille);
  end;
  inherited Destroy;
end;

procedure TMatrice.Rempli(P,Q : integer; Poids, K : double);
begin
  if abs(K)>1 then begin
    K:=K+1-1;
  end;
  if (P<>0) then begin
    if (Q<>0) then begin
      Gauche[P-1]^[P-1]:=Gauche[P-1]^[P-1]+1/Poids;
      Gauche[Q-1]^[Q-1]:=Gauche[Q-1]^[Q-1]+1/Poids;
      Gauche[P-1]^[Q-1]:=Gauche[P-1]^[Q-1]-1/Poids;
      Gauche[Q-1]^[P-1]:=Gauche[Q-1]^[P-1]-1/Poids;
      Creux[P-1]^[P-1]:=false;
      Creux[Q-1]^[Q-1]:=false;
      Creux[P-1]^[Q-1]:=false;
      Creux[Q-1]^[P-1]:=false;
      Droite^[P-1]:=Droite^[P-1]-K/Poids;
      Droite^[Q-1]:=Droite^[Q-1]+K/Poids;
    end
    else begin
      Gauche[P-1]^[P-1]:=Gauche[P-1]^[P-1]+1/Poids;
      Droite^[P-1]:=Droite^[P-1]-K/Poids;
    end;
  end
  else begin
    if (Q<>0) then begin
      Gauche[Q-1]^[Q-1]:=Gauche[Q-1]^[Q-1]+1/Poids;
      Droite^[Q-1]:=Droite^[Q-1]+K/Poids;
    end
    else begin
      MessageDlg('Points de ref  chaque bout !',mtWarning,[mbOk],0);
    end;
  end;
end;

procedure TMatrice.Inverse;
var
  I, J, K : integer;
  Ptt_Reel : double;
begin
  for I:=0 to Taille-1 do begin
    for J:=0 to Taille-1 do if (I<>J) and not Creux[J]^[I] then begin
      Ptt_Reel:=Gauche[J]^[I]/Gauche[I]^[I];
      Droite^[J]:=Droite^[J]-Ptt_Reel*Droite^[I];
      for K:=I+1 to Taille-1 do if not Creux[I]^[K] then begin
        Gauche[J]^[K]:=Gauche[J]^[K]-Ptt_Reel*Gauche[I]^[K];
        Creux[J]^[K]:=false;
      end;
    end;
    Ptt_Reel:=1/Gauche[I]^[I];
    Droite^[I]:=Droite^[I]*Ptt_Reel;
    for K:=I to Taille-1 do if not Creux[I]^[K] then Gauche[I]^[K]:=Gauche[I]^[K]*Ptt_Reel;
  end;
end;

(********************** TConcentre *********************)
constructor TConcentre.Create;
begin
  inherited Create;
  Nombre_Fiche:=0;
end;

destructor TConcentre.Destroy;
begin
  FreeMem(Anciennes_Galerie,sizeOf(Longint)*Nombre_Fiche);
  FreeMem(Anciennes_Fiche,sizeOf(Longint)*Nombre_Fiche);
  FreeMem(Inverse,sizeOf(boolean)*Nombre_Fiche);
  Depart.Free; Depart:=nil;
  Arrivee.Free; Arrivee:=nil;
  inherited Destroy;
end;

procedure TConcentre.Resume(Une_Galerie : TGalerie; AF, AG : longint);
begin
  Nombre_Fiche:=1;
  GetMem(Anciennes_Galerie,sizeOf(Longint));
  GetMem(Anciennes_Fiche,sizeOf(Longint));
  GetMem(Inverse,sizeOf(boolean));
  Depart:=TPointTopo.create;
  Depart.Duplique(Une_Galerie.Depart);
  Arrivee:=TPointTopo.Create;
  Arrivee.Duplique(Une_Galerie.Arrivee);
  L:=Une_Galerie.SigmaL;
  LP:=Une_Galerie.SigmaL*Une_Galerie.Precision;
  L2:=Une_Galerie.SigmaL2*Une_Galerie.Precision;
  DX:=Une_Galerie.DeltaX;
  DY:=Une_Galerie.DeltaY;
  DZ:=Une_Galerie.DeltaZ;
  Approximation:=Une_Galerie.Approximation;
  Anciennes_Fiche^[1]:=AF;
  Anciennes_Galerie^[1]:=AG;
  Inverse^[1]:=false;
end;

(*********************** TCalculateur *******************)
constructor TCalculateur.Create;
var
  I : integer;
begin
  inherited Create;
  Fiches:=TList.Create;
  Fiches.Capacity:=25;
  Reference:=TList.Create;
  Reference.Capacity:=5;
  Noeud:=TList.Create;
  Noeud.Capacity:=25;
  Boucle:=TList.Create;
  Boucle.Capacity:=25;
  Cul_de_Sac:=TList.Create;
  Cul_de_Sac.Capacity:=25;
  GetMem(Approx,sizeOf(boolean)*Max_Point);
  for I:=1 to Max_Point do Approx^[I]:=false;
  Imparfait:=false;
  Tout_Faux:=false;
end;

destructor TCalculateur.Destroy;
var
  I : integer;
begin
  Calcul_En_Cours:=false;
  FreeMem(Approx,sizeOf(boolean)*Max_Point);
  if Utilise then FreeMem(Vue,sizeOf(boolean)*Noeud.Count);
  for I:=Fiches.Count-1 downto 0 do begin
    TConcentre(Fiches[I]).Free;
    Fiches.Delete(I);
  end;
  Fiches.Free; Fiches:=nil;
  for I:=Reference.Count-1 downto 0 do begin
    TConcentre(Reference[I]).Free;
    Reference.Delete(I);
  end;
  Reference.Free; Reference:=nil;
  for I:=Boucle.Count-1 downto 0 do begin
    TConcentre(Boucle[I]).Free;
    Boucle.Delete(I);
  end;
  Boucle.Free; Boucle:=nil;
  for I:=Cul_de_sac.Count-1 downto 0 do begin
    TConcentre(Cul_de_sac[I]).Free;
    Cul_de_sac.Delete(I);
  end;
  Cul_de_sac.Free; Cul_de_sac:=nil;
  for I:=Noeud.Count-1 downto 0 do begin
    TPointTopo(Noeud[I]).Free;
    Noeud.Delete(I);
  end;
  Noeud.Free; Noeud:=nil;
  inherited Destroy;
end;

procedure TCalculateur.Initialise(var Les_Fiches, Les_Points : TList;Mode : integer);
var
  I, J : integer;
  Con : TConcentre;
  Point : TPointTopo;
begin
  Source:=Les_Fiches;
  for I:=0 to Les_Fiches.Count-1 do with TFiche(Les_Fiches[I]) do begin
    for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
      if (Mode=moBoucle) or (not(En_Coupe) and not Depart.Terminus
            and not Arrivee.Terminus) then begin
        Con:=TConcentre.Create;
        Con.Resume(TGalerie(Liste_Galerie[J]),I,J);
        Fiches.Add(Con);
      end;
    end;
  end;
  for I:=0 to Les_Points.Count-1 do begin
    Point:=TPointTopo.Create;
    Point.Duplique(TPointTopo(Les_Points[I]));
    Reference.Add(Point);
  end;
end;

function TCalculateur.Ajouter_Point(Le_Point : TPointTopo) : integer;
var
  I : integer;
  Nvx_Point : TPointTopo;
  Nvx : boolean;
begin
  if not Le_Point.Terminus then begin
    I:=0;
    Nvx:=true;
    repeat
      Nvx_Point:=Reference[I];
      if Nvx_Point.Identique(Le_Point) then Nvx:=false;
      inc(I);
    until (I>=Reference.Count) or not Nvx;
    if not Nvx then Ajouter_Point:=0
    else begin
      I:=Noeud.Count-1;
      Nvx:=true;
      if Noeud.Count>0 then
      repeat
        Nvx_Point:=Noeud[I];
        if Nvx_Point.Identique(Le_Point) then Nvx:=false;
        dec(I);
      until not Nvx or (I<0);
      if Nvx then begin
        Nvx_Point:=TPointTopo.Create;
        Nvx_Point.Duplique(Le_Point);
        Noeud.Add(Nvx_Point);
        Ajouter_Point:=Noeud.Count;
        if Noeud.Count>Max_Point then begin
          if not Calcul_Arrete then ShowMessage('Massif trop gros,'#13'Demandez une version spciale de CyberTopo');
          Calcul_Arrete:=true;
        end;
      end
      else Ajouter_Point:=I+2;
    end;
  end
  else Ajouter_Point:=-1;
end;

function TCalculateur.Atteint(Echo : boolean) : boolean;
type
  TCocher = array[0..10000] of boolean;
  PCocher = ^TCocher;
var
  Cocher : PCocher;
  Changement, Reussite, PasRien : boolean;
  I, J, Nb_Suppr : integer;
  Resultat, S : string;

  Premier : boolean;
  Virage, Borne, Borne2 : integer;
begin
  if Reference.Count=0 then begin
    Reussite:=false;
    if Echo=Bruyant then MessageDlg('Il n''y a pas de point de rfrence.', mtConfirmation,
                   [mbOk],0);
  end
  else begin
    for J:=0 to Reference.Count-1 do with TPointTopo(Reference[J]) do begin
      X:=X*1000;
      Y:=Y*1000;
    end;
    GetMem(Cocher,sizeOf(Boolean)*Fiches.Count);
    for I:=0 to Fiches.Count-1 do Cocher^[I]:=false;
(*    for J:=0 to Reference.Count-1 do begin
      Un_Point:=Reference[J];
      for I:=0 to Fiches.Count-1 do if not Cocher^[I] then begin
        Conc:=Fiches[I];
        if Conc.Depart.Identique(Un_Point) then begin
          Conc.Depart.X:=Un_Point.X;
          Conc.Depart.Y:=Un_Point.Y;
          Conc.Depart.Z:=Un_Point.Z;
          Cocher^[I]:=true;
          Conc.N_Arrivee:=Ajouter_Point(Conc.Arrivee);
          if Conc.Approximation and (Conc.N_Arrivee>0) then Approx^[Conc.N_Arrivee]:=true;
          Changement:=true;
        end
        else if Conc.Arrivee.Identique(Un_Point) then begin
          Conc.Arrivee.X:=Un_Point.X;
          Conc.Arrivee.Y:=Un_Point.Y;
          Conc.Arrivee.Z:=Un_Point.Z;
          Cocher^[I]:=true;
          Conc.N_Depart:=Ajouter_Point(Conc.Depart);
          if Conc.Approximation and (Conc.N_Depart>0) then Approx^[Conc.N_Depart]:=true;
          Changement:=true;
        end;
      end;
    end;

    J:=0;
    if Changement and (Noeud.Count<>0) then repeat
      Un_Point:=Noeud[J];
      for I:=0 to Fiches.Count-1 do if not Cocher^[I] then begin
        Conc:=Fiches[I];
        if Conc.Depart.Identique(Un_Point) then begin
          Cocher^[I]:=true;
          Conc.N_Arrivee:=Ajouter_Point(Conc.Arrivee);
          Conc.N_Depart:=J+1;
          if Conc.Approximation and (Conc.N_Arrivee>0) then Approx^[Conc.N_Arrivee]:=true;
          if Conc.Approximation and (Conc.N_Depart>0) then Approx^[Conc.N_Depart]:=true;
        end
        else if Conc.Arrivee.Identique(Un_Point) then begin
          Cocher^[I]:=true;
          Conc.N_Depart:=Ajouter_Point(Conc.Depart);
          Conc.N_Arrivee:=J+1;
          if Conc.Approximation and (Conc.N_Arrivee>0) then Approx^[Conc.N_Arrivee]:=true;
          if Conc.Approximation and (Conc.N_Depart>0) then Approx^[Conc.N_Depart]:=true;
        end;
      end;
      inc(J);
    until (J>=Noeud.Count) or Calcul_Arrete; *)

    Premier:=true;
    Virage:=-1;
    Borne:=0; Borne2:=0;
    repeat
      Changement:=false;
      for I:=Virage+1 to Fiches.Count-1 do if not Cocher^[I] then with TConcentre(Fiches[I]) do begin
        if Premier then begin
          for J:=0 to Reference.Count-1 do with TPointTopo(Reference[J]) do begin
            if Identique(Depart) then begin
              Depart.X:=X;
              Depart.Y:=Y;
              Depart.Z:=Z;
              Cocher^[I]:=true;
              N_Arrivee:=Ajouter_Point(Arrivee);
              if Approximation and (N_Arrivee>0) then Approx^[N_Arrivee]:=true;
              Changement:=true;
            end
            else if Identique(Arrivee) then begin
              Arrivee.X:=X;
              Arrivee.Y:=Y;
              Arrivee.Z:=Z;
              Cocher^[I]:=true;
              N_Depart:=Ajouter_Point(Depart);
              if Approximation and (N_Depart>0) then Approx^[N_Depart]:=true;
              Changement:=true;
            end;
          end;
        end
        else begin
          J:=Noeud.Count-1;
          repeat
            with TPointTopo(Noeud[J]) do
            if Identique(Depart) then begin
              Cocher^[I]:=true;
              N_Arrivee:=Ajouter_Point(Arrivee);
              N_Depart:=J+1;
              if Approximation and (N_Arrivee>0) then Approx^[N_Arrivee]:=true;
              if Approximation and (N_Depart>0) then Approx^[N_Depart]:=true;
              Changement:=true; Virage:=I;
            end
            else if Identique(Arrivee) then begin
              Cocher^[I]:=true;
              N_Depart:=Ajouter_Point(Depart);
              N_Arrivee:=J+1;
              if Approximation and (N_Arrivee>0) then Approx^[N_Arrivee]:=true;
              if Approximation and (N_Depart>0) then Approx^[N_Depart]:=true;
              Changement:=true; Virage:=I;
            end;
            dec(J);
          until (J<Borne) or Cocher^[I];
        end;
      end;
      if Premier then begin
        Virage:=Fiches.Count;
        Premier:=false;
      end;
      Borne:=Borne2;
      Borne2:=Noeud.Count-1;
      if Changement then begin
        Changement:=false;
        for I:=Virage-1 downto 0 do if not Cocher^[I] then with TConcentre(Fiches[I]) do begin
          J:=Noeud.Count-1;
          repeat
            with TPointTopo(Noeud[J]) do
            if Identique(Arrivee) then begin
              Cocher^[I]:=true;
              N_Depart:=Ajouter_Point(Depart);
              N_Arrivee:=J+1;
              if Approximation and (N_Arrivee>0) then Approx^[N_Arrivee]:=true;
              if Approximation and (N_Depart>0) then Approx^[N_Depart]:=true;
              Changement:=true; Virage:=I;
            end
            else if Identique(Depart) then begin
              Cocher^[I]:=true;
              N_Arrivee:=Ajouter_Point(Arrivee);
              N_Depart:=J+1;
              if Approximation and (N_Arrivee>0) then Approx^[N_Arrivee]:=true;
              if Approximation and (N_Depart>0) then Approx^[N_Depart]:=true;
              Changement:=true; Virage:=I;
            end;
            dec(J);
          until (J<Borne) or Cocher^[I];
        end;
      end;
      Borne:=Borne2;
      Borne2:=Noeud.Count-1;
    until not Changement;

{  MessageDlg(Fin+' s', mtInformation,[mbOk],0); }
    if not Calcul_Arrete then begin
      Reussite:=true;
      PasRien:=false;
      for I:=0 to Fiches.Count-1 do begin
        Reussite:=(Reussite and Cocher^[I]);
        PasRien:=(PasRien or Cocher^[I]);
      end;
      Nb_Suppr:=0;
      if not PasRien then begin
        Reussite:=false;
        Tout_Faux:=true;
        if Echo=Bruyant then begin
          Screen.Cursor:=crDefault;
          ShowMessage('Aucune fiche n''est atteinte, vrifiez les noms des points de rfrence.');
          Screen.Cursor:=crHourGlass;
        end;
      end else
      if not Reussite then begin
        Imparfait:=true;
        Lacunes:=TLacunes.Create(application);
        if Echo=Bruyant then begin
          for I:=0 to Fiches.Count-1 do begin
            if not Cocher^[I] then with TConcentre(Fiches[I]) do begin
              for J:=1 to Nombre_Fiche do with TFiche(Source[Anciennes_Fiche^[J]]) do
               with TGalerie(Liste_Galerie[Anciennes_Galerie^[J]]) do begin
                Systeme.Lit(S); Resultat:=S;
                Cavite.Lit(S); Resultat:=Resultat+'/'+S;
                Reseau.Lit(S); Resultat:=Resultat+'/'+S;
                Lieu.Lit(S); Resultat:=Resultat+'/'+S;
                if Depart.Terminus then Resultat:=Resultat+' Terminus - '
                else begin
                  Depart.Nom.Lit(S); Resultat:=Resultat+' '+S+' - ';
                end;
                if Arrivee.Terminus then Resultat:=Resultat+'Terminus'
                else begin
                  Arrivee.Nom.Lit(S); Resultat:=Resultat+S;
                end;
                Lacunes.LaListe.Lines.Add(Resultat);
              end;
            end;
          end;
        end;
        Screen.Cursor:=crDefault;
        if (Echo=Silencieux) or (mrOk=Lacunes.ShowModal) then begin
          Reussite:=true;
          for I:=0 to Fiches.Count-1 do begin
            if not Cocher^[I] then begin
              TConcentre(Fiches[I-Nb_Suppr]).Free;
              Fiches.Delete(I-Nb_Suppr);
              inc(Nb_Suppr);
            end;
          end;
          Fiches.pack;
        end;
        Lacunes.Free;
        if Echo=Bruyant then Screen.Cursor:=crHourGlass;
      end;
      FreeMem(Cocher,sizeOf(Boolean)*Fiches.Count);
    end;
  end;
  Atteint:=Reussite; Utilise:=Reussite;
end;

procedure TCalculateur.Fusion(N_Point : integer);
var
  Fiche_1, Fiche_2, Nvlle : TConcentre;
  I : integer;
begin
  I:=0;
  while (TConcentre(Fiches[I]).N_Depart<>N_Point) and (TConcentre(Fiches[I]).N_Arrivee<>N_Point) do inc(I);
  Fiche_1:=Fiches[I];
  inc(I);
  while (TConcentre(Fiches[I]).N_Depart<>N_Point) and (TConcentre(Fiches[I]).N_Arrivee<>N_Point) do inc(I);
  Fiche_2:=Fiches[I];
  Nvlle:=TConcentre.Create;
  with Nvlle do begin
    Nombre_Fiche:=Fiche_1.Nombre_Fiche+Fiche_2.Nombre_Fiche;
    GetMem(Anciennes_Fiche,sizeOf(Longint)*Nombre_Fiche);
    GetMem(Anciennes_Galerie,sizeOf(Longint)*Nombre_Fiche);
    GetMem(Inverse,sizeOf(boolean)*Nombre_Fiche);
    Depart:=TPointTopo.Create;
    Arrivee:=TPointTopo.Create;
    if Fiche_1.N_Depart<>N_Point then begin
      Depart.Duplique(Fiche_1.Depart);
      if Fiche_2.N_Arrivee<>N_Point then begin
        Arrivee.Duplique(Fiche_2.Arrivee);
        N_Depart:=Fiche_1.N_Depart;
        N_Arrivee:=Fiche_2.N_Arrivee;
        DX:=Fiche_1.DX+Fiche_2.DX;
        DY:=Fiche_1.DY+Fiche_2.DY;
        DZ:=Fiche_1.DZ+Fiche_2.DZ;
        for I:=1 to Fiche_1.Nombre_Fiche do begin
          Anciennes_Fiche^[I]:=Fiche_1.Anciennes_Fiche^[I];
          Anciennes_Galerie^[I]:=Fiche_1.Anciennes_Galerie^[I];
          Inverse^[I]:=Fiche_1.Inverse^[I];
        end;
        for I:=1 to Fiche_2.Nombre_Fiche do begin
          Anciennes_Fiche^[I+Fiche_1.Nombre_Fiche]:=Fiche_2.Anciennes_Fiche^[I];
          Anciennes_Galerie^[I+Fiche_1.Nombre_Fiche]:=Fiche_2.Anciennes_Galerie^[I];
          Inverse^[I+Fiche_1.Nombre_Fiche]:=Fiche_2.Inverse^[I];
        end;
      end
      else begin
        Arrivee.Duplique(Fiche_2.Depart);
        N_Depart:=Fiche_1.N_Depart;
        N_Arrivee:=Fiche_2.N_Depart;
        DX:=Fiche_1.DX-Fiche_2.DX;
        DY:=Fiche_1.DY-Fiche_2.DY;
        DZ:=Fiche_1.DZ-Fiche_2.DZ;
        for I:=1 to Fiche_1.Nombre_Fiche do begin
          Anciennes_Fiche^[I]:=Fiche_1.Anciennes_Fiche^[I];
          Anciennes_Galerie^[I]:=Fiche_1.Anciennes_Galerie^[I];
          Inverse^[I]:=Fiche_1.Inverse^[I];
        end;
        for I:=1 to Fiche_2.Nombre_Fiche do begin
          Anciennes_Fiche^[Nombre_Fiche+1-I]:=Fiche_2.Anciennes_Fiche^[I];
          Anciennes_Galerie^[Nombre_Fiche+1-I]:=Fiche_2.Anciennes_Galerie^[I];
          Inverse^[Nombre_Fiche+1-I]:=not Fiche_2.Inverse^[I];
        end;
      end;
    end
    else begin
      if Fiche_2.N_Arrivee<>N_Point then begin
        Depart.Duplique(Fiche_1.Arrivee);
        Arrivee.Duplique(Fiche_2.Arrivee);
        N_Depart:=Fiche_1.N_Arrivee;
        N_Arrivee:=Fiche_2.N_Arrivee;
        DX:=-Fiche_1.DX+Fiche_2.DX;
        DY:=-Fiche_1.DY+Fiche_2.DY;
        DZ:=-Fiche_1.DZ+Fiche_2.DZ;
        for I:=1 to Fiche_1.Nombre_Fiche do begin
          Anciennes_Fiche^[Fiche_1.Nombre_Fiche+1-I]:=Fiche_1.Anciennes_Fiche^[I];
          Anciennes_Galerie^[Fiche_1.Nombre_Fiche+1-I]:=Fiche_1.Anciennes_Galerie^[I];
          Inverse^[Fiche_1.Nombre_Fiche+1-I]:=not Fiche_1.Inverse^[I];
        end;
        for I:=1 to Fiche_2.Nombre_Fiche do begin
          Anciennes_Fiche^[I+Fiche_1.Nombre_Fiche]:=Fiche_2.Anciennes_Fiche^[I];
          Anciennes_Galerie^[I+Fiche_1.Nombre_Fiche]:=Fiche_2.Anciennes_Galerie^[I];
          Inverse^[I+Fiche_1.Nombre_Fiche]:=Fiche_2.Inverse^[I];
        end;
      end
      else begin
        Arrivee.Duplique(Fiche_1.Arrivee);
        Depart.Duplique(Fiche_2.Depart);
        N_Depart:=Fiche_2.N_Depart;
        N_Arrivee:=Fiche_1.N_Arrivee;
        DX:=Fiche_1.DX+Fiche_2.DX;
        DY:=Fiche_1.DY+Fiche_2.DY;
        DZ:=Fiche_1.DZ+Fiche_2.DZ;
        for I:=1 to Fiche_2.Nombre_Fiche do begin
          Anciennes_Fiche^[I]:=Fiche_2.Anciennes_Fiche^[I];
          Anciennes_Galerie^[I]:=Fiche_2.Anciennes_Galerie^[I];
          Inverse^[I]:=Fiche_2.Inverse^[I];
        end;
        for I:=1 to Fiche_1.Nombre_Fiche do begin
          Anciennes_Fiche^[Fiche_2.Nombre_Fiche+I]:=Fiche_1.Anciennes_Fiche^[I];
          Anciennes_Galerie^[Fiche_2.Nombre_Fiche+I]:=Fiche_1.Anciennes_Galerie^[I];
          Inverse^[Fiche_2.Nombre_Fiche+I]:=Fiche_1.Inverse^[I];
        end;
      end;
    end;
    L:=Fiche_1.L+Fiche_2.L;
    LP:=Fiche_1.LP+Fiche_2.LP;
    L2:=Fiche_1.L2+Fiche_2.L2;
  end;
  Fiches.Remove(Fiche_1);
  Fiches.Remove(Fiche_2);
  Fiches.Add(Nvlle);
  Fiches.Pack;
  Fiche_1.Free; Fiche_1:=nil;
  Fiche_2.Free; Fiche_2:=nil;
end;


procedure TCalculateur.Reduction(Affichage : boolean);
type
  TCompteur = array[1..10000] of byte;
  PCompteur = ^TCompteur;
var
  Compteur : PCompteur;
  Conc : TConcentre;
  I, J, CND, CNA : integer;
  Changement : Boolean;
  S : string;
begin
  S:=IntTostr(Fiches.Count)+' galeries,';
  GetMem(Compteur,sizeOf(byte)*Noeud.Count);
  for I:=1 to Noeud.Count do Compteur^[I]:=0;
  for I:=0 to Fiches.Count-1 do with TConcentre(Fiches[I]) do begin
    J:=N_Depart;
    if J>0 then inc(Compteur^[J]);
    J:=N_Arrivee;
    if J>0 then inc(Compteur^[J]);
  end;
  for I:=1 to Noeud.Count do if Compteur^[I]=2 then begin
    Fusion(I);
    Compteur^[I]:=0;
  end;
  repeat
    Changement:=false;
    I:=0;
    repeat

{----------- Elimination des boucles ---------------}
      Conc:=Fiches[I];
      inc(I);
      CND:=Conc.N_Depart; CNA:=Conc.N_Arrivee;
      if (Conc.N_Depart>=0) and (Conc.N_Depart=Conc.N_Arrivee) then begin
        Changement:=true;
        Fiches.Remove(Conc);
        Fiches.pack;
        if Conc.N_Depart>0 then begin
          dec(Compteur^[Conc.N_Depart],2);
          if Compteur^[Conc.N_Depart]=2 then begin
            Fusion(Conc.N_Depart);
            Compteur^[Conc.N_Depart]:=0;
          end;
        end;
        Boucle.Add(Conc);
        I:=0;
      end;
    until I>=Fiches.Count;
    if Fiches.Count>0 then begin
      I:=0;
      repeat
{------------- Elimination des culs de sacs -------------}
        Conc:=Fiches[I];
        inc(I);
        CND:=Conc.N_Depart; CNA:=Conc.N_Arrivee;
        if (CND=-1) or (CNA=-1)  or ((CND>0) and (Compteur^[CND]=1))
                                 or ((CNA>0) and (Compteur^[CNA]=1)) then begin
          Changement:=true;
          Fiches.Remove(Conc);
          Fiches.Pack;
          if (CND<>-1) and ((CND<=0) or (Compteur^[CND]<>1)) then begin
            if CND<>0 then begin
              dec(Compteur^[CND]);
              if Compteur^[CND]=2 then begin
                Fusion(CND);
                Compteur^[CND]:=0;
              end;
            end;
          end
          else if CNA<>0 then begin
            dec(Compteur^[CNA]);
            if Compteur^[CNA]=2 then begin
              Fusion(CNA);
              Compteur^[CNA]:=0;
            end;
          end;
          Cul_de_Sac.Add(Conc);
        end;
      until I>=Fiches.Count;
    end;
  until not Changement or (Fiches.Count=0) or Calcul_Arrete;
  if Affichage then begin
    Screen.Cursor:=crDefault;

    S:=S+' '+IntTostr(Fiches.Count)+' branches,';
    S:=S+' '+IntTostr(Boucle.Count)+' boucles,';
    S:=S+' '+IntTostr(Cul_de_sac.Count)+' cul de sac,';
    MessageDlg(S, mtInformation,[mbOk],0);
    Screen.Cursor:=crHourGlass;
  end;
  FreeMem(Compteur,sizeOf(byte)*Noeud.Count);
end;

procedure TCalculateur.Approxime;
var
  I, Dep, Arr : integer;
  Changement : boolean;
  Longueur : double;
  Destination : TPointTopo;
begin
  for I:=1 to Noeud.Count do Vue^[I]:=false;
  for I:=1 to Noeud.Count do if Approx^[I] then begin
    Distance^[I]:=0;
  end;
  for I:=1 to Fiches.Count do with TConcentre(Fiches[I-1]) do begin
    Dep:=N_Depart;
    Arr:=N_Arrivee;
//    if Dep<>0 then if Approx^[Dep] then Vue^[Dep]:=true;
//    if Arr<>0 then if Approx^[Arr] then Vue^[Arr]:=true;
    if (Dep=0) or (Arr=0) then begin
      Longueur:=LP;
    end;
    if (Dep=0) and (Arr<>0) then begin
      if not Vue^[Arr] or (Vue^[Arr] and (Distance^[Arr]>Longueur)) then begin
        Vue^[Arr]:=true;
        Distance^[Arr]:=Longueur;
        with TPointTopo(Noeud[Arr-1]) do begin
          X:=Depart.X+DX;
          Y:=Depart.Y+DY;
          Z:=Depart.Z+DZ;
        end;
      end;
    end;
    if (Dep<>0) and (Arr=0) then begin
      if not Vue^[Dep] or (Vue^[Dep] and (Distance^[Dep]>Longueur)) then begin
        Vue^[Dep]:=true;
        Distance^[Dep]:=Longueur;
        with TPointTopo(Noeud[Dep-1]) do begin
          X:=Arrivee.X-DX;
          Y:=Arrivee.Y-DY;
          Z:=Arrivee.Z-DZ;
        end;
      end;
    end;
  end;
  repeat
    Changement:=false;
    for I:=1 to Fiches.Count do with TConcentre(Fiches[I-1]) do begin
      Dep:=N_Depart;
      Arr:=N_Arrivee;
      Longueur:=LP;
      if (Dep<>0) and (Arr<>0) then begin
        if (Vue^[Dep]) and
        (not Vue^[Arr]
        or (Vue^[Arr] and (Distance^[Arr]>Distance^[Dep]+Longueur+1e-5))) then begin
          Changement:=true;
          Vue^[Arr]:=true;
          Distance^[Arr]:=Distance^[Dep]+Longueur;
          with TPointTopo(Noeud[Dep-1]) do begin
            Destination:=Noeud[Arr-1];
            Destination.X:=X+DX;
            Destination.Y:=Y+DY;
            Destination.Z:=Z+DZ;
          end;
        end;
        if (Vue^[Arr]) and
        (not Vue^[Dep]
        or (Vue^[Dep] and (Distance^[Dep]>Distance^[Arr]+Longueur+1e-5))) then begin
          Changement:=true;
          Vue^[Dep]:=true;
          Distance^[Dep]:=Distance^[Arr]+Longueur;
          with TPointTopo(Noeud[Arr-1]) do begin
            Destination:=Noeud[Dep-1];
            Destination.X:=X-DX;
            Destination.Y:=Y-DY;
            Destination.Z:=Z-DZ;
          end;
        end;
      end;
    end;
  until not Changement;
end;

procedure TCalculateur.La_Totale(Affichage : boolean);
var
  I, J, P, Q, Compteur : integer;
  MatriceX, MatriceY, MatriceZ : TMatrice;
  K : double;
  NP, NQ : TPointTopo;
  S : string;
begin
  GetMem(Vue,sizeOf(boolean)*Noeud.Count);
  GetMem(Distance,sizeOf(double)*Noeud.Count);
  GetMem(Numero,sizeOf(longint)*Noeud.Count);
  Approxime;
  Compteur:=0;
  for I:=1 to Noeud.Count do if Vue^[I] then begin
    inc(Compteur);
    Numero^[I]:=Compteur;
  end;
  if Affichage then begin
    S:=IntTostr(Compteur)+' carrefours.';
    Screen.Cursor:=crDefault;
    MessageDlg(S, mtInformation,[mbOk],0);
    Screen.Cursor:=crHourGlass;
    Demarre;
  end;
  MatriceX:=TMatrice.Create(Compteur);
  MatriceY:=TMatrice.Create(Compteur);
  MatriceZ:=TMatrice.Create(Compteur);
  for I:=0 to Fiches.Count-1 do with TConcentre(Fiches[I]) do begin
    P:=N_Depart;
    Q:=N_Arrivee;
    if P=0 then NP:=Depart
           else begin
      NP:=Noeud[P-1];
      P:=Numero^[P];
    end;
    if Q=0 then NQ:=Arrivee
           else begin
      NQ:=Noeud[Q-1];
      Q:=Numero^[Q];
    end;
    K:=DX-(NQ.X-NP.X);
    MatriceX.Rempli(P,Q,L2,K);
    K:=DY-(NQ.Y-NP.Y);

    MatriceY.Rempli(P,Q,L2,K);
    K:=DZ-(NQ.Z-NP.Z);
    MatriceZ.Rempli(P,Q,L2,K);
  end;
  if not Calcul_Arrete then MatriceX.Inverse;
  if not Calcul_Arrete then MatriceY.Inverse;
  if not Calcul_Arrete then MatriceZ.Inverse;
  if not Calcul_Arrete then for I:=1 to Noeud.Count do if Vue^[I] then
   with TPointTopo(Noeud[I-1]) do begin
    J:=Numero^[I];
    X:=X+MatriceX.Droite^[J-1];
    Y:=Y+MatriceY.Droite^[J-1];
    Z:=Z+MatriceZ.Droite^[J-1];
  end;
  MatriceX.Free; MatriceX:=nil;
  MatriceY.Free; MatriceY:=nil;
  MatriceZ.Free; MatriceZ:=nil;
  FreeMem(Numero,sizeOf(longint)*Noeud.Count);
  FreeMem(Distance,sizeOf(double)*Noeud.Count);
  if Affichage then begin
    Screen.Cursor:=crDefault;
    MessageDlg(Fin+' s', mtInformation,[mbOk],0);
    Screen.Cursor:=crHourGlass;
  end;
end;


procedure TCalculateur.Fission;
var
  I, ND, NA : integer;
  Changement : boolean;
  Une_Fiche : TConcentre;
  Traite_B, Traite_Cds : PSens;
begin
  GetMem(Traite_B,sizeOf(Boolean)*Boucle.Count);
  for I:=1 to Boucle.Count do Traite_B^[I]:=false;
  GetMem(Traite_cds,sizeOf(Boolean)*Cul_de_sac.Count);
  for I:=1 to Cul_De_Sac.Count do Traite_Cds^[I]:=false;
  for I:=0 to Fiches.Count-1 do Decompresse(TConcentre(Fiches[I]),deBranche);
  repeat
    Changement:=false;
    for I:=Boucle.Count-1 downto 0 do if not Traite_B^[I+1] then begin
      Une_Fiche:=Boucle[I];
      ND:=Une_Fiche.N_Depart;
      if (ND=0) or Vue^[ND] then begin
        Changement:=true;
        Decompresse(Une_Fiche,deBoucle);
        Traite_B^[I+1]:=true;
      end;
    end;
    for I:=Cul_de_sac.Count-1 downto 0 do if not Traite_cds^[I+1] then begin
      Une_Fiche:=Cul_de_sac[I];
      ND:=Une_Fiche.N_Depart;
      NA:=Une_Fiche.N_Arrivee;
      if (ND=0) or ((ND>0) and Vue^[ND]) or
         (NA=0) or ((NA>0) and Vue^[NA]) then begin
        Changement:=true;
        Decompresse(Une_Fiche,deCul_de_sac);
        Traite_cds^[I+1]:=true;
      end;
    end;
  until not Changement;
  FreeMem(Traite_B,sizeOf(Boolean)*Boucle.Count);
  FreeMem(Traite_cds,sizeOf(Boolean)*Cul_de_sac.Count);
end;

procedure TCalculateur.Decompresse(La_Fiche : TConcentre; Nature : shortInt);
var
  I, J : integer;
  CX, CY, CZ, UX, UY, UZ : double;
  Bon_Sens : boolean;
  Un_Point, Pt_Debut, Pt_Fin : TPointTopo;
  Anc_Fiche : TFiche;
  Anc_Galerie : TGalerie;
  S : string;
begin
  Bon_Sens:=true;
  case La_Fiche.N_Depart of
    -1 : Bon_Sens:=false;
    0 : begin
      for I:=0 to Reference.Count-1 do begin
        Un_Point:=Reference[I];
        if La_Fiche.Depart.Identique(Un_Point) then Pt_Debut:=Un_Point;
      end;
    end;
    else begin
      if Vue^[La_Fiche.N_Depart] then Pt_Debut:=Noeud[La_Fiche.N_Depart-1]
      else Bon_Sens:=false;
    end;
  end;
  case La_Fiche.N_Arrivee of
    -1 : begin end;
    0 : begin
      for I:=0 to Reference.Count-1 do with TPointTopo(Reference[I]) do
        if La_Fiche.Arrivee.Identique(TPointTopo(Reference[I])) then Pt_Fin:=TPointTopo(Reference[I]);
    end;
    else begin
      if Vue^[La_Fiche.N_Arrivee] then Pt_Fin:=Noeud[La_Fiche.N_Arrivee-1];
    end;
  end;

  case Nature of
    deBranche : with La_Fiche do begin
      CX:=((Pt_Fin.X-Pt_Debut.X)-DX)/LP;
      CY:=((Pt_Fin.Y-Pt_Debut.Y)-DY)/LP;
      CZ:=((Pt_Fin.Z-Pt_Debut.Z)-DZ)/LP;
      Contrainte_Plan:=sqrt(sqr(((Pt_Fin.X-Pt_Debut.X)-DX)/L2)+
                                sqr(((Pt_Fin.Y-Pt_Debut.Y)-DY)/L2));
      Contrainte_Z:=abs(((Pt_Fin.Z-Pt_Debut.Z)-DZ)/L2);
    end;
    deBoucle : with La_Fiche do begin
      if Pt_Debut.Identique(Pt_Fin) then begin
        CX:=-DX/LP;
        CY:=-DY/LP;
        CZ:=-DZ/LP;
        Contrainte_Plan:=sqrt(sqr(DX/L2)+
                                  sqr(DY/L2));
        Contrainte_Z:=abs(DZ/L2);
      end
      else begin
        CX:=((Pt_Fin.X-Pt_Debut.X)-DX)/LP;
        CY:=((Pt_Fin.Y-Pt_Debut.Y)-DY)/LP;
        CZ:=((Pt_Fin.Z-Pt_Debut.Z)-DZ)/LP;
        Contrainte_Plan:=sqrt(sqr(((Pt_Fin.X-Pt_Debut.X)-DX)/L2)+
                                  sqr(((Pt_Fin.Y-Pt_Debut.Y)-DY)/L2));
        Contrainte_Z:=abs(((Pt_Fin.Z-Pt_Debut.Z)-DZ)/L2);
      end;
    end;
    deCul_de_sac : with La_Fiche do begin
      CX:=0;
      CY:=0;
      CZ:=0;
      Contrainte_Plan:=0;
      Contrainte_Z:=0;
    end;
  end;
  if Bon_Sens then begin
    UX:=Pt_Debut.X;
    UY:=Pt_Debut.Y;
    UZ:=Pt_Debut.Z;
  end
  else begin
    UX:=Pt_Fin.X;
    UY:=Pt_Fin.Y;
    UZ:=Pt_Fin.Z;
  end;
  if Bon_Sens then with La_Fiche do begin
    for I:=1 to Nombre_Fiche do begin
      Anc_Fiche:=Source[La_Fiche.Anciennes_Fiche^[I]];
      Anc_Galerie:=Anc_Fiche.Liste_Galerie[La_Fiche.Anciennes_Galerie^[I]];
      Anc_Galerie.Contrainte_Plan:=La_Fiche.Contrainte_Plan;
      Anc_Galerie.Contrainte_Z:=La_Fiche.Contrainte_Z;
      if (I<La_Fiche.Nombre_Fiche) or (Nature=deCul_de_sac) then begin
        if La_Fiche.Inverse^[I] then begin
          UX:=UX-Anc_Galerie.DeltaX+CX*Anc_Galerie.Precision*Anc_Galerie.SigmaL;
          UY:=UY-Anc_Galerie.DeltaY+CY*Anc_Galerie.Precision*Anc_Galerie.SigmaL;
          UZ:=UZ-Anc_Galerie.DeltaZ+CZ*Anc_Galerie.Precision*Anc_Galerie.SigmaL;
          J:=Ajouter_Point(Anc_Galerie.Depart);
          if J<>-1 then begin
            Un_Point:=Noeud[J-1];
            Vue^[J]:=true;
            Un_Point.X:=UX; Un_Point.Y:=UY; Un_Point.Z:=UZ;
          end;
        end
        else begin
          UX:=UX+Anc_Galerie.DeltaX+CX*Anc_Galerie.Precision*Anc_Galerie.SigmaL;
          UY:=UY+Anc_Galerie.DeltaY+CY*Anc_Galerie.Precision*Anc_Galerie.SigmaL;
          UZ:=UZ+Anc_Galerie.DeltaZ+CZ*Anc_Galerie.Precision*Anc_Galerie.SigmaL;
          J:=Ajouter_Point(Anc_Galerie.Arrivee);
          Anc_Galerie.Arrivee.Nom.Lit(S);
          if J<>-1 then begin
            Un_Point:=Noeud[J-1];
            Vue^[J]:=true;
            Un_Point.X:=UX; Un_Point.Y:=UY; Un_Point.Z:=UZ;
          end;
        end;
      end;
    end;
  end
  else with La_Fiche do begin
    for I:=Nombre_Fiche downto 1 do begin
      Anc_Fiche:=Source[Anciennes_Fiche^[I]];
      Anc_Galerie:=Anc_Fiche.Liste_Galerie[Anciennes_Galerie^[I]];
      Anc_Galerie.Contrainte_Plan:=Contrainte_Plan;
      Anc_Galerie.Contrainte_Z:=Contrainte_Z;
      if Inverse^[I] then begin
        UX:=UX+Anc_Galerie.DeltaX;
        UY:=UY+Anc_Galerie.DeltaY;
        UZ:=UZ+Anc_Galerie.DeltaZ;
        J:=Ajouter_Point(Anc_Galerie.Arrivee);
        if J<>-1 then begin
          Un_Point:=Noeud[J-1];
          Vue^[J]:=true;
          Un_Point.X:=UX; Un_Point.Y:=UY; Un_Point.Z:=UZ;
        end;
      end
      else begin
        UX:=UX-Anc_Galerie.DeltaX;
        UY:=UY-Anc_Galerie.DeltaY;
        UZ:=UZ-Anc_Galerie.DeltaZ;
        J:=Ajouter_Point(Anc_Galerie.Depart);
        if J<>-1 then begin
          Un_Point:=Noeud[J-1];
          Vue^[J]:=true;
          Un_Point.X:=UX; Un_Point.Y:=UY; Un_Point.Z:=UZ;
        end;
      end;
    end;
  end;
end;

procedure TCalculateur.Retour_Info;
var
  I, J, ND, NA, Decalage, Numero : integer;
  Un_Noeud : TPointTopo;
  Deuxieme : boolean;

  function Hors_Jeu(var Un_Point : TPointTopo) : boolean;
  var
    I : integer;
    Vu : boolean;
  begin
    Vu:=false;
    if Un_Point.Terminus then begin
      Numero:=-1;
    end
    else begin
      for I:=0 to Reference.Count-1 do begin
        if Un_Point.Identique(Reference[I]) then begin
          Vu:=true;
          Numero:=0;
          Un_Point.X:=TPointTopo(Reference[I]).X;
          Un_Point.Y:=TPointTopo(Reference[I]).Y;
          Un_Point.Z:=TPointTopo(Reference[I]).Z;
        end;
      end;
      if not Vu then begin
        I:=0;
        while (I<Noeud.Count) and not Un_Point.Identique(Noeud[(I+Decalage) mod Noeud.Count]) do inc(I);
        Vu:=I<>Noeud.Count;
        if Vu then begin
          Decalage:=(I+Decalage) mod Noeud.Count;
          Numero:=Decalage+1;
        end;
      end;
    end;
    Hors_Jeu:=not Vu;
  end;


begin
  Decalage:=0;
  for I:=0 to Source.Count-1 do with TFiche(Source[I]) do begin
    for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
      Deuxieme:=Hors_Jeu(Depart);
      ND:=Numero;
      if Not Hors_Jeu(Arrivee) or not Deuxieme then begin
        NA:=Numero;
        case ND of
          0 : begin
            Approximation:=true;
            if (NA>0) then begin
              Un_Noeud:=Noeud[Na-1];
              Arrivee.X:=Un_Noeud.X;
              Arrivee.Y:=Un_Noeud.Y;
              Arrivee.Z:=Un_Noeud.Z;
            end;
          end;
          -1 : if (NA>0) then begin
            Approximation:=true;
            Un_Noeud:=Noeud[Na-1];
            Arrivee.X:=Un_Noeud.X;
            Arrivee.Y:=Un_Noeud.Y;
            Arrivee.Z:=Un_Noeud.Z;
          end
          else if NA=0 then Approximation:=true;
          else begin
            if NA<=0 then begin
              Approximation:=true;
              Un_Noeud:=Noeud[Nd-1];
              Depart.X:=Un_Noeud.X;
              Depart.Y:=Un_Noeud.Y;
              Depart.Z:=Un_Noeud.Z;
            end
            else begin
              Approximation:=true;
              Un_Noeud:=Noeud[Nd-1];
              Depart.X:=Un_Noeud.X;
              Depart.Y:=Un_Noeud.Y;
              Depart.Z:=Un_Noeud.Z;
              Un_Noeud:=Noeud[Na-1];
              Arrivee.X:=Un_Noeud.X;
              Arrivee.Y:=Un_Noeud.Y;
              Arrivee.Z:=Un_Noeud.Z;
            end;
          end;
        end;
      end;
    end;
  end;
end;

procedure TCalculateur.Resoud(Echo : boolean);
begin
  if Echo=Bruyant then Screen.Cursor:=crHourGlass;
  Calcul_En_Cours:=true;
  Calcul_Arrete:=false;
  if Atteint(Echo) then begin
    if not Calcul_Arrete then Reduction(Echo);
    if not Calcul_Arrete then La_Totale(Echo) else Utilise:=false;
    if not Calcul_Arrete then Fission;
    if not Calcul_Arrete then Retour_Info;
  end;
  Screen.Cursor:=crDefault;
end;


end.
