{$N-}
unit Fichetpo;

interface

uses Classes, SysUtils, Dialogs, Forms, Graphics, BufferedFile, Controls;
const

  Version : SmallInt=1005;

type

  TUniteDistance = (m, cm);
  TUniteAngle = (deg, gr, mp, cmp, vdeg, ndeg, ngr, nmp, ncmp, nvdeg);

const

  tuDistance : array[TUniteDistance] of string=('m','cm');
  tuAngle: array[TUniteAngle] of string=('','gr','m','cm','vul','-','-gr','-m','-cm','-vul');

var

  Ver : smallInt;

type
  reel = real48;

  TTotal = array[1..512] of char;
  PTotal = ^TTotal;

  TNom = string[50];

  TPoint3D = record
    X, Y, Z, LH : reel;
  end;

  TEco = class
  public
    Existe : boolean;
    Taille : word;
    Contenu : PTotal;
    constructor Create;
    destructor Destroy; override;
    procedure Ecrit(S : string);
    procedure Lit(var S : string);
    function AsString: string;
    procedure UpString(var S : string);
    procedure Sauve(var F : TFileBufferStream);
    procedure Loade(var F : TFileBufferStream);
    procedure SauveTxt(var F : textFile);
    procedure LoadeTxt(S : string);
    procedure Duplique(Original : TEco);
    function Identique(L_Autre : TEco) : boolean;
  end;

//  TUniteDistance = (m, cm);
//  TUniteAngle = (deg, gr, mp, cmp, vdeg, ndeg, ngr, nmp, ncmp, nvdeg);

  TVisee = class
  public
    Longueur, Azimuth, Pente, X, Y, Z, LH, Debut_Topofil : reel;
    LargeurDroite, LargeurGauche, HauteurSup, HauteurInf : string[10];
    Commentaires : TEco;
    ULong, ULarg, UHauteur : TUniteDistance;
    UAzimuth, UPente : TUniteAngle;
    Schema : byte;
    Longueur_Horizontale, Longueur_Dlt : reel;
    Correction_Horizontale, Correction_Dlt, Topofil : boolean;
    constructor Create;
    destructor Destroy; override;
  end;
  TListeVisee = array[1..128] of TVisee;


  TPointTopo = class
  public
    Terminus : boolean;
    Nom : TEco;
    Indice : byte;
    X, Y, Z : reel;
    constructor Create;
    destructor Destroy; override;
    procedure Loade(var F : TFileBufferStream);
    procedure Sauve(var F : TFileBufferStream);
    procedure SauveTxt(var F : textFile);
    procedure LoadeTxt(var F : textFile);
    function Identique(L_autre : TPointTopo): boolean;
    procedure Duplique(Original : TPointTopo);
  end;

  TListeCoordonnees = array[1..128] of TPoint3D;
  PListeCoordonnees = ^TListeCoordonnees;

  TGalerie = class(Tobject)  // une galerie rejoint deux carrefours topos ou un terminus
  public
    Visees : TListeVisee;  // liste des vise d'une galerie
    Coordonnees : PListeCoordonnees;  // liste des coordonnes de chaque vise
    Depart,                           // point de dpart d'une galerie
    Arrivee : TPointTopo;             // point d'arrive d'une galerie
    DeltaX,                           // diffrence en X entre le dbut et la fin
                                      // de la galerie avant dformation
    DeltaY,                           // idem en Y
    DeltaZ,                           // idem en Z
    SigmaL,                           // somme des longueurs de vises
    SigmaL2 : reel;                   // somme des carrs des longueurs
    Contrainte_Plan,                  // coefficient de dformation en plan suite aux rebouclages
    Contrainte_Z : reel;              // idem en z
    Nb_Visee,                         // nombre de vises
    Taille_Coord : byte;
    Approximation,                    // indique si une approximation des coordonnes
                                      // de chaque vise existe (=Coordonnees renseign)
    Existe_Plan,                      // indique s'il existe un habillage en plan (non implment)
    Existe_Coupe,                     // idem en coupe (non implment)
    Fantome : boolean;                // galerie vide (une seule vise de longueur nulle)
    Developement,                     // developpement de la galerie
    Precision,                        // coefficient de dformation de la galerie
                                      // dans les rebouclages
    Correction_Topofil : reel;        // coefficient de correction  appliquer aux vises Topofil
    Information : TEco;               // texte d'information gnral sur la galerie
    Couleur : TColor;                 // couleur de dessin de la galerie

    En_Coupe,                         // indique si la galerie appartient  une coupe dveloppe
    Accrochage_Debut,                 // indique si la galerie s'accroche  la prcdente
                                      // par le point de dpart ou celui d'arrive
    Gauche_Droite : boolean;          // sens de droulement de la coupe
    Nb_Suite : shortInt;              // nombre de branches de coupe qui font suite
    Ordre_Suite,                      // numro d'ordre des fiches qui contiennent les suites de la coupe
    Galerie_Suite : array[1..5] of longint; // numro des galeries qui font suite
    Decal_Hori,                       // coordonnes horizontale dans la coupe dveloppe
                                      // du dbut de la galerie
    Decal_Z,                          // dcalage par rapport  l'altitude relle de la coupe
    Fin_Hori : reel;                  // coordonnes horizontale dans la coupe dveloppe
                                      // de la fin de la galerie

    constructor Create;
    destructor Destroy; override;
    procedure Detailler;
    procedure Compile(La_Declinaison: reel);
    procedure Sauve(var F : TFileBufferStream);
    procedure Loade(var F : TFileBufferStream);
    procedure SauveTxt(var F : textFile);
    procedure LoadeTxt(var F : textFile);
    function Identique(var L_autre : TGalerie) : boolean;
  end;

  TFiche = class(TObject)
  private
  public
    Liste_Galerie : TList;
    Date : string[8];
    Unite_Decli : TUniteAngle;
    Topographes : TList;
    Systeme, Cavite, Reseau, Lieu, Temps : TEco;
    Declinaison, Premiere : reel;
    Ordre : longint;
    Sommet_Tableau, Base_Tableau : longint;
    constructor Create;
    destructor Destroy; override;
    procedure Compile(Geodesique : reel);
    procedure SauveTxt(var F : textFile);
    procedure LoadeTxt(var F : textFile);
    procedure Sauve(var F : TFileBufferStream);
    procedure Loade(var F : TFileBufferStream);
    function Meme_Equipe(var L_autre : TFiche) : boolean;
    function Meme_Lieu(var L_autre : TFiche) : boolean;
    function Identique(var L_autre : TFiche) : boolean;
    procedure Copie_de(Source : TFiche);
    procedure Les_Hauteurs(N_Galerie,N_Visee : integer;var En_Haut, En_Bas : double);
    procedure Les_Largeurs(N_Galerie,N_Visee : integer;var A_Gauche, A_Droite : double);
  end;

  TRaccourci = class(TObject) // raccourci vers un fichier, non implment actuellement
  private
  public
    constructor Create;
    destructor Destroy; override;
    procedure SauveTxt(var F : textFile);
    procedure LoadeTxt(var F : textFile);
    procedure Sauve(var F : TFileBufferStream);
    procedure Loade(var F : TFileBufferStream);
  end;

  TPointCoupe = class  // point de dmarrage d'une branche de coupe
  public
    Nom : TEco;        // nom du point de dpart
    Indice : byte;     // indice du point de dpart
    Decal_Hori,        // coordonne horizontale dveloppe du point
    Decal_Z : reel;    // dcalage verticale par rapport  l'altitude relle
    Ordre_Suite,       // numro d'ordre de la fiche qui dmarre  partir du point
    Galerie_Suite : longint; // numro de la galerie dans la fiche ci dessus
    constructor Create;
    destructor Destroy; override;
    procedure SauveTxt(var F : textFile);
    procedure LoadeTxt(var F : textFile);
    procedure Loade(var F : TFileBufferStream);
    procedure Sauve(var F : TFileBufferStream);
  end;


  TMassif = class(TObject)
  public
    Selection, Corbeille, Raccourci : TList;
    Liste_Courante, Liste_Fiche, Liste_Reference, Germe_Coupe : TList;
    Existe_Selection, Sauvegarde, Finition, Finition_Coupe, Gabarit_Tableau, Alerte_Perte : boolean;
    Fichier : TFileName;
    Origine_Decli, Variation_Decli, Angle_Geodesique, Seuil_Deformation : reel;
    Date_Decli : string[8];

    Fiche_Recherche, Galerie_Recherche, Visee_Recherche : smallint;
    constructor Create;
    destructor Destroy; override;
    function Nouveau_Point(Proposition : String; I0 : integer) : integer;
    procedure Enregistrer(Destination : TFileName);
{$IFDEF FUSION}
    function Recuperer : boolean;
{$ELSE}
    procedure Recuperer;
{$ENDIF}
    procedure Remplacer(var Ancienne, Nouvelle : TFiche);
    procedure Ajouter(var Nouvelle : TFiche;Local : boolean);
    procedure Retirer(var Ancienne: TFiche);
    procedure Trier;
    procedure Trier_Selection;
    procedure Ecrire_Point(var Nouveau : TPointTopo);
    procedure Effacer_Points;
    procedure Detailler;
    procedure Calculer_Coupe;
    procedure Coupe_Une_Galerie(N_Ordre,N_Galerie : smallInt; DX, DZ : reel);
    procedure Detailler_Coupe;
    procedure Nettoyage_Coupe;
    procedure Detecteur_Erreur_Coupe;
    function Grosse_Deformation(var La_Fiche, La_Galerie : integer):reel;
    procedure Retirer_Coupe(N_Ordre : longint; N_Galerie : integer; Racine : boolean);

  end;

    function Calibre(var PMin, PMax : TPoint3D;La_Liste : TList) : boolean;
    function Calibre_Coupe(var PMin, PMax : TPoint3D;La_Liste : TList) : boolean;
    procedure Tuer_les_Coupe(Une_Fiche : TFiche);
    function Date2Jour(S : string):integer;
    function StrToIntE(S : string) : integer;
    function StrToFloatE(S : string) : extended;

implementation

{$IFNDEF FUSION}
uses Unit1, Unit2;
{$ENDIF}

(*********************** TEco ***************************)

constructor TEco.Create;
begin
  inherited Create;
  Existe:=false;
  Taille:=0;
end;

destructor TEco.Destroy;
{$IFDEF DEBUG}
var
  I : integer;
{$ENDIF}
begin
  if Existe then begin
  {$IFDEF DEBUG}
    for I:=1 to Taille do Contenu^[I]:=chr($A3);
  {$ENDIF}
    FreeMem(Contenu,Taille*sizeOf(Char));
  end;
  inherited Destroy;
end;

procedure TEco.Ecrit(S : string);
var
  I : SmallInt;
begin
  if Existe then begin
  {$IFDEF DEBUG}
    for I:=1 to Taille do Contenu^[I]:=chr($A3);
  {$ENDIF}
    FreeMem(Contenu,Taille*sizeOf(Char));
  end;
  Taille:=length(S);
  GetMem(Contenu,Taille*sizeOf(Char));
  Existe:=true;
  for I:=1 to Taille do Contenu^[I]:=S[I];
end;

procedure TEco.Lit(var S : string);
var
  I : integer;
begin
  SetLength(S, Taille);
  if taille<>0 then begin
    for I:=1 to taille do S[I]:=Contenu^[I];
  end;
end;

function TEco.AsString: string;
var
  p: integer;
begin
  Lit(Result);
  // remplace les espaces par des souligns
  while true do begin
    p := Pos(' ', Result);
    if p=0 then break;
    Result[p] := '_';
  end;
end;


procedure TEco.UpString(var S : string);
var
  I : SmallInt;
begin
  S:='';
  for I:=1 to Taille do S:=S+UpCase(Contenu^[I]);
end;


procedure TEco.Sauve(var F : TFileBufferStream);
begin
  F.write(Taille,sizeOf(Taille));
  F.write(Contenu^,Taille*SizeOf(Char));
end;

procedure TEco.Loade(var F : TFileBufferStream);
{$IFDEF DEBUG}
var
  I : integer;
{$ENDIF}
begin
  if Existe then begin
  {$IFDEF DEBUG}
    for I:=1 to Taille do Contenu^[I]:=chr($A3);
  {$ENDIF}
    FreeMem(Contenu,Taille*sizeOf(Char));
  end;
  F.Read(Taille,sizeOf(Taille));
  GetMem(Contenu,Taille*sizeOf(Char));
  Existe:=true;
  F.Read(Contenu^,Taille*SizeOf(Char));
end;

procedure TEco.SauveTxt(var F : textFile);
var
  I : SmallInt;
  S : string;
begin
  S:='';
  for I:=1 to Taille do S:=S+Contenu^[I];
  writeln(F,S);
end;

procedure TEco.LoadeTxt(S : string);
begin
  Ecrit(S);
end;

procedure TEco.Duplique(Original : TEco);
var
  S : string;
begin
  Original.Lit(S);
  Ecrit(S);
end;

function TEco.Identique(L_Autre : TEco) : boolean;
var
  S1, S2 : string;
begin
  Lit(S1);
  L_Autre.Lit(S2);
  Identique:=(S1=S2);
end;

(******************** TVisee *********************)

constructor TVisee.Create;
begin
  inherited Create;
  Commentaires:=TEco.Create;
end;

destructor TVisee.Destroy;
begin
  Commentaires.Free;
  Commentaires:=nil;
  inherited Destroy;
end;

(******************** TPointTopo ******************)

constructor TPointTopo.Create;
begin
  inherited Create;
  Nom:=TEco.Create;
end;

destructor TPointTopo.Destroy;
begin
  Nom.Free; Nom:=nil;
  inherited Destroy;
end;

procedure TPointTopo.Loade(var F : TFileBufferStream);
begin
  F.Read(Terminus,sizeOf(Terminus));
  Nom.Loade(F);
  F.Read(Indice,sizeOf(Indice));
  F.Read(X,sizeOf(X));
  F.Read(Y,sizeOf(Y));
  F.Read(Z,sizeOf(Z));
end;

procedure TPointTopo.Sauve(var F : TFileBufferStream);
begin
  F.write(Terminus,sizeOf(Terminus));
  Nom.Sauve(F);
  F.write(Indice,sizeOf(Indice));
  F.write(X,sizeOf(X));
  F.write(Y,sizeOf(Y));
  F.write(Z,sizeOf(Z));
end;

procedure TPointTopo.LoadeTxt(var F : textFile);
var
  S : string;
begin
  readln(F,S);
  if S='Terminus' then Terminus:=true
  else begin
    Terminus:=false;
    readln(F,S);
    Nom.LoadeTxt(S);
    readln(F,S); Indice:=StrToInt(S);
  end;
  readln(F,S);
  X:=StrToFloat(copy(S,1,pos(chr(9),S)-1));
  delete(S,1,pos(chr(9),S));
  Y:=StrToFloat(copy(S,1,pos(chr(9),S)-1));
  delete(S,1,pos(chr(9),S));
  Z:=StrToFloat(S);
end;

procedure TPointTopo.SauveTxt(var F : textFile);
begin
  if Terminus then writeln(F,'Terminus') else begin
    writeln(F,'Pt');
    Nom.SauveTxt(F);
    writeln(F,IntToStr(Indice));
  end;
  write(F,FloatToStrF(X,ffGeneral,9,3)+chr(9));
  write(F,FloatToStrF(Y,ffGeneral,9,3)+chr(9));
  writeln(F,FloatToStrF(Z,ffGeneral,9,3));
end;

function TPointTopo.Identique(L_autre : TPointTopo) : boolean;
var
  I : integer;
begin
  if L_autre.Terminus or Terminus or (Indice<>L_autre.Indice) or (Nom.Taille<>L_Autre.Nom.Taille) then Identique:=false
  else begin
    I:=Nom.Taille;
    while (I>0) and ((Nom.Contenu^[I]=L_Autre.Nom.Contenu^[I]) or (UpCase(Nom.Contenu^[I])=UpCase(L_Autre.Nom.Contenu^[I]))) do dec(I);
    Identique:=(I=0);
  end;
end;

procedure TPointTopo.Duplique(Original : TPointTopo);
var
 S : string;
begin
  Terminus:=Original.Terminus;
  if not Terminus then begin
    Indice:=Original.Indice;

    Original.Nom.Lit(S);
    Nom.Ecrit(S);

    X:=Original.X;
    Y:=ORiginal.Y;
    Z:=Original.Z;
  end;
end;

(************************ TPointCoupe *****************)
constructor TPointCoupe.Create;
begin
  inherited Create;
  Nom:=TEco.Create;
end;

destructor TPointCoupe.Destroy;
begin
  Nom.Free; Nom:=nil;
  inherited Destroy;
end;

procedure TPointCoupe.Loade(var F : TFileBufferStream);
begin
  Nom.Loade(F);
  F.Read(Indice,sizeOf(Indice));
  F.Read(Decal_Hori,sizeOf(Decal_Hori));
  F.Read(Decal_Z,sizeOf(Decal_Z));
  F.Read(Ordre_Suite,sizeOf(Ordre_Suite));
  F.Read(Galerie_Suite,sizeOf(Galerie_Suite));
end;

procedure TPointCoupe.Sauve(var F : TFileBufferStream);
begin
  Nom.Sauve(F);
  F.write(Indice,sizeOf(Indice));
  F.write(Decal_Hori,sizeOf(Decal_Hori));
  F.write(Decal_Z,sizeOf(Decal_Z));
  F.write(Ordre_Suite,sizeOf(Ordre_Suite));
  F.write(Galerie_Suite,sizeOf(Galerie_Suite));
end;

procedure TPointCoupe.LoadeTxt(var F : textFile);
var
  S : string;
begin
  readln(F,S);
  Nom.LoadeTxt(S);
  readln(F,S); Indice:=StrToInt(S);
  readln(F,S); Decal_Hori:=StrToFloat(S);
  readln(F,S); Decal_Z:=StrToFloat(S);
  readln(F,S); Ordre_Suite:=StrToInt(S);
  readln(F,S); Galerie_Suite:=StrToInt(S);
end;

procedure TPointCoupe.SauveTxt(var F : textFile);
begin
  Nom.SauveTxt(F);
  writeln(F,IntTostr(Indice));
  writeln(F,FloatTostrF(Decal_Hori,ffGeneral,7,3));
  writeln(F,FloatTostrF(Decal_Z,ffGeneral,7,3));
  writeln(F,IntTostr(Ordre_Suite));
  writeln(F,IntTostr(Galerie_Suite));
end;


(******************* TGalerie *******************)

constructor TGalerie.Create;
begin
  inherited Create;
  Nb_Visee:=0;
  Taille_Coord:=0;
  Existe_Plan:=false;
  Existe_Coupe:=false;
  Fantome:=false;
  Information:=TEco.Create;
  Depart:=TPointTopo.Create;
  Arrivee:=TPointTopo.Create;
  Contrainte_Plan:=0;
  Contrainte_Z:=0;
  Correction_Topofil:=1;
  Couleur:=$000000;
  En_Coupe:=false;
end;

destructor TGalerie.Destroy;
var
  I : SmallInt;
begin
  for I:=1 to {Nb_Visee}128 do begin
    Visees[I].Free;
    Visees[I]:=nil;
  end;
  if Taille_Coord<>0 then FreeMem(Coordonnees,Taille_Coord*sizeOf(TPoint3D));
  Information.Free; Information:=nil;
  Depart.Free; Depart:=nil;
  Arrivee.Free; Arrivee:=nil;
  inherited Destroy;
end;

procedure TGalerie.Detailler;
var
  K : integer;
  ContrainteX, ContrainteY, ContrainteZ, L, X0, Y0, Z0 : reel;
begin
  if Taille_Coord<>0 then FreeMem(Coordonnees,Taille_Coord*sizeOf(TPoint3D));
  Taille_Coord:=Nb_Visee;
  GetMem(Coordonnees,Taille_Coord*sizeOf(TPoint3D));
  if Approximation then begin
    if Depart.Terminus or Arrivee.Terminus then begin
      ContrainteX:=0;
      ContrainteY:=0;
      ContrainteZ:=0;
    end
    else begin
      ContrainteX:=((Arrivee.X-Depart.X)-DeltaX)/SigmaL;
      ContrainteY:=((Arrivee.Y-Depart.Y)-DeltaY)/SigmaL;
      ContrainteZ:=((Arrivee.Z-Depart.Z)-DeltaZ)/SigmaL;
    end;
    L:=0;
    if Depart.Terminus then begin
      X0:=Arrivee.X-DeltaX; Y0:=Arrivee.Y-DeltaY; Z0:=Arrivee.Z-DeltaZ;
      Depart.X:=X0; Depart.Y:=Y0; Depart.Z:=Z0;
    end
    else begin
      X0:=Depart.X; Y0:=Depart.Y; Z0:=Depart.Z;
    end;
    for K:=1 to Nb_Visee do begin
      if Visees[K].ULong=cm then L:=L+Visees[K].Longueur*0.01 else L:=L+Visees[K].Longueur;
      Coordonnees^[K].X:=X0+Visees[K].X+ContrainteX*L;
      Coordonnees^[K].Y:=Y0+Visees[K].Y+ContrainteY*L;
      Coordonnees^[K].Z:=Z0+Visees[K].Z+ContrainteZ*L;
    end;
  end;
end;

procedure TGalerie.Compile(La_Declinaison: reel);
var
  I : SmallInt;
  P, L, Angle : reel;
  Metrique : boolean;
begin
  DeltaX:=0; DeltaY:=0; DeltaZ:=0;
  SigmaL:=0; SigmaL2:=0;
  Developement:=0;
  Approximation:=false;
  for I:=1 to Nb_Visee do begin
    with Visees[I] do begin
      if ULong=cm then L:=Longueur/100 else L:=Longueur;
      case UAzimuth of
        deg : Angle:=Azimuth/180*PI;
        ndeg: Angle:=(Azimuth/180+1)*PI;
        gr  : Angle:=Azimuth/200*PI;
        ngr : Angle:=(Azimuth/200+1)*PI;
      end;
      if Correction_Dlt then Developement:=Developement+Longueur_Dlt
                        else Developement:=Developement+L;
      Metrique :=false;
      case UPente of
        deg : P:=Pente/180*PI;
        gr  : P:=Pente/200*PI;
        vdeg: P:=(90-Pente)/180*PI;
        mp  : begin
          P:=Pente;
          Metrique:=true;
        end;
        cmp : begin
          P:=Pente/100;
          Metrique:=true;
        end;
        ndeg : P:=-Pente/180*PI;
        ngr  : P:=-Pente/200*PI;
        nvdeg: P:=-(90-Pente)/180*PI;
        nmp  : begin
          P:=-Pente;
          Metrique:=true;
        end;
        ncmp : begin
          P:=-Pente/100;
          Metrique:=true;
        end;
      end;
      SigmaL:=SigmaL+L;
      SigmaL2:=SigmaL2+sqr(L);
      if Metrique then begin
        if sqr(L)>sqr(P) then LH:=sqrt(sqr(L)-sqr(P))
                         else LH:=0;
        DeltaZ:=DeltaZ+P;
      end
      else begin
        LH:=L*cos(P);
        DeltaZ:=DeltaZ+L*sin(P);
      end;
      DeltaX:=DeltaX+LH*sin(Angle-La_Declinaison);
      DeltaY:=DeltaY+LH*cos(Angle-La_Declinaison);
      X:=DeltaX;
      Y:=DeltaY;
      Z:=DeltaZ;
    end;
  end;
end;

procedure TGalerie.Sauve(var F : TFileBufferStream);
var
  I : SmallInt;
begin
  Depart.Sauve(F);
  Arrivee.Sauve(F);
  F.write(DeltaX,sizeOf(DeltaX));
  F.write(DeltaY,sizeOf(DeltaY));
  F.write(DeltaZ,sizeOf(DeltaZ));
  F.write(SigmaL,sizeOf(SigmaL));
  F.write(SigmaL2,sizeOf(SigmaL2));
  F.write(Nb_Visee,sizeOf(Nb_Visee));
  F.write(Developement,sizeOf(Developement));
  F.write(Precision,sizeOf(Precision));
  Information.Sauve(F);
  for I:=1 to Nb_Visee do with Visees[I] do begin
    F.write(Longueur,sizeOf(Longueur));
    F.write(Azimuth,sizeOf(Azimuth));
    F.write(Pente,sizeOf(Pente));
    F.write(LargeurDroite,sizeOf(LargeurDroite));
    F.write(LargeurGauche,sizeOf(LargeurGauche));
    F.write(HauteurSup,sizeOf(HauteurSup));
    F.write(HauteurInf,sizeOf(HauteurInf));
    Commentaires.Sauve(F);
    F.write(ULong,sizeOf(ULong));
    F.write(ULarg,sizeOf(ULarg));
    F.write(UHauteur,sizeOf(UHauteur));
    F.write(UAzimuth,sizeOf(UAzimuth));
    F.write(UPente,sizeOf(UPente));
    F.write(Schema,sizeOf(Schema));
    F.write(Longueur_Horizontale,sizeOf(Longueur_Horizontale));
    F.write(Correction_Horizontale,sizeOf(Correction_Horizontale));
    F.write(Longueur_Dlt,sizeOf(Longueur_Dlt));
    F.write(Correction_Dlt,sizeOf(Correction_Dlt));
    F.write(X,sizeOf(X));
    F.write(Y,sizeOf(Y));
    F.write(Z,sizeOf(Z));
    F.write(LH,sizeOf(LH));
    F.write(Topofil,sizeOf(Topofil));
    if Topofil then F.write(Debut_Topofil,sizeOf(Debut_Topofil));
  end;
  F.write(Correction_Topofil,sizeOf(Correction_Topofil));
  F.write(Approximation,sizeOf(Approximation));
  F.write(Existe_Plan,sizeOf(Existe_Plan));
  F.write(Existe_Coupe,sizeOf(Existe_Coupe));
  F.write(Fantome,sizeOf(Fantome));
  F.write(En_Coupe,sizeOf(En_Coupe));
  F.write(Accrochage_Debut,sizeOf(Accrochage_Debut));
  F.write(Gauche_Droite,sizeOf(Gauche_Droite));
  F.write(Nb_Suite,sizeOf(Nb_Suite));
  F.write(Ordre_Suite,sizeOf(Ordre_Suite));
  F.write(Galerie_Suite,sizeOf(Galerie_Suite));
  F.write(Couleur,sizeOf(Couleur));
end;

procedure TGalerie.Loade(var F : TFileBufferStream);
var
  I : SmallInt;
begin
  Depart.Loade(F);
  Arrivee.Loade(F);

  F.Read(DeltaX,sizeOf(DeltaX));
  F.Read(DeltaY,sizeOf(DeltaY));
  F.Read(DeltaZ,sizeOf(DeltaZ));
  F.Read(SigmaL,sizeOf(SigmaL));
  F.Read(SigmaL2,sizeOf(SigmaL2));
  F.Read(Nb_Visee,sizeOf(Nb_Visee));
  F.Read(Developement,sizeOf(Developement));
  F.Read(Precision,sizeOf(Precision));
  Information.Loade(F);
  for I:=1 to Nb_Visee do begin
    Visees[I]:=TVisee.Create;
    with Visees[I] do begin
      F.Read(Longueur,sizeOf(Longueur));
      F.Read(Azimuth,sizeOf(Azimuth));
      F.Read(Pente,sizeOf(Pente));
      F.Read(LargeurDroite,sizeOf(LargeurDroite));
      F.Read(LargeurGauche,sizeOf(LargeurGauche));
      F.Read(HauteurSup,sizeOf(HauteurSup));
      F.Read(HauteurInf,sizeOf(HauteurInf));
      Commentaires.loade(F);
      F.Read(ULong,sizeOf(ULong));
      F.Read(ULarg,sizeOf(ULarg));
      F.Read(UHauteur,sizeOf(UHauteur));
      F.Read(UAzimuth,sizeOf(UAzimuth));
      F.Read(UPente,sizeOf(UPente));
      F.Read(Schema,sizeOf(Schema));
      F.Read(Longueur_Horizontale,sizeOf(Longueur_Horizontale));
      F.Read(Correction_Horizontale,sizeOf(Correction_Horizontale));
      F.Read(Longueur_Dlt,sizeOf(Longueur_Dlt));
      F.Read(Correction_Dlt,sizeOf(Correction_Dlt));
      F.Read(X,sizeOf(X));
      F.Read(Y,sizeOf(Y));
      F.Read(Z,sizeOf(Z));
      F.Read(LH,sizeOf(LH));
      if Ver>=1003 then begin
        F.Read(Topofil,sizeOf(Topofil));
        if Topofil then F.Read(Debut_Topofil,sizeOf(Debut_Topofil));
      end
      else Topofil:=false;
    end;
  end;
  if Ver>=1003 then F.Read(Correction_Topofil,sizeOf(Correction_Topofil))
  else Correction_Topofil:=1;
  F.Read(Approximation,sizeOf(Approximation));
  F.Read(Existe_Plan,sizeOf(Existe_Plan));
  F.Read(Existe_Coupe,sizeOf(Existe_Coupe));
  F.Read(Fantome,sizeOf(Fantome));
  F.Read(En_Coupe,sizeOf(En_Coupe));
  F.Read(Accrochage_Debut,sizeOf(Accrochage_Debut));
  F.Read(Gauche_Droite,sizeOf(Gauche_Droite));
  F.Read(Nb_Suite,sizeOf(Nb_Suite));
  F.Read(Ordre_Suite,sizeOf(Ordre_Suite));
  F.Read(Galerie_Suite,sizeOf(Galerie_Suite));
  if Ver>=1004 then F.Read(Couleur,sizeOf(Couleur));
end;

procedure TGalerie.SauveTxt(var F : textFile);
var
  I : SmallInt;
begin
  Depart.SauveTxt(F);
  Arrivee.SauveTxt(F);
  writeln(F,FloatTostrF(DeltaX,ffGeneral,10,4));
  writeln(F,FloatTostrF(DeltaY,ffGeneral,10,4));
  writeln(F,FloatTostrF(DeltaZ,ffGeneral,10,4));
  writeln(F,FloatTostrF(SigmaL,ffGeneral,7,4));
  writeln(F,FloatTostrF(SigmaL2,ffGeneral,8,4));
  writeln(F,FloatTostrF(Developement,ffGeneral,7,4));
  writeln(F,FloatTostrF(Precision,ffGeneral,5,4));
  Information.SauveTxt(F);
  writeln(F,'Nb_Visee '+IntTostr(Nb_Visee));
  for I:=1 to Nb_Visee do with Visees[I] do begin
    write(F,FloatTostrF(Longueur,ffGeneral,5,3)+chr(9));
    write(F,tuDistance[ULong]+chr(9));
    write(F,FloatTostrF(Azimuth,ffGeneral,4,1)+chr(9));
    case UAzimuth of
      deg : write(F,'deg'+chr(9));
      gr : write(F,'grade'+chr(9));
      ndeg : write(F,'-deg'+chr(9));
      ngr : write(F,'-grade'+chr(9));
    end;
    write(F,FloatTostrF(Pente,ffGeneral,3,1)+chr(9));
    case UPente of
      deg : write(F,'deg'+chr(9));
      gr : write(F,'grade'+chr(9));
      ndeg : write(F,'-deg'+chr(9));
      ngr : write(F,'-grade'+chr(9));
      mp : write(F,'mp'+chr(9));
      cmp : write(F,'cmp'+chr(9));
      nmp : write(F,'-mp'+chr(9));
      ncmp : write(F,'-cmp'+chr(9));
    end;
    write(F,LargeurDroite+chr(9));
    Write(F,LargeurGauche+chr(9));
    write(F,tuDistance[ULarg]+chr(9));
    Write(F,HauteurSup+chr(9));
    Write(F,HauteurInf+chr(9));
    write(F,tuDistance[UHauteur]+chr(9));
    write(F,IntTostr(Schema)+chr(9));
    write(F,FloatTostrF(Longueur_Horizontale,ffGeneral,4,2)+chr(9));
    if Correction_Horizontale then write(F,'O'+chr(9)) else write(F,'N'+chr(9));
    write(F,FloatTostrF(Longueur_Dlt,ffGeneral,4,2)+chr(9));
    if Correction_Dlt then write(F,'O'+chr(9)) else write(F,'N'+chr(9));
    write(F,FloatTostrF(X,ffGeneral,9,3)+chr(9));
    write(F,FloatTostrF(Y,ffGeneral,9,3)+chr(9));
    write(F,FloatTostrF(Z,ffGeneral,9,3)+chr(9));
    write(F,FloatTostrF(LH,ffGeneral,9,3)+chr(9));
    if Topofil then write(F,'O'+chr(9)) else write(F,'N'+chr(9));
    if Topofil then write(F,FloatTostrF(Debut_Topofil,ffGeneral,7,1)+chr(9));
    Commentaires.SauveTxt(F);
  end;
  writeln(F,FloatTostrF(Correction_Topofil,ffGeneral,5,4));
  if Approximation then writeln(F,'O') else writeln(F,'N');
  if Existe_Plan then writeln(F,'O') else writeln(F,'N');
  if Existe_Coupe then writeln(F,'O') else writeln(F,'N');
  if Fantome then writeln(F,'O') else writeln(F,'N');
  if En_Coupe then writeln(F,'O') else writeln(F,'N');
  if Accrochage_Debut then writeln(F,'O') else writeln(F,'N');
  if Gauche_Droite then writeln(F,'O') else writeln(F,'N');
  writeln(F,IntTostr(Nb_Suite));
  for I:=1 to Nb_Suite do begin
    write(F,IntTostr(Ordre_Suite[i])+chr(9));
  end;
  writeln(F);
  for I:=1 to Nb_Suite do begin
    write(F,IntTostr(Galerie_Suite[i])+chr(9));
  end;
  writeln(F);
  writeln(F,'Couleur '+ColorToString(Couleur));
  writeln(F,'---');
end;

procedure TGalerie.LoadeTxt(var F : textFile);
var
  I : SmallInt;
  S, Morceau : string;
begin
  Depart.LoadeTxt(F);
  Arrivee.LoadeTxt(F);


  readln(F,S); DeltaX:=StrToFloat(S);
  readln(F,S); DeltaY:=StrToFloat(S);
  readln(F,S); DeltaZ:=StrToFloat(S);
  readln(F,S); SigmaL:=StrToFloat(S);
  readln(F,S); SigmaL2:=StrToFloat(S);
  readln(F,S); Developement:=StrToFloat(S);
  readln(F,S); Precision:=StrToFloat(S);
  readln(F,S);
  Information.LoadeTxt(S);
  readln(F,S); delete(S,1,8); Nb_Visee:=StrToInt(S);
  for I:=1 to Nb_Visee do begin
    Visees[I]:=TVisee.Create;
    readln(F,S);
    with Visees[I] do begin
      Longueur:=StrToFloat(copy(S,1,pos(chr(9),S)-1));
      delete(S,1,pos(chr(9),S));
      if copy(S,1,pos(chr(9),S)-1)=tuDistance[m] then ULong:=m else ULong:=cm;
      delete(S,1,pos(chr(9),S));
      Azimuth:=StrToFloat(copy(S,1,pos(chr(9),S)-1));
      delete(S,1,pos(chr(9),S));
      Morceau:=copy(S,1,pos(chr(9),S)-1); delete(S,1,pos(chr(9),S));
      if Morceau='deg' then UAzimuth:=deg else
        if Morceau='grade' then UAzimuth:=gr else
          if Morceau='-deg' then UAzimuth:=ndeg else
            if Morceau='-grade' then UAzimuth:=ngr;
      Pente:=StrToFloat(copy(S,1,pos(chr(9),S)-1));
      delete(S,1,pos(chr(9),S));
      Morceau:=copy(S,1,pos(chr(9),S)-1); delete(S,1,pos(chr(9),S));
      if Morceau='deg' then UPente:=deg else
        if Morceau='grade' then UPente:=gr else
          if Morceau='-deg' then UPente:=ndeg else
            if Morceau='-grade' then UPente:=ngr else
              if Morceau='mp' then UPente:=mp else
                if Morceau='cmp' then UPente:=cmp else
                  if Morceau='-mp' then UPente:=nmp else
                    if Morceau='-cmp' then UPente:=ncmp;
      LargeurDroite:=copy(S,1,pos(chr(9),S)-1);
      delete(S,1,pos(chr(9),S));
      LargeurGauche:=copy(S,1,pos(chr(9),S)-1); delete(S,1,pos(chr(9),S));
      if copy(S,1,pos(chr(9),S)-1)=tuDistance[m] then ULarg:=m else ULarg:=cm;
      delete(S,1,pos(chr(9),S));
      HauteurSup:=copy(S,1,pos(chr(9),S)-1); delete(S,1,pos(chr(9),S));
      HauteurInf:=copy(S,1,pos(chr(9),S)-1); delete(S,1,pos(chr(9),S));
      if copy(S,1,pos(chr(9),S)-1)=tuDistance[m] then UHauteur:=m else UHauteur:=cm;
      delete(S,1,pos(chr(9),S));
      Schema:=StrToInt(copy(S,1,pos(chr(9),S)-1));
      delete(S,1,pos(chr(9),S));
      Longueur_Horizontale:=StrToFloat(copy(S,1,pos(chr(9),S)-1));
      delete(S,1,pos(chr(9),S));
      Correction_Horizontale:=(copy(S,1,pos(chr(9),S)-1)='O');
      delete(S,1,pos(chr(9),S));
      Longueur_Dlt:=strToFloat(copy(S,1,pos(chr(9),S)-1));
      delete(S,1,pos(chr(9),S));
      Correction_Dlt:=(copy(S,1,pos(chr(9),S)-1)='O');
      delete(S,1,pos(chr(9),S));
      X:=strToFloat(copy(S,1,pos(chr(9),S)-1));
      delete(S,1,pos(chr(9),S));
      Y:=strToFloat(copy(S,1,pos(chr(9),S)-1));
      delete(S,1,pos(chr(9),S));
      Z:=strToFloat(copy(S,1,pos(chr(9),S)-1));
      delete(S,1,pos(chr(9),S));
      LH:=strToFloat(copy(S,1,pos(chr(9),S)-1));
      delete(S,1,pos(chr(9),S));
      Topofil:=(copy(S,1,pos(chr(9),S)-1)='O');
      delete(S,1,pos(chr(9),S));
      if Topofil then begin
        Debut_Topofil:=strToFloat(copy(S,1,pos(chr(9),S)-1));
        delete(S,1,pos(chr(9),S));
      end;
      Commentaires.loadeTxt(S);
    end;
  end;
  readln(F,S); Correction_Topofil:=strToFloat(S);
  readln(F,S); Approximation:=S='O';
  readln(F,S); Existe_Plan:=S='O';
  readln(F,S); Existe_Coupe:=S='O';
  readln(F,S); Fantome:=S='O';
  readln(F,S); En_Coupe:=S='O';
  readln(F,S); Accrochage_Debut:=S='O';
  readln(F,S); Gauche_Droite:=S='O';
  readln(F,S); Nb_Suite:=StrToInt(S);
  readln(F,S);
  for I:=1 to Nb_Suite do begin
    Ordre_Suite[I]:=strToInt(copy(S,1,pos(chr(9),S)-1));
    delete(S,1,pos(chr(9),S));
  end;
  readln(F,S);
  for I:=1 to Nb_Suite do begin
    Galerie_Suite[I]:=strToInt(copy(S,1,pos(chr(9),S)-1));
    delete(S,1,pos(chr(9),S));
  end;
  if Ver>=1004 then begin
    readln(F,S);
    Couleur:=StringToColor(copy(S,9,length(S)))
  end
  else Couleur:=$0;
  readln(F);
end;

function TGalerie.Identique(var L_autre : TGalerie) : boolean;
var
  Echec : boolean;
  I : integer;
  Une_Visee, L_Autre_Visee : TVisee;
begin
  Echec:=false;
  if not Information.Identique(L_autre.Information) then
    Echec:=true
  else begin
    if Depart.Terminus then begin
      if not L_Autre.Depart.Terminus or not Arrivee.Nom.Identique(L_Autre.Arrivee.Nom) then
        Echec:=true;
    end
    else begin
      if Arrivee.Terminus then begin
        if not L_Autre.Arrivee.Terminus or not Depart.Nom.Identique(L_Autre.Depart.Nom) then
          Echec:=true;
      end
      else if not Depart.Nom.Identique(L_autre.Depart.Nom) or
              not Arrivee.Nom.Identique(L_Autre.Arrivee.Nom) then Echec:=true;
    end;
    if not Echec then begin
      if Nb_Visee<>L_Autre.Nb_Visee then
        Echec:=true
      else begin
        for I:=1 to Nb_Visee do begin
          Une_Visee:=Visees[I];
          L_Autre_Visee:=L_Autre.Visees[I];
          if Une_Visee.Longueur<>L_Autre_Visee.Longueur then Echec:=true;
          if Une_Visee.Azimuth<>L_Autre_Visee.Azimuth then Echec:=true;
          if Une_Visee.Pente<>L_Autre_Visee.Pente then Echec:=true;
          if Une_Visee.LargeurDroite<>L_Autre_Visee.LargeurDroite then Echec:=true;
          if Une_Visee.LargeurGauche<>L_Autre_Visee.LargeurGauche then Echec:=true;
          if Une_Visee.HauteurSup<>L_Autre_Visee.HauteurSup then Echec:=true;
          if Une_Visee.HauteurInf<>L_Autre_Visee.HauteurInf then Echec:=true;
          if Une_Visee.ULong<>L_Autre_Visee.ULong then Echec:=true;
          if Une_Visee.ULarg<>L_Autre_Visee.ULarg then Echec:=true;
          if Une_Visee.UPente<>L_Autre_Visee.UPente then Echec:=true;
          if Une_Visee.Schema<>L_Autre_Visee.Schema then Echec:=true;
          if Une_Visee.Longueur_Horizontale<>L_Autre_Visee.Longueur_Horizontale then Echec:=true;
          if Une_Visee.Longueur_Dlt<>L_Autre_Visee.Longueur_Dlt then Echec:=true;
          if Une_Visee.Correction_Horizontale<>L_Autre_Visee.Correction_Horizontale then Echec:=true;
          if Une_Visee.Correction_Dlt<>L_Autre_Visee.Correction_Dlt then Echec:=true;
          if not Une_Visee.Commentaires.Identique(L_Autre_Visee.Commentaires) then Echec:=true;
        end;
      end;
    end;
  end;
  Identique:=not Echec;
end;

(******************* TFiche *******************)

constructor TFiche.Create;
begin
  inherited Create;
  Liste_Galerie:=TList.Create;
  Liste_Galerie.Capacity:=5;
  Systeme:=TEco.Create;
  Cavite:=TEco.Create;
  Reseau:=Teco.Create;
  Lieu:=TEco.Create;
  Temps:=TEco.Create;
  Topographes:=TList.Create;
end;

destructor TFiche.Destroy;
var
  I : SmallInt;
begin
  Systeme.Free; Systeme:=nil;
  Cavite.Free; Cavite:=nil;
  Reseau.Free; Reseau:=nil;
  Lieu.Free; Lieu:=nil;
  Temps.Free; Temps:=nil;
  for I:=0 to Liste_Galerie.Count-1 do TGalerie(Liste_Galerie[I]).Free;
  Liste_Galerie.Free; Liste_Galerie:=nil;
  for I:=0 to Topographes.Count-1 do TEco(Topographes[I]).Free;
  Topographes.Free; Topographes:=nil;
  inherited Destroy;
end;


procedure TFiche.Compile(Geodesique : reel);
var
  I : SmallInt;
  Decli : reel;
begin
  if Unite_Decli=deg then Decli:=Declinaison/180*PI
                     else Decli:=Declinaison/200*PI;
  for I:=0 to Liste_Galerie.Count-1 do TGalerie(Liste_Galerie[I]).Compile(Decli+Geodesique/180*PI);
end;

procedure TFiche.Sauve(var F : TFileBufferStream);
var
  I : SmallInt;
begin
  F.write(Date,sizeOf(Date));
  F.write(Unite_Decli,sizeOf(Unite_Decli));
  I:=Topographes.Count;
  F.write(I,sizeOf(I));
  for I:=0 to Topographes.Count-1 do TEco(Topographes[I]).Sauve(F);
  Systeme.Sauve(F);
  Cavite.Sauve(F);
  Reseau.Sauve(F);
  Lieu.Sauve(F);
  F.write(Declinaison,sizeOf(Declinaison));
  F.write(Premiere,sizeOf(Premiere));
  Temps.Sauve(F);
  F.write(Ordre,sizeOf(Ordre));
  I:=Liste_Galerie.Count;
  F.write(I,SizeOf(I));
  for I:=0 to Liste_Galerie.Count-1 do TGalerie(Liste_Galerie[I]).Sauve(F);
end;

procedure TFiche.Loade(var F : TFileBufferStream);
var
  I, J, Taille : SmallInt;
  Topog : TEco;
  La_Galerie : TGalerie;
begin
  F.Read(Date,sizeOf(Date));
  F.Read(Unite_Decli,sizeOf(Unite_Decli));
  F.Read(I,sizeOf(I));
  for J:=1 to I do begin
    Topog:=TEco.Create;
    Topog.Loade(F);
    Topographes.Add(Topog);
  end;
  Systeme.Loade(F);
  Cavite.Loade(F);
  Reseau.Loade(F);
  Lieu.Loade(F);
  F.Read(Declinaison,sizeOf(Declinaison));
  F.Read(Premiere,sizeOf(Premiere));
  Temps.Loade(F);
  F.Read(Ordre,sizeOf(Ordre));
  F.Read(Taille,sizeOf(Taille));
  for I:=1 to Taille do begin
    La_Galerie:=TGalerie.Create;
    La_Galerie.Loade(F);
    Liste_Galerie.Add(La_Galerie);
  end;
end;

procedure TFiche.SauveTxt(var F : textFile);
var
  I : SmallInt;
begin
  writeln(F,'Date '+Date);
  writeln(F,'Declinaison '+FloatTostrF(Declinaison,ffGeneral,7,5));
  if Unite_Decli=deg then writeln(F,'Unite_Decli deg')
                     else writeln(F,'Unite_Decli grade');
  writeln(F,'Nb_Topographes '+IntTostr(Topographes.Count));
  for I:=0 to Topographes.Count-1 do TEco(Topographes[I]).SauveTxt(F);
  write(F,'Systeme '); Systeme.SauveTxt(F);
  write(F,'Cavite '); Cavite.SauveTxt(F);
  write(F,'Reseau '); Reseau.SauveTxt(F);
  write(F,'Lieu '); Lieu.SauveTxt(F);
  writeln(F,'Premiere '+FloatTostrF(Premiere,ffGeneral,6,3));
  write(F,'TPST '); Temps.SauveTxt(F);
  writeln(F,'Ordre '+IntTostr(Ordre));
  writeln(F,'Nb_Galerie '+IntToStr(Liste_Galerie.Count));
  for I:=0 to Liste_Galerie.Count-1 do TGalerie(Liste_Galerie[I]).SauveTxt(F);
end;

procedure TFiche.LoadeTxt(var F : textFile);
var
  I, J, Taille : SmallInt;
  Topog : TEco;
  La_Galerie : TGalerie;
  S : string;
begin
  readln(F,S); Date:=copy(S,6,8);
  readln(F,S); delete(S,1,12); Declinaison:=StrToFloat(S);
  readln(F,S); if S='Unite_Decli deg' then Unite_Decli:=deg else Unite_Decli:=gr;
  readln(F,S); delete(S,1,15); I:=StrToInt(S);
  for J:=1 to I do begin
    Topog:=TEco.Create;
    readln(F,S);
    Topog.LoadeTxt(S);
    Topographes.Add(Topog);
  end;
  readln(F,S); delete(S,1,8); Systeme.LoadeTxt(S);
  readln(F,S); delete(S,1,7); Cavite.LoadeTxt(S);
  readln(F,S); delete(S,1,7); Reseau.LoadeTxt(S);
  readln(F,S); delete(S,1,5); Lieu.LoadeTxt(S);
  readln(F,S); delete(S,1,9); Premiere:=strToFloat(S);
  readln(F,S); delete(S,1,5); Temps.LoadeTxt(S);
  readln(F,S); delete(S,1,6); Ordre:=StrToInt(S);
  readln(F,S); delete(S,1,11); Taille:=StrToInt(S);
  for I:=1 to Taille do begin
    La_Galerie:=TGalerie.Create;
    La_Galerie.LoadeTxt(F);
    Liste_Galerie.Add(La_Galerie);
  end;
end;

function TFiche.Meme_Equipe(var L_autre : TFiche) : boolean;
var
  Echec : boolean;
  I : SmallInt;
  Topog : TEco;
  S1, S2 : string;
begin
  if (Date<>L_Autre.Date) or (Topographes.Count<>L_Autre.Topographes.Count)
  then Meme_Equipe:=false
  else begin
    Echec:=false;
    for I:=0 to Topographes.Count-1 do begin
      Topog:=Topographes[I];
      Topog.UpString(S1);
      Topog:=L_Autre.Topographes[I];
      Topog.UpString(S2);
      if S1<>S2 then Echec:=true;
    end;
    Meme_Equipe:=not Echec;
  end;
end;

function TFiche.Meme_Lieu(var L_autre : TFiche) : boolean;
var
  Echec : boolean;
  S1, S2 : string;
begin
  Echec:=false;
  Systeme.Lit(S1); L_autre.Systeme.Lit(S2);
  if S1<>S2 then Echec:=true;
  Cavite.Lit(S1); L_autre.Cavite.Lit(S2);
  if S1<>S2 then Echec:=true;
  Reseau.Lit(S1); L_autre.Reseau.Lit(S2);
  if S1<>S2 then Echec:=true;
  lieu.Lit(S1); L_autre.Lieu.Lit(S2);
  if S1<>S2 then Echec:=true;
  Meme_Lieu:=not Echec;
end;

function TFiche.Identique(var L_autre : TFiche) : boolean;
var
  Echec : boolean;
  Une_Galerie, Autre_Galerie : TGalerie;
  I : integer;
begin
  Echec:=false;
  if not Meme_Lieu(L_Autre) or not Meme_Equipe(L_Autre) then Echec:=true
  else begin
    if (Declinaison<>L_autre.Declinaison) or
       (Unite_Decli<>L_Autre.Unite_Decli) or
       (Liste_Galerie.Count<>L_Autre.Liste_Galerie.Count) then Echec:=true
    else begin
      for I:=0 to Liste_Galerie.Count-1 do if not Echec then begin
        Une_Galerie:=Liste_Galerie[I];
        Autre_Galerie:=L_autre.Liste_Galerie[I];
        if not Une_Galerie.Identique(Autre_Galerie) then
          Echec:=true;
      end;
    end;
  end;
  Identique:=not Echec;
end;

procedure TFiche.Copie_de(Source : TFiche);
var
  Topog : TEco;
  Une_Galerie, Source_Galerie : TGalerie;
  I, J : integer;
begin
  for I:=0 to Source.Liste_Galerie.Count-1 do begin
    Source_Galerie:=Source.Liste_Galerie[I];
    Une_Galerie:=TGalerie.Create;
    Liste_Galerie.Add(Une_Galerie);
    for J:=1 to Source_Galerie.Nb_Visee do begin
      Une_Galerie.Visees[J]:=TVisee.Create;
      Une_Galerie.Visees[J].Longueur:=Source_Galerie.Visees[J].Longueur;
      Une_Galerie.Visees[J].Azimuth:=Source_Galerie.Visees[J].Azimuth;
      Une_Galerie.Visees[J].Pente:=Source_Galerie.Visees[J].Pente;
      Une_Galerie.Visees[J].LargeurDroite:=Source_Galerie.Visees[J].LargeurDroite;
      Une_Galerie.Visees[J].LargeurGauche:=Source_Galerie.Visees[J].LargeurGauche;
      Une_Galerie.Visees[J].HauteurSup:=Source_Galerie.Visees[J].HauteurSup;
      Une_Galerie.Visees[J].HauteurInf:=Source_Galerie.Visees[J].HauteurInf;
      Une_Galerie.Visees[J].Commentaires.Duplique(Source_Galerie.Visees[J].Commentaires);
      Une_Galerie.Visees[J].ULong:=Source_Galerie.Visees[J].ULong;
      Une_Galerie.Visees[J].ULarg:=Source_Galerie.Visees[J].ULarg;
      Une_Galerie.Visees[J].UHauteur:=Source_Galerie.Visees[J].UHauteur;
      Une_Galerie.Visees[J].UAzimuth:=Source_Galerie.Visees[J].UAzimuth;
      Une_Galerie.Visees[J].UPente:=Source_Galerie.Visees[J].UPente;
      Une_Galerie.Visees[J].Schema:=Source_Galerie.Visees[J].Schema;
      Une_Galerie.Visees[J].Longueur_Horizontale:=Source_Galerie.Visees[J].Longueur_Horizontale;
      Une_Galerie.Visees[J].Longueur_Dlt:=Source_Galerie.Visees[J].Longueur_Dlt;
      Une_Galerie.Visees[J].Correction_Dlt:=Source_Galerie.Visees[J].Correction_Dlt;
      Une_Galerie.Visees[J].X:=Source_Galerie.Visees[J].X;
      Une_Galerie.Visees[J].Y:=Source_Galerie.Visees[J].Y;
      Une_Galerie.Visees[J].Z:=Source_Galerie.Visees[J].Z;
      Une_Galerie.Visees[J].LH:=Source_Galerie.Visees[J].LH;
      Une_Galerie.Visees[J].Topofil:=Source_Galerie.Visees[J].Topofil;
      Une_Galerie.Visees[J].Debut_Topofil:=Source_Galerie.Visees[J].Debut_Topofil;
    end;
    Une_Galerie.Depart.Duplique(Source_Galerie.Depart);
    Une_Galerie.Arrivee.Duplique(Source_Galerie.Arrivee);
    Une_Galerie.DeltaX:=Source_Galerie.DeltaX;
    Une_Galerie.DeltaY:=Source_Galerie.DeltaY;
    Une_Galerie.DeltaZ:=Source_Galerie.DeltaZ;
    Une_Galerie.SigmaL:=Source_Galerie.SigmaL;
    Une_Galerie.SigmaL2:=Source_Galerie.SigmaL2;
    Une_Galerie.Contrainte_Plan:=Source_Galerie.Contrainte_Plan;
    Une_Galerie.Contrainte_Z:=Source_Galerie.Contrainte_Z;
    Une_Galerie.Nb_Visee:=Source_Galerie.Nb_Visee;
    Une_Galerie.Taille_Coord:=0;
    Une_Galerie.Approximation:=Source_Galerie.Approximation;
    Une_Galerie.Existe_Plan:=Source_Galerie.Existe_Plan;
    Une_Galerie.Existe_Coupe:=Source_Galerie.Existe_Coupe;
    Une_Galerie.Fantome:=Source_Galerie.Fantome;
    Une_Galerie.Developement:=Source_Galerie.Developement;
    Une_Galerie.Precision:=Source_Galerie.Precision;
    Une_Galerie.Correction_Topofil:=Source_Galerie.Correction_Topofil;
    Une_Galerie.Information.Duplique(Source_Galerie.Information);
    Une_Galerie.Couleur:=Source_Galerie.Couleur;
    Une_Galerie.En_Coupe:=false;
  end;
  Date:=Source.date;
  Unite_Decli:=Source.Unite_Decli;
  for I:=0 to Source.Topographes.Count-1 do begin
    Topog:=TEco.Create;
    Topog.Duplique(Source.Topographes[I]);
    Topographes.Add(Topog);
  end;
  Systeme.Duplique(Source.Systeme);
  Cavite.Duplique(Source.Cavite);
  Reseau.Duplique(Source.Reseau);
  Lieu.Duplique(Source.Lieu);
  Temps.Duplique(Source.Temps);
  Declinaison:=Source.Declinaison;
  Premiere:=Source.Premiere;
  Sommet_Tableau:=0;
  Base_Tableau:=0;
end;

procedure TFiche.Les_Hauteurs(N_Galerie,N_Visee : integer;var En_Haut, En_Bas : double);
// fournit les hauteurs, si elles existent
// renvoie -1 dans le cas contraire
var
  L_Autre : TGalerie;
  S : string;
  Trouve_Haut, Trouve_Bas : boolean;
  K, Erreur : integer;
begin
  if N_Visee=0 then begin
    Trouve_Haut:=false;
    Trouve_Bas:=false;
    for K:=N_Galerie-1 downto 0 do if not Trouve_Haut then begin
      L_autre:=Liste_Galerie[K];
      if L_Autre.Arrivee.Identique(TGalerie(Liste_Galerie[N_Galerie]).Depart) then begin
        S:=L_Autre.Visees[L_Autre.Nb_Visee].HauteurSup;
        val(S,En_Haut,Erreur);
        if Erreur<>0 then begin
          S:=copy(S,1,length(S)-1);
          val(S,En_Haut,Erreur);
        end;
        if L_autre.Visees[L_Autre.Nb_Visee].UHauteur=cm then En_Haut:=En_Haut/100;
        Trouve_Haut:=(Erreur=0);
      end;
    end;
    if not Trouve_Haut then En_Haut:=-1;
    for K:=N_Galerie-1 downto 0 do if not Trouve_Bas then begin
      L_autre:=Liste_Galerie[K];
      if L_Autre.Arrivee.Identique(TGalerie(Liste_Galerie[N_Galerie]).Depart) then begin
        S:=L_Autre.Visees[L_Autre.Nb_Visee].HauteurInf;
        if (S='') and (L_Autre.Visees[1].HauteurSup<>'') then begin
          Erreur:=0; En_Bas:=0;
        end
        else begin
          val(S,En_Bas,Erreur);
          if Erreur<>0 then begin
            S:=copy(S,1,length(S)-1);
            val(S,En_Bas,Erreur);
          end;
        end;
        if L_Autre.Visees[L_Autre.Nb_Visee].UHauteur=cm then En_Bas:=En_Bas/100;
        Trouve_Bas:=(Erreur=0);
      end;
    end;
    En_Bas:=abs(En_Bas);
    if not Trouve_Bas then En_Bas:=-1;
  end
  else with TGalerie(Liste_Galerie[N_Galerie]) do begin
    S:=Visees[N_Visee].HauteurSup;
    val(S,En_Haut,Erreur);
    if Erreur<>0 then begin
      S:=copy(S,1,length(S)-1);
      val(S,En_Haut,Erreur);
    end;
    if Visees[N_Visee].UHauteur=cm then En_Haut:=En_Haut/100;
    if Erreur<>0 then En_Haut:=-1;
    S:=Visees[N_Visee].HauteurInf;
    if (S='') and (Visees[N_Visee].HauteurSup<>'') then begin
      Erreur:=0; En_bas:=0;
    end
    else begin
      val(S,En_bas,Erreur);
      if Erreur<>0 then begin
        S:=copy(S,1,length(S)-1);
        val(S,En_bas,Erreur);
      end;
    end;
    if Visees[N_Visee].UHauteur=cm then En_bas:=En_bas/100;
    En_bas:=abs(En_bas);
    if Erreur<>0 then En_bas:=-1;
  end;
end;

procedure TFiche.Les_Largeurs(N_Galerie,N_Visee : integer;var A_Gauche, A_Droite : double);
// fournit les largeurs, si elles existent
// renvoie -1 dans le cas contraire
var
  L_Autre : TGalerie;
  S : string;
  Trouve_Gauche, Trouve_Droite : boolean;
  K, Erreur : integer;
begin
  if N_Visee=0 then begin
    Trouve_Gauche:=false;
    Trouve_Droite:=false;
    for K:=N_Galerie-1 downto 0 do if not Trouve_Gauche then begin
      L_autre:=Liste_Galerie[K];
      if L_Autre.Arrivee.Identique(TGalerie(Liste_Galerie[N_Galerie]).Depart) then begin
        S:=L_Autre.Visees[L_Autre.Nb_Visee].LargeurGauche;
        val(S,A_Gauche,Erreur);
        if Erreur<>0 then begin
          S:=copy(S,1,length(S)-1);
          val(S,A_Gauche,Erreur);
        end;
        if L_autre.Visees[L_Autre.Nb_Visee].ULarg=cm then A_Gauche:=A_Gauche/100;
        Trouve_Gauche:=(Erreur=0);
        S:=L_Autre.Visees[L_Autre.Nb_Visee].LargeurDroite;
        if Trouve_Gauche and (S='') then begin
          A_Gauche:=A_Gauche/2; A_Droite:=A_Gauche;
          Trouve_Droite:=true;
        end
        else begin
          val(S,A_Droite,Erreur);
          if Erreur<>0 then begin
            S:=copy(S,1,length(S)-1);
            val(S,A_Droite,Erreur);
          end;
          if L_autre.Visees[L_Autre.Nb_Visee].ULarg=cm then A_Droite:=A_Droite/100;
          Trouve_Droite:=(Erreur=0);
        end;
      end;
    end;
  end
  else with TGalerie(Liste_Galerie[N_Galerie]) do begin
    S:=Visees[N_Visee].LargeurGauche;
    val(S,A_Gauche,Erreur);
    if Erreur<>0 then begin
      S:=copy(S,1,length(S)-1);
      val(S,A_Gauche,Erreur);
    end;
    if Visees[N_Visee].ULarg=cm then A_Gauche:=A_Gauche/100;
    Trouve_Gauche:=(Erreur=0);
    S:=Visees[N_Visee].LargeurDroite;
    if Trouve_Gauche and (S='') then begin
      A_Gauche:=A_Gauche/2; A_Droite:=A_Gauche;
      Trouve_Droite:=true;
    end
    else begin
      val(S,A_Droite,Erreur);
      if Erreur<>0 then begin
        S:=copy(S,1,length(S)-1);
        val(S,A_Droite,Erreur);
      end;
      if Visees[N_Visee].ULarg=cm then A_Droite:=A_Droite/100;
      Trouve_Droite:=(Erreur=0);
    end;
  end;
  if not Trouve_Gauche then A_Gauche:=-1;
  if not Trouve_Droite then A_Droite:=-1;
end;

(***************  TMassif ********************)

constructor TMassif.Create;
var
  Zero : reel;
begin
  inherited Create;
  Liste_Fiche:=TList.Create;
  Liste_Fiche.Capacity:=16;
  Liste_Courante:=Liste_Fiche;
  Liste_Reference:=TList.Create;
  Liste_Reference.Capacity:=16;
  Germe_Coupe:=TList.Create;
  Germe_Coupe.Capacity:=16;
  Corbeille:=TList.Create;
  Corbeille.Capacity:=5;
  Raccourci:=TList.Create;
  Raccourci.Capacity:=16;
  Sauvegarde:=true;
  Fichier:='';
  Date_Decli:='01/01/80';
  Zero:=0;
  Origine_Decli:=Zero;
  Angle_Geodesique:=Zero;
  Variation_Decli:=1;
  Existe_Selection:=false;
  Finition:=false;
  Gabarit_Tableau:=false;
  Seuil_Deformation:=0.05;
  Alerte_Perte:=true;
end;

destructor TMassif.Destroy;
var
  I : SmallInt;
begin
  for I:=0 to Liste_Fiche.Count-1 do TFiche(Liste_Fiche[I]).Free;
  Liste_Fiche.Free; Liste_Fiche:=nil;
  for I:=0 to Liste_Reference.Count-1 do TPointTopo(Liste_Reference[I]).Free;
  Liste_Reference.Free; Liste_Reference:=nil;
  for I:=0 to Germe_Coupe.Count-1 do TPointCoupe(Germe_Coupe[I]).Free;
  Germe_Coupe.Free; Germe_Coupe:=nil;
  for I:=0 to Corbeille.Count-1 do
    TFiche(Corbeille[I]).Free;
  Corbeille.Free; Corbeille:=nil;
  for I:=0 to Raccourci.Count-1 do TRaccourci(Raccourci[I]).Free;
  Raccourci.Free; Raccourci:=nil;
  Selection.Free; Selection:=nil;
  inherited Destroy;
end;

function TMassif.Nouveau_Point(Proposition : string; I0 : integer) : integer;
var
  I, J, K : integer;
  Trouve : boolean;
  S : string;

begin
  Liste_Fiche.Pack;
  if Liste_Fiche.Count=0 then Nouveau_Point:=I0+1
  else begin
    I:=I0;
    repeat
      inc(I);
      Trouve:=false;
      J:=Liste_Fiche.Count;
      repeat
        with TFiche(Liste_Fiche[J-1]) do
        if Liste_Galerie.Count<>0 then begin
          K:=0;
          repeat
            with TGalerie(Liste_Galerie[K]) do begin
              if (Depart.Indice=I) and (not Depart.Terminus) then begin
                Depart.Nom.Lit(S);
                if UpperCase(S)=Proposition then Trouve:=true;
              end;
              if (Arrivee.Indice=I) and (not Arrivee.Terminus) then begin
                Arrivee.Nom.Lit(S);
                if UpperCase(S)=Proposition then Trouve:=true;
              end;
              inc(K);
            end;
          until (K=Liste_Galerie.Count) or Trouve;
        end;
        dec(J);
      until (J=0) or Trouve;
    until not Trouve;
    Nouveau_Point:=I;
  end;
end;

procedure TMassif.Enregistrer(Destination : TFileName);
var
  F : TFileBufferStream;
  G : textFile;
  I : longint;
  Taille : SmallInt;
  Fiche_En_Cours : boolean;
begin
{$IFNDEF FUSION}
  Form1.Gauge1.Position:=0;
  Form1.Gauge1.visible:=true;
  Form1.StatusBar.Panels[0].Text:='Sauvegarde du massif ...';
  Form1.Refresh;
{$ENDIF}
  Liste_Fiche.Pack;
  if Destination='' then Destination:=Fichier;
  if UpperCase(ExtractFileExt(Destination))<>'.MAS' then begin
    if Destination='' then F:=TFileBufferStream.Create(Fichier,fmCreate)
                      else F:=TFileBufferStream.Create(Destination,fmCreate);
    F.write(Version,sizeOf(Version));
    Taille:=Liste_Fiche.Count;
    F.write(Taille,SizeOf(Taille));
    for I:=0 to Liste_Fiche.Count-1 do with TFiche(Liste_Fiche[I]) do begin;
      Sauve(F);
{$IFNDEF FUSION}
      Form1.Gauge1.Position:=round(I/Taille*100);
{$ENDIF}
      Application.ProcessMessages;
    end;
    Taille:=Liste_Reference.Count;
    F.write(Taille,SizeOf(Taille));
    for I:=0 to Liste_Reference.Count-1 do TPointTopo(Liste_Reference[I]).Sauve(F);
    F.write(Origine_Decli,SizeOf(Origine_Decli));
    F.write(Date_Decli,SizeOf(Date_Decli));
    F.write(Variation_Decli,SizeOf(Variation_Decli));
    F.write(Angle_Geodesique,SizeOf(Angle_Geodesique));
    Taille:=Germe_Coupe.Count;
    F.write(Taille,SizeOf(Taille));
    for I:=0 to Germe_Coupe.Count-1 do TPointCoupe(Germe_Coupe[I]).Sauve(F);
    Taille:=Corbeille.Count;
    F.write(Taille,SizeOf(Taille));
    for I:=0 to Corbeille.Count-1 do TFiche(Corbeille[I]).Sauve(F);
    Taille:=Raccourci.Count;
    F.write(Taille,SizeOf(Taille));
    for I:=0 to Raccourci.Count-1 do TRaccourci(Raccourci[I]).Sauve(F);
{$IFNDEF FUSION}
    Fiche_En_Cours:=MultPageDlg.Visible;
    F.write(Fiche_En_Cours,SizeOf(Fiche_En_Cours));
    if Fiche_En_Cours then begin
      MultPageDlg.Sauvegarder(f);
    end;
    Form1.Gauge1.visible:=false;
    if Existe_Selection then Form1.StatusBar.Panels[0].Text:='Slection'
                              else Form1.StatusBar.Panels[0].Text:='';
{$ENDIF}
    F.Free;
  end
  else begin
    assignFile(G,Destination);
    rewrite(G);
    writeln(G,'Version '+IntTostr(Version));
    Taille:=Liste_Fiche.Count;
    writeln(G,'Nb_Fiche '+IntTostr(Taille));
    writeln(G);
    for I:=0 to Liste_Fiche.Count-1 do with TFiche(Liste_Fiche[I]) do begin
      SauveTxt(G);
{$IFNDEF FUSION}
      Form1.Gauge1.Position:=round(I/Taille*100);
{$ENDIF}
      Application.ProcessMessages;
      writeln(G);
    end;
    Taille:=Liste_Reference.Count;
    writeln(G,'Nb_Reference '+IntTostr(Taille));
    for I:=0 to Liste_Reference.Count-1 do TPointTopo(Liste_Reference[I]).SauveTxt(G);
    writeln(G,'Origine_Declinaison '+FloatTostrF(Origine_Decli,ffGeneral,6,4));
    writeln(G,'Date_Declinaison ',Date_Decli);
    writeln(G,'Variation_Declinaison '+FloatTostrF(Variation_Decli,ffGeneral,6,4));
    writeln(G,'Angle_Geodesique '+FloatTostrF(Angle_Geodesique,ffGeneral,6,4));
    Taille:=Germe_Coupe.Count;
    writeln(G,'Nb_Coupe '+IntTostr(Taille));
    for I:=0 to Germe_Coupe.Count-1 do TPointCoupe(Germe_Coupe[I]).SauveTxt(G);

    Taille:=Corbeille.Count; writeln(G,'Corbeille '+IntTostr(Taille));
    writeln(G);
    for I:=0 to Corbeille.Count-1 do begin
      TFiche(Corbeille[I]).SauveTxt(G);
      writeln(G);
    end;

    Taille:=Raccourci.Count; writeln(G,'Nb_Raccourci '+IntTostr(Taille));
    writeln(G);
    for I:=0 to Raccourci.Count-1 do begin
      TRaccourci(Raccourci[I]).SauveTxt(G);
      writeln(G);
    end;
{    Fiche_En_Cours:=MultPageDlg.Visible;
    F.write(Fiche_En_Cours,SizeOf(Fiche_En_Cours));
    if Fiche_En_Cours then begin
      MultPageDlg.Sauvegarder(f);
    end; }
{$IFNDEF FUSION}
    Form1.Gauge1.visible:=false;
    if Existe_Selection then Form1.StatusBar.Panels[0].Text:='Slection'
                        else Form1.StatusBar.Panels[0].Text:='';
{$ENDIF}
    CloseFile(G);
  end;
end;

{$IFDEF FUSION}
function TMassif.Recuperer : boolean;
{$ELSE}
procedure TMassif.Recuperer;
{$ENDIF}
var
  F : TFileBufferStream;
  G : textFile;
  I : longint;
  Taille : SmallInt;
  La_Fiche : TFiche;
  Le_Point : TPointTopo;
  Le_Germe : TPointCoupe;
  Le_Raccourci : TRaccourci;
  Fiche_En_Cours : boolean;
  S : string;
begin
  Fiche_En_Cours:=false;
  if UpperCase(ExtractFileExt(Fichier))<>'.MAS' then begin
    F:=TFileBufferStream.Create(Fichier,fmOpenRead);
    F.Read(Ver,SizeOf(Version));
    if Ver>Version then begin
      MessageDlg('Ce fichier correspond  une version plus rcente du logiciel',
      mtWarning,[mbOk],0);
    end
    else begin
{$IFNDEF FUSION}
      Form1.Gauge1.Position:=0;
      Form1.Gauge1.visible:=true;
      Form1.StatusBar.Panels[0].Text:='Lecture du massif ...';
      Form1.Refresh;
{$ENDIF}
      F.Read(Taille,sizeOf(Taille));
      for I:=1 to Taille do begin
        La_Fiche:=TFiche.Create;
        La_Fiche.Loade(F);
        Liste_Fiche.Add(La_Fiche);
{$IFNDEF FUSION}
        Form1.Gauge1.Position:=round(I/Taille*100);
{$ENDIF}
      end;
      F.Read(Taille,sizeOf(Taille));
      for I:=1 to Taille do begin
        Le_Point:=TPointTopo.Create;
        Le_Point.Loade(F);
        Liste_Reference.Add(Le_Point);
      end;
      F.Read(Origine_Decli,SizeOf(Origine_Decli));
      F.Read(Date_Decli,SizeOf(Date_Decli));
      F.Read(Variation_Decli,SizeOf(Variation_Decli));
      if Ver>=1002 then F.Read(Angle_Geodesique,SizeOf(Angle_Geodesique))
      else Angle_Geodesique:=0;
      F.Read(Taille,sizeOf(Taille));
      for I:=1 to Taille do begin
        Le_Germe:=TPointCoupe.Create;
        Le_Germe.Loade(F);
        Germe_Coupe.Add(Le_Germe);
      end;
      if Ver>=1005 then begin
        F.Read(Taille,sizeOf(Taille));
        for I:=1 to Taille do begin
          La_Fiche:=TFiche.Create;
          La_Fiche.Loade(F);
          Corbeille.Add(La_Fiche);
        end;
        F.Read(Taille,sizeOf(Taille));
        for I:=1 to Taille do begin
          Le_Raccourci:=TRaccourci.Create;
          Le_Raccourci.Loade(F);
          Raccourci.Add(Le_Raccourci);
        end;
      end;
      F.Read(Fiche_En_Cours,SizeOf(Fiche_En_Cours));
{$IFDEF FUSION}
      Recuperer:=not Fiche_En_Cours;
{$ELSE}
      if Fiche_En_Cours then begin
        MultPageDlg.Recuperer(f);
        Form1.Timer2.Enabled:=true;
      end;
      Form1.Gauge1.visible:=false;
      Form1.StatusBar.Panels[0].Text:='';
{$ENDIF}
    end;
    F.Free;
  end
  else begin
    AssignFile(G,Fichier);
    Reset(G);
    readln(G,S); delete(S,1,8); Ver:=strToInt(S);
    if Ver>Version then begin
      MessageDlg('Ce fichier correspond  une version plus rcente du logiciel',
      mtWarning,[mbOk],0);
    end
    else begin
{$IFNDEF FUSION}
      Form1.Gauge1.Position:=0;
      Form1.Gauge1.visible:=true;
      Form1.StatusBar.Panels[0].Text:='Lecture du massif ...';
      Form1.Refresh;
{$ENDIF}
      readln(G,S); delete(S,1,9); Taille:=StrToInt(S);
      readln(G);
      for I:=1 to Taille do begin
        La_Fiche:=TFiche.Create;
        La_Fiche.LoadeTxt(G);
        Liste_Fiche.Add(La_Fiche);
{$IFNDEF FUSION}
        Form1.Gauge1.Position:=round(I/Taille*100);
{$ENDIF}
        readln(G);
      end;
      readln(G,S); delete(S,1,13); Taille:=StrToInt(S);
      for I:=1 to Taille do begin
        Le_Point:=TPointTopo.Create;
        Le_Point.LoadeTxt(G);
        Liste_Reference.Add(Le_Point);
      end;
      readln(G,S); delete(S,1,20); Origine_Decli:=StrToFloat(S);
      readln(G,S); Date_Decli:=copy(S,18,length(S));
      readln(G,S); delete(S,1,22); Variation_Decli:=StrToFloat(S);
      readln(G,S); delete(S,1,17); Angle_Geodesique:=StrToFloat(S);
      readln(G,S); delete(S,1,9); Taille:=StrToInt(S); // Nb_coupe
      for I:=1 to Taille do begin
        Le_Germe:=TPointCoupe.Create;
        Le_Germe.LoadeTxt(G);
        Germe_Coupe.Add(Le_Germe);
      end;
      if Ver>=1005 then begin
        readln(G,S); delete(S,1,10); Taille:=StrToInt(S); // Corbeille
        readln(G);
        for I:=1 to Taille do begin
          La_Fiche:=TFiche.Create;
          La_Fiche.LoadeTxt(G);
          Corbeille.Add(La_Fiche);
          readln(G);
        end;
        readln(G,S); delete(S,1,13); Taille:=StrToInt(S);
        readln(G);
        for I:=1 to Taille do begin
          Le_Raccourci:=TRaccourci.Create;
          Le_Raccourci.LoadeTxt(G);
          Raccourci.Add(Le_Raccourci);
          readln(G);
        end;
      end;
{      F.Read(Fiche_En_Cours,SizeOf(Fiche_En_Cours));
      if Fiche_En_Cours then begin
        MultPageDlg.Recuperer(f);
        Form1.Timer2.Enabled:=true;
      end; }
{$IFNDEF FUSION}
      Form1.Gauge1.visible:=false;
      Form1.StatusBar.Panels[0].Text:='';
{$ENDIF}
    end;
    CloseFile(G);
  end;
  Finition:=false;
  Finition_Coupe:=false;
end;

procedure TMassif.Remplacer(var Ancienne, Nouvelle : TFiche);
{var
  Ancienne_Galerie, Nouvelle_Galerie : TGalerie;
  A_Point, N_Point : TPointTopo;
  I, Decalage : integer;
  Alerte : boolean;
  Equivalent : array[1..340] of integer;
  Bloque : array[1..340] of boolean; }
begin
  Nouvelle.Ordre:=Ancienne.Ordre;
{  Alerte:=false;
  for I:=0 to Ancienne.Liste_Galerie.Count-1 do begin
    Ancienne_Galerie:=Ancienne.Liste_Galerie[I];
    if Ancienne_Galerie.En_Coupe then begin
      Alerte:=true;
      Equivalent[I]:=-1;
    end
    else Equivalent[I]:=-2;
  end;
  if Alerte then begin
    Decalage:=0;
    I:=0;
    repeat
      Ancienne_Galerie:=Ancienne.Liste_Galerie[I];
      Nouvelle_Galerie:=Nouvelle.Liste_Galerie[I+Decalage];
      A_Point:=Ancienne_Galerie.Depart;
      N_Point:=Nouvelle_Galerie.Depart;
      if A_Point.Identique(N_Point) then begin
        A_Point:=Ancienne_Galerie.Arrivee;
        N_Point:=Nouvelle_Galerie.Arrivee;
        if A_Point.Identique(N_Point) then begin
          if Ancienne_Galerie.En_Coupe then with Nouvelle_Galerie do begin
            Equivalent[I]:=I+Decalage;
            Accrochage_Debut:=Ancienne_Galerie.Accrochage_Debut;
            Gauche_Droite:=Ancienne_Galerie.Gauche_Droite;
            Nb_Suite:=Ancienne_Galerie.Nb_Suite;
            Ordre_Suite:=Ancienne_Galerie.Ordre_Suite;
            Galerie_Suite:=Ancienne_Galerie.Galerie_Suite;
            Decal_Hori:=Ancienne_Galerie.Decal_Hori;
            Decal_Z:=Ancienne_Galerie.Decal_Z;
            Fin_Hori:=Ancienne_Galerie.Fin_Hori;
          end;
        end
        else begin
          if I+Decalage+1<=Nouvelle.Liste_Galerie.Count-1
  end; }
  Liste_Fiche.Remove(Ancienne);
  Liste_Fiche.Add(Nouvelle);
  if Existe_Selection and (Selection.IndexOf(Ancienne)>=0) then begin
    Selection.Remove(Ancienne);
    Selection.Add(Nouvelle);
    Selection.Pack;
  end;
  Finition:=false;
end;

procedure TMassif.Ajouter(var Nouvelle : TFiche;Local : boolean);
var
  I : SmallInt;
  Maximum : longint;
begin
  Liste_Fiche.Pack;
  Maximum:=0;
  for I:=0 to Liste_Fiche.Count-1 do with TFiche(Liste_Fiche[I]) do
    if Maximum<Ordre then Maximum:=Ordre;
  for I:=0 to Corbeille.Count-1 do with TFiche(Corbeille[I]) do
    if Maximum<Ordre then Maximum:=Ordre;
  Nouvelle.Ordre:=Maximum+1;
  Liste_Fiche.Add(Nouvelle);
  if Local and (Selection<>nil) then Selection.Add(Nouvelle);
  Finition:=false;
end;

procedure TMassif.Retirer(var Ancienne : TFiche);
begin
  Liste_Fiche.Remove(Ancienne);
  Liste_Fiche.Pack;
  if Existe_Selection and (Selection.IndexOf(Ancienne)>=0) then begin
    Selection.Remove(Ancienne);
    Selection.Pack;
  end;
{$IFNDEF FUSION}
  Corbeille.Add(Ancienne);
{$ENDIF}  
  Finition:=false;
end;

procedure TMassif.Trier;
var
  I, J : SmallInt;
  FicheA, FicheB : TFiche;
begin
  for I:=0 to Liste_Fiche.Count-2 do begin
    for J:=Liste_Fiche.Count-1 downto I+1 do begin
      FicheA:=Liste_Fiche[J-1];
      FicheB:=Liste_Fiche[J];
      if FicheA.Ordre>FicheB.Ordre then begin
        Liste_Fiche[J-1]:=FicheB;
        Liste_Fiche[J]:=FicheA;
      end;
    end;
  end;
  Trier_Selection;
end;

procedure TMassif.Trier_Selection;
var
  I, J : SmallInt;
  FicheA, FicheB : TFiche;
begin
  if Existe_Selection then
  for I:=0 to Selection.Count-2 do begin
    for J:=Selection.Count-1 downto I+1 do begin
      FicheA:=Selection[J-1];
      FicheB:=Selection[J];
      if FicheA.Ordre>FicheB.Ordre then begin
        Selection[J-1]:=FicheB;
        Selection[J]:=FicheA;
      end;
    end;
  end;
end;

procedure TMassif.Ecrire_Point(var Nouveau : TPointTopo);
begin
  Liste_Reference.Add(Nouveau);
  Finition:=false;
end;

procedure TMassif.Effacer_Points;
var
  Un_Point : TPointTopo;
  I : SmallInt;
begin
  for I:=0 to Liste_Reference.Count-1 do begin
    Un_Point:=Liste_Reference[0];
    Liste_Reference.Remove(Un_Point);
    Un_Point.Free;
  end;
  Finition:=false;
end;

procedure TMassif.Detailler;
var
  I, J : integer;
begin
  Finition:=true;
  for I:=0 to Liste_Fiche.Count-1 do with TFiche(Liste_Fiche[I]) do
    for J:=0 to Liste_Galerie.Count-1 do TGalerie(Liste_Galerie[J]).Detailler;
end;

procedure TMassif.Calculer_Coupe;
var
  I : integer;
begin
  try
    for I:=0 to Germe_Coupe.Count-1 do with TPointCoupe(Germe_Coupe[I]) do
      Coupe_Une_Galerie(Ordre_Suite,Galerie_Suite,Decal_Hori,Decal_Z);
  except
    // on arrive ici s'il y a un bug dans les coupes
    // alors, on fait le mnage
{$IFDEF DEBUG}
    if MessageDlg('La coupe du massif prsente une incohrence, supprimer toutes les coupes?',
     mtWarning,[mbOk,mbCancel],0)=mrOk then
{$ENDIF}
    begin
      Nettoyage_Coupe;
{$IFNDEF FUSION}
      if Form1.LaCoupe<>nil then Form1.LaCoupe.Mise_A_Jour;
{$ENDIF}
    end;
  end;
end;

procedure TMassif.Coupe_Une_Galerie(N_Ordre,N_Galerie : smallInt; DX, DZ : reel);
var
  I : integer;
  Longueur_Plane : reel;
begin
  I:=0;
  while TFiche(Liste_Fiche[I]).Ordre<>N_Ordre do inc(I);
  with TGalerie(TFiche(Liste_Fiche[I]).Liste_Galerie[N_Galerie]) do begin
    Decal_Z:=DZ;
    Decal_Hori:=DX;
    Longueur_Plane:=0;
    for I:=1 to Nb_Visee do with Visees[I] do begin
      if Correction_Horizontale then Longueur_Plane:=Longueur_Plane+Longueur_Horizontale
      else Longueur_Plane:=Longueur_Plane+LH;
    end;
    if not Gauche_Droite then Longueur_Plane:=-Longueur_Plane;
    for I:=1 to Nb_Suite do
       Coupe_Une_Galerie(Ordre_Suite[I],Galerie_Suite[I],DX+Longueur_Plane,DZ);
    Finition_Coupe:=false;
  end;
end;

procedure TMassif.Detailler_Coupe;
var
  I, J, K : integer;
  Longueur_Plane : reel;
begin
  if not Finition then Detailler;
  for I:=0 to Liste_Fiche.Count-1 do with TFiche(Liste_Fiche[I]) do begin
    for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
      if En_Coupe then begin
        Longueur_Plane:=0;
        if Accrochage_Debut then begin
          for K:=1 to Nb_Visee do begin
            Longueur_Plane:=Longueur_Plane+Visees[K].LH;
            if Gauche_Droite then Coordonnees^[K].LH:=Decal_Hori+Longueur_Plane
            else Coordonnees^[K].LH:=Decal_Hori-Longueur_Plane;
          end;
          if Gauche_Droite then Fin_Hori:=Decal_Hori+Longueur_Plane
          else Fin_Hori:=Decal_Hori-Longueur_Plane;
        end
        else begin
          for K:=Nb_Visee downto 1 do begin
            if Gauche_Droite then Coordonnees^[K].LH:=Decal_Hori+Longueur_Plane
            else Coordonnees^[K].LH:=Decal_Hori-Longueur_Plane;
            Longueur_Plane:=Longueur_Plane+Visees[K].LH;
          end;
          if Gauche_Droite then Fin_Hori:=Decal_Hori+Longueur_Plane
          else Fin_Hori:=Decal_Hori-Longueur_Plane;
        end;
      end;
    end;
  end;
  Finition_Coupe:=true;
end;

procedure TMassif.Detecteur_Erreur_Coupe;
var
  I, J, K, U : integer;
  Une_Fiche : TFiche;
  Une_Galerie : TGalerie;
  Erreur : boolean;
begin
  Erreur:=false;
  for I:=0 to Liste_Fiche.Count-1 do with TFiche(Liste_Fiche[I]) do begin
    for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
      if En_Coupe then begin
        for K:=1 to Nb_Suite do begin
          U:=0;
          repeat
            Une_Fiche:=Liste_Fiche[U];
            inc(U);
          until Une_Fiche.Ordre=Ordre_Suite[K];
          Une_Galerie:=Une_Fiche.Liste_Galerie[Galerie_Suite[K]];
          if not Une_Galerie.En_Coupe then begin
            Erreur:=true;
          end;
        end;
      end;
    end;
  end;
  if Erreur then ShowMessage('Problme dans la coupe');
  for I:=0 to Liste_Fiche.Count-1 do with TFiche(Liste_Fiche[I]) do begin
    for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
      if En_Coupe then begin
        Erreur:=false;
        for K:=1 to Nb_Suite do begin
          if not Erreur then begin
            U:=0;
            repeat
              Une_Fiche:=Liste_Fiche[U];
              inc(U);
            until Une_Fiche.Ordre=Ordre_Suite[K];
            Une_Galerie:=Une_Fiche.Liste_Galerie[Galerie_Suite[K]];
            if not Une_Galerie.En_Coupe then begin
              Erreur:=true;
            end;
          end
          else begin
            Ordre_Suite[K-1]:=Ordre_Suite[K];
            Galerie_Suite[K-1]:=Galerie_Suite[K];
          end;
        end;
        if Erreur then dec(Nb_Suite);
      end;
    end;
  end;
end;

function TMassif.Grosse_Deformation(var La_Fiche, La_Galerie : integer):reel;
var
  Contrainte_Max, Pourcent : double;
  I, J : integer;
begin
  if not Finition then Detailler;
  Contrainte_Max:=0;
  for I:=0 to liste_Fiche.Count-1 do with TFiche(Liste_Fiche[I]) do begin
    for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
      if Approximation and (Nb_Visee<>0) then begin
        if Nb_Visee=1 then begin  // s'il n'y a qu'une vise, elle ne peut tre de longueur nulle
          if Visees[1].Longueur>1e-5 then begin
            Pourcent:=sqr((Coordonnees^[1].X-Depart.X)-Visees[1].X);
            Pourcent:=Pourcent+sqr((Coordonnees^[1].Y-Depart.Y)-Visees[1].Y);
            Pourcent:=Pourcent+sqr((Coordonnees^[1].Z-Depart.Z)-Visees[1].Z);
            Pourcent:=sqrt(Pourcent)/Visees[1].Longueur/Precision;
            if Visees[1].ULong=cm then Pourcent:=Pourcent*100;
            if Pourcent>Contrainte_Max then begin
              Contrainte_Max:=Pourcent;
              La_Fiche:=I;
              La_Galerie:=J;
            end;
          end;
        end
        else begin  // s'il y a plusieurs vises, on vite de prendre la premire qui peut
                    // tre de longueur nulle
          if Nb_Visee=0 then
            ShowMessage('Tout faux');
          Pourcent:=sqr((Coordonnees^[2].X-Coordonnees^[1].X)-(Visees[2].X-Visees[1].X));
          Pourcent:=Pourcent+sqr((Coordonnees^[2].Y-Coordonnees^[1].Y)-(Visees[2].Y-Visees[1].Y));
          Pourcent:=Pourcent+sqr((Coordonnees^[2].Z-Coordonnees^[1].Z)-(Visees[2].Z-Visees[1].Z));
          Pourcent:=sqrt(Pourcent)/Visees[2].Longueur/Precision;
          if Visees[2].ULong=cm then Pourcent:=Pourcent*100;
          if Pourcent>Contrainte_Max then begin
            Contrainte_Max:=Pourcent;
            La_Fiche:=I;
            La_Galerie:=J;
          end;
        end;
      end;
    end;
  end;
  Grosse_Deformation:=Contrainte_Max;
end;

procedure TMassif.Retirer_Coupe(N_Ordre : longint; N_Galerie : integer; Racine : boolean);
var
  I, J, K : integer;
  Une_Fiche : TFiche;
  Un_Germe : TPointCoupe;
  Detecte : boolean;
begin
  I:=0;
  repeat
    Une_Fiche:=Liste_Fiche[I];
    inc(I);
  until Une_Fiche.Ordre=N_Ordre;
  with TGalerie(Une_Fiche.Liste_Galerie[N_Galerie]) do begin
    En_Coupe:=false;
    for I:=1 to Nb_Suite do
       Retirer_coupe(Ordre_Suite[I],Galerie_Suite[I],false);
  end;
  if Racine then begin
    Detecte:=false;
    for I:=0 to Germe_Coupe.Count-1 do begin
      if Detecte then Un_Germe:=Germe_Coupe[I-1]
                 else Un_Germe:=Germe_Coupe[I];
      if (Un_Germe.Ordre_Suite=N_Ordre) and (Un_Germe.Galerie_Suite=N_Galerie) then begin
        Germe_Coupe.Remove(Un_Germe);
        Germe_Coupe.Pack;
        Un_Germe.Free;
        Detecte:=true;
      end;
    end;
    for I:=0 to Liste_Fiche.Count-1 do with TFiche(Liste_Fiche[I]) do begin
      for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
        Detecte:=false;
        if En_Coupe then begin
          for K:=1 to Nb_Suite do begin
            if not Detecte and (Ordre_Suite[K]=N_Ordre)
                           and (Galerie_Suite[K]=N_Galerie) then Detecte:=true;
            if Detecte then begin
              if K<Nb_Suite then begin
                Ordre_Suite[K]:=Ordre_Suite[K+1];
                Galerie_Suite[K]:=Galerie_Suite[K+1];
              end;
            end;
          end;
          if Detecte then dec(Nb_Suite);
        end;
      end;
    end;
  end;
end;

procedure TMassif.Nettoyage_Coupe;
var
  I, J : integer;
begin
  for I:=0 to Liste_Fiche.Count-1 do with TFiche(Liste_Fiche[I]) do begin
    for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
      En_Coupe:=false;
      Nb_Suite:=0;
    end;
  end;
  for I:=0 to Germe_Coupe.Count-1 do TPointCoupe(Germe_Coupe[I]).Free;
  Germe_Coupe.Free; Germe_Coupe:=nil; Germe_Coupe:=TList.Create;
  Sauvegarde:=false;
  Detecteur_Erreur_Coupe;
end;

(*****************************************************************************)
(*                            TRaccourci                                     *)
(*****************************************************************************)


constructor TRaccourci.Create;
begin
  inherited Create;
end;

destructor TRaccourci.Destroy; 
begin
  inherited Destroy;
end;

procedure TRaccourci.SauveTxt(var F : textFile);
begin
end;

procedure TRaccourci.LoadeTxt(var F : textFile);
begin
end;

procedure TRaccourci.Sauve(var F : TFileBufferStream);
begin
end;

procedure TRaccourci.Loade(var F : TFileBufferStream);
begin
end;


function Calibre(var PMin, PMax : TPoint3D;La_Liste : TList) : boolean;
var
  I, J, K : integer;
  X1, Y1, Z1 : reel;
  Trouve : boolean;
begin
  Trouve:=false;
  for I:=0 to La_Liste.Count-1 do with TFiche(La_Liste[I]) do begin
    for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
      if Approximation then begin
        if not Trouve then begin
          Trouve:=true;
          PMin.X:=Depart.X;
          PMin.Y:=Depart.Y;
          PMin.Z:=Depart.Z;
          PMax.X:=PMin.X; PMax.Y:=PMin.Y; PMax.Z:=PMin.Z;
        end
        else begin
          X1:=Depart.X;
          Y1:=Depart.Y;
          Z1:=Depart.Z;
          if X1>PMax.X then PMax.X:=X1; if X1<PMin.X then PMin.X:=X1;
          if Y1>PMax.Y then PMax.Y:=Y1; if Y1<PMin.Y then PMin.Y:=Y1;
          if Z1>PMax.Z then PMax.Z:=Z1; if Z1<PMin.Z then PMin.Z:=Z1;
        end;
        for K:=1 to Nb_Visee do begin
          X1:=Coordonnees^[K].X;
          Y1:=Coordonnees^[K].Y;
          Z1:=Coordonnees^[K].Z;
          if X1>PMax.X then PMax.X:=X1; if X1<PMin.X then PMin.X:=X1;
          if Y1>PMax.Y then PMax.Y:=Y1; if Y1<PMin.Y then PMin.Y:=Y1;
          if Z1>PMax.Z then PMax.Z:=Z1; if Z1<PMin.Z then PMin.Z:=Z1;
        end;
      end;
    end;
  end;
  Calibre:=Trouve;
end;

function Calibre_Coupe(var PMin, PMax : TPoint3D;La_Liste : TList) : boolean;
var
  I, J, K : integer;
  Z1 : reel;
  Trouve : boolean;
begin
  Trouve:=false;
  PMin.LH:=1e38; PMax.LH:=-1e38;
  for I:=0 to La_Liste.Count-1 do with TFiche(La_Liste[I]) do begin
    for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
      if En_Coupe and Approximation then begin
        if PMin.LH>Decal_Hori then PMin.LH:=Decal_Hori;
        if PMin.LH>Fin_Hori then PMin.LH:=Fin_Hori;
        if PMax.LH<Decal_Hori then PMax.LH:=Decal_Hori;
        if PMax.LH<Fin_Hori then PMax.LH:=Fin_Hori;
        if not Trouve then begin
          Trouve:=true;
          PMin.Z:=Depart.Z+Decal_Z;
          PMax.Z:=PMin.Z;
        end
        else begin
          Z1:=Depart.Z+Decal_Z;
          if Z1>PMax.Z then PMax.Z:=Z1; if Z1<PMin.Z then PMin.Z:=Z1;
        end;
        for K:=1 to Nb_Visee do begin
          Z1:=Coordonnees^[K].Z+Decal_Z;
          if Z1>PMax.Z then PMax.Z:=Z1; if Z1<PMin.Z then PMin.Z:=Z1;
        end;
      end;
    end;
  end;
  Calibre_Coupe:=Trouve and (PMin.LH<PMax.LH);
end;

procedure Tuer_les_Coupe(Une_Fiche : TFiche);
var
  I : integer;
  Galerie : TGalerie;
begin
  for I:=0 to Une_Fiche.Liste_Galerie.Count-1 do begin
    Galerie:=Une_Fiche.Liste_Galerie[I];
{$IFNDEF FUSION}
    if Galerie.En_Coupe then Form1.Massif.Retirer_Coupe(Une_Fiche.Ordre,I,true);
{$ENDIF}    
  end;
end;

function Date2Jour(S : string):integer;
var
  I : integer;
begin
  result:=0;
  try
    result:=StrToInt(S[1]+S[2]);
  except
  end;
  try
    inc(result,StrToInt(S[4]+S[5])*32);
  except
  end;
  try
    I:=StrToInt(S[7]+S[8]);
    if I<50 then inc(I,100);
    inc(result,I*400);
  except
    assert(false,'Il n''y a mme pas d''anne dans Date2Jour');
  end;
end;

function StrToIntE(S : string) : integer;
// fonction StrToInt tendue qui renvoie 0 en cas d'erreur
begin
  try
    result:=StrToInt(S);
  except
    result:=0;
  end;
end;

function StrToFloatE(S : string) : extended;
// fonction StrToFloat tendue qui renvoie 0 en cas d'erreur
begin
  try
    result:=StrToFloat(S);
  except
    result:=0;
  end;
end;

end.
