unit FRose;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Fichetpo, ComCtrls, Stats;

const
  RoseStep    = 10;
  RoseCount   = 360 div RoseStep;
  LegendStep  = 30;
  LegendCount = 360 div LegendStep;

type
  TfrmRose = class(TForm)
    imgRose: TImage;
    StatusBar1: TStatusBar;
    procedure ComputeRose(Sender: TObject);
    procedure DrawRose(Sender: TObject);
    procedure ShowPie(Sender: TObject; Shift: TShiftState; X, Y: Integer);
  private
    { Dclarations prives }
    FRoses: array[0..RoseCount-1] of integer;
    FShots: integer;
  protected
    FMassif: TMassif;
  public
    { Dclarations publiques }
    class procedure ShowIt(AMassif: TMassif);
  end;

var

  frmRose : TfrmRose;  

implementation

uses Math;
{$R *.DFM}

class procedure TfrmRose.ShowIt(AMassif: TMassif);
begin

  with TfrmRose.Create(nil) do begin
    FMassif := AMassif;
    ShowModal;
    Free;
  end;
end;

// Il faut ventuellement ne pas tenir compte des vises "verticales"
procedure TfrmRose.ComputeRose(Sender: TObject);
var
  i,j,k: integer;
  angle: double;
  r: integer;
begin

  // Nettoie la rose des vents
  for i:=0 to RoseCount-1 do FRoses[i]:=0;

  // Parcours les fiches du massif et remplie la rose
  FShots := 0;
  with FMassif do begin
    with Statistique.Les_Fiches do begin
      for i:=0 to Count-1 do with TFiche(Items[i]) do begin
        with Liste_Galerie do begin
          for j:=0 to Count-1 do with TGalerie(Items[j]) do begin
            for k:=1 to Nb_Visee do with Visees[k] do if (Longueur>1e-5) and (LH>1e-5) then begin
              // Gestion rose des vents
              case UAzimuth of
                deg : angle := Azimuth/180*PI;
                ndeg: angle := (Azimuth/180+1)*PI;
                gr  : angle := Azimuth/200*PI;
                ngr : angle := (Azimuth/200+1)*PI;
              end;
              angle := trunc(angle*180/PI) mod 360;
              r := trunc(angle / RoseStep);
              if Pente<-1e-5 then inc(FRoses[r],2)
              else if Pente>1e-5 then inc(FRoses[(r+RoseCount div 2) mod RoseCount],2)
              else begin
                inc(FRoses[r]);
                inc(FRoses[(r+RoseCount div 2) mod RoseCount]);
              end;
              inc(FShots);
            end;
          end;
        end;
      end;
    end;
  end;
  DrawRose(Self);
end;

procedure TfrmRose.DrawRose(Sender: TObject);
var
  c,p1,p2,p3,p4: TPoint;
  r,ri,i: integer;
  a,rp:   double;
  rm:     integer;
  s:      string;
begin

  with imgRose,Canvas do begin
    // Redimensionne le bitmap
    Picture.Bitmap.Width  := Width;
    Picture.Bitmap.Height := Height;
    Brush.Color := clBtnFace;
    FillRect(ClientRect);

    // Dtermine le rayon le plus grand
    rm := 0;
    for i:=0 to RoseCount-1 do
      if FRoses[i] > rm then rm:=FRoses[i];

    with c do begin
      x := ClientWidth  div 2;
      y := ClientHeight div 2;
      if x<y then r:=x else r:=y;
      r := r-TextWidth('Ouest'); //*8 div 10;

      p1 := Point(x-r,y-r);
      p2 := Point(x+r,y+r);

      Pen.Color   := clBlack;
      Brush.Color := clWhite;
      Ellipse(p1.x,p1.y, p2.X,p2.y);

      Brush.Color := clAqua;
      for i:=0 to RoseCount-1 do begin
      //rp := Trunc(r*i*RoseStep/360); // Test
        rp := r*FRoses[i]/rm; //FShots);
        ri := round(rp);

        a := (i*RoseStep+RoseStep - 90)/180*PI;
        p3.x := Trunc(c.x + Cos(a)*rp);
        p3.y := Trunc(c.y + Sin(a)*rp);

        a  := (i*RoseStep - 90)/180*PI; // + PI/2 ;
        p4.x := round(c.x + Cos(a)*rp);
        p4.y := round(c.y + Sin(a)*rp);
        Pie(c.x-ri,c.y-ri, c.x+ri,c.y+ri, p3.x,p3.y, p4.x,p4.y);
      end;

      // Trac lgende
      SetTextAlign(Handle,TA_CENTER);
      SetBkMode(Handle, 1); //TRANSPARENT);

      Pen.Color := clBtnFace;
      for i:=0 to LegendCount-1 do begin
        rp := r + TextHeight('123');

        a := (i*LegendStep - 90)/180*PI;
        p4.x := round(c.x + Cos(a)*r);
        p4.y := round(c.y + Sin(a)*r);
        MoveTo(c.x,  c.y);
        LineTo(p4.x, p4.y);

        p3.x := Trunc(c.x + Cos(a)*rp);
        p3.y := Trunc(c.y + Sin(a)*rp);
        Font.Style := [fsBold];
        case i*LegendStep of
            0:  s:='Nord';
           90:  s:='Est';
          180:  s:='Sud';
          270:  s:='Ouest';
        else
          Font.Style := [];
          s:=format('%.3d%s', [i*LegendStep,tuAngle[deg]]);
        end;
        TextOut(p3.x, p3.y-TextHeight(s) div 2, s);
      end;
    end;
    Pen.Color   := clBlack;
    Brush.Color := clWhite;
    ri := Trunc(r*0.03);
    Ellipse(c.x-ri,c.y-ri, c.x+ri,c.y+ri);
  end;
end;

procedure TfrmRose.ShowPie(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
  dx,dy: integer;
  a:     double;
  c:     integer;
begin

  with TWinControl(Sender) do begin
    dx := ClientWidth  div 2 - X;
    dy := ClientHeight div 2 - Y;
  end;

  if (dx=0) and (dy>=0) then a:=0
  else a:=PI+ArcTan2(dx,-dy);
  a := a/PI*180;
  c := FRoses[trunc(a / RoseStep)];
  StatusBar1.SimpleText := format('%.3d%s -> %d vises sur %d (%d%%)', [trunc(a), tuAngle[deg],c div 2, FShots, c * 50 div FShots]);
end;

end.
