unit Grapheco;

interface

uses Graphes, Fichetpo, Classes, ExtCtrls, Printers, Windows, Graphics,
     SysUtils, Options, Forms, Dialogs, Controls, Couleurs, Branche, Recherch, Simple ;

type

  TCoupe= class (TProjection)
    constructor Create;
    procedure Dessine(Pixel_X, Pixel_Y : longint; Resolution, Rapport : double; Canevas : TImage; Etat : smallInt); override;
    procedure Imprime(Page_X, Page_Y, Sup_X, Sup_Y : integer; Etat : smallInt;
                              Fond : TCanvas; Reduc : double); override;
    procedure Gabarit(var Pixel_Hori, Pixel_Verti  : longint; Resolution, Rapport : double; Canevas : TImage); override;
    procedure Gabarit_Imprimante(var Page_X, Page_Y : integer; MargeX, MargeY : integer); override;
    function Trouve(X, Y : longint;var N_Fiche, N_Galerie, N_Visee : integer) : double; override;
    procedure Isoler(X, Y, Rayon2 : longint); override;
    function Met_En_Relief(Nouveau : boolean; N_Fiche, N_Galerie, N_Visee : integer; Etat : smallInt) : string; override;
    function Nouvelle_Branche : boolean;
    function Continuer(var N_Fiche, N_Galerie : integer) : boolean;
    procedure Inverser(N_Ordre : longint; N_Galerie : integer);
    procedure Retirer(N_Ordre : longint; N_Galerie : integer; Racine : boolean);
    function Decaler(N_Ordre : longint; N_Galerie : integer) : boolean;
    function Raccrocher(N_Ordre : longint; N_Galerie : integer) : boolean;
    procedure Export_Svg(Fichier_SVG: TFileName; Illustrator, AllLayers : boolean); override;
  end;

implementation

uses Unit1, svgUtils;

{********************** TCoupe ****************************************}
constructor TCoupe.Create;
begin
  inherited Create;
  Echelle:=100;
end;

procedure TCoupe.Dessine(Pixel_X, Pixel_Y : longint; Resolution, Rapport : double; Canevas : TImage; Etat : smallInt);
var
  I, J, K, Erreur : integer;
  L_autre : TGalerie;
  Contrainte_Max, Xi, Poids_1, Poids_2, X_Precedent, Z_Precedent, X_Suivant,
  Z_Suivant, Delta, X0, Y0, Dessus_Precedent, Dessus_Suivant , Dessous_Precedent,
  Dessous_Suivant, Ref : double;
  Trouve : boolean;
  S, S2 : string;
  Hors_Plan : TBitMap;
begin
  PixelX:=Pixel_X; PixelY:=Pixel_Y;

//  Le_Caneva:=Canevas;

  Hors_Plan:=TBitMap.Create;

  Hors_Plan.Width:=Canevas.Width;
  Hors_Plan.Height:=Canevas.Height;
  Le_Caneva:=Hors_Plan.Canvas;
  Le_Caneva.Pen.Color:=clWhite; Le_Caneva.Pen.Style:=psSolid; Le_Caneva.Pen.Mode:=pmCopy;
  Le_Caneva.Brush.Color:=clWhite; Le_Caneva.Brush.Style:=bsSolid;
//  Le_Caneva.Rectangle(0,0,Le_Caneva.Width,Le_Caneva.Height);
  La_Resolution:=Resolution*Rapport;
  if Affichable then begin
    Poids_1:=Form1.Les_Options_Coupe.Pourcent/200+0.5;
    Poids_2:=1-Poids_1;
    Le_Caneva.Pen.Color:=clBlack;
    Le_Caneva.Brush.Color:=clBlack; Le_Caneva.Brush.Style:=bsSolid;
    Le_Caneva.Pen.Width:=round(Form1.Les_Options_Coupe.Visee_Epaisseur/10*Resolution*Rapport);
    if Form1.Les_Options_Coupe.Carrefour>0 then begin
      Le_Caneva.Font:=Form1.Les_Options_Coupe.Station_police;
      Le_Caneva.Font.Height:=round(Le_Caneva.Font.Height*Rapport);
      Delta:=Le_Caneva.TextHeight('O')/Resolution/2000*Echelle/Rapport;
    end;
    if Etat=etContrainte then Contrainte_Max:=Contrainte_Maximum(Form1.Massif.Liste_Courante);
    for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
      for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
        if Approximation and En_Coupe then begin
          case Etat of
            etContrainte : Le_Caneva.Pen.Color:=Degrade[round(Contrainte_Z/Contrainte_Max*60)];
            etNoir, etRotation : Le_Caneva.Pen.Color:=clBlack;
            etCouleur :  Le_Caneva.Pen.Color:=Couleur;
          end;
          if Accrochage_Debut then begin
            X_Precedent:=Decal_Hori;
            Z_Precedent:=Decal_Z+Depart.Z;
          end
          else begin
            X_Precedent:=Fin_Hori;
            Z_Precedent:=Decal_Z+Depart.Z;
          end;
          if Form1.Les_Options_Coupe.Habit_Affiche or (Form1.Les_Options_Coupe.Type_Hauteur<>0) then begin
            Les_Hauteurs(J,0,dessus_precedent, dessous_precedent);
            if Dessus_Precedent>-0.5 then begin
              case Form1.Les_Options_Coupe.Type_Hauteur of
                1 : begin
                  Le_Caneva.Pen.Width:=round(Form1.Les_Options_Coupe.Epaisseur_Hauteur/10*Resolution*Rapport);
                  MoveToRel(X_Precedent,Z_Precedent);
                  LineToRel(X_Precedent,Z_Precedent+Dessus_Precedent);
                end;
                2 : begin
                  Le_Caneva.Pen.Width:=round(Form1.Les_Options_Coupe.Epaisseur_Hauteur/10*Resolution*Rapport);
                  Cercle(X_Precedent,Z_Precedent+Dessus_Precedent,Form1.Les_Options_Coupe.Epaisseur_Hauteur div 2);
                end;
              end;
            end;
            if Dessous_Precedent>-0.5 then begin
              case Form1.Les_Options_Coupe.Type_Hauteur of
                1 : begin
                  Le_Caneva.Pen.Width:=round(Form1.Les_Options_Coupe.Epaisseur_Hauteur/10*Resolution*Rapport);
                  MoveToRel(X_Precedent,Z_Precedent);
                  LineToRel(X_Precedent,Z_Precedent+Dessous_Precedent);
                end;
                2 : begin
                  Le_Caneva.Pen.Width:=round(Form1.Les_Options_Coupe.Epaisseur_Hauteur/10*Resolution*Rapport);
                  Cercle(X_Precedent,Z_Precedent+Dessous_Precedent,Form1.Les_Options_Coupe.Epaisseur_Hauteur div 2);
                end;
              end;
            end;
          end;
          if Form1.Les_Options_Coupe.Diametre<>0 then begin
            Le_Caneva.Brush.Color:=clBlack; Le_Caneva.Brush.Style:=bsSolid;
            Cercle(X_Precedent,Z_Precedent,Form1.Les_Options_Coupe.Diametre);
          end;
          if (Form1.Les_Options_Coupe.Carrefour>0) and not Depart.Terminus then begin
            Le_Caneva.Brush.Color:=clWhite; Le_Caneva.Brush.Style:=bsClear;
            Depart.Nom.lit(S);
            if Form1.Les_Options_Coupe.Carrefour=2 then begin
              S:=S+'('+IntTostr(Depart.Indice)+')';
            end;
            TextRel(X_Precedent+Delta,Z_Precedent+Delta,S);
          end;
          for K:=1 to Nb_Visee do begin
            X_Suivant:=Coordonnees^[K].LH;
            Z_Suivant:=Decal_Z+Coordonnees^[K].Z;
            MovetoRel(X_Precedent*Poids_1+X_Suivant*Poids_2,
                      Z_Precedent*Poids_1+Z_Suivant*Poids_2);
            LinetoRel(X_Precedent*Poids_2+X_Suivant*Poids_1,
                      Z_Precedent*Poids_2+Z_Suivant*Poids_1);

            Le_Caneva.Brush.Color:=clBlack; Le_Caneva.Brush.Style:=bsSolid;
            if Form1.Les_Options_Coupe.Habit_Affiche or (Form1.Les_Options_Coupe.Type_Hauteur<>0) then begin
              Les_Hauteurs(J,K,dessus_suivant, dessous_suivant);
              if (Dessus_Precedent>-0.5) and (Dessus_Suivant>-0.5) then begin
                if Form1.Les_Options_Coupe.Habit_Affiche then begin
                  Le_Caneva.Pen.Width:=round(Form1.Les_Options_Coupe.Habit_Epaisseur/10*Resolution*Rapport);
                  MoveToRel(X_Precedent,Z_Precedent+Dessus_Precedent);
                  LineToRel(X_Suivant,Z_Suivant+Dessus_Suivant);
                end;
              end;
              if Dessus_Suivant>-0.5 then begin
                case Form1.Les_Options_Coupe.Type_Hauteur of
                  1 : begin
                    Le_Caneva.Pen.Width:=round(Form1.Les_Options_Coupe.Epaisseur_Hauteur/10*Resolution*Rapport);
                    MoveToRel(X_Suivant,Z_Suivant);
                    LineToRel(X_Suivant,Z_Suivant+Dessus_Suivant);
                  end;
                  2 : begin
                    Le_Caneva.Pen.Width:=round(Form1.Les_Options_Coupe.Epaisseur_Hauteur/10*Resolution*Rapport);
                    Cercle(X_Suivant,Z_Suivant+Dessus_Suivant,Form1.Les_Options_Coupe.Epaisseur_Hauteur div 2);
                  end;
                end;
              end;

              if (Dessous_Precedent>-0.5) and (Dessous_Suivant>-0.5) then begin
                if Form1.Les_Options_Coupe.Habit_Affiche then begin
                  Le_Caneva.Pen.Width:=round(Form1.Les_Options_Coupe.Habit_Epaisseur/10*Resolution*Rapport);
                  MoveToRel(X_Precedent,Z_Precedent-Dessous_Precedent);
                  LineToRel(X_Suivant,Z_Suivant-Dessous_Suivant);
                end;
              end;
              if Dessous_Suivant>-0.5 then begin
                case Form1.Les_Options_Coupe.Type_Hauteur of
                  1 : begin
                    Le_Caneva.Pen.Width:=round(Form1.Les_Options_Coupe.Epaisseur_Hauteur/10*Resolution*Rapport);
                    MoveToRel(X_Suivant,Z_Suivant);
                    LineToRel(X_Suivant,Z_Suivant-Dessous_Suivant);
                  end;
                  2 : begin
                    Le_Caneva.Pen.Width:=round(Form1.Les_Options_Coupe.Epaisseur_Hauteur/10*Resolution*Rapport);
                    Cercle(X_Suivant,Z_Suivant-Dessous_Suivant,Form1.Les_Options_Coupe.Epaisseur_Hauteur div 2);
                  end;
                end;
              end;
              Le_Caneva.Pen.Width:=round(Form1.Les_Options_Coupe.Visee_Epaisseur/10*Resolution*Rapport);
              Dessus_Precedent:=Dessus_Suivant;
              Dessous_Precedent:=Dessous_Suivant;
            end;
            X_Precedent:=X_Suivant; Z_Precedent:=Z_Suivant;
            if Form1.Les_Options_Coupe.Diametre<>0 then begin
              Le_Caneva.Brush.Color:=clBlack; Le_Caneva.Brush.Style:=bsSolid;
              Cercle(X_Precedent,Z_Precedent,Form1.Les_Options_Coupe.Diametre);
            end;
          end;
          if (Form1.Les_Options_Coupe.Carrefour>0) and not Arrivee.Terminus then begin
            Le_Caneva.Brush.Color:=clWhite; Le_Caneva.Brush.Style:=bsClear;
            Arrivee.Nom.lit(S);
            if Form1.Les_Options_Coupe.Carrefour=2 then begin
              S:=S+'('+IntTostr(Arrivee.Indice)+')';
            end;
            TextRel(X_Precedent+Delta,Z_Precedent+Delta,S);
          end;
        end;
      end;
    end;
    if Form1.Les_Options_Coupe.Horizontale_Affiche then with Form1.Les_Options_Coupe do begin
      Le_Caneva.Pen.Width:=round(Horizontale_Epaisseur/10*Resolution);
      Le_Caneva.Pen.Color:=clBlack;
      MoveToRel(Horizontale_X,Horizontale_Z);
      LineToRel(Horizontale_X+Horizontale_Longueur,Horizontale_Z);
      LineToRel(Horizontale_X+Horizontale_Longueur,Horizontale_Z+Horizontale_Largeur/1000*Echelle);
      MoveToRel(Horizontale_X,Horizontale_Z);
      LineToRel(Horizontale_X,Horizontale_Z+Horizontale_Largeur/1000*Echelle);
      if Horizontale_Pas>1 then for I:=1 to Horizontale_Pas-1 do begin
        MoveToRel(Horizontale_X+Horizontale_Longueur/Horizontale_Pas*I,Horizontale_Z);
        LineToRel(Horizontale_X+Horizontale_Longueur/Horizontale_Pas*I,Horizontale_Z+Horizontale_Largeur/1000*Echelle);
      end;
      Le_Caneva.Font:=Horizontale_police;
      Le_Caneva.Font.Height:=round(Le_Caneva.Font.Height*Rapport);
      Le_Caneva.Brush.Color:=clWhite; Le_Caneva.Brush.Style:=bsClear;
      X0:=Le_Caneva.TextWidth('0')/Resolution/2;
      Y0:=Le_Caneva.TextHeight('0')/Resolution;
      TextRel(Horizontale_X-X0/1000*Echelle,Horizontale_Z+(Horizontale_Largeur+Y0)/1000*Echelle,'0');
      S:=IntTostr(Horizontale_Longueur)+tuDistance[m];
      X0:=Le_Caneva.TextWidth(S)/Resolution/2;
      TextRel(Horizontale_X+Horizontale_Longueur-X0/1000*Echelle,Horizontale_Z+(Horizontale_Largeur+Y0)/1000*Echelle,S);
    end;
    Le_Caneva.Pen.Width:=round(Form1.Les_Options_Coupe.Verticale_Epaisseur/10*Resolution);
    Le_Caneva.Pen.Color:=clBlack;
    Le_Caneva.Brush.Color:=clWhite; Le_Caneva.Brush.Style:=bsClear;
    for I:=1 to Form1.Les_Options_Coupe.Nb_Verticale do
                      with Form1.Les_Options_Coupe do with Verticale[I] do begin
      Ref:=0;
      Trouve:=false;
      for J:=0 to Form1.Massif.Liste_Fiche.Count-1 do if not Trouve then with TFiche(Form1.Massif.Liste_Fiche[J]) do begin
        for K:=0 to Liste_Galerie.Count-1 do if not Trouve then with TGalerie(Liste_Galerie[K]) do begin
          if Approximation then begin
            if not Depart.Terminus and (Depart.Indice=Indice) then begin
              Depart.Nom.UpString(S);
              if UpperCase(Point)=S then begin
                Trouve:=true; Ref:=Depart.Z;
              end;
            end;
            if not Arrivee.Terminus and (Arrivee.Indice=Indice) then begin
              Arrivee.Nom.UpString(S);
              if UpperCase(Point)=S then begin
                Trouve:=true; Ref:=Arrivee.Z;
              end;
            end;
          end;
        end;
      end;

      MoveToRel(XDev,Zmin+Decal+ref);
      LineToRel(XDev,Zmax+Decal+Ref);
      for J:=round(Zmin/Verticale_Pas+0.4999999) to round(Zmax/Verticale_Pas-0.4999999) do begin
        MoveToRel(XDev,Decal+Ref+J*Verticale_Pas);
        S:=IntTostr(J*Verticale_Pas);
        Y0:=Le_Caneva.TextHeight('0')/Resolution/2;
        if Gauche then begin
          LineToRel(XDev-Verticale_Largeur/1000*Echelle,Decal+Ref+J*Verticale_Pas);
          X0:=Le_Caneva.TextWidth(S)/Resolution;
          TextRel(XDev-(Verticale_Largeur+X0)/1000*Echelle,Decal+Ref+J*Verticale_Pas+Y0/1000*Echelle,S);
        end
        else begin
          LineToRel(XDev+Verticale_Largeur/1000*Echelle,Decal+Ref+J*Verticale_Pas);
          X0:=Le_Caneva.TextWidth('0')/Resolution/2;
          TextRel(XDev+(Verticale_Largeur+X0)/1000*Echelle,Decal+Ref+J*Verticale_Pas+Y0/1000*Echelle,S);
        end;
      end;


    end;

  end;

  Le_Caneva:=Canevas.Canvas;
  Le_Caneva.CopyMode:=cmSrcCopy;
  Le_Caneva.Draw(0,0,Hors_Plan);
  Hors_Plan.Free;

end;

procedure TCoupe.Imprime(Page_X, Page_Y, Sup_X, Sup_Y : integer; Etat : smallInt; Fond : TCanvas; Reduc : double);
var
  I, J, K, Page_Horizontale, Page_Verticale, Erreur : integer;
  L_Autre : TGalerie;
  Poids_1, Poids_2, Delta, X_Precedent, Z_Precedent, X_Suivant, Z_Suivant,
  X0, Y0, Contrainte_Max, Xi, Dessus_Precedent, Dessus_Suivant , Dessous_Precedent,
  Dessous_Suivant, Ref  : double;
  Trouve : boolean;
  S, S2 : string;
begin

  Supl_X:=Sup_X; Supl_Y:=Sup_Y;

  Support:=Fond; Correction:=Reduc;

  if Form1.Les_Options_Coupe.Blocage_Origine then with Form1.Les_Options_Coupe do begin
    Page_Horizontale:=trunc((XMax-Origine_X-0.01*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MX))+1+
            trunc((Origine_X-XMin-0.01*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MX))+1;
    Page_Verticale:=trunc((YMax-Origine_Z-0.01*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MY))+1+
            trunc((Origine_Z-YMin-0.01*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MY))+1;
    { Marge_X et Marge_Y sont les marges globales en fraction de page }
    Marge_X:=1-frac((Origine_X-XMin)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MX));
    Marge_Y:=1-frac((YMax-Origine_Z)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MY));
  end
  else begin
    Page_Horizontale:=trunc((XMax-XMin-0.02*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MX))+1;
    Page_Verticale:=trunc((YMax-YMin-0.02*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MY))+1;
    Marge_X:=(Page_Horizontale-(XMax-XMin)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MX))/2;
    Marge_Y:=(Page_Verticale-(YMax-YMin)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MY))/2;
  end;
{  Page_Horizontale:=trunc((XMax-XMin)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MX))+1;
  Page_Verticale:=trunc((YMax-YMin)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MY))+1;
  Marge_X:=(Page_Horizontale-(XMax-XMin)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MX))/2;
  Marge_Y:=(Page_Verticale-(YMax-YMin)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MY))/2; }
  PageX:=Page_X; PageY:=Page_Y;
  Resolution_Imprimante:=Printer.PageWidth/GetDeviceCaps(Printer.Handle,HorzSize);
  if Affichable then begin
    Poids_1:=Form1.Les_Options_Coupe.Pourcent/200+0.5;
    Poids_2:=1-Poids_1;
    Support.Pen.Color:=clBlack;
    Support.Brush.Color:=clBlack; Support.Brush.Style:=bsSolid;
    Support.Pen.Width:=round(Form1.Les_Options_Coupe.Visee_Epaisseur/10*Resolution_Imprimante*Correction);
    if Form1.Les_Options_Coupe.Carrefour>0 then begin
      Support.Font:=Form1.Les_Options_Coupe.Station_police;
      Printer.Canvas.Font:=Form1.Les_Options_Coupe.Station_police;
      Delta:=Printer.Canvas.TextHeight('O')/Resolution_Imprimante/2000*Echelle;
      if Correction<>1 then Support.Font.Height:=round(Support.Font.Height*Correction*
           Resolution_Imprimante/Form1.PixelsPerInch*25.4);
    end;
    if Etat=etContrainte then Contrainte_Max:=Contrainte_Maximum(Form1.Massif.Liste_Courante);
    for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
      for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
        if Approximation and En_Coupe then begin
          case Etat of
            etContrainte : Support.Pen.Color:=Degrade[round(Contrainte_Z/Contrainte_Max*60)];
            etNoir, etRotation : Support.Pen.Color:=clBlack;
            etCouleur :  Support.Pen.Color:=Couleur;
          end;
          if Accrochage_Debut then begin
            X_Precedent:=Decal_Hori;
            Z_Precedent:=Decal_Z+Depart.Z;
          end
          else begin
            X_Precedent:=Fin_Hori;
            Z_Precedent:=Decal_Z+Depart.Z;
          end;
          if Form1.Les_Options_Coupe.Habit_Affiche or (Form1.Les_Options_Coupe.Type_Hauteur<>0) then begin
            Les_Hauteurs(J,0,dessus_precedent, dessous_precedent);
            if Dessus_Precedent>-0.5 then begin
              case Form1.Les_Options_Coupe.Type_Hauteur of
                1 : begin
                  Support.Pen.Width:=round(Form1.Les_Options_Coupe.Epaisseur_Hauteur/10*Resolution_Imprimante*Correction);
                  MoveToImp(X_Precedent,Z_Precedent);
                  LineToImp(X_Precedent,Z_Precedent+Dessus_Precedent);
                end;
                2 : begin
                  Support.Pen.Width:=round(Form1.Les_Options_Coupe.Epaisseur_Hauteur/10*Resolution_Imprimante*Correction);
                  Cercle_Imp(X_Precedent,Z_Precedent+Dessus_Precedent,Form1.Les_Options_Coupe.Epaisseur_Hauteur div 2);
                end;
              end;
            end;
            if Dessous_Precedent>-0.5 then begin
              case Form1.Les_Options_Coupe.Type_Hauteur of
                1 : begin
                  Support.Pen.Width:=round(Form1.Les_Options_Coupe.Epaisseur_Hauteur/10*Resolution_Imprimante*Correction);
                  MoveToImp(X_Precedent,Z_Precedent);
                  LineToImp(X_Precedent,Z_Precedent+Dessous_Precedent);
                end;
                2 : begin
                  Support.Pen.Width:=round(Form1.Les_Options_Coupe.Epaisseur_Hauteur/10*Resolution_Imprimante*Correction);
                  Cercle_Imp(X_Precedent,Z_Precedent+Dessous_Precedent,Form1.Les_Options_Coupe.Epaisseur_Hauteur div 2);
                end;
              end;
            end;
          end;
          if Form1.Les_Options_Coupe.Diametre<>0 then begin
            Support.Brush.Color:=clBlack; Support.Brush.Style:=bsSolid;
            Cercle_Imp(X_Precedent,Z_Precedent,Form1.Les_Options_Coupe.Diametre);
          end;
          if (Form1.Les_Options_Coupe.Carrefour>0) and not Depart.Terminus then begin
            Support.Brush.Color:=clWhite; Support.Brush.Style:=bsClear;
            Depart.Nom.lit(S);
            if Form1.Les_Options_Coupe.Carrefour=2 then begin
              S:=S+'('+IntTostr(Depart.Indice)+')';
            end;
            TextImp(X_Precedent+Delta,Z_Precedent+Delta,S);
          end;
          for K:=1 to Nb_Visee do begin

            X_Suivant:=Coordonnees^[K].LH;
            Z_Suivant:=Decal_Z+Coordonnees^[K].Z;
            MovetoImp(X_Precedent*Poids_1+X_Suivant*Poids_2,
                      Z_Precedent*Poids_1+Z_Suivant*Poids_2);
            LinetoImp(X_Precedent*Poids_2+X_Suivant*Poids_1,
                      Z_Precedent*Poids_2+Z_Suivant*Poids_1);
            Support.Brush.Color:=clBlack; Support.Brush.Style:=bsSolid;
            Les_Hauteurs(J,K,dessus_suivant, dessous_suivant);
            if (Dessus_Precedent>-0.5) and (Dessus_Suivant>-0.5) then begin
              if Form1.Les_Options_Coupe.Habit_Affiche then begin
                Support.Pen.Width:=round(Form1.Les_Options_Coupe.Habit_Epaisseur/10*Resolution_Imprimante*Correction);
                MoveToImp(X_Precedent,Z_Precedent+Dessus_Precedent);
                LineToImp(X_Suivant,Z_Suivant+Dessus_Suivant);
              end;
            end;
            if Dessus_Suivant>-0.5 then begin
              case Form1.Les_Options_Coupe.Type_Hauteur of
                1 : begin
                  Support.Pen.Width:=round(Form1.Les_Options_Coupe.Epaisseur_Hauteur/10*Resolution_Imprimante*Correction);
                  MoveToImp(X_Suivant,Z_Suivant);
                  LineToImp(X_Suivant,Z_Suivant+Dessus_Suivant);
                end;
                2 : begin
                  Support.Pen.Width:=round(Form1.Les_Options_Coupe.Epaisseur_Hauteur/10*Resolution_Imprimante*Correction);
                  Cercle_Imp(X_Suivant,Z_Suivant+Dessus_Suivant,Form1.Les_Options_Coupe.Epaisseur_Hauteur div 2);
                end;
              end;
            end;
            if (Dessous_Precedent>-0.5) and (Dessous_Suivant>-0.5) then begin
              if Form1.Les_Options_Coupe.Habit_Affiche then begin
                Support.Pen.Width:=round(Form1.Les_Options_Coupe.Habit_Epaisseur/10*Resolution_Imprimante*Correction);
                MoveToImp(X_Precedent,Z_Precedent-Dessous_Precedent);
                LineToImp(X_Suivant,Z_Suivant-Dessous_Suivant);
              end;
            end;
            if Dessous_Suivant>-0.5 then begin
              case Form1.Les_Options_Coupe.Type_Hauteur of
                1 : begin
                  Support.Pen.Width:=round(Form1.Les_Options_Coupe.Epaisseur_Hauteur/10*Resolution_Imprimante*Correction);
                  MoveToImp(X_Suivant,Z_Suivant);
                  LineToImp(X_Suivant,Z_Suivant-Dessous_Suivant);
                end;
                2 : begin
                  Support.Pen.Width:=round(Form1.Les_Options_Coupe.Epaisseur_Hauteur/10*Resolution_Imprimante*Correction);
                  Cercle_Imp(X_Suivant,Z_Suivant-Dessous_Suivant,Form1.Les_Options_Coupe.Epaisseur_Hauteur div 2);
                end;
              end;
            end;
            Support.Pen.Width:=round(Form1.Les_Options_Coupe.Visee_Epaisseur/10*Resolution_Imprimante*Correction);
            Dessus_Precedent:=Dessus_Suivant;
            Dessous_Precedent:=Dessous_Suivant;
            X_Precedent:=X_Suivant; Z_Precedent:=Z_Suivant;
            if Form1.Les_Options_Coupe.Diametre<>0 then begin
              Support.Brush.Color:=clBlack; Support.Brush.Style:=bsSolid;
              Cercle_Imp(X_Precedent,Z_Precedent,Form1.Les_Options_Coupe.Diametre);
            end;
          end;
          if (Form1.Les_Options_Coupe.Carrefour>0) and not Arrivee.Terminus then begin
            Support.Brush.Color:=clWhite; Support.Brush.Style:=bsClear;
            Arrivee.Nom.lit(S);
            if Form1.Les_Options_Coupe.Carrefour=2 then begin
              S:=S+'('+IntTostr(Arrivee.Indice)+')';
            end;
            TextImp(X_Precedent+Delta,Z_Precedent+Delta,S);
          end;
        end;
      end;
    end;
    if Form1.Les_Options_Coupe.Horizontale_Affiche then with Form1.Les_Options_Coupe do begin
      Support.Pen.Width:=round(Horizontale_Epaisseur/10*Resolution_Imprimante*Correction);
      Support.Pen.Color:=clBlack;
      MoveToImp(Horizontale_X,Horizontale_Z);
      LineToImp(Horizontale_X+Horizontale_Longueur,Horizontale_Z);
      LineToImp(Horizontale_X+Horizontale_Longueur,Horizontale_Z+Horizontale_Largeur/1000*Echelle);
      MoveToImp(Horizontale_X,Horizontale_Z);
      LineToImp(Horizontale_X,Horizontale_Z+Horizontale_Largeur/1000*Echelle);
      if Horizontale_Pas>1 then for I:=1 to Horizontale_Pas-1 do begin
        MoveToImp(Horizontale_X+Horizontale_Longueur/Horizontale_Pas*I,Horizontale_Z);
        LineToImp(Horizontale_X+Horizontale_Longueur/Horizontale_Pas*I,Horizontale_Z+Horizontale_Largeur/1000*Echelle);
      end;
      Support.Font:=Horizontale_police; Printer.Canvas.Font:=Horizontale_police;
      if Correction<>1 then Support.Font.Height:=round(Support.Font.Height*Correction*
           Resolution_Imprimante/Form1.PixelsPerInch*25.4);
      Support.Brush.Color:=clWhite; Support.Brush.Style:=bsClear;
      X0:=Printer.Canvas.TextWidth('0')/Resolution_Imprimante/2;
      Y0:=Printer.Canvas.TextHeight('0')/Resolution_Imprimante;
      TextImp(Horizontale_X-X0/1000*Echelle,Horizontale_Z+(Horizontale_Largeur+Y0)/1000*Echelle,'0');
      S:=IntTostr(Horizontale_Longueur)+tuDistance[m];
      X0:=Printer.Canvas.TextWidth(S)/Resolution_Imprimante/2;
      TextImp(Horizontale_X+Horizontale_Longueur-X0/1000*Echelle,Horizontale_Z+(Horizontale_Largeur+Y0)/1000*Echelle,S);
    end;
    Support.Pen.Width:=round(Form1.Les_Options_Coupe.Verticale_Epaisseur/10*Resolution_Imprimante*Correction);
    Support.Pen.Color:=clBlack;
    Support.Brush.Color:=clWhite; Support.Brush.Style:=bsClear;
    for I:=1 to Form1.Les_Options_Coupe.Nb_Verticale do
                      with Form1.Les_Options_Coupe do with Verticale[I] do begin

      Ref:=0;
      Trouve:=false;
      for J:=0 to Form1.Massif.Liste_Fiche.Count-1 do if not Trouve then with TFiche(Form1.Massif.Liste_Fiche[J]) do begin
        for K:=0 to Liste_Galerie.Count-1 do if not Trouve then with TGalerie(Liste_Galerie[K]) do begin
          if Approximation then begin
            if not Depart.Terminus and (Depart.Indice=Indice) then begin
              Depart.Nom.UpString(S);
              if Point=S then begin
                Trouve:=true; Ref:=Depart.Z;
              end;
            end;
            if not Arrivee.Terminus and (Arrivee.Indice=Indice) then begin
              Arrivee.Nom.UpString(S);
              if Point=S then begin
                Trouve:=true; Ref:=Arrivee.Z;
              end;
            end;
          end;
        end;
      end;
      MoveToImp(XDev,Zmin+Decal+Ref);
      LineToImp(XDev,Zmax+Decal+Ref);
      for J:=round(Zmin/Verticale_Pas+0.4999999) to round(Zmax/Verticale_Pas-0.4999999) do begin
        MoveToImp(XDev,Decal+Ref+J*Verticale_Pas);
        S:=IntTostr(J*Verticale_Pas);
        Y0:=Printer.Canvas.TextHeight('0')/Resolution_Imprimante/2;
        if Gauche then begin
          LineToImp(XDev-Verticale_Largeur/1000*Echelle,Decal+Ref+J*Verticale_Pas);
          X0:=Printer.Canvas.TextWidth(S)/Resolution_Imprimante;
          TextImp(XDev-(Verticale_Largeur+X0)/1000*Echelle,Decal+Ref+J*Verticale_Pas+Y0/1000*Echelle,S);
        end
        else begin
          LineToImp(XDev+Verticale_Largeur/1000*Echelle,Decal+Ref+J*Verticale_Pas);
          X0:=Printer.Canvas.TextWidth('0')/Resolution_Imprimante/2;
          TextImp(XDev+(Verticale_Largeur+X0)/1000*Echelle,Decal+Ref+J*Verticale_Pas+Y0/1000*Echelle,S);
        end;
      end;
    end;
  end;
end;

procedure TCoupe.Gabarit(var Pixel_Hori, Pixel_Verti  : longint; Resolution, Rapport : double; Canevas : TImage);
var
  Pmin, PMax : TPoint3D;
begin
  Affichable:=Calibre_Coupe(PMin, PMax,Form1.Massif.Liste_Courante);
  if Affichable then begin
    XMin:=PMin.LH; XMax:=PMax.LH;
    YMin:=PMin.Z;  YMax:=PMax.Z;
    Xmin:=XMin-0.01*Echelle; Xmax:=XMax+0.01*Echelle;
    Ymin:=YMin-0.01*Echelle; Ymax:=YMax+0.01*Echelle;
    Pixel_Hori:=round((XMax-XMin)/Echelle*1000*Resolution*Rapport);
    Pixel_Verti:=round((YMax-YMin)/Echelle*1000*Resolution*Rapport);
  end
  else begin
    Pixel_Hori:=1;
    Pixel_Verti:=1;
  end;
end;

procedure TCoupe.Gabarit_Imprimante(var Page_X, Page_Y : integer; MargeX, MargeY : integer);
begin
  if Form1.Les_Options_Coupe.Blocage_Origine then with Form1.Les_Options_Coupe do begin
    Page_X:=trunc((XMax-Origine_X-0.01*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MargeX))+1+
            trunc((Origine_X-XMin-0.01*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MargeX))+1;
    Page_Y:=trunc((YMax-Origine_Z-0.01*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MargeY))+1+
            trunc((Origine_Z-YMin-0.01*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MargeY))+1;
  end
  else begin
    Page_X:=trunc((XMax-XMin-0.02*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MargeX))+1;
    Page_Y:=trunc((YMax-YMin-0.02*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MargeY))+1;
  end;
  MX:=MargeX;
  MY:=MargeY;
end;

function TCoupe.Trouve(X, Y : longint;var N_Fiche, N_Galerie, N_Visee : integer) : double;
var
  I, J, K : integer;
  H_Abs, Z_Abs, DMin, Ante_X, Ante_Z, N_X, N_Z : double;
begin
  H_Abs:=X/1000*Echelle/La_Resolution+XMin;
  Z_Abs:=YMax-Y/1000*Echelle/La_Resolution;
  DMin:=1e30;
  if Affichable then begin
    Le_Caneva.Pen.Color:=clBlack;
    for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
      for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
        if Approximation and En_Coupe then begin
          if Accrochage_Debut then begin
            Ante_X:=Decal_Hori; Ante_Z:=Decal_Z+Depart.Z;
            for K:=1 to Nb_Visee do begin
              N_X:=Coordonnees^[K].LH; N_Z:=Decal_Z+Coordonnees^[K].Z;
              if Abs(Ante_X-N_X)<0.5 then begin
                if ((N_Z-Z_Abs)*(Ante_Z-Z_Abs)<0) and (abs((N_X+Ante_X)/2-H_Abs)<Dmin) then begin
                  Dmin:=abs((N_X+Ante_X)/2-H_Abs);
                  N_Fiche:=I; N_Galerie:=J; N_Visee:=K;
                end;
              end
              else
              if ((N_X-H_Abs)*(Ante_X-H_Abs)<0) and
                 (abs((N_Z-Ante_Z)*(H_Abs-N_X)/(N_X-Ante_X)+N_Z-Z_Abs)<Dmin) then begin
                Dmin:=abs((N_Z-Ante_Z)*(H_Abs-N_X)/(N_X-Ante_X)+N_Z-Z_Abs);
                N_Fiche:=I; N_Galerie:=J; N_Visee:=K;
              end;
              Ante_X:=N_X; Ante_Z:=N_Z;
            end;
          end
          else begin
            Ante_X:=Fin_Hori; Ante_Z:=Decal_Z+Depart.Z;
            for K:=1 to Nb_Visee do begin
              N_X:=Coordonnees^[K].LH; N_Z:=Decal_Z+Coordonnees^[K].Z;
              if Abs(Ante_X-N_X)<0.5 then begin
                if ((N_Z-Z_Abs)*(Ante_Z-Z_Abs)<0) and (abs((N_X+Ante_X)/2-H_Abs)<Dmin) then begin
                  Dmin:=abs((N_X+Ante_X)/2-H_Abs);
                  N_Fiche:=I; N_Galerie:=J; N_Visee:=K;
                end;
              end
              else
              if ((N_X-H_Abs)*(Ante_X-H_Abs)<0) and
                 (abs((N_Z-Ante_Z)*(H_Abs-N_X)/(N_X-Ante_X)+N_Z-Z_Abs)<Dmin) then begin
                Dmin:=abs((N_Z-Ante_Z)*(H_Abs-N_X)/(N_X-Ante_X)+N_Z-Z_Abs);
                N_Fiche:=I; N_Galerie:=J; N_Visee:=K;
              end;
              Ante_X:=N_X; Ante_Z:=N_Z;
            end;
          end;
        end;
      end;
    end;
  end;
  if Dmin>1e29 then N_Visee:=0;
  Trouve:=Dmin*1000/Echelle*La_Resolution;;
end;

procedure TCoupe.Isoler(X, Y, Rayon2 : longint);
var
  I, J, K : integer;
  Nouvelle_Selection : TList;
  H_Abs, Z_Abs, N_X, N_Z : double;
  Repere : boolean;
begin
  H_Abs:=X/1000*Echelle/La_Resolution+XMin;
  Z_Abs:=YMax-Y/1000*Echelle/La_Resolution;
  Nouvelle_Selection:=TList.Create;
  if Affichable then begin
    for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
      Repere:=false;
      for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
        if Approximation and En_Coupe then begin
          if Accrochage_Debut then N_X:=Decal_Hori
                              else N_X:=Fin_Hori;
          N_Z:=Decal_Z+Depart.Z;
          if sqr((N_X-H_Abs))+sqr((N_Z-Z_Abs))<Rayon2*sqr(Echelle/La_Resolution/1000) then Repere:=true;
          for K:=1 to Nb_Visee do begin
            N_X:=Coordonnees^[K].LH; N_Z:=Decal_Z+Coordonnees^[K].Z;
            if sqr((N_X-H_Abs))+sqr((N_Z-Z_Abs))<Rayon2*sqr(Echelle/La_Resolution/1000) then Repere:=true;
          end;
        end;
      end;
      if Repere then Nouvelle_Selection.Add(Form1.Massif.Liste_Courante[I]);
    end;
  end;
  if Form1.Massif.Existe_Selection then Form1.Massif.Selection.Free;
  Form1.Massif.Selection:=Nouvelle_Selection;
  Form1.Massif.Liste_Courante:=Nouvelle_Selection;
  Form1.Massif.Existe_Selection:=true;
end;

function TCoupe.Met_En_Relief(Nouveau : boolean; N_Fiche, N_Galerie, N_Visee : integer;Etat : smallInt) : string;
var
  K : integer;
  Resultat, S, SP : string;
  X_Precedent, Z_Precedent : double;

  procedure Ensemble(N_Ordre, N_Galerie : integer);
  var
    I, K : integer;
    Une_Fiche : TFiche;
  begin
    I:=0;
    repeat
      Une_Fiche:=Form1.Massif.Liste_Courante[I];
      inc(I);
    until (Une_Fiche.Ordre=N_Ordre) or (I>=Form1.Massif.Liste_Courante.Count);
    if Une_Fiche.Ordre=N_Ordre then with TGalerie(Une_Fiche.Liste_Galerie[N_Galerie]) do begin
      if Approximation and En_Coupe then begin
        if Accrochage_Debut then begin
          X_Precedent:=Decal_Hori; Z_Precedent:=Decal_Z+Depart.Z;
          for K:=1 to Nb_Visee do begin
            MoveToRel(X_Precedent*(0.5+Form1.Les_Options_Coupe.Pourcent/200)+
                      Coordonnees^[K].LH*(0.5-Form1.Les_Options_Coupe.Pourcent/200),
                      Z_Precedent*(0.5+Form1.Les_Options_Coupe.Pourcent/200)+
                      (Decal_Z+Coordonnees^[K].Z)*(0.5-Form1.Les_Options_Coupe.Pourcent/200));
            LineToRel(X_Precedent*(0.5-Form1.Les_Options_Coupe.Pourcent/200)+
                      Coordonnees^[K].LH*(0.5+Form1.Les_Options_Coupe.Pourcent/200),
                      Z_Precedent*(0.5-Form1.Les_Options_Coupe.Pourcent/200)+
                      (Decal_Z+Coordonnees^[K].Z)*(0.5+Form1.Les_Options_Coupe.Pourcent/200));
            X_Precedent:=Coordonnees^[K].LH;
            Z_Precedent:=Decal_Z+Coordonnees^[K].Z;
          end;
        end
        else begin
          X_Precedent:=Fin_Hori; Z_Precedent:=Decal_Z+Depart.Z;
          for K:=1 to Nb_Visee do begin
            MoveToRel(X_Precedent*(0.5+Form1.Les_Options_Coupe.Pourcent/200)+
                      Coordonnees^[K].LH*(0.5-Form1.Les_Options_Coupe.Pourcent/200),
                      Z_Precedent*(0.5+Form1.Les_Options_Coupe.Pourcent/200)+
                      (Decal_Z+Coordonnees^[K].Z)*(0.5-Form1.Les_Options_Coupe.Pourcent/200));
            LineToRel(X_Precedent*(0.5-Form1.Les_Options_Coupe.Pourcent/200)+
                      Coordonnees^[K].LH*(0.5+Form1.Les_Options_Coupe.Pourcent/200),
                      Z_Precedent*(0.5-Form1.Les_Options_Coupe.Pourcent/200)+
                      (Decal_Z+Coordonnees^[K].Z)*(0.5+Form1.Les_Options_Coupe.Pourcent/200));
            X_Precedent:=Coordonnees^[K].LH;
            Z_Precedent:=Decal_Z+Coordonnees^[K].Z;
          end;
        end;

        for I:=1 to Nb_Suite do Ensemble(Ordre_Suite[I],Galerie_Suite[I]);
      end;
    end;
  end;

begin
  if Nouveau then Le_Caneva.Pen.Color:=65280
             else Le_Caneva.Pen.Color:=clBlack;
  Le_Caneva.Pen.Style:=psSolid; Le_Caneva.Pen.Mode:=pmCopy;
  with TFiche(Form1.Massif.Liste_Courante[N_Fiche]) do with TGalerie(Liste_Galerie[N_Galerie]) do begin

{ cration de la chaine rsum }
    if Nouveau then begin
      Systeme.Lit(S); Resultat:=S;
      Cavite.Lit(S); Resultat:=Resultat+'/'+S;
      Reseau.Lit(S); Resultat:=Resultat+'/'+S;
      Lieu.Lit(S); Resultat:=Resultat+'/'+S;
      if Depart.Terminus then S:='Terminus'
      else Depart.Nom.Lit(S);
      if Arrivee.Terminus then SP:='Terminus'
      else Arrivee.Nom.Lit(SP);
      if Accrochage_Debut xor Gauche_Droite then Resultat:=Resultat+' '+SP+' - '+S
                                                                    else Resultat:=Resultat+' '+S+' - '+SP;
      Met_En_Relief:=Resultat;
    end
    else Met_En_Relief:='';

    if Approximation and En_Coupe then begin
      if Etat<>etContrainte then for K:=1 to Nb_Suite do
                               Ensemble(ordre_Suite[K],Galerie_Suite[K]);
      if Accrochage_Debut then begin
        X_Precedent:=Decal_Hori; Z_Precedent:=Decal_Z+Depart.Z;
        for K:=1 to Nb_Visee do begin
          if K=N_Visee then begin
            Le_Caneva.Pen.Mode:=pmNot;
            if Nouveau then begin
              Le_Caneva.Pen.Color:=clBlack;
              Le_Caneva.Brush.Color:=clBlack;
            end
            else begin
              Le_Caneva.Pen.Color:=clWhite;
              Le_Caneva.Brush.Color:=clWhite;
            end;
            Carre(X_Precedent,Z_Precedent);
            Carre(Coordonnees^[K].LH,Decal_Z+Coordonnees^[K].Z);
            Le_Caneva.Pen.Mode:=pmCopy;
          end;
          if Nouveau then begin
            if K=N_Visee then Le_Caneva.Pen.Color:=clRed
                         else Le_Caneva.Pen.Color:=65280;
          end
          else case Etat of
            etNoir : Le_Caneva.Pen.Color:=clBlack;
            etCouleur : Le_Caneva.Pen.Color:=Couleur;
          end;
          if Etat<>etContrainte then begin
            MoveToRel(X_Precedent*(0.5+Form1.Les_Options_Coupe.Pourcent/200)+
                      Coordonnees^[K].LH*(0.5-Form1.Les_Options_Coupe.Pourcent/200),
                      Z_Precedent*(0.5+Form1.Les_Options_Coupe.Pourcent/200)+
                      (Decal_Z+Coordonnees^[K].Z)*(0.5-Form1.Les_Options_Coupe.Pourcent/200));
            LineToRel(X_Precedent*(0.5-Form1.Les_Options_Coupe.Pourcent/200)+
                      Coordonnees^[K].LH*(0.5+Form1.Les_Options_Coupe.Pourcent/200),
                      Z_Precedent*(0.5-Form1.Les_Options_Coupe.Pourcent/200)+
                      (Decal_Z+Coordonnees^[K].Z)*(0.5+Form1.Les_Options_Coupe.Pourcent/200));
          end;
          X_Precedent:=Coordonnees^[K].LH;
          Z_Precedent:=Decal_Z+Coordonnees^[K].Z;
        end;
      end
      else begin
        X_Precedent:=Fin_Hori; Z_Precedent:=Decal_Z+Depart.Z;
        for K:=1 to Nb_Visee do begin
          if K=N_Visee then begin
            Le_Caneva.Pen.Mode:=pmNot;
            if Nouveau then begin
              Le_Caneva.Pen.Color:=clBlack;
              Le_Caneva.Brush.Color:=clBlack;
            end
            else begin
              Le_Caneva.Pen.Color:=clWhite;
              Le_Caneva.Brush.Color:=clWhite;
            end;
            Carre(X_Precedent,Z_Precedent);
            Carre(Coordonnees^[K].LH,Decal_Z+Coordonnees^[K].Z);
            Le_Caneva.Pen.Mode:=pmCopy;
          end;
          if Nouveau then begin
            if K=N_Visee then Le_Caneva.Pen.Color:=clRed
                         else Le_Caneva.Pen.Color:=65280;
          end
          else case Etat of
            etNoir : Le_Caneva.Pen.Color:=clBlack;
            etCouleur : Le_Caneva.Pen.Color:=Couleur;
          end;
          if Etat<>etContrainte then begin
            MoveToRel(X_Precedent*(0.5+Form1.Les_Options_Coupe.Pourcent/200)+
                      Coordonnees^[K].LH*(0.5-Form1.Les_Options_Coupe.Pourcent/200),
                      Z_Precedent*(0.5+Form1.Les_Options_Coupe.Pourcent/200)+
                      (Decal_Z+Coordonnees^[K].Z)*(0.5-Form1.Les_Options_Coupe.Pourcent/200));
            LineToRel(X_Precedent*(0.5-Form1.Les_Options_Coupe.Pourcent/200)+
                      Coordonnees^[K].LH*(0.5+Form1.Les_Options_Coupe.Pourcent/200),
                      Z_Precedent*(0.5-Form1.Les_Options_Coupe.Pourcent/200)+
                      (Decal_Z+Coordonnees^[K].Z)*(0.5+Form1.Les_Options_Coupe.Pourcent/200));
          end;
          X_Precedent:=Coordonnees^[K].LH;
          Z_Precedent:=Decal_Z+Coordonnees^[K].Z;
        end;
      end;
    end;
  end;
end;

function TCoupe.Nouvelle_Branche : boolean;
var
  Quelle_Fiche : TFiche;
  Position : integer;
  Ptt_Text, Grand_Nom : string;
  Le_Point : TPointCoupe;
  Echec : boolean;
begin
  Echec:=true;
  Position:=-2;
  if Recherche_Simple(Form1.Massif.Liste_Courante,Form1.Massif,Quelle_Fiche,Position,'','','','','')
  then begin
    if Position=-1 then begin
      ShowMessage('Il faut obligatoirement passer par la visualisation pour cette option');
{      Position:=-1; }
    end
    else with TGalerie(Quelle_Fiche.Liste_Galerie[Position]) do begin
      if not Depart.Terminus then begin
        Depart.Nom.Lit(Ptt_Text);
        Ptt_Text:=Ptt_Text+'('+IntToStr(Depart.Indice)+')';
      end
      else Ptt_Text:='Terminus';
      if not Arrivee.Terminus then begin
        Arrivee.Nom.Lit(Grand_Nom);
        Grand_Nom:=Grand_Nom+'('+IntToStr(Arrivee.Indice)+')';
      end
      else Grand_Nom:='Terminus';
      with Nvlle_Branche do begin
        Nvlle_Branche.HelpContext:=29;
        Caption:='Nouvelle branche';
        CheckBox1.Visible:=true;
        Point.Visible:=true;
        Point.Items[1]:=Grand_Nom;
        Point.Items[0]:=Ptt_Text;
      end;
      Nvlle_Branche.HelpContext:=29;
      if Nvlle_Branche.ShowModal=mrOk then begin
        Echec:=false;
        Le_Point:=TPointCoupe.Create;
        Le_Point.Decal_Hori:=StrToFloatE(Nvlle_Branche.Horizontal.Text);
        Le_Point.Decal_Z:=StrToFloatE(Nvlle_Branche.Vertical.Text);
        Le_Point.Nom.Create;
        if Nvlle_Branche.Point.ItemIndex=0 then begin
          if Ptt_Text='Terminus' then begin
            Le_Point.Nom.Ecrit('Terminus');
            Le_Point.Indice:=0;
          end
          else begin
            Le_Point.Nom.Duplique(Depart.Nom);
            Le_Point.Indice:=Depart.Indice;
          end;
        end
        else begin
          if Grand_Nom='Terminus' then begin
            Le_Point.Nom.Ecrit('Terminus');
            Le_Point.Indice:=0;
          end
          else begin
            Le_Point.Nom.Duplique(Arrivee.Nom);
            Le_Point.Indice:=Arrivee.Indice;
          end;
        end;
        Le_Point.Ordre_Suite:=Quelle_Fiche.Ordre;
        Le_Point.Galerie_Suite:=Position;
        En_Coupe:=true;
        Gauche_Droite:=Nvlle_Branche.CheckBox1.Checked;
        Nb_Suite:=0;
        Accrochage_Debut:=(Nvlle_Branche.Point.ItemIndex=0);
        Form1.Massif.Germe_Coupe.Add(Le_Point);
        Form1.Massif.Calculer_Coupe;
      end;
    end;
  end;
  Nouvelle_Branche:=not Echec;
end;

function TCoupe.Continuer(var N_Fiche, N_Galerie : integer) : boolean;
var
  Une_Fiche : TFiche;
  Une_Galerie, Precedente : TGalerie;
  Les_Suite : TList;
  Un_Point : TPointTopo;
  Echec : boolean;
  I, J : integer;

  procedure Raboute(Classement : integer);
  var
    I, J : integer;
    Compteur : integer;
  begin
    Compteur:=0;
    for I:=0 to Form1.Massif.Liste_Courante.Count-1 do begin
      Une_Fiche:=Form1.Massif.Liste_Courante[I];
      for J:=0 to Une_Fiche.Liste_Galerie.Count-1 do begin
        Une_Galerie:=Une_Fiche.Liste_Galerie[J];
        if not Une_Galerie.En_Coupe then begin
          if Une_Galerie.Depart.Identique(Un_Point) or
             Une_Galerie.Arrivee.Identique(Un_Point) then begin
            inc(Compteur);
            if Compteur=Classement then begin
              Une_Galerie.En_Coupe:=true;
              Une_Galerie.Nb_Suite:=0;
              Une_Galerie.Accrochage_Debut:=Une_Galerie.Depart.Identique(Un_Point);
              with Precedente do begin
                inc(Nb_Suite);
                Ordre_Suite[Nb_Suite]:=Une_Fiche.Ordre;
                Galerie_Suite[Nb_Suite]:=J;
                Une_Galerie.Gauche_Droite:=Gauche_Droite;
              end;
              N_Fiche:=I; N_Galerie:=J;
            end;
          end;
        end;
      end;
    end;
  end;


begin
  Echec:=false;
  Une_Fiche:=Form1.Massif.Liste_Courante[N_Fiche];
  Precedente:=Une_Fiche.Liste_Galerie[N_Galerie];
  if Precedente.Accrochage_Debut then Un_Point:=Precedente.Arrivee
                                  else Un_Point:=Precedente.Depart;
  if (Un_Point.Terminus) or (Precedente.Nb_Suite=5) then begin
    if Un_Point.Terminus then MessageDlg('Galerie en cul-de-sac',mtWarning,[mbOk],0)
                         else MessageDlg('Pas plus de 5 suites par galerie',mtWarning,[mbOk],0);
    Echec:=true;
  end
  else begin
    Les_Suite:=TList.Create;
    for I:=0 to Form1.Massif.Liste_Courante.Count-1 do begin
      Une_Fiche:=Form1.Massif.Liste_Courante[I];
      for J:=0 to Une_Fiche.Liste_Galerie.Count-1 do begin
        Une_Galerie:=Une_Fiche.Liste_Galerie[J];
        if not Une_Galerie.En_Coupe then begin
          if Une_Galerie.Depart.Identique(Un_Point) then
            Les_Suite.Add(Une_Galerie.Arrivee)
          else if Une_Galerie.Arrivee.Identique(Un_Point) then Les_Suite.Add(Une_Galerie.Depart);
        end;
      end;
    end;
    case Les_Suite.Count of
      0 : begin
        MessageDlg('Pas de suite (disponible)',mtWarning,[MbOk],0);
        Echec:=true;
      end;
      1 : begin
        Raboute(1);
        Form1.Massif.Calculer_Coupe;
      end;
      else begin
        RechSimp.Preparation(RPoint,Les_Suite,Form1.Massif,'');
        if RechSimp.ShowModal=mrOk then begin
          I:=RechSimp.La_Quelle+1;
          Raboute(I);
          Form1.Massif.Calculer_Coupe;
        end;
      end;
    end;
    Les_Suite.Free;
  end;
  Continuer:=not Echec;
end;

procedure TCoupe.Inverser(N_Ordre : longint; N_Galerie : integer);
var
  I : integer;
  Une_Fiche : TFiche;
begin
  I:=0;
  repeat
    Une_Fiche:=Form1.Massif.Liste_Fiche[I];
    inc(I);
  until Une_Fiche.Ordre=N_Ordre;
  with TGalerie(Une_Fiche.Liste_Galerie[N_Galerie]) do begin
    Gauche_Droite:=not Gauche_Droite;
    for I:=1 to Nb_Suite do
       Inverser(Ordre_Suite[I],Galerie_Suite[I]);
   end;
end;

procedure TCoupe.Retirer(N_Ordre : longint; N_Galerie : integer; Racine : boolean);
var
  I, J, K : integer;
  Une_Fiche : TFiche;
  Un_Germe : TPointCoupe;
  Detecte : boolean;
begin
  I:=0;
  repeat
    Une_Fiche:=Form1.Massif.Liste_Fiche[I];
    inc(I);
  until Une_Fiche.Ordre=N_Ordre;
  with TGalerie(Une_Fiche.Liste_Galerie[N_Galerie]) do begin
    En_Coupe:=false;
    for I:=1 to Nb_Suite do
       Retirer(Ordre_Suite[I],Galerie_Suite[I],false);
  end;
  if Racine then begin
    Detecte:=false;
    for I:=0 to Form1.Massif.Germe_Coupe.Count-1 do begin
      if Detecte then Un_Germe:=Form1.Massif.Germe_Coupe[I-1]
                 else Un_Germe:=Form1.Massif.Germe_Coupe[I];
      if (Un_Germe.Ordre_Suite=N_Ordre) and (Un_Germe.Galerie_Suite=N_Galerie) then begin
        Form1.Massif.Germe_Coupe.Remove(Un_Germe);
        Form1.Massif.Germe_Coupe.Pack;
        Un_Germe.Free; Un_Germe:=nil;
        Detecte:=true;
      end;
    end;
    for I:=0 to Form1.Massif.Liste_Fiche.Count-1 do with TFiche(Form1.Massif.Liste_Fiche[I]) do begin
      for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
        Detecte:=false;
        if En_Coupe then begin
          for K:=1 to Nb_Suite do begin
            if not Detecte and (Ordre_Suite[K]=N_Ordre)
                           and (Galerie_Suite[K]=N_Galerie) then Detecte:=true;
            if Detecte then begin
              if K<Nb_Suite then begin
                Ordre_Suite[K]:=Ordre_Suite[K+1];
                Galerie_Suite[K]:=Galerie_Suite[K+1];
              end;
            end;
          end;
          if Detecte then dec(Nb_Suite);
        end;
      end;
    end;
  end;
end;

function TCoupe.Decaler(N_Ordre : longint; N_Galerie : integer) : boolean;
var
  I, J, K : integer;
  Une_Fiche : TFiche;
  Une_Galerie : TGalerie;
  Ptt_X, Ptt_Z : double;
  Un_Germe : TPointCoupe;
  Detecte : boolean;
begin
  I:=0;
  repeat
    Une_Fiche:=Form1.Massif.Liste_Fiche[I];
    inc(I);
  until Une_Fiche.Ordre=N_Ordre;
  Une_Galerie:=Une_Fiche.Liste_Galerie[N_Galerie];
  with Nvlle_Branche do begin
    Caption:='Dcaler une branche';
    CheckBox1.Visible:=false;
    Point.Visible:=false;
    Horizontal.Text:=FloatTostrF(Une_Galerie.Decal_Hori,ffGeneral,6,2);
    Vertical.Text:=FloatTostrF(Une_Galerie.Decal_Z,ffGeneral,4,1);
  end;
  Nvlle_Branche.HelpContext:=32;
  if Nvlle_Branche.ShowModal=mrOk then begin
    try
      Ptt_X:=StrToFloat(Nvlle_Branche.Horizontal.Text);
      Ptt_Z:=StrToFloat(Nvlle_Branche.Vertical.Text);
{ recherche du germe antcdent s'il existe }
      Detecte:=false;
      for I:=0 to Form1.Massif.Germe_Coupe.Count-1 do begin
        Un_Germe:=Form1.Massif.Germe_Coupe[I];
        if (Un_Germe.Ordre_Suite=N_Ordre) and (Un_Germe.Galerie_Suite=N_Galerie) then begin
          Un_Germe.Decal_Hori:=Ptt_X;
          Un_Germe.Decal_Z:=Ptt_Z;
          Detecte:=true;
        end;
      end;
// s'il n'y a pas de germe, on en fait un et on dcroche la branche
      if not Detecte then begin
        Un_Germe:=TPointCoupe.Create;
        with Un_Germe do begin
          Decal_Hori:=Ptt_X;
          Decal_Z:=Ptt_Z;
          Ordre_Suite:=N_Ordre;
          Galerie_Suite:=N_Galerie;
          if Une_Galerie.Accrochage_Debut then begin
            Nom.Duplique(Une_Galerie.Depart.Nom);
            Indice:=Une_Galerie.Depart.Indice;
          end
          else begin
            Nom.Duplique(Une_Galerie.Arrivee.Nom);
            Indice:=Une_Galerie.Arrivee.Indice;
          end;
        end;
        Form1.Massif.Germe_Coupe.Add(Un_Germe);
        for I:=0 to Form1.Massif.Liste_Fiche.Count-1 do begin
          Une_Fiche:=Form1.Massif.Liste_Fiche[I];
          for J:=0 to Une_Fiche.Liste_Galerie.Count-1 do begin
            Une_Galerie:=Une_Fiche.Liste_Galerie[J];
            Detecte:=false;
            if Une_Galerie.En_Coupe then begin
              for K:=1 to Une_Galerie.Nb_Suite do begin
                if not Detecte and (Une_Galerie.Ordre_Suite[K]=N_Ordre)
                               and (Une_Galerie.Galerie_Suite[K]=N_Galerie) then Detecte:=true;
                if Detecte then begin
                  if K<Une_Galerie.Nb_Suite then begin
                    Une_Galerie.Ordre_Suite[K]:=Une_Galerie.Ordre_Suite[K+1];
                    Une_Galerie.Galerie_Suite[K]:=Une_Galerie.Galerie_Suite[K+1];
                  end;
                end;
              end;
              if Detecte then dec(Une_Galerie.Nb_Suite);
            end;
          end;
        end;
      end;
      Decaler:=true;
    except
      Decaler:=false;
    end;
  end
  else Decaler:=false;
end;

function TCoupe.Raccrocher(N_Ordre : longint; N_Galerie : integer) : boolean;
var
  Detecte : boolean;
  I, J, Numero_Germe : integer;
  Un_Germe : TPointCoupe;
  Lautre_Point : TPointTopo;
  Liste_Proposition, Suite : TList;
  Une_Fiche : TFiche;
  Une_Galerie : TGalerie;
begin
  Detecte:=false;
  for I:=0 to Form1.Massif.Germe_Coupe.Count-1 do begin
    Un_Germe:=Form1.Massif.Germe_Coupe[I];
    if (Un_Germe.Ordre_Suite=N_Ordre) and (Un_Germe.Galerie_Suite=N_Galerie) then begin
      Numero_Germe:=I;
      Detecte:=true;
    end;
  end;
  if not Detecte then begin
    ShowMessage('Vous n''tes pas au dbut d''une branche, raccrochage impossible.');
    Raccrocher:=false;
  end
  else begin
    Liste_Proposition:=TList.Create;
    Un_Germe:=Form1.Massif.Germe_Coupe[Numero_Germe];
    for I:=0 to Form1.Massif.Liste_Fiche.Count-1 do begin
      Une_Fiche:=Form1.Massif.Liste_Fiche[I];
      for J:=0 to Une_Fiche.Liste_Galerie.Count-1 do begin
        Une_Galerie:=Une_Fiche.Liste_Galerie[J];
        if Une_Galerie.En_Coupe and (Une_Galerie.Nb_Suite<5) then begin
          if Une_Galerie.Accrochage_Debut then Lautre_Point:=Une_Galerie.Arrivee
                                          else Lautre_Point:=Une_Galerie.Depart;
          if LAutre_Point.Nom.Identique(Un_Germe.Nom) and (LAutre_Point.Indice=Un_Germe.Indice) then
                Liste_Proposition.Add(Une_Galerie);
        end;
      end;
    end;
    case Liste_Proposition.Count of
      0 : begin
        ShowMessage('Pas de branche disponible pour un raccrochage');
        Raccrocher:=false;
      end;
      1 : begin
        Form1.Massif.Germe_Coupe.Remove(Un_Germe);
        Form1.Massif.Germe_Coupe.Pack;
        Un_Germe.Free; Un_Germe:=nil;
        Une_Galerie:=Liste_Proposition[0];
        inc(Une_Galerie.Nb_Suite);
        Une_Galerie.Ordre_Suite[Une_Galerie.Nb_Suite]:=N_Ordre;
        Une_Galerie.Galerie_Suite[Une_Galerie.Nb_Suite]:=N_Galerie;
        Raccrocher:=true;
      end;
      else begin
        Suite:=TList.Create;
        for I:=0 to Liste_Proposition.Count-1 do begin
          Une_Galerie:=Liste_Proposition[I];
          if Une_Galerie.Accrochage_Debut then Suite.Add(Une_Galerie.Depart)
                                          else Suite.Add(Une_Galerie.Arrivee);
        end;
        RechSimp.Preparation(RPoint,Suite,Form1.Massif,'');
        if RechSimp.ShowModal=mrOk then begin
          Form1.Massif.Germe_Coupe.Remove(Un_Germe);
          Form1.Massif.Germe_Coupe.Pack;
          Un_Germe.Free; Un_Germe:=nil;
          Une_Galerie:=Liste_Proposition[RechSimp.La_Quelle];
          inc(Une_Galerie.Nb_Suite);
          Une_Galerie.Ordre_Suite[Une_Galerie.Nb_Suite]:=N_Ordre;
          Une_Galerie.Galerie_Suite[Une_Galerie.Nb_Suite]:=N_Galerie;
          Raccrocher:=true;
        end
        else Raccrocher:=false;
        Suite.Free;
      end;
    end;
    Liste_Proposition.Free;
  end;
end;

procedure TCoupe.Export_Svg(Fichier_SVG: TFileName; Illustrator, AllLayers : boolean);
const
  coVisee = 0;
  coParois = 1;
  coHauteur = 2;
  coStation = 3;
  coCarrefour = 4;
var
  F : TextFile;
  I, J, K, Couche : integer;
  Poids_1, Poids_2, Delta : double;
  X_Precedent, Z_Precedent, X_Suivant, Z_Suivant : double;
  Dessus_Suivant, Dessous_Suivant, Dessus_Precedent, Dessous_Precedent : double;
  Ref : double;
  Trouve : boolean;
  S, sLayer : string;
begin
  inherited;
  AssignFile(F,Fichier_SVG);
  try
    rewrite(F);
    EnteteSvg(F,round((XMax-XMin)*1000/Echelle),round((YMax-YMin)*1000/Echelle),Illustrator);
    writeln(F,Format('<title>%s</title>',[ExtractFileName(Form1.Massif.Fichier)]));
    writeln(F,'<desc>Coupe</desc>');

    if Affichable and (Form1.Les_Options_Coupe<>nil) then begin
      Poids_1:=Form1.Les_Options_Coupe.Pourcent/200+0.5;
      Poids_2:=1-Poids_1;
      Delta:=Form1.Les_Options_Coupe.Station_police.Size*25.4/72/2;
      if Illustrator then
        sLayer:=' i:layer="yes"'
      else
        sLayer:='';
      for Couche:=0 to 4 do
      begin
        case Couche of
          coVisee : writeln(F,Format('<g id="visees"%s style="stroke-width:%fmm; stroke:black;">',
                                     [sLayer,Form1.Les_Options_Coupe.Visee_Epaisseur/10]));
          coParois : writeln(F,Format('<g id="parois"%s style="stroke-width:%fmm; stroke:black;">',
                                     [sLayer,Form1.Les_Options_Coupe.Habit_Epaisseur/10]));
          coHauteur : writeln(F,Format('<g id="hauteurs"%s style="stroke-width:%fmm; stroke:black; fill:black;">',
                                     [sLayer,Form1.Les_Options_Coupe.Epaisseur_Hauteur/10]));
          coStation : writeln(F,Format('<g id="stations"%s style="stroke:black; fill:black;">',
                                     [sLayer]));
          coCarrefour : writeln(F,Format('<g id="carrefours"%s style="font-size:%dpt; font-family:%s;">',
                                     [sLayer,
                                     Form1.Les_Options_Coupe.Station_police.Size,
                                     Form1.Les_Options_Coupe.Station_police.Name]));
        end;
        for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
          writeln(F,'<g>'); // un groupe par fiche
          for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
            if Approximation and En_Coupe then begin
              writeln(F,'<g>'); // un groupe par galerie
              if Accrochage_Debut then begin
                X_Precedent:=Decal_Hori;
                Z_Precedent:=Decal_Z+Depart.Z;
              end
              else begin
                X_Precedent:=Fin_Hori;
                Z_Precedent:=Decal_Z+Depart.Z;
              end;
              Les_Hauteurs(J,0,dessus_precedent, dessous_precedent);
              if Couche=coHauteur then
              begin
                if (Form1.Les_Options_Coupe.Type_Hauteur=1) or AllLayers then
                begin
                  if Dessus_Precedent>-0.5 then
                    writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                                     [(X_Precedent-XMin)/Echelle*1000,
                                      (YMax-Z_Precedent)/Echelle*1000,
                                      (X_Precedent-XMin)/Echelle*1000,
                                      (YMax-(Z_Precedent+Dessus_Precedent))/Echelle*1000]));
                  if Dessous_Precedent>-0.5 then
                    writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                                     [(X_Precedent-XMin)/Echelle*1000,
                                      (YMax-Z_Precedent)/Echelle*1000,
                                      (X_Precedent-XMin)/Echelle*1000,
                                      (YMax-(Z_Precedent-Dessous_Precedent))/Echelle*1000]));
                end;
                if (Form1.Les_Options_Coupe.Type_Hauteur=2) and not  AllLayers then
                begin
                  if Dessus_Precedent>-0.5 then
                    writeln(F,Format('<circle cx="%fmm" cy="%fmm" r="%fmm"/>',
                                     [(X_Precedent-XMin)/Echelle*1000,
                                      (YMax-(Z_Precedent+Dessus_Precedent))/Echelle*1000,
                                       Form1.Les_Options_Plan.Epaisseur_Largeur/20]));
                  if Dessous_Precedent>-0.5 then
                    writeln(F,Format('<circle cx="%fmm" cy="%fmm" r="%fmm"/>',
                                     [(X_Precedent-XMin)/Echelle*1000,
                                      (YMax-(Z_Precedent+Dessous_Precedent))/Echelle*1000,
                                       Form1.Les_Options_Plan.Epaisseur_Largeur/20]));
                end;
              end;
              if (Couche=coStation) and ((Form1.Les_Options_Coupe.Diametre<>0) or AllLayers) then begin
                writeln(F,Format('<circle cx="%fmm" cy="%fmm" r="%fmm"/>',
                                  [(X_Precedent-XMin)/Echelle*1000,
                                   (YMax-Z_Precedent)/Echelle*1000,
                                   Form1.Les_Options_Coupe.Diametre/20]));
              end;
              if Couche=coCarrefour then
              if ((Form1.Les_Options_Coupe.Carrefour>0) or AllLayers) and not Depart.Terminus then begin
                Depart.Nom.lit(S);
                if Form1.Les_Options_Coupe.Carrefour=2 then begin
                  S:=S+'('+IntTostr(Depart.Indice)+')';
                end;
                writeln(F,Format('<text x="%fmm" y="%fmm">%s</text>',
                                 [(X_Precedent-XMin)/Echelle*1000+Delta,(Ymax-Z_Precedent)/Echelle*1000+Delta,S]));
              end;
              for K:=1 to Nb_Visee do begin
                X_Suivant:=Coordonnees^[K].LH;
                Z_Suivant:=Decal_Z+Coordonnees^[K].Z;
                if Couche=coVisee then
                  writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                                   [(X_Precedent*Poids_1+X_Suivant*Poids_2-XMin)/Echelle*1000,
                                    (YMax-(Z_Precedent*Poids_1+Z_Suivant*Poids_2))/Echelle*1000,
                                    (X_Precedent*Poids_2+X_Suivant*Poids_1-XMin)/Echelle*1000,
                                    (YMax-(Z_Precedent*Poids_2+Z_Suivant*Poids_1))/Echelle*1000]));
                if Form1.Les_Options_Coupe.Habit_Affiche or (Form1.Les_Options_Coupe.Type_Hauteur<>0) or AllLayers then begin
                  Les_Hauteurs(J,K,dessus_Suivant, dessous_Suivant);
                  if (Couche=coParois) and
                     (Form1.Les_Options_Coupe.Habit_Affiche or AllLayers) then
                  begin
                    if (Dessus_Precedent>-0.5) and (Dessus_Suivant>-0.5) then begin
                      writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                                       [(X_Precedent-XMin)/Echelle*1000,
                                        (YMax-(Z_Precedent+Dessus_Precedent))/Echelle*1000,
                                        (X_Suivant-XMin)/Echelle*1000,
                                        (YMax-(Z_Suivant+Dessus_Suivant))/Echelle*1000]));
                    end;
                    if (Dessous_Precedent>-0.5) and (Dessous_Suivant>-0.5) then begin
                      writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                                       [(X_Precedent-XMin)/Echelle*1000,
                                        (YMax-(Z_Precedent-Dessous_Precedent))/Echelle*1000,
                                        (X_Suivant-XMin)/Echelle*1000,
                                        (YMax-(Z_Suivant-Dessous_Suivant))/Echelle*1000]));
                    end;
                  end;
                  if Couche=coHauteur then
                  begin
                    if (Form1.Les_Options_Coupe.Type_Hauteur=1) or AllLayers then
                    begin
                      if Dessus_Suivant>-0.5 then
                        writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                                         [(X_Suivant-XMin)/Echelle*1000,
                                          (YMax-Z_Suivant)/Echelle*1000,
                                          (X_Suivant-XMin)/Echelle*1000,
                                          (YMax-(Z_Suivant+Dessus_Suivant))/Echelle*1000]));
                      if Dessous_Suivant>-0.5 then
                        writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                                         [(X_Suivant-XMin)/Echelle*1000,
                                          (YMax-Z_Suivant)/Echelle*1000,
                                          (X_Suivant-XMin)/Echelle*1000,
                                          (YMax-(Z_Suivant-Dessous_Suivant))/Echelle*1000]));
                    end;
                    if (Form1.Les_Options_Coupe.Type_Hauteur=2) and not  AllLayers then
                    begin
                      if Dessus_Suivant>-0.5 then
                        writeln(F,Format('<circle cx="%fmm" cy="%fmm" r="%fmm"/>',
                                         [(X_Suivant-XMin)/Echelle*1000,
                                          (YMax-(Z_Suivant+Dessus_Suivant))/Echelle*1000,
                                           Form1.Les_Options_Plan.Epaisseur_Largeur/20]));
                      if Dessous_Suivant>-0.5 then
                        writeln(F,Format('<circle cx="%fmm" cy="%fmm" r="%fmm"/>',
                                         [(X_Suivant-XMin)/Echelle*1000,
                                          (YMax-(Z_Suivant+Dessous_Suivant))/Echelle*1000,
                                           Form1.Les_Options_Plan.Epaisseur_Largeur/20]));
                    end;
                  end;
                end;
                Dessus_Precedent:=Dessus_Suivant;
                Dessous_Precedent:=Dessous_Suivant;
                if ((Form1.Les_Options_Coupe.Diametre<>0) or AllLayers) and (Couche=coStation) then
                  writeln(F,Format('<circle cx="%fmm" cy="%fmm" r="%fmm"/>',
                                    [(X_Suivant-XMin)/Echelle*1000,
                                     (YMax-Z_Suivant)/Echelle*1000,
                                     Form1.Les_Options_Coupe.Diametre/20]));
                X_Precedent:=X_Suivant; Z_Precedent:=Z_Suivant;
              end;
              if Couche=coCarrefour then
              if ((Form1.Les_Options_Coupe.Carrefour>0) or AllLayers) and not Arrivee.Terminus then begin
                Arrivee.Nom.lit(S);
                if Form1.Les_Options_Coupe.Carrefour=2 then begin
                  S:=S+'('+IntTostr(Arrivee.Indice)+')';
                end;
                writeln(F,Format('<text x="%fmm" y="%fmm">%s</text>',
                                 [(X_Precedent-XMin)/Echelle*1000+Delta,
                                  (Ymax-Z_Precedent)/Echelle*1000+Delta,S]));
              end;
              writeln(F,'</g>'); // un groupe par galerie
            end;
          end;
          writeln(F,'</g>'); // un groupe par fiche
        end;
        writeln(F,'</g>');
      end;
      if Form1.Les_Options_Coupe.Horizontale_Affiche or AllLayers then
      with Form1.Les_Options_Coupe do begin
        writeln(F,Format('<g id="echelle horizontale"%s>',[sLayer]));
        writeln(F,Format('<g style="stroke-width:%fmm; stroke:black;">',[Horizontale_Epaisseur/10]));
        writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                         [(Horizontale_X-XMin)/Echelle*1000,
                          (YMax-Horizontale_Z)/Echelle*1000,
                          (Horizontale_X+Horizontale_Longueur-XMin)/Echelle*1000,
                          (YMax-Horizontale_Z)/Echelle*1000]));
        writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                         [(Horizontale_X+Horizontale_Longueur-XMin)/Echelle*1000,
                          (YMax-Horizontale_Z)/Echelle*1000,
                          (Horizontale_X+Horizontale_Longueur-XMin)/Echelle*1000,
                          (YMax-Horizontale_Z)/Echelle*1000-Horizontale_Largeur]));
        writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                         [(Horizontale_X-XMin)/Echelle*1000,
                          (YMax-Horizontale_Z)/Echelle*1000,
                          (Horizontale_X-XMin)/Echelle*1000,
                          (YMax-Horizontale_Z)/Echelle*1000-Horizontale_Largeur]));
        if Horizontale_Pas>1 then for I:=1 to Horizontale_Pas-1 do begin
          writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                           [(Horizontale_X+Horizontale_Longueur/Horizontale_Pas*I-XMin)/Echelle*1000,
                            (YMax-Horizontale_Z)/Echelle*1000,
                            (Horizontale_X+Horizontale_Longueur/Horizontale_Pas*I-XMin)/Echelle*1000,
                            (YMax-Horizontale_Z)/Echelle*1000-Horizontale_Largeur]));
        end;
        writeln(F,'</g>');
        writeln(F,Format('<g style="font-size:%dpt; font-family:%s; text-anchor: middle">',
                         [Horizontale_police.Size,Horizontale_police.Name]));
        Delta:=Form1.Les_Options_Coupe.Horizontale_police.Size*25.4/72/2;
        writeln(F,Format('<text x="%fmm" y="%fmm">%s</text>',
                         [(Horizontale_X-XMin)/Echelle*1000,
                          (Ymax-Horizontale_Z)/Echelle*1000-Delta-Horizontale_Largeur,'0']));
        S:=IntTostr(Horizontale_Longueur)+tuDistance[m];
        writeln(F,Format('<text x="%fmm" y="%fmm">%s</text>',
                         [(Horizontale_X+Horizontale_Longueur-XMin)/Echelle*1000,
                          (Ymax-Horizontale_Z)/Echelle*1000-Delta-Horizontale_Largeur,S]));
        writeln(F,'</g>');
        writeln(F,'</g>');
      end;
      writeln(F,Format('<g id="echelles verticales"%s>',[sLayer]));
      Delta:=Form1.Les_Options_Coupe.Verticale_police.Size*25.4/72/2;
      for I:=1 to Form1.Les_Options_Coupe.Nb_Verticale do
                        with Form1.Les_Options_Coupe do with Verticale[I] do begin
        writeln(F,'<g>');
        Ref:=0;
        Trouve:=false;
        for J:=0 to Form1.Massif.Liste_Fiche.Count-1 do if not Trouve then with TFiche(Form1.Massif.Liste_Fiche[J]) do begin
          for K:=0 to Liste_Galerie.Count-1 do if not Trouve then with TGalerie(Liste_Galerie[K]) do begin
            if Approximation then begin
              if not Depart.Terminus and (Depart.Indice=Indice) then begin
                Depart.Nom.UpString(S);
                if UpperCase(Point)=S then begin
                  Trouve:=true; Ref:=Depart.Z;
                end;
              end;
              if not Arrivee.Terminus and (Arrivee.Indice=Indice) then begin
                Arrivee.Nom.UpString(S);
                if UpperCase(Point)=S then begin
                  Trouve:=true; Ref:=Arrivee.Z;
                end;
              end;
            end;
          end;
        end;
        writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm" style="stroke-width:%fmm; stroke:black;"/>',
                         [(XDev-XMin)/Echelle*1000,
                          (YMax-(Zmin+Decal+ref))/Echelle*1000,
                          (XDev-XMin)/Echelle*1000,
                          (YMax-(Zmax+Decal+Ref))/Echelle*1000,
                          Verticale_Epaisseur/10]));
        for J:=round(Zmin/Verticale_Pas+0.4999999) to round(Zmax/Verticale_Pas-0.4999999) do begin
          write(F,Format('<line x1="%fmm" y1="%fmm" ',
                         [(XDev-XMin)/Echelle*1000,
                          (YMax-(J*Verticale_Pas+Decal+ref))/Echelle*1000]));
          S:=IntTostr(J*Verticale_Pas);
          if Gauche then begin
            writeln(F,Format('x2="%fmm" y2="%fmm" style="stroke-width:%fmm; stroke:black;"/>',
                           [(XDev-XMin)/Echelle*1000-Verticale_Largeur,
                            (YMax-(J*Verticale_Pas+Decal+ref))/Echelle*1000,
                            Verticale_Epaisseur/10]));
            writeln(F,Format('<text x="%fmm" y="%fmm" style="font-size:%dpt; font-family:%s; text-anchor: end">%s</text>',
                             [(XDev-XMin)/Echelle*1000-Verticale_Largeur-Delta,
                              (YMax-(J*Verticale_Pas+Decal+ref))/Echelle*1000+Delta,
                              Verticale_police.Size,Verticale_police.Name,
                              S]));
          end
          else begin
            writeln(F,Format('x2="%fmm" y2="%fmm" style="stroke-width:%fmm; stroke:black;"/>',
                           [(XDev-XMin)/Echelle*1000+Verticale_Largeur,
                            (YMax-(J*Verticale_Pas+Decal+ref))/Echelle*1000,
                            Verticale_Epaisseur/10]));
            writeln(F,Format('<text x="%fmm" y="%fmm" style="font-size:%dpt; font-family:%s; text-anchor: start">%s</text>',
                             [(XDev-XMin)/Echelle*1000+Verticale_Largeur+Delta,
                              (YMax-(J*Verticale_Pas+Decal+ref))/Echelle*1000+Delta,
                              Verticale_police.Size,Verticale_police.Name,
                              S]));
          end;
        end;
        writeln(F,'</g>');
      end;
      writeln(F,'</g>');
    end;
    writeln(F,'</svg>');
  finally
    CloseFile(F);
  end;
end;


end.
