unit Graphere;

interface

uses Fichetpo, Classes, ExtCtrls, Printers, windows, Options, Forms, Graphics,
     Controls, Couleurs, Graphes, UrickGL, U3Dpolys, OpenGLPerso, SysUtils;

type

  TPerspective = class (TProjection)
    CosH, SinH, SinP, Rotation_Horizontale, Elevation, ZCentral,
    Centre_X, Centre_Y, Centre_Z : double;
    Vue_Azimuth, Vue_Pente : double;
    Vue_Position, PMin, PMax : TPoint3D;
    Deja_Vu, Relief, OpenGL_Actif, Premiere_Fois : boolean;
    Scene : TMaScene;
    constructor Create;
    procedure Dessine(Pixel_X, Pixel_Y : longint; Resolution, Rapport : double; Canevas : TImage; Etat : smallInt); override;
    procedure Imprime(Page_X, Page_Y, Sup_X, Sup_Y : integer; Etat : smallInt;
                              Fond : TCanvas; Reduc : double); override;
    procedure Gabarit(var Pixel_Hori, Pixel_Verti : longint; Resolution, Rapport : double; Canevas : TImage); override;
    procedure Gabarit_Imprimante(var Page_X, Page_Y : integer; MargeX, MargeY : integer); override;
    function Trouve(X, Y : longint;var N_Fiche, N_Galerie, N_Visee : integer) : double; override;
    procedure Isoler(X, Y, Rayon2 : longint); override;
    function Met_En_Relief(Nouveau : boolean; N_Fiche, N_Galerie, N_Visee : integer; Etat : smallInt) : string; override;
    function Essai(Hori, Verti : double): string;
    procedure Definitif(Hori, Verti : double);
    procedure Construit_Scene(Le_Support:TPanel; Etat : smallInt;var F : textFile);
    procedure Redessine_Scene;
    procedure Export_Svg(Fichier_SVG: TFileName; Illustrator, AllLayers : boolean); override;
  end;

implementation

uses Unit1, vrml, svgUtils;

(***************************** TPerspective **********************************)

constructor TPerspective.Create;
begin
  inherited Create;
  Echelle:=100;
  Rotation_Horizontale:=0; Elevation:=0;
  SinH:=0; CosH:=1; sinP:=0;
  Relief:=false;
  Premiere_Fois:=true;
end;

procedure TPerspective.Dessine(Pixel_X, Pixel_Y : longint; Resolution, Rapport : double; Canevas: TImage; Etat : smallInt);
var
  I, J, K : integer;
  X0, Y0, Z0, X1, Y1, Z1 : double;
  Xi, Yi, Zi, X_Precedent, Y_Precedent, Z_Precedent : double;
  Contrainte_Max, Poids_1, Poids_2, Delta : double;
  S, S2 : string;
  Hors_Plan : TBitMap;
begin
  PixelX:=Pixel_X; PixelY:=Pixel_Y;

  Hors_Plan:=TBitMap.Create;
  Hors_Plan.Width:=Canevas.Width;
  Hors_Plan.Height:=Canevas.Height;
  Le_Caneva:=Hors_Plan.Canvas;
  if Relief then begin
    Le_Caneva.Pen.Color:=clBlack; Le_Caneva.Pen.Style:=psSolid; Le_Caneva.Pen.Mode:=pmCopy;
    Le_Caneva.Brush.Color:=clBlack; Le_Caneva.Brush.Style:=bsSolid;
    Le_Caneva.FloodFill(1,1,clWhite,fsSurface);
    Le_Caneva.Pen.Mode:=pmMerge;
  end
  else begin
    Le_Caneva.Pen.Color:=clWhite; Le_Caneva.Pen.Style:=psSolid; Le_Caneva.Pen.Mode:=pmCopy;
    Le_Caneva.Brush.Color:=clWhite; Le_Caneva.Brush.Style:=bsSolid;
  end;
  La_Resolution:=Resolution*Rapport;
  if Affichable then begin
    Poids_1:=Form1.Les_Options_Relief.Pourcent/200+0.5;
    Poids_2:=1-Poids_1;
    Le_Caneva.Pen.Color:=clBlack;
    Le_Caneva.Brush.Color:=clBlack; Le_Caneva.Brush.Style:=bsSolid;
    Le_Caneva.Pen.Width:=round(Form1.Les_Options_Relief.Visee_Epaisseur/10*Resolution*Rapport);
    if Form1.Les_Options_Relief.Carrefour>0 then begin
      Le_Caneva.Font:=Form1.Les_Options_Relief.Station_police;
      Le_Caneva.Font.Height:=round(Le_Caneva.Font.Height*Rapport);
      Delta:=Le_Caneva.TextHeight('O')/Resolution/2000*Echelle/Rapport;
    end;
    if Etat=etContrainte then Contrainte_Max:=Contrainte_Maximum(Form1.Massif.Liste_Courante);
    for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
      for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
        if Approximation then begin
          case Etat of
            etContrainte : Le_Caneva.Pen.Color:=
             Degrade[round(sqrt(sqr(Contrainte_Plan)+sqr(Contrainte_Z))/Contrainte_Max*60)];
            etNoir, etRotation : Le_Caneva.Pen.Color:=clBlack;
            etCouleur :  Le_Caneva.Pen.Color:=Couleur;
          end;
          if not Relief then begin
            X0:=Depart.X; Y0:=Depart.Y; Z0:=Depart.Z;
            X_Precedent:=X0*cosH+Y0*sinH;
            Y_Precedent:=-X0*sinH+Y0*cosH;
            Z_Precedent:=Z0-Y_Precedent*sinP;
            if Form1.Les_Options_Relief.Diametre<>0 then begin
              Le_Caneva.Brush.Color:=clBlack; Le_Caneva.Brush.Style:=bsSolid;
              Cercle(X_Precedent,Z_Precedent,Form1.Les_Options_Relief.Diametre);
            end;
            if (Form1.Les_Options_Relief.Carrefour>0) and not Depart.Terminus then begin
              Le_Caneva.Brush.Color:=clWhite; Le_Caneva.Brush.Style:=bsClear;
              Depart.Nom.lit(S);
              if Form1.Les_Options_Relief.Carrefour=2 then begin
                S:=S+'('+IntTostr(Depart.Indice)+')';
              end;
              TextRel(X_Precedent+Delta,Z_Precedent+Delta,S);
            end;
            for K:=1 to Nb_Visee do begin
              X0:=Coordonnees^[K].X;
              Y0:=Coordonnees^[K].Y;
              Z0:=Coordonnees^[K].Z;
              Xi:=X0*cosH+Y0*sinH;
              Yi:=-X0*sinH+Y0*cosH;
              Zi:=Z0-Yi*sinP;
              MoveToRel(X_Precedent*Poids_1+Xi*Poids_2,
                        Z_Precedent*Poids_1+Zi*Poids_2);
              LineToRel(X_Precedent*Poids_2+Xi*Poids_1,
                        Z_Precedent*Poids_2+Zi*Poids_1);
              if Form1.Les_Options_Relief.Diametre<>0 then
                                    Cercle(Xi,Zi,Form1.Les_Options_Relief.Diametre);
              X_Precedent:=Xi; Z_Precedent:=Zi;
            end;
            if (Form1.Les_Options_Relief.Carrefour>0) and not Arrivee.Terminus then begin
              Le_Caneva.Brush.Color:=clWhite; Le_Caneva.Brush.Style:=bsClear;
              Arrivee.Nom.lit(S);
              if Form1.Les_Options_Relief.Carrefour=2 then begin
                S:=S+'('+IntTostr(Arrivee.Indice)+')';
              end;
              TextRel(X_Precedent+Delta,Z_Precedent+Delta,S);
            end;
          end
          else begin
            Le_Caneva.Pen.Color:=clBlue;
            X0:=Depart.X; Y0:=Depart.Y; Z0:=Depart.Z;
            Xi:=X0*cosH+Y0*sinH;
            Yi:=-X0*sinH+Y0*cosH;
            Zi:=Z0-Yi*sinP;
            Xi:=(Xi-(XMin+XMax)/2-Entre_Yeux*Echelle/Rapport/2)*Observateur/((Yi-ZCentral)/Echelle*Rapport+Observateur)
                 +Entre_Yeux*Echelle/Rapport/2+(XMin+XMax)/2;
            Zi:=(Zi-(YMin+YMax)/2)*Observateur/((Yi-ZCentral)/Echelle*Rapport+Observateur)+(YMin+YMax)/2;
            MoveToRel(Xi,Zi);
            for K:=1 to Nb_Visee do begin
              X1:=Coordonnees^[K].X;
              Y1:=Coordonnees^[K].Y;
              Z1:=Coordonnees^[K].Z;
              Xi:=X1*cosH+Y1*sinH;
              Yi:=-X1*sinH+Y1*cosH;
              Zi:=Z1-Yi*sinP;
              Xi:=(Xi-(XMin+XMax)/2-Entre_Yeux*Echelle/Rapport/2)*Observateur/((Yi-ZCentral)/Echelle*Rapport+Observateur)
                           +Entre_Yeux*Echelle/Rapport/2+(XMin+XMax)/2;
              Zi:=(Zi-(YMin+YMax)/2)*Observateur/((Yi-ZCentral)/Echelle*Rapport+Observateur)+(YMin+YMax)/2;
              LineToRel(Xi, Zi);
            end;
            Le_Caneva.Pen.Color:=RGB(192,0,0);
            X0:=Depart.X; Y0:=Depart.Y; Z0:=Depart.Z;
            Xi:=X0*cosH+Y0*sinH;
            Yi:=-X0*sinH+Y0*cosH;
            Zi:=Z0-Yi*sinP;
            Xi:=(Xi-(XMin+XMax)/2+Entre_Yeux*Echelle/Rapport/2)*Observateur/((Yi-ZCentral)/Echelle*Rapport+Observateur)
                  -Entre_Yeux*Echelle/Rapport/2+(XMin+XMax)/2;
            Zi:=(Zi-(YMin+YMax)/2)*Observateur/((Yi-ZCentral)/Echelle*Rapport+Observateur)+(YMin+YMax)/2;
            MoveToRel(Xi,Zi);
            for K:=1 to Nb_Visee do begin
              X1:=Coordonnees^[K].X;
              Y1:=Coordonnees^[K].Y;
              Z1:=Coordonnees^[K].Z;
              Xi:=X1*cosH+Y1*sinH;
              Yi:=-X1*sinH+Y1*cosH;
              Zi:=Z1-Yi*sinP;
              Xi:=(Xi-(XMin+XMax)/2+Entre_Yeux*Echelle/Rapport/2)*Observateur/((Yi-ZCentral)/Echelle*Rapport+Observateur)
                      -Entre_Yeux*Echelle/Rapport/2+(XMin+XMax)/2;
              Zi:=(Zi-(YMin+YMax)/2)*Observateur/((Yi-ZCentral)/Echelle*Rapport+Observateur)+(YMin+YMax)/2;
              LineToRel(Xi, Zi);
            end;
          end;
        end;
      end;
    end;
    if not Relief then begin
      if Form1.Les_Options_Relief.Repere_Affiche then with Form1.Les_Options_Relief do begin
        Le_Caneva.Pen.Width:=round(Repere_Epaisseur/10*Resolution);
        Le_Caneva.Pen.Color:=clBlack;
        X0:=Repere_X*1000; Y0:=Repere_Y*1000; Z0:=Repere_Z;
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        MoveToRel(Xi,Zi);
        if Inverse_X then X0:=X0-Repere_Longueur else X0:=X0+Repere_Longueur;
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        LineToRel(Xi,Zi);
        X0:=Repere_X*1000; Y0:=Repere_Y*1000; Z0:=Repere_Z;
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        MoveToRel(Xi,Zi);
        if Inverse_Y then Y0:=Y0-Repere_Longueur else Y0:=Y0+Repere_Longueur;
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        LineToRel(Xi,Zi);
        X0:=Repere_X*1000; Y0:=Repere_Y*1000; Z0:=Repere_Z;
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        MoveToRel(Xi,Zi);
        if Inverse_Z then Z0:=Z0-Repere_Longueur else Z0:=Z0+Repere_Longueur;
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        LineToRel(Xi,Zi);
{        LineToRel(Xi+Repere_Longueur,Yi+Repere_Largeur/1000*Echelle);
        MoveToRel(Xi,Yi);
        LineToRel(Xi,Yi+Repere_Largeur/1000*Echelle);
        if Repere_Sub>1 then for I:=1 to Repere_Sub-1 do begin
          MoveToRel(Xi+Repere_Longueur/Repere_Sub*I,Yi);
          LineToRel(Xi+Repere_Longueur/Repere_Sub*I,Yi+Repere_Largeur/1000*Echelle);
        end;
        Le_Caneva.Font:=Repere_police;
        Le_Caneva.Font.Height:=round(Le_Caneva.Font.Height*Rapport);
        Le_Caneva.Brush.Color:=clWhite;
        X0:=Le_Caneva.TextWidth('0')/Resolution/2;
        Y0:=Le_Caneva.TextHeight('0')/Resolution;
        TextRel(Xi-X0/1000*Echelle,Yi+(Repere_Largeur+Y0)/1000*Echelle,'0');
        str(Repere_Longueur,S); S:=S+tuDistance[m];
        X0:=Le_Caneva.TextWidth(S)/Resolution/2;
        TextRel(Xi+Repere_Longueur-X0/1000*Echelle,Yi+(Repere_Largeur+Y0)/1000*Echelle,S);}
      end;
      if Form1.Les_Options_Relief.Nord_Affiche then with Form1.Les_Options_Relief do begin
        Le_Caneva.Pen.Width:=round(Nord_Epaisseur/10*Resolution);
        Le_Caneva.Pen.Color:=clBlack;
        X0:=Nord_X*1000; Y0:=Nord_Y*1000; Z0:=Nord_Z;
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        MoveToRel(Xi,Zi);
        if NG then begin
          Y0:=Y0+Nord_Longueur/1000*Echelle*cos(Form1.Massif.Angle_Geodesique/180*PI);
          X0:=X0-Nord_Longueur/1000*Echelle*sin(Form1.Massif.Angle_Geodesique/180*PI);
          Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
          LineToRel(Xi,Zi);
          Y0:=Y0-Nord_Longueur/1000*Echelle*
           (cos(Form1.Massif.Angle_Geodesique/180*PI)/2-sin(Form1.Massif.Angle_Geodesique/180*PI)/10);
          X0:=X0+Nord_Longueur/1000*Echelle*
           (sin(Form1.Massif.Angle_Geodesique/180*PI)/2+cos(Form1.Massif.Angle_Geodesique/180*PI)/10);
          Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
          LineToRel(Xi,Zi);
          Le_Caneva.Font:=Nord_police;
          Le_Caneva.Font.Height:=round(Le_Caneva.Font.Height*Rapport);
          Le_Caneva.Brush.Color:=clWhite; Le_Caneva.Brush.Style:=bsClear;
          TextRel(Xi,Zi,Texte_Nord);
        end
        else begin
          Y0:=Y0+Nord_Longueur/1000*Echelle;
          Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
          LineToRel(Xi,Zi);
          Y0:=Y0-Nord_Longueur/1000*Echelle/2; X0:=X0+Nord_Longueur/1000*Echelle/10;
          Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
          LineToRel(Xi,Zi);
          Le_Caneva.Font:=Nord_police;
          Le_Caneva.Font.Height:=round(Le_Caneva.Font.Height*Rapport);
          Le_Caneva.Brush.Color:=clWhite; Le_Caneva.Brush.Style:=bsClear;
          TextRel(Xi,Zi,Texte_Nord);
        end;
      end;
      if Form1.Les_Options_Relief.Rectangle_Affiche then begin
        Le_Caneva.Pen.Color:=Form1.Les_Options_Relief.Rectangle_Couleur;
        Le_Caneva.Pen.Width:=round(Form1.Les_Options_Relief.Rectangle_Epaisseur/10*Resolution*Rapport);
        X1:=PMin.X; Y1:=PMin.Y; Z1:=PMin.Z;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        MoveToRel(Xi, Zi);
        X1:=PMax.X;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        LineToRel(Xi, Zi);
        Y1:=PMax.Y;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        LineToRel(Xi, Zi);
        X1:=PMin.X;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        LineToRel(Xi, Zi);
        Y1:=PMin.Y;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        LineToRel(Xi, Zi);
        Z1:=PMax.Z;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        LineToRel(Xi, Zi);
        X1:=PMax.X;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        LineToRel(Xi, Zi);
        Y1:=PMax.Y;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        LineToRel(Xi, Zi);
        X1:=PMin.X;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        LineToRel(Xi, Zi);
        Y1:=PMin.Y;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        LineToRel(Xi, Zi);
        X1:=PMax.X;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        MoveToRel(Xi, Zi);
        Z1:=Pmin.Z;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        LineToRel(Xi, Zi);
        Y1:=PMax.Y;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        MoveToRel(Xi, Zi);
        Z1:=Pmax.Z;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        LineToRel(Xi, Zi);
        X1:=Pmin.X;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        MoveToRel(Xi, Zi);
        Z1:=Pmin.Z;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        LineToRel(Xi, Zi);
      end;
    end;
  end;

  Le_Caneva.Pen.Mode:=pmCopy;

  Le_Caneva:=Canevas.Canvas;
  Le_Caneva.CopyMode:=cmSrcCopy;
  Le_Caneva.Draw(0,0,Hors_Plan);
  Hors_Plan.Free;
end;

procedure TPerspective.Imprime(Page_X, Page_Y, Sup_X, Sup_Y : integer; Etat : smallInt; Fond : TCanvas; Reduc : double);
var
  I, J, K, Page_Horizontale,  Page_Verticale: integer;
  X0, Y0, Z0, X1, Y1, Z1 : double;
  Xi, Yi, Zi, X_Precedent, Y_Precedent, Z_Precedent, Poids_1, Poids_2, Delta,
  Contrainte_Max : double;
  S, S2 : string;
begin
  Support:=Fond; Correction:=Reduc;
  Supl_X:=Sup_X; Supl_Y:=Sup_Y;

  Page_Horizontale:=trunc((XMax-XMin-0.02*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MX))+1;
  Page_Verticale:=trunc((YMax-YMin-0.02*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MY))+1;
  Marge_X:=(Page_Horizontale-(XMax-XMin)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MX))/2;
  Marge_Y:=(Page_Verticale-(YMax-YMin)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MY))/2;
  PageX:=Page_X; PageY:=Page_Y;
  Resolution_Imprimante:=Printer.PageWidth/GetDeviceCaps(Printer.Handle,HorzSize);
  if Affichable then begin
    Poids_1:=Form1.Les_Options_Relief.Pourcent/200+0.5;
    Poids_2:=1-Poids_1;
    Support.Pen.Color:=clBlack;
    Support.Brush.Color:=clBlack; Support.Brush.Style:=bsSolid;
    Support.Pen.Width:=round(Form1.Les_Options_Relief.Visee_Epaisseur/10*Resolution_Imprimante*Correction);
    if Form1.Les_Options_Relief.Carrefour>0 then begin
      Support.Font:=Form1.Les_Options_Relief.Station_police;
      Printer.Canvas.Font:=Form1.Les_Options_Relief.Station_police;
      if Correction<>1 then Support.Font.Height:=round(Support.Font.Height*Correction*
           Resolution_Imprimante/Form1.PixelsPerInch*25.4);
      Delta:=Printer.Canvas.TextHeight('O')/Resolution_Imprimante/2000*Echelle;
    end;
    if  Etat=etContrainte then Contrainte_Max:=Contrainte_Maximum(Form1.Massif.Liste_Courante);
    for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
      for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
        if Approximation then begin
          case Etat of
            etContrainte : Support.Pen.Color:=
             Degrade[round(sqrt(sqr(Contrainte_Plan)+sqr(Contrainte_Z))/Contrainte_Max*60)];
            etNoir, etRotation : Support.Pen.Color:=clBlack;
            etCouleur :  Support.Pen.Color:=Couleur;
          end;
          X0:=Depart.X; Y0:=Depart.Y; Z0:=Depart.Z;
          X_Precedent:=X0*cosH+Y0*sinH;
          Y_Precedent:=-X0*sinH+Y0*cosH;
          Z_Precedent:=Z0-Y_Precedent*sinP;
          if Form1.Les_Options_Relief.Diametre<>0 then begin
            Support.Brush.Color:=clBlack; Support.Brush.Style:=bsSolid;
            Cercle_Imp(X_Precedent,Z_Precedent,Form1.Les_Options_Relief.Diametre);
          end;
          if (Form1.Les_Options_Relief.Carrefour>0) and not Depart.Terminus then begin
            Support.Brush.Color:=clWhite; Support.Brush.Style:=bsClear;
            Depart.Nom.lit(S);
            if Form1.Les_Options_Relief.Carrefour=2 then begin
              S:=S+'('+IntTostr(Depart.Indice)+')';
            end;
            TextImp(X_Precedent+Delta,Z_Precedent+Delta,S);
          end;
          for K:=1 to Nb_Visee do begin
            X0:=Coordonnees^[K].X;
            Y0:=Coordonnees^[K].Y;
            Z0:=Coordonnees^[K].Z;
            Xi:=X0*cosH+Y0*sinH;
            Yi:=-X0*sinH+Y0*cosH;
            Zi:=Z0-Yi*sinP;
            MoveToImp(X_Precedent*Poids_1+Xi*Poids_2,
                      Z_Precedent*Poids_1+Zi*Poids_2);
            LineToImp(X_Precedent*Poids_2+Xi*Poids_1,
                      Z_Precedent*Poids_2+Zi*Poids_1);
            if Form1.Les_Options_Relief.Diametre<>0 then
                                  Cercle_Imp(Xi,Zi,Form1.Les_Options_Relief.Diametre);
            X_Precedent:=Xi; Z_Precedent:=Zi;
          end;
          if (Form1.Les_Options_Relief.Carrefour>0) and not Arrivee.Terminus then begin
            Support.Brush.Color:=clWhite; Support.Brush.Style:=bsClear;
            Arrivee.Nom.lit(S);
            if Form1.Les_Options_Relief.Carrefour=2 then begin
              S:=S+'('+IntTostr(Arrivee.Indice)+')';
            end;
            TextImp(X_Precedent+Delta,Z_Precedent+Delta,S);
          end;
        end;
      end;
    end;
    if Form1.Les_Options_Relief.Repere_Affiche then with Form1.Les_Options_Relief do begin
      Support.Pen.Width:=round(Repere_Epaisseur/10*Resolution_Imprimante*Correction);
      Support.Pen.Color:=clBlack;
      X0:=Repere_X*1000; Y0:=Repere_Y*1000; Z0:=Repere_Z;
      Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
      MoveToImp(Xi,Zi);
      if Inverse_X then X0:=X0-Repere_Longueur else X0:=X0+Repere_Longueur;
      Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
      LineToImp(Xi,Zi);
      X0:=Repere_X*1000; Y0:=Repere_Y*1000; Z0:=Repere_Z;
      Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
      MoveToImp(Xi,Zi);
      if Inverse_Y then Y0:=Y0-Repere_Longueur else Y0:=Y0+Repere_Longueur;
      Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
      LineToImp(Xi,Zi);
      X0:=Repere_X*1000; Y0:=Repere_Y*1000; Z0:=Repere_Z;
      Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
      MoveToImp(Xi,Zi);
      if Inverse_Z then Z0:=Z0-Repere_Longueur else Z0:=Z0+Repere_Longueur;
      Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
      LineToImp(Xi,Zi);
    end;
    if Form1.Les_Options_Relief.Nord_Affiche then with Form1.Les_Options_Relief do begin
      Support.Pen.Width:=round(Nord_Epaisseur/10*Resolution_Imprimante*Correction);
      Support.Pen.Color:=clBlack;
      X0:=Nord_X*1000; Y0:=Nord_Y*1000; Z0:=Nord_Z;
      Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
      MoveToImp(Xi,Zi);
      if NG then begin
        Y0:=Y0+Nord_Longueur/1000*Echelle*cos(Form1.Massif.Angle_Geodesique/180*PI);
        X0:=X0-Nord_Longueur/1000*Echelle*sin(Form1.Massif.Angle_Geodesique/180*PI);
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        LineToImp(Xi,Zi);
        Y0:=Y0-Nord_Longueur/1000*Echelle*
         (cos(Form1.Massif.Angle_Geodesique/180*PI)/2-sin(Form1.Massif.Angle_Geodesique/180*PI)/10);
        X0:=X0+Nord_Longueur/1000*Echelle*
         (sin(Form1.Massif.Angle_Geodesique/180*PI)/2+cos(Form1.Massif.Angle_Geodesique/180*PI)/10);
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        LineToImp(Xi,Zi);
        Support.Font:=Nord_police;
        if Correction<>1 then Support.Font.Height:=round(Support.Font.Height*Correction*
             Resolution_Imprimante/Form1.PixelsPerInch*25.4);
        Support.Brush.Color:=clWhite; Support.Brush.Style:=bsClear;
        TextImp(Xi,Zi,Texte_Nord);
      end
      else begin
        Y0:=Y0+Nord_Longueur/1000*Echelle;
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        LineToImp(Xi,Zi);
        Y0:=Y0-Nord_Longueur/1000*Echelle/2; X0:=X0+Nord_Longueur/1000*Echelle/10;
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        LineToImp(Xi,Zi);
        Support.Font:=Nord_police;
        if Correction<>1 then Support.Font.Height:=round(Support.Font.Height*Correction*
             Resolution_Imprimante/Form1.PixelsPerInch*25.4);
        Support.Brush.Color:=clWhite; Support.Brush.Style:=bsClear;
        TextImp(Xi,Zi,Texte_Nord);
      end;
    end;

    if Form1.Les_Options_Relief.Rectangle_Affiche then begin
      Support.Pen.Color:=Form1.Les_Options_Relief.Rectangle_Couleur;
      Support.Pen.Width:=round(Form1.Les_Options_Relief.Rectangle_Epaisseur/10*Resolution_Imprimante*Correction);
      X1:=PMin.X; Y1:=PMin.Y; Z1:=PMin.Z;
      Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
      MoveToImp(Xi, Zi);
      X1:=PMax.X;
      Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
      LineToImp(Xi, Zi);
      Y1:=PMax.Y;
      Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
      LineToImp(Xi, Zi);
      X1:=PMin.X;
      Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
      LineToImp(Xi, Zi);
      Y1:=PMin.Y;
      Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
      LineToImp(Xi, Zi);
      Z1:=PMax.Z;
      Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
      LineToImp(Xi, Zi);
      X1:=PMax.X;
      Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
      LineToImp(Xi, Zi);
      Y1:=PMax.Y;
      Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
      LineToImp(Xi, Zi);
      X1:=PMin.X;
      Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
      LineToImp(Xi, Zi);
      Y1:=PMin.Y;
      Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
      LineToImp(Xi, Zi);
      X1:=PMax.X;
      Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
      MoveToImp(Xi, Zi);
      Z1:=Pmin.Z;
      Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
      LineToImp(Xi, Zi);
      Y1:=PMax.Y;
      Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
      MoveToImp(Xi, Zi);
      Z1:=Pmax.Z;
      Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
      LineToImp(Xi, Zi);
      X1:=Pmin.X;
      Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
      MoveToImp(Xi, Zi);
      Z1:=Pmin.Z;
      Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
      LineToImp(Xi, Zi);
    end;
  end;
  Support.Brush.Color:=clWhite; Support.Brush.Style:=bsClear;
end;

procedure TPerspective.Gabarit(var Pixel_Hori, Pixel_Verti : longint; Resolution, Rapport : double; Canevas : TImage);
var
  I : integer;
  Les_Angles : array[1..8] of TPoint3D;
  Xi, Yi, Zi, ZMin, ZMax : double;
begin
  Affichable:=Calibre(PMin, PMax,Form1.Massif.Liste_Courante);
  if Affichable then begin
    Les_Angles[1].X:=PMin.X; Les_Angles[1].Y:=PMin.Y; Les_Angles[1].Z:=PMin.Z;
    Les_Angles[2].X:=PMin.X; Les_Angles[2].Y:=PMin.Y; Les_Angles[2].Z:=PMax.Z;
    Les_Angles[3].X:=PMin.X; Les_Angles[3].Y:=PMax.Y; Les_Angles[3].Z:=PMin.Z;
    Les_Angles[4].X:=PMin.X; Les_Angles[4].Y:=PMax.Y; Les_Angles[4].Z:=PMax.Z;
    Les_Angles[5].X:=PMax.X; Les_Angles[5].Y:=PMin.Y; Les_Angles[5].Z:=PMin.Z;
    Les_Angles[6].X:=PMax.X; Les_Angles[6].Y:=PMin.Y; Les_Angles[6].Z:=PMax.Z;
    Les_Angles[7].X:=PMax.X; Les_Angles[7].Y:=PMax.Y; Les_Angles[7].Z:=PMin.Z;
    Les_Angles[8].X:=PMax.X; Les_Angles[8].Y:=PMax.Y; Les_Angles[8].Z:=PMax.Z;
    for I:=1 to 8 do with Les_Angles[I] do begin
      Xi:=X*cosH+Y*sinH;
      Yi:=-X*sinH+Y*cosH;
      Zi:=Z-Yi*sinP;
      X:=Xi; Y:=Zi; Z:=Yi;
    end;
    XMin:=Les_Angles[1].X; XMax:=XMin;
    YMin:=Les_Angles[1].Y; YMax:=YMin;
    ZMin:=Les_Angles[1].Z; ZMax:=ZMin;
    for I:=2 to 8 do with Les_Angles[I] do begin
      if X<XMin then XMin:=X;
      if X>XMax then XMax:=X;
      if Y<YMin then YMin:=Y;
      if Y>YMax then YMax:=Y;
      if Z<ZMin then ZMin:=Z;
      if Z>ZMax then ZMax:=Z;
    end;
    Xmin:=XMin-0.01*Echelle; Xmax:=XMax+0.01*Echelle;
    Ymin:=YMin-0.01*Echelle; Ymax:=YMax+0.01*Echelle;
    Pixel_Hori:=round((XMax-XMin)/Echelle*1000*Resolution*Rapport);
    Pixel_Verti:=round((YMax-YMin)/Echelle*1000*Resolution*Rapport);
    ZCentral:=(ZMin+Zmax)/2;
  end
  else begin
    Pixel_Hori:=1;
    Pixel_Verti:=1;
  end;
end;

procedure TPerspective.Gabarit_Imprimante(var Page_X, Page_Y : integer; MargeX, MargeY : integer);
begin
  Page_X:=trunc((XMax-XMin-0.02*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MargeX))+1;
  Page_Y:=trunc((YMax-YMin-0.02*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MargeY))+1;
  MX:=MargeX;
  MY:=MargeY;
end;

function TPerspective.Essai(Hori, Verti : double):string;
begin
  sinH:=sin((Rotation_Horizontale+Hori)/180*PI);
  cosH:=cos((Rotation_Horizontale+Hori)/180*PI);
  if Elevation+Hori>90 then Hori:=89-Elevation;
  if Elevation+Hori<-90 then Hori:=-89-Elevation;
  sinP:=sin((Elevation+Verti)/180*PI);
  Essai:=FloatTostrF(Rotation_Horizontale+Hori,ffGeneral,3,0)+'   '+
    FloatTostrF(-Elevation-Verti,ffGeneral,2,0)+' ';
end;

procedure TPerspective.Definitif(Hori, Verti : double);
begin
  Rotation_Horizontale:=Rotation_Horizontale+Hori;
  if Rotation_Horizontale>360 then Rotation_Horizontale:=Rotation_Horizontale-360;
  if Rotation_Horizontale<0 then Rotation_Horizontale:=Rotation_Horizontale+360;
  Elevation:=Elevation+Verti;
  if Elevation>90 then Elevation:=89;
  if Elevation<-90 then Elevation:=-89;
  sinH:=sin(Rotation_Horizontale/180*PI);
  cosH:=cos(Rotation_Horizontale/180*PI);
  sinP:=sin(Elevation/180*PI);
end;

function TPerspective.Trouve(X, Y : longint;var N_Fiche, N_Galerie, N_Visee : integer) : double;
var
  I, J, K : integer;
  X0, Y0, Z0, X1, Y1, Z1 : double;
  Xi, Yi, X_Abs, Y_Abs, DMin, Zone : double;
  Repere : boolean;
begin
  X_Abs:=X/1000*Echelle/La_Resolution+XMin;
  Y_Abs:=YMax-Y/1000*Echelle/La_Resolution;
  Repere:=false;
  if Affichable then begin
    for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
      for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
        if Approximation then begin
          X0:=Depart.X; Y0:=Depart.Y; Z0:=Depart.Z;
          Xi:=X0*cosH+Y0*sinH;
          Yi:=-X0*sinH+Y0*cosH;
          Yi:=Z0-Yi*sinP;
          if not Repere then begin
            Repere:=true;
            Dmin:=sqr((Xi-X_Abs))+sqr((Yi-Y_Abs));
            N_Fiche:=I; N_Galerie:=J; N_Visee:=1;
          end
          else if sqr((Xi-X_Abs))+sqr((Yi-Y_Abs))<Dmin then begin
            Dmin:=sqr((Xi-X_Abs))+sqr((Yi-Y_Abs));
            N_Fiche:=I; N_Galerie:=J; N_Visee:=1;
          end;
          for K:=1 to Nb_Visee do begin
            X1:=Coordonnees^[K].X;
            Y1:=Coordonnees^[K].Y;
            Z1:=Coordonnees^[K].Z;
            X0:=X1*cosH+Y1*sinH;
            Y0:=-X1*sinH+Y1*cosH;
            Y0:=Z1-Y0*sinP;
            Zone:=(X_Abs-Xi)*(X0-Xi)+(Y_Abs-Yi)*(Y0-Yi);
            if Zone>sqr(X0-Xi)+sqr(Y0-Yi) then begin
              if sqr((X0-X_Abs))+sqr((Y0-Y_Abs))<Dmin then begin
                Dmin:=sqr((X0-X_Abs))+sqr((Y0-Y_Abs));
                N_Fiche:=I; N_Galerie:=J; N_Visee:=K;
              end;
            end
            else if (Zone>0) and ((Xi<>X0) or (Yi<>Y0)) then begin
              Zone:=(X0-Xi)*(Y_Abs-Yi)-(Y0-Yi)*(X_Abs-Xi);
              Zone:=sqr(Zone)/(sqr(X0-Xi)+sqr(Y0-Yi));
              if Zone<Dmin then begin
                Dmin:=Zone;
                N_Fiche:=I; N_Galerie:=J; N_Visee:=K;
              end;
            end;
            Xi:=X0; Yi:=Y0;
          end;
        end;
      end;
    end;
  end
  else Dmin:=1e6;
  Trouve:=sqrt(Dmin)*1000/Echelle*La_Resolution;;
end;

procedure TPerspective.Isoler(X, Y, Rayon2 : longint);
var
  I, J, K : integer;
  X0, Y0, Z0 : double;
  Nouvelle_Selection : TList;
  Xi, Yi, Zi, X_Abs, Y_Abs : double;
  Repere : boolean;
begin
  X_Abs:=X/1000*Echelle/La_Resolution+XMin;
  Y_Abs:=YMax-Y/1000*Echelle/La_Resolution;
  Nouvelle_Selection:=TList.Create;
  if Affichable then begin
    for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
      Repere:=false;
      for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
        if Approximation then begin
          X0:=Depart.X; Y0:=Depart.Y; Z0:=Depart.Z;
          Xi:=X0*cosH+Y0*sinH;
          Yi:=-X0*sinH+Y0*cosH;
          Yi:=Z0-Yi*sinP;
          if sqr((Xi-X_Abs))+sqr((Yi-Y_Abs))<Rayon2*sqr(Echelle/La_Resolution/1000) then Repere:=true;
          for K:=1 to Nb_Visee do begin
            Xi:=Coordonnees^[K].X;
            Yi:=Coordonnees^[K].Y;
            Zi:=Coordonnees^[K].Z;
            X0:=Xi*cosH+Yi*sinH;
            Y0:=-Xi*sinH+Yi*cosH;
            Y0:=Zi-Y0*sinP;
            if sqr((X0-X_Abs))+sqr((Y0-Y_Abs))<Rayon2*sqr(Echelle/La_Resolution/1000) then Repere:=true;
          end;
        end;
      end;
      if Repere then Nouvelle_Selection.Add(Form1.Massif.Liste_Courante[I]);
    end;
  end;
  if Form1.Massif.Existe_Selection then Form1.Massif.Selection.Free;
  Form1.Massif.Selection:=Nouvelle_Selection;
  Form1.Massif.Liste_Courante:=Nouvelle_Selection;
  Form1.Massif.Existe_Selection:=true;
end;

function TPerspective.Met_En_Relief(Nouveau : boolean; N_Fiche, N_Galerie, N_Visee : integer;Etat : smallInt) : string;
var
  K : integer;
  X0, Y0, Z0, X1, Y1, Z1, X_Precedent, Z_Precedent : double;
  Xi, Yi, Zi : double;
  S, Resultat : string;
begin
  with TFiche(Form1.Massif.Liste_Courante[N_Fiche]) do
  with TGalerie(Liste_Galerie[N_Galerie]) do begin
    Le_Caneva.Pen.Style:=psSolid; Le_Caneva.Pen.Mode:=pmCopy;

{ cration de la chaine rsum }
    if Nouveau then begin
      Systeme.Lit(S); Resultat:=S;
      Cavite.Lit(S); Resultat:=Resultat+'/'+S;
      Reseau.Lit(S); Resultat:=Resultat+'/'+S;
      Lieu.Lit(S); Resultat:=Resultat+'/'+S;
      if Depart.Terminus then Resultat:=Resultat+' Terminus - '
      else begin
        Depart.Nom.Lit(S); Resultat:=Resultat+' '+S+' - ';
      end;
      if Arrivee.Terminus then Resultat:=Resultat+'Terminus'
      else begin
        Arrivee.Nom.Lit(S); Resultat:=Resultat+S;
      end;
      Met_En_Relief:=Resultat;
    end
    else Met_En_Relief:='';

    if Approximation then begin
      X0:=Depart.X; Y0:=Depart.Y; Z0:=Depart.Z;
      X_Precedent:=X0*cosH+Y0*sinH;
      Yi:=-X0*sinH+Y0*cosH;
      Z_Precedent:=Z0-Yi*sinP;
      for K:=1 to Nb_Visee do begin
        X1:=Coordonnees^[K].X;
        Y1:=Coordonnees^[K].Y;
        Z1:=Coordonnees^[K].Z;
        Xi:=X1*cosH+Y1*sinH;
        Yi:=-X1*sinH+Y1*cosH;
        Zi:=Z1-Yi*sinP;
        if K=N_Visee then begin
          Le_Caneva.Pen.Mode:=pmNot;
          if Nouveau then begin
            Le_Caneva.Pen.Color:=clBlack;
            Le_Caneva.Brush.Color:=clBlack;
          end
          else begin
            Le_Caneva.Pen.Color:=clWhite;
            Le_Caneva.Brush.Color:=clWhite;
          end;
          Carre(X_Precedent,Z_Precedent);
          Carre(Xi,Zi);
          Le_Caneva.Pen.Mode:=pmCopy;
          if Etat=etNoir then begin
            if Nouveau then Le_Caneva.Pen.Color:=clRed
                       else Le_Caneva.Pen.Color:=clBlack;
            MoveToRel(X_Precedent*(0.5+Form1.Les_Options_Relief.Pourcent/200)+
                      Xi*(0.5-Form1.Les_Options_Relief.Pourcent/200),
                      Z_Precedent*(0.5+Form1.Les_Options_Relief.Pourcent/200)+
                      Zi*(0.5-Form1.Les_Options_Relief.Pourcent/200));
            LineToRel(X_Precedent*(0.5-Form1.Les_Options_Relief.Pourcent/200)+
                      Xi*(0.5+Form1.Les_Options_Relief.Pourcent/200),
                      Z_Precedent*(0.5-Form1.Les_Options_Relief.Pourcent/200)+
                      Zi*(0.5+Form1.Les_Options_Relief.Pourcent/200));
          end;
        end;
        X_Precedent:=Xi;
        Z_Precedent:=Zi;
      end;
    end;
  end;
end;


procedure TPerspective.Construit_Scene(Le_Support:TPanel;Etat : smallInt;var F : textFile);
var
  Morceau : TEntity;
  Face : TFace;
  light : TLight;
  La_Couleur : TColor;
  Les_Points : TList;
  I, J, K, N, Erreur : integer;
  S : string;
  Contrainte_Max, X0 : double;
  Haut_Gauche, Bas_Gauche, Haut_Droit, Bas_Droit,
  Haut_Gauche2, Bas_Gauche2, Haut_Droit2, Bas_Droit2,
  Prec_Haut_Gauche, Prec_Bas_Gauche, Prec_Haut_Droit, Prec_Bas_Droit, Normal : TPoint3D;
  A_Gauche, A_Droite, En_Haut, En_Bas, Angle : double;
  L_Autre : TGalerie;
  Horizontal, Plus, Moins : array[0..10000] of single;

  Une_Fiche : TFiche;
  Une_Galerie : TGalerie;
  Les_Mur : array[0..512] of TPoint3D;

  procedure Perpendiculaire(Un, Deux, Trois : TPoint3D;var N : TPoint3D);
  var
    V1, V2 : TPoint3D;
    D : single;
  begin
    V1.X:=Deux.X-Un.X; V1.Y:=Deux.Y-Un.Y; V1.Z:=Deux.Z-Un.Z;
    V2.X:=Trois.X-Un.X; V2.Y:=Trois.Y-Un.Y; V2.Z:=Trois.Z-Un.Z;
    N.X:=V1.Y*V2.Z-V1.Z*V2.Y;
    N.Y:=V1.Z*V2.X-V1.X*V2.Z;
    N.Z:=V1.X*V2.Y-V1.Y*V2.X;
    D:=sqrt(sqr(N.X)+sqr(N.Y)+sqr(N.Z));
    if D>1e-3 then begin
      N.X:=N.X/D;
      N.Y:=N.Y/D;
      N.Z:=N.Z/D;
    end;
  end;

  procedure Enveloppe(Une_Visee : TVisee;var HG,HD,BG,BD : TPoint3D);
  var
    Angle : double;
  begin
    with Une_Visee do begin
      case UAzimuth of
        deg : Angle:=Azimuth/180*PI;
        ndeg: Angle:=(Azimuth/180+1)*PI;
        gr  : Angle:=Azimuth/200*PI;
        ngr : Angle:=(Azimuth/200+1)*PI;
      end;
      if LH<0.01 then begin
        HD.X:=HD.X+A_Droite*cos(Angle)+En_Haut*sin(Angle);
        HD.Y:=HD.Y-A_Droite*sin(Angle)+En_Haut*cos(Angle);
        HG.X:=HG.X-A_Gauche*cos(Angle)+En_Haut*sin(Angle);
        HG.Y:=HG.Y+A_Gauche*sin(Angle)+En_Haut*cos(Angle);
        BD.X:=BD.X+A_Droite*cos(Angle)-En_Bas*sin(Angle);
        BD.Y:=BD.Y-A_Droite*sin(Angle)-En_Bas*cos(Angle);
        BG.X:=BG.X-A_Gauche*cos(Angle)-En_Bas*sin(Angle);
        BG.Y:=BG.Y+A_Gauche*sin(Angle)-En_Bas*cos(Angle);
      end
      else begin
        HG.Z:=HG.Z+En_Haut;
        HD.Z:=HD.Z+En_Haut;
        BG.Z:=BG.Z-En_Bas;
        BD.Z:=BD.Z-En_Bas;
        HD.X:=HD.X+A_Droite*cos(Angle);
        HD.Y:=HD.Y-A_Droite*sin(Angle);
        HG.X:=HG.X-A_Gauche*cos(Angle);
        HG.Y:=HG.Y+A_Gauche*sin(Angle);
        BD.X:=BD.X+A_Droite*cos(Angle);
        BD.Y:=BD.Y-A_Droite*sin(Angle);
        BG.X:=BG.X-A_Gauche*cos(Angle);
        BG.Y:=BG.Y+A_Gauche*sin(Angle);
      end;
    end;
  end;

  procedure Calcul_les_points(La_Fiche : TFiche; La_Galerie : TGalerie; J : integer;var Le_tour : array of TPoint3D);
  var
    K : integer;
  begin
    with La_Galerie do begin
      if (not Depart.Terminus) and (Visees[1].Longueur=0) then begin
        K:=0;
        while (K<Les_Points.Count) and not Depart.Identique(TPointTopo(Les_Points[K])) do inc(K);
        if K=Les_Points.Count then begin
          Les_Points.Add(Depart);
          Horizontal[K]:=0; Plus[K]:=0; Moins[K]:=0;
        end;

        if Form1.Les_Options_Relief.Squelette then begin
          Horizontal[K]:=Form1.Les_Options_Relief.Diametre_Mini/2;
          Plus[K]:=Form1.Les_Options_Relief.Diametre_Mini;
          Moins[K]:=0;
        end
        else begin
          La_Fiche.Les_Largeurs(J,1,A_Gauche,A_Droite);
          if A_Gauche<-0.5 then A_Gauche:=Form1.Les_Options_Relief.Diametre_Mini/2;
          if A_Droite<-0.5 then A_Droite:=Form1.Les_Options_Relief.Diametre_Mini/2;
          if A_Gauche>Horizontal[K] then Horizontal[K]:=A_Gauche;
          if A_Droite>Horizontal[K] then Horizontal[K]:=A_Droite;

          La_Fiche.Les_Hauteurs(J,1,En_Haut,En_Bas);
          if En_Haut<-0.5 then En_Haut:=Form1.Les_Options_Relief.Diametre_Mini;
          if En_Haut>Plus[K] then Plus[K]:=En_Haut;
          if En_Bas>Moins[K] then Moins[K]:=En_Bas;
        end;
      end;

      if not Arrivee.Terminus then begin
        K:=0;
        while (K<Les_Points.Count) and not Arrivee.Identique(TPointTopo(Les_Points[K])) do inc(K);
        if K=Les_Points.Count then begin
          Les_Points.Add(Arrivee);
          Horizontal[K]:=0; Plus[K]:=0; Moins[K]:=0;
        end;

        if Form1.Les_Options_Relief.Squelette then begin
          Horizontal[K]:=Form1.Les_Options_Relief.Diametre_Mini/2;
          Plus[K]:=Form1.Les_Options_Relief.Diametre_Mini;
          Moins[K]:=0;
        end
        else begin
          La_Fiche.Les_Largeurs(J,Nb_Visee,A_Gauche,A_Droite);
          if A_Gauche<-0.5 then A_Gauche:=Form1.Les_Options_Relief.Diametre_Mini/2;
          if A_Droite<-0.5 then A_Droite:=Form1.Les_Options_Relief.Diametre_Mini/2;
          if A_Gauche>Horizontal[K] then Horizontal[K]:=A_Gauche;
          if A_Droite>Horizontal[K] then Horizontal[K]:=A_Droite;

          La_Fiche.Les_Hauteurs(J,Nb_Visee,En_Haut,En_Bas);
          if En_Haut<-0.5 then En_Haut:=Form1.Les_Options_Relief.Diametre_Mini;
          if En_Haut>Plus[K] then Plus[K]:=En_Haut;
          if En_Bas>Moins[K] then Moins[K]:=En_Bas;
        end;

      end;

      Prec_Haut_Gauche.X:=Depart.X; Prec_Haut_Gauche.Y:=Depart.Y; Prec_Haut_Gauche.Z:=Depart.Z;
      Prec_Bas_Gauche:=Prec_Haut_Gauche;
      Prec_Haut_Droit:=Prec_Haut_Gauche; Prec_Bas_Droit:=Prec_Haut_Gauche;
      if Form1.Les_Options_Relief.Squelette then begin
        En_Haut:=Form1.Les_Options_Relief.Diametre_Mini;
        En_Bas:=0;
        A_Gauche:=Form1.Les_Options_Relief.Diametre_Mini/2;
        A_Droite:=Form1.Les_Options_Relief.Diametre_Mini/2;
      end;
      if (not Depart.Terminus) and (Visees[1].Longueur<>0) then begin
        if not Form1.Les_Options_Relief.Squelette then begin
          La_Fiche.Les_Hauteurs(J,0,En_Haut,En_Bas);
          if En_Haut<-0.5 then En_Haut:=Form1.Les_Options_Relief.Diametre_Mini;
          if En_Bas<-0.5 then En_Bas:=0;
          La_Fiche.Les_Largeurs(J,0,A_Gauche,A_Droite);
          if A_Gauche<-0.5 then A_Gauche:=Form1.Les_Options_Relief.Diametre_Mini/2;
          if A_Droite<-0.5 then A_Droite:=Form1.Les_Options_Relief.Diametre_Mini/2;
        end;
        Enveloppe(Visees[1],Prec_Haut_Gauche,Prec_Haut_Droit,Prec_Bas_Gauche,Prec_Bas_Droit);
      end;
      Le_tour[0]:=Prec_Haut_Gauche; Le_tour[1]:=Prec_Haut_Droit;
      Le_tour[3]:=Prec_Bas_Gauche; Le_tour[2]:=Prec_Bas_Droit;
      for K:=1 to Nb_Visee do begin

        if not Form1.Les_Options_Relief.Squelette then begin
          La_Fiche.Les_Largeurs(J,K,A_Gauche,A_Droite);
          if A_Gauche<-0.5 then A_Gauche:=Form1.Les_Options_Relief.Diametre_Mini/2;
          if A_Droite<-0.5 then A_Droite:=Form1.Les_Options_Relief.Diametre_Mini/2;

          La_Fiche.Les_Hauteurs(J,K,En_Haut,En_Bas);
          if En_Haut<-0.5 then En_Haut:=Form1.Les_Options_Relief.Diametre_Mini;
          if En_Bas<-0.5 then En_Bas:=0;
        end;
        Haut_Gauche:=Coordonnees^[K]; Bas_Gauche:=Coordonnees^[K];
        Haut_Droit:=Coordonnees^[K]; Bas_Droit:=Coordonnees^[K];
        Enveloppe(Visees[K],Haut_Gauche,Haut_Droit,Bas_Gauche,Bas_Droit);
        if K<Nb_Visee then begin
          Haut_Gauche2:=Coordonnees^[K]; Bas_Gauche2:=Coordonnees^[K];
          Haut_Droit2:=Coordonnees^[K]; Bas_Droit2:=Coordonnees^[K];
          Enveloppe(Visees[K+1],Haut_Gauche2,Haut_Droit2,Bas_Gauche2,Bas_Droit2);
          Haut_Gauche.X:=(Haut_Gauche.X+Haut_Gauche2.X)/2; Haut_Gauche.Y:=(Haut_Gauche.Y+Haut_Gauche2.Y)/2; Haut_Gauche.Z:=(Haut_Gauche.Z+Haut_Gauche2.Z)/2;
          Haut_Droit.X:=(Haut_Droit.X+Haut_Droit2.X)/2; Haut_Droit.Y:=(Haut_Droit.Y+Haut_Droit2.Y)/2; Haut_Droit.Z:=(Haut_Droit.Z+Haut_Droit2.Z)/2;
          Bas_Gauche.X:=(Bas_Gauche.X+Bas_Gauche2.X)/2; Bas_Gauche.Y:=(Bas_Gauche.Y+Bas_Gauche2.Y)/2; Bas_Gauche.Z:=(Bas_Gauche.Z+Bas_Gauche2.Z)/2;
          Bas_Droit.X:=(Bas_Droit.X+Bas_Droit2.X)/2; Bas_Droit.Y:=(Bas_Droit.Y+Bas_Droit2.Y)/2; Bas_Droit.Z:=(Bas_Droit.Z+Bas_Droit2.Z)/2;
        end;
        Le_tour[0+4*K]:=Haut_Gauche; Le_tour[1+4*K]:=Haut_Droit;
        Le_tour[3+4*K]:=Bas_Gauche; Le_tour[2+4*K]:=Bas_Droit;
        Prec_Haut_Gauche:=Haut_Gauche;
        Prec_Bas_Gauche:=Bas_Gauche;
        Prec_Haut_Droit:=Haut_Droit;
        Prec_Bas_Droit:=Bas_Droit;
      end;
{      if Arrivee.Terminus then begin
        Le_tour[0+4*(Nb_Visee+1)]:=Coordonnees^[Nb_Visee]; Le_tour[1+4*(Nb_Visee+1)]:=Coordonnees^[Nb_Visee];
        Le_tour[3+4*(Nb_Visee+1)]:=Coordonnees^[Nb_Visee]; Le_tour[2+4*(Nb_Visee+1)]:=Coordonnees^[Nb_Visee];
        Calcul_les_points:=Nb_Visee+1;
      end
      else}
    end;
  end;

begin
  Screen.Cursor:=crHourGlass;
  if Le_Support<>nil then begin
    Scene.Free;
    Scene:=TMaScene.Create;
    Scene.BackR:=Form1.Les_Options_Relief.Fond_Couleur mod 256;
    Scene.BackG:=(Form1.Les_Options_Relief.Fond_Couleur div 256) mod 256;
    Scene.BackB:=(Form1.Les_Options_Relief.Fond_Couleur div 65536) mod 256;
  end;
  if Affichable then begin
    Centre_X:=(PMax.X+PMin.X)/2;
    Centre_Y:=(PMax.Y+PMin.Y)/2;
    Centre_Z:=(PMax.Z+PMin.Z)/2;

    if Le_Support=nil then begin
      if Export_Vrml.Centre.Checked then writeln(F,'#Cybertopo ',Centre_X:6:2,' ',
         Centre_Z:6:2,' ',-Centre_Y:6:2)
      else writeln(F,'#Cybertopo 0 0 0');
      writeln(F);
    end;  

    Les_Points:=TList.Create;
    if Etat=etContrainte then Contrainte_Max:=Contrainte_Maximum(Form1.Massif.Liste_Courante);
    if (Le_Support=nil) and (Export_Vrml.Texture.Checked or (Etat=etNoir)) then begin
      writeln(F,'DEF AspectGalerie Appearance {');
      if Export_Vrml.Texture.Checked then writeln(F,'  texture ImageTexture { url ["',Export_Vrml.Texture_UNIX,'"] }');
      writeln(F,'  material Material {');
      write(F,'    diffuseColor ');
      if Export_Vrml.Texture.Checked then writeln(F,' 1 1 1') else writeln(F,' 0.75 0.75 0.75');
      with Export_Vrml.Ambiance do writeln(F,'    ambientIntensity ',Position/Max:1:2);
      writeln(F,'  }');
      writeln(F,'}');
      writeln(F);

      if Export_Vrml.Texture.Checked then begin
        writeln(F,' DEF MaTextureCoordinate TextureCoordinate {');
        writeln(F,' point [ 0 1, 0 0, 1 0, 1 1 ]');
        writeln(F,'}');
        writeln(F);
      end;

    end;
    for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
      for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
        if Approximation then begin
          case Etat of
            etContrainte : La_Couleur:=
             Degrade[round(sqrt(sqr(Contrainte_Plan)+sqr(Contrainte_Z))/Contrainte_Max*60)];
            etNoir, etRotation : La_Couleur:=Form1.Les_Options_Relief.Galerie_Couleur;
            etCouleur :  La_Couleur:=Couleur;
          end;
          if La_Couleur=clBlack then La_Couleur:=Form1.Les_Options_Relief.Galerie_Couleur;

          if Le_support<>nil then begin
            Morceau:=TEntity.Create;
            Morceau.SetColor(La_Couleur mod 256, (La_Couleur div 256) mod 256, (La_couleur div 65536) mod 256);
            Calcul_les_points(TFiche(Form1.Massif.Liste_Courante[I]),TGalerie(Liste_Galerie[J]),J,les_mur);
            with Morceau.AddFace do begin
              Perpendiculaire(Les_Mur[0],Les_Mur[1],Les_Mur[2],Normal);
              AddVertex(Les_Mur[0].X-Centre_X,Les_Mur[0].Z-Centre_Z,-Les_Mur[0].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
              AddVertex(Les_Mur[3].X-Centre_X,Les_Mur[3].Z-Centre_Z,-Les_Mur[3].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
              AddVertex(Les_Mur[2].X-Centre_X,Les_Mur[2].Z-Centre_Z,-Les_Mur[2].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
              AddVertex(Les_Mur[1].X-Centre_X,Les_Mur[1].Z-Centre_Z,-Les_Mur[1].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
            end;
            with Morceau.AddFace do begin
              Perpendiculaire(Les_Mur[4*Nb_Visee],Les_Mur[4*Nb_Visee+1],Les_Mur[4*Nb_Visee+2],Normal);
              AddVertex(Les_Mur[4*Nb_Visee].X-Centre_X,Les_Mur[4*Nb_Visee].Z-Centre_Z,-Les_Mur[4*Nb_Visee].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
              AddVertex(Les_Mur[4*Nb_Visee+1].X-Centre_X,Les_Mur[4*Nb_Visee+1].Z-Centre_Z,-Les_Mur[4*Nb_Visee+1].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
              AddVertex(Les_Mur[4*Nb_Visee+2].X-Centre_X,Les_Mur[4*Nb_Visee+2].Z-Centre_Z,-Les_Mur[4*Nb_Visee+2].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
              AddVertex(Les_Mur[4*Nb_Visee+3].X-Centre_X,Les_Mur[4*Nb_Visee+3].Z-Centre_Z,-Les_Mur[4*Nb_Visee+3].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
            end;
            for K:=0 to Nb_Visee-1 do begin
              with Morceau.AddFace do begin
                Perpendiculaire(Les_Mur[4*K],Les_Mur[4*K+1],Les_Mur[4*K+5],Normal);
                AddVertex(Les_Mur[4*K].X-Centre_X,Les_Mur[4*K].Z-Centre_Z,-Les_Mur[4*K].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
                AddVertex(Les_Mur[4*K+1].X-Centre_X,Les_Mur[4*K+1].Z-Centre_Z,-Les_Mur[4*K+1].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
                AddVertex(Les_Mur[4*K+5].X-Centre_X,Les_Mur[4*K+5].Z-Centre_Z,-Les_Mur[4*K+5].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
                AddVertex(Les_Mur[4*K+4].X-Centre_X,Les_Mur[4*K+4].Z-Centre_Z,-Les_Mur[4*K+4].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
              end;
              with Morceau.AddFace do begin
                Perpendiculaire(Les_Mur[4*K+1],Les_Mur[4*K+2],Les_Mur[4*K+6],Normal);
                AddVertex(Les_Mur[4*K+1].X-Centre_X,Les_Mur[4*K+1].Z-Centre_Z,-Les_Mur[4*K+1].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
                AddVertex(Les_Mur[4*K+2].X-Centre_X,Les_Mur[4*K+2].Z-Centre_Z,-Les_Mur[4*K+2].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
                AddVertex(Les_Mur[4*K+6].X-Centre_X,Les_Mur[4*K+6].Z-Centre_Z,-Les_Mur[4*K+6].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
                AddVertex(Les_Mur[4*K+5].X-Centre_X,Les_Mur[4*K+5].Z-Centre_Z,-Les_Mur[4*K+5].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
              end;
              with Morceau.AddFace do begin
                Perpendiculaire(Les_Mur[4*K+2],Les_Mur[4*K+3],Les_Mur[4*K+7],Normal);
                AddVertex(Les_Mur[4*K+2].X-Centre_X,Les_Mur[4*K+2].Z-Centre_Z,-Les_Mur[4*K+2].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
                AddVertex(Les_Mur[4*K+3].X-Centre_X,Les_Mur[4*K+3].Z-Centre_Z,-Les_Mur[4*K+3].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
                AddVertex(Les_Mur[4*K+7].X-Centre_X,Les_Mur[4*K+7].Z-Centre_Z,-Les_Mur[4*K+7].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
                AddVertex(Les_Mur[4*K+6].X-Centre_X,Les_Mur[4*K+6].Z-Centre_Z,-Les_Mur[4*K+6].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
              end;
              with Morceau.AddFace do begin
                Perpendiculaire(Les_Mur[4*K+3],Les_Mur[4*K],Les_Mur[4*K+4],Normal);
                AddVertex(Les_Mur[4*K+3].X-Centre_X,Les_Mur[4*K+3].Z-Centre_Z,-Les_Mur[4*K+3].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
                AddVertex(Les_Mur[4*K].X-Centre_X,Les_Mur[4*K].Z-Centre_Z,-Les_Mur[4*K].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
                AddVertex(Les_Mur[4*K+4].X-Centre_X,Les_Mur[4*K+4].Z-Centre_Z,-Les_Mur[4*K+4].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
                AddVertex(Les_Mur[4*K+7].X-Centre_X,Les_Mur[4*K+7].Z-Centre_Z,-Les_Mur[4*K+7].Y+Centre_Y,Normal.X,Normal.Z,-Normal.Y);
              end;
            end;
            Scene.Entities.add(Morceau);
          end
          else begin
            writeln(F,'Shape {');
            writeln(F,'  geometry IndexedFaceSet {');
            if Export_VRML.Lissage.Checked then writeln(F,'    creaseAngle 6.28');
            writeln(F,'    coord Coordinate {');
            writeln(F,'      point [ ');
            Calcul_les_points(TFiche(Form1.Massif.Liste_Courante[I]),TGalerie(Liste_Galerie[J]),J,les_mur);
            for K:=0 to 4*Nb_Visee+3 do begin
              if Export_VRML.Centre.Checked
                then writeln(F,'        ',Les_mur[K].X-Centre_X:6:2,' ',Les_mur[K].Z-Centre_Z:6:2,' ',Centre_Y-Les_mur[K].Y:6:2,',')
                else writeln(F,'        ',Les_mur[K].X:6:2,' ',Les_mur[K].Z:6:2,' ',-Les_mur[K].Y:6:2,',');
            end;
            writeln(F,'      ]');
            writeln(F,'    }');
            writeln(F,'    coordIndex	[');
            writeln(F,'      0, 3, 2, 1, -1');
            writeln(F,'      ',4*Nb_visee,', ',4*Nb_visee+1,', ',4*Nb_visee+2,', ',4*Nb_visee+3,', -1');
            for K:=0 to Nb_visee-1 do begin
              writeln(F,'      ',4*K,', ',4*K+1,', ',4*K+5,', -1');
              writeln(F,'      ',4*K,', ',4*K+5,', ',4*K+4,', -1');

              writeln(F,'      ',4*K+1,', ',4*K+2,', ',4*K+6,', -1');
              writeln(F,'      ',4*K+1,', ',4*K+6,', ',4*K+5,', -1');

              writeln(F,'      ',4*K+2,', ',4*K+3,', ',4*K+7,', -1');
              writeln(F,'      ',4*K+2,', ',4*K+7,', ',4*K+6,', -1');

              writeln(F,'      ',4*K+3,', ',4*K,', ',4*K+4,', -1');
              writeln(F,'      ',4*K+3,', ',4*K+4,', ',4*K+7,', -1');
            end;
            writeln(F,'    ]');

            if (Le_Support=nil) and (Export_Vrml.Texture.Checked) then begin
              writeln(F,'    texCoord	USE MaTextureCoordinate');

              writeln(F,'    texCoordIndex	[');
              writeln(F,'      0, 1, 2, 3, -1');
              writeln(F,'      0, 1, 2, 3, -1');
              for K:=0 to 4*Nb_visee-1 do begin
                writeln(F,'      0, 1, 2, -1');
                writeln(F,'      0, 2, 3, -1');
              end;
              writeln(F,'    ]');
            end;


            writeln(F,'  }');
            if (Le_Support=nil) and (Export_Vrml.Texture.Checked or (Etat=etNoir)) then begin
              writeln(F,'  appearance USE AspectGalerie');
            end
            else begin
              writeln(F,'  appearance Appearance {');
              writeln(F,'    material Material {');
              write(F,'      diffuseColor ');
              write(F,(La_Couleur mod 256)/256:1:3,' ');
              write(F,((La_Couleur div 256) mod 256)/256:1:3,' ');
              writeln(F,((La_Couleur div 65536) mod 256)/256:1:3,' ');
              with Export_Vrml.Ambiance do writeln(F,'      ambientIntensity ',Position/Max:1:2);
              writeln(F,'    }');
              writeln(F,'  }');
            end; 
            writeln(F,'}');
            writeln(F);

          end;
        end
      end;
    end;
    if Form1.Les_Options_Relief.Les_Cylindres then begin
      if (Le_Support=nil) and not Export_Vrml.Texture.Checked then begin
        writeln(F,'DEF AspectCylindre Appearance {');
        writeln(F,'  material Material {');
        write(F,'    diffuseColor ');
        write(F,(Form1.Les_Options_Relief.Cylindre_Couleur mod 256)/256:1:3,' ');
        write(F,((Form1.Les_Options_Relief.Cylindre_Couleur div 256) mod 256)/256:1:3,' ');
        writeln(F,((Form1.Les_Options_Relief.Cylindre_Couleur div 65536) mod 256)/256:1:3,' ');
        with Export_Vrml.Ambiance do writeln(F,'    ambientIntensity ',Position/Max:1:2);
        writeln(F,'  }');
        writeln(F,'}');
      end;
      for I:=0 to Les_Points.Count-1 do with TPointTopo(Les_Points[I]) do begin
        Plus[I]:=Plus[I]+0.01; Moins[I]:=Moins[I]+0.01;
        if Le_Support<>nil then begin
          Morceau:=TEntity.Create;
          Morceau.SetColor(Form1.Les_Options_Relief.Cylindre_Couleur mod 256,
             (Form1.Les_Options_Relief.Cylindre_Couleur div 256) mod 256,
             (Form1.Les_Options_Relief.Cylindre_Couleur div 65536) mod 256);
          for J:=0 to Form1.Les_Options_Relief.Nb_Faces-1 do begin
            with Morceau.addFace do begin
              AddVertex(Horizontal[I]*cos(J/Form1.Les_Options_Relief.Nb_Faces*2*PI)+X-Centre_X,
                        Plus[I]+Z-Centre_Z,
                        -Horizontal[I]*sin(J/Form1.Les_Options_Relief.Nb_Faces*2*PI)+Centre_Y-Y,
                        cos(J/Form1.Les_Options_Relief.Nb_Faces*2*PI),0,-sin(J/Form1.Les_Options_Relief.Nb_Faces*2*PI));
              AddVertex(Horizontal[I]*cos(J/Form1.Les_Options_Relief.Nb_Faces*2*PI)+X-Centre_X,
                        -Moins[I]+Z-Centre_Z,
                        -Horizontal[I]*sin(J/Form1.Les_Options_Relief.Nb_Faces*2*PI)+Centre_Y-Y,
                        cos(J/Form1.Les_Options_Relief.Nb_Faces*2*PI),0,-sin(J/Form1.Les_Options_Relief.Nb_Faces*2*PI));
              AddVertex(Horizontal[I]*cos((J+1)/Form1.Les_Options_Relief.Nb_Faces*2*PI)+X-Centre_X,
                        -Moins[I]+Z-Centre_Z,
                        -Horizontal[I]*sin((J+1)/Form1.Les_Options_Relief.Nb_Faces*2*PI)+Centre_Y-Y,
                        cos((J+1)/Form1.Les_Options_Relief.Nb_Faces*2*PI),0,-sin((J+1)/Form1.Les_Options_Relief.Nb_Faces*2*PI));
              AddVertex(Horizontal[I]*cos((J+1)/Form1.Les_Options_Relief.Nb_Faces*2*PI)-Centre_X+X,
                        Plus[I]+Z-Centre_Z,
                        -Horizontal[I]*sin((J+1)/Form1.Les_Options_Relief.Nb_Faces*2*PI)+Centre_Y-Y,
                        cos((J+1)/Form1.Les_Options_Relief.Nb_Faces*2*PI),0,-sin((J+1)/Form1.Les_Options_Relief.Nb_Faces*2*PI));
            end;
            with Morceau.addFace do begin
              AddVertex(Horizontal[I]*cos(J/Form1.Les_Options_Relief.Nb_Faces*2*PI)+X-Centre_X,
                        Plus[I]+Z-Centre_Z,
                        -Horizontal[I]*sin(J/Form1.Les_Options_Relief.Nb_Faces*2*PI)+Centre_Y-Y,
                        0,1,0);
              AddVertex(Horizontal[I]*cos((J+1)/Form1.Les_Options_Relief.Nb_Faces*2*PI)+X-Centre_X,
                        Plus[I]+Z-Centre_Z,
                        -Horizontal[I]*sin((J+1)/Form1.Les_Options_Relief.Nb_Faces*2*PI)+Centre_Y-Y,
                        0,1,0);
              AddVertex(X-Centre_X,Plus[I]+Z-Centre_Z,Centre_Y-Y,0,1,0);
            end;
            with Morceau.addFace do begin
              AddVertex(Horizontal[I]*cos(J/Form1.Les_Options_Relief.Nb_Faces*2*PI)+X-Centre_X,
                        -Moins[I]+Z-Centre_Z,
                        -Horizontal[I]*sin(J/Form1.Les_Options_Relief.Nb_Faces*2*PI)+Centre_Y-Y,
                        0,-1,0);
              AddVertex(X-Centre_X,-Moins[I]+Z-Centre_Z,Centre_Y-Y,0,-1,0);
              AddVertex(Horizontal[I]*cos((J+1)/Form1.Les_Options_Relief.Nb_Faces*2*PI)+X-Centre_X,
                        -Moins[I]+Z-Centre_Z,
                        -Horizontal[I]*sin((J+1)/Form1.Les_Options_Relief.Nb_Faces*2*PI)+Centre_Y-Y,
                        0,-1,0);
            end;
          end;
          Scene.Entities.add(Morceau);
        end
        else begin
          writeln(F,'Transform { ');
          if Export_VRML.Centre.Checked
            then writeln(F,'  translation ',X-Centre_X:6:2,' ',(Z-Centre_Z+(Plus[I]-Moins[I])/2):6:2,' ',Centre_Y-Y:6:2)
            else writeln(F,'  translation ',X:6:2,' ',(Z+(Plus[I]-Moins[I])/2):6:2,' ',-Y:6:2);
          writeln(F,'  children [');
          writeln(F,'    Shape {');
          writeln(F,'      geometry Cylinder {');
          writeln(F,'        radius ',Horizontal[I]:2:2);
          writeln(F,'        height ',(Plus[I]+Moins[I]):2:2);
          writeln(F,'      }');
          if Export_Vrml.Texture.Checked then writeln(F,'      appearance USE AspectGalerie')
          else writeln(F,'      appearance USE AspectCylindre');
          writeln(F,'    }');
          writeln(F,'  ]');
          writeln(F,'}');
        end;
      end;
    end;
    Les_Points.Free;
    Les_Points:=nil;
  end;


  if Le_Support<>nil then begin
    if Premiere_Fois then begin
      Premiere_Fois:=false;
      Vue_Azimuth:=360-Rotation_Horizontale;
      Vue_Pente:=Elevation;
      X0:=abs(PMin.X-Pmax.X)+abs(PMin.Y-Pmax.Y);
      with Vue_Position do begin
        X:=-X0*cos(Vue_Pente/180*PI)*sin(Vue_Azimuth/180*PI);
        Y:=-X0*sin(Vue_Pente/180*PI);
        Z:=X0*cos(Vue_Pente/180*PI)*cos(Vue_Azimuth/180*PI);
      end;
    end;

    light:=Tlight.create(1);
    light.LightType:=CLSpot;
    light.CutOffAngle:=90;
    Light.SpotExponent:=0;

    Light.Fdiffuse[0]:=1;
    Light.Fdiffuse[1]:=1;
    Light.Fdiffuse[2]:=1;
{    Light.Fdiffuse[0]:=0.5;
    Light.Fdiffuse[1]:=0.5;
    Light.Fdiffuse[2]:=0.5; }

    Light.Fspecular[0]:=0;
    Light.Fspecular[1]:=0;
    Light.Fspecular[2]:=0;

    Light.Fambient[0]:=0.5;
    Light.Fambient[1]:=0.5;
    Light.Fambient[2]:=0.5;

    Light.SetOrientation(0,-1,0);
    Light.Source.SetPosition(0,(Pmax.Z-PMin.Z)+1000,0);

  //  light.attenuationType:=CLconstant;
  //  light.attenuationType:=CLlinear;
  //  light.attenuationType:=CLquadratic;

//    Scene.Lights.add(light);
    light:=Tlight.create(2);
    Scene.Lights.add(light);

    Scene.DistFar:=1e6;
    Scene.InitRC(Le_Support.Handle);
    Scene.UpDateArea(Le_Support.Width,Le_Support.Height);
  //  Form1.StatusBar.Panels[1].Text:=Form1.StatusBar.Panels[1].Text+'-';
    Redessine_Scene;
    Screen.Cursor:=crDefault;
  end;
end;

procedure TPerspective.Redessine_Scene;
var
  XP, YP, ZP : single;
begin
//  Form1.StatusBar.Panels[1].Text:=Form1.StatusBar.Panels[1].Text+'+';
  with Vue_Position do begin
    XP:=X*cos(Vue_Azimuth/180*PI)+Z*sin(Vue_Azimuth/180*PI);
    YP:=-X*sin(Vue_Azimuth/180*PI)*sin(Vue_Pente/180*PI)+Y*cos(Vue_Pente/180*PI)+Z*cos(Vue_Azimuth/180*PI)*sin(Vue_Pente/180*PI);
    ZP:=-X*sin(Vue_Azimuth/180*PI)-Y*cos(Vue_Azimuth/180*PI)*sin(Vue_Pente/180*PI)+Z*cos(Vue_Azimuth/180*PI)*cos(Vue_Pente/180*PI);
  end;
  Scene.move(-XP,-YP,-ZP);
  Scene.Rotate(-Vue_Pente,Vue_Azimuth,0);
  Scene.Redraw;
end;

procedure TPerspective.Export_Svg(Fichier_SVG: TFileName; Illustrator, AllLayers : boolean);
const
  coVisee = 0;
  coStation = 1;
  coCarrefour = 2;
var
  F : TextFile;
  I, J, K, Couche : integer;
  Poids_1, Poids_2, Delta : double;
  X0, Y0, Z0, Xi, Yi, Zi, X_Precedent, Y_Precedent, Z_Precedent : double;
  X1, Y1, Z1 : double;
  S, sLayer : string;
begin
  inherited;
  AssignFile(F,Fichier_SVG);
  try
    rewrite(F);
    EnteteSvg(F,round((XMax-XMin)*1000/Echelle),round((YMax-YMin)*1000/Echelle),Illustrator);
    writeln(F,Format('<title>%s</title>',[ExtractFileName(Form1.Massif.Fichier)]));
    writeln(F,'<desc>Perspective</desc>');

    if Affichable and (Form1.Les_Options_Relief<>nil) then begin
      Poids_1:=Form1.Les_Options_Relief.Pourcent/200+0.5;
      Poids_2:=1-Poids_1;
      Delta:=Form1.Les_Options_Relief.Station_police.Size*25.4/72/2;
      if Illustrator then
        sLayer:=' i:layer="yes"'
      else
        sLayer:='';
      for Couche:=0 to 2 do
      begin
        case Couche of
          coVisee : writeln(F,Format('<g id="visees"%s style="stroke-width:%fmm; stroke:black;">',
                                     [sLayer,Form1.Les_Options_Relief.Visee_Epaisseur/10]));
          coStation : writeln(F,Format('<g id="stations"%s style="stroke:black; fill:black;">',
                                     [sLayer]));
          coCarrefour : writeln(F,Format('<g id="carrefours"%s style="font-size:%dpt; font-family:%s;">',
                                     [sLayer,
                                     Form1.Les_Options_Relief.Station_police.Size,
                                     Form1.Les_Options_Relief.Station_police.Name]));
        end;
        for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
          writeln(F,'<g>'); // un groupe par fiche
          for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
            if Approximation then begin
              writeln(F,'<g>'); // un groupe par galerie
              X0:=Depart.X; Y0:=Depart.Y; Z0:=Depart.Z;
              X_Precedent:=X0*cosH+Y0*sinH;
              Y_Precedent:=-X0*sinH+Y0*cosH;
              Z_Precedent:=Z0-Y_Precedent*sinP;
              if ((Form1.Les_Options_Relief.Diametre<>0) or AllLayers) and (Couche=coStation) then
              begin
                writeln(F,Format('<circle cx="%fmm" cy="%fmm" r="%fmm"/>',
                                  [(X_Precedent-XMin)/Echelle*1000,
                                   (YMax-Z_Precedent)/Echelle*1000,
                                   Form1.Les_Options_Relief.Diametre/20]));
              end;
              if (Couche=coCarrefour) and ((Form1.Les_Options_Relief.Carrefour>0) or AllLayers)
                  and not Depart.Terminus then begin
                Depart.Nom.lit(S);
                if Form1.Les_Options_Relief.Carrefour=2 then begin
                  S:=S+'('+IntTostr(Depart.Indice)+')';
                end;
                writeln(F,Format('<text x="%fmm" y="%fmm">%s</text>',
                                 [(X_Precedent-XMin)/Echelle*1000+Delta,
                                  (Ymax-Z_Precedent)/Echelle*1000+Delta,S]));
              end;

              for K:=1 to Nb_Visee do begin
                X0:=Coordonnees^[K].X;
                Y0:=Coordonnees^[K].Y;
                Z0:=Coordonnees^[K].Z;
                Xi:=X0*cosH+Y0*sinH;
                Yi:=-X0*sinH+Y0*cosH;
                Zi:=Z0-Yi*sinP;
                if Couche=coVisee then
                  writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                                   [(X_Precedent*Poids_1+Xi*Poids_2-XMin)/Echelle*1000,
                                    (YMax-(Z_Precedent*Poids_1+Zi*Poids_2))/Echelle*1000,
                                    (X_Precedent*Poids_2+Xi*Poids_1-XMin)/Echelle*1000,
                                    (YMax-(Z_Precedent*Poids_2+Zi*Poids_1))/Echelle*1000]));
                if ((Form1.Les_Options_Relief.Diametre<>0) or AllLayers) and (Couche=coStation) then
                  writeln(F,Format('<circle cx="%fmm" cy="%fmm" r="%fmm"/>',
                                    [(Xi-XMin)/Echelle*1000,
                                     (YMax-Zi)/Echelle*1000,
                                     Form1.Les_Options_Relief.Diametre/20]));
                X_Precedent:=Xi; Z_Precedent:=Zi;
              end;

              if Couche=coCarrefour then
              if ((Form1.Les_Options_Relief.Carrefour>0) or AllLayers) and not Arrivee.Terminus then begin
                Arrivee.Nom.lit(S);
                if Form1.Les_Options_Relief.Carrefour=2 then begin
                  S:=S+'('+IntTostr(Arrivee.Indice)+')';
                end;
                writeln(F,Format('<text x="%fmm" y="%fmm">%s</text>',
                                 [(X_Precedent-XMin)/Echelle*1000+Delta,
                                  (Ymax-Z_Precedent)/Echelle*1000+Delta,S]));
              end;
              writeln(F,'</g>'); // un groupe par galerie
            end;
          end;
          writeln(F,'</g>'); // un groupe par fiche
        end;
        writeln(F,'</g>');
      end;
      if Form1.Les_Options_Relief.Nord_Affiche or AllLayers then
      with Form1.Les_Options_Relief do begin
        writeln(F,Format('<g id="nord"%s>',[sLayer]));
        Delta:=Nord_police.Size*25.4/72/2;
        X0:=Nord_X*1000; Y0:=Nord_Y*1000; Z0:=Nord_Z;
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        writeln(F,Format('<g style="stroke-width:%fmm; stroke:black;">',[Nord_Epaisseur/10])); // on fait un groupe pour runir les 2 trais de la flche
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        if NG then begin
          Y0:=Y0+Nord_Longueur/1000*Echelle*cos(Form1.Massif.Angle_Geodesique/180*PI);
          X0:=X0-Nord_Longueur/1000*Echelle*sin(Form1.Massif.Angle_Geodesique/180*PI);
          Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
          writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
          write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
          Y0:=Y0-Nord_Longueur/1000*Echelle*
           (cos(Form1.Massif.Angle_Geodesique/180*PI)/2-sin(Form1.Massif.Angle_Geodesique/180*PI)/10);
          X0:=X0+Nord_Longueur/1000*Echelle*
           (sin(Form1.Massif.Angle_Geodesique/180*PI)/2+cos(Form1.Massif.Angle_Geodesique/180*PI)/10);
          Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
          writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        end
        else begin
          Y0:=Y0+Nord_Longueur/1000*Echelle;
          Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
          writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
          write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
          Y0:=Y0-Nord_Longueur/1000*Echelle/2; X0:=X0+Nord_Longueur/1000*Echelle/10;
          Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
          writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        end;
        writeln(F,'</g>');
        writeln(F,Format('<g style="font-size:%dpt; font-family:%s;">',[Nord_police.Size,Nord_police.Name]));
        writeln(F,Format('<text x="%fmm" y="%fmm">%s</text>',
                         [(Xi-XMin)/Echelle*1000+Delta,
                          (Ymax-Zi)/Echelle*1000+Delta,Texte_Nord]));
        writeln(F,'</g>');
        writeln(F,'</g>');
      end;

      if Form1.Les_Options_Relief.Repere_Affiche or AllLayers then
      with Form1.Les_Options_Relief do begin
        writeln(F,Format('<g id="repere"%s style="stroke-width:%fmm; stroke:black;">',
                         [sLayer,Repere_Epaisseur/10]));
        X0:=Repere_X*1000; Y0:=Repere_Y*1000; Z0:=Repere_Z;
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        if Inverse_X then X0:=X0-Repere_Longueur else X0:=X0+Repere_Longueur;
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        X0:=Repere_X*1000; Y0:=Repere_Y*1000; Z0:=Repere_Z;
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        if Inverse_Y then Y0:=Y0-Repere_Longueur else Y0:=Y0+Repere_Longueur;
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        X0:=Repere_X*1000; Y0:=Repere_Y*1000; Z0:=Repere_Z;
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        if Inverse_Z then Z0:=Z0-Repere_Longueur else Z0:=Z0+Repere_Longueur;
        Xi:=X0*cosH+Y0*sinH; Yi:=-X0*sinH+Y0*cosH; Zi:=Z0-Yi*sinP;
        writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        writeln(F,'</g>');
      end;

      if Form1.Les_Options_Relief.Rectangle_Affiche or AllLayers then
      with Form1.Les_Options_Relief do begin
        writeln(F,Format('<g id="quadrillatere"%s style="stroke-width:%fmm; stroke:black;">',
                         [sLayer,Rectangle_Epaisseur/10]));
        X1:=PMin.X; Y1:=PMin.Y; Z1:=PMin.Z;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        X1:=PMax.X;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        Y1:=PMax.Y;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        X1:=PMin.X;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        Y1:=PMin.Y;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        Z1:=PMax.Z;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        X1:=PMax.X;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        Y1:=PMax.Y;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        X1:=PMin.X;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        Y1:=PMin.Y;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        X1:=PMax.X;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        Z1:=Pmin.Z;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        Y1:=PMax.Y;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        Z1:=Pmax.Z;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        X1:=Pmin.X;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        Z1:=Pmin.Z;
        Xi:=X1*cosH+Y1*sinH; Yi:=-X1*sinH+Y1*cosH; Zi:=Z1-Yi*sinP;
        writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Zi)/Echelle*1000]));
        writeln(F,'</g>');
      end;
    end;
    writeln(F,'</svg>');
  finally
    CloseFile(F);
  end;
end;


end.
