unit Graphes;

interface

uses Fichetpo, Classes, ExtCtrls, Printers, Windows, SysUtils, Options,
     Forms, Controls, Graphics, Couleurs, Dialogs;

const

  Entre_Yeux=0.07; (*distance entre les yeux de l'observateur en m*)
  Observateur=0.5; (*distance de l'observateur  l'cran en m*)
  etNoir = 0;
  etRotation = 1;
  etCouleur = 2;
  etContrainte = 3;

type

  TGraphique = class
    Echelle, PageX, PageY, MX, MY, Supl_X, Supl_Y : integer;
    Xmin, XMax, YMin, YMax, Marge_X, Marge_Y, La_Resolution,
    Resolution_Imprimante, Correction : double;
    Affichable : boolean;
    PixelX, PixelY : longint;
    Support, Le_Caneva : TCanvas;
    procedure Dessine(Pixel_X, Pixel_Y : longint; Resolution, Rapport : double;
                      Canevas : TImage; Etat : smallInt); virtual; abstract;
    procedure Imprime(Page_X, Page_Y, Sup_X, Sup_Y : integer; Etat : smallInt;
                      Fond : TCanvas; Reduc : double); virtual; abstract;
    procedure Gabarit(var Pixel_Hori, Pixel_Verti : longint; Resolution, Rapport : double; Canevas : TImage); virtual; abstract;
    procedure Gabarit_Imprimante(var Page_X, Page_Y : integer; MargeX, MargeY : integer); virtual; abstract;
    function Trouve(X, Y : longint;var N_Fiche, N_Galerie, N_Visee : integer) : double; virtual; abstract;
    procedure Isoler(X, Y, Rayon2 : longint); virtual; abstract;
    function Met_En_Relief(Nouveau : boolean; N_Fiche, N_Galerie, N_Visee : integer;
                      Etat : smallInt) : string; virtual; abstract;
    function Bulle(N_Fiche, N_Galerie, N_Visee : integer) : string; virtual; abstract;
    function Mauvais_Rebouclage(var N_Fiche, N_Galerie, N_Visee : integer) : boolean; //virtual; abstract;
    function Contrainte_Maximum(var Les_Fiches : TList) : single;
  end;

  TProjection = class(TGraphique)
    procedure MoveToRel(X, Y : double);
    procedure LineToRel(X, Y : double);
    procedure TextRel(X, Y : double;S : string);
    procedure MoveToImp(X, Y : double);
    procedure LineToImp(X, Y : double);
    procedure TextImp(X, Y : double;S : string);
    procedure Cercle(X_Centre, Y_Centre : double; Diametre : integer);
    procedure Carre(X_Centre, Y_Centre : double);
    procedure Cercle_Imp(X_Centre, Y_Centre : double; Diametre : integer);
    function Bulle(N_Fiche, N_Galerie, N_Visee : integer) : string; override;
    procedure Export_Svg(Fichier_SVG : TFileName; Illustrator, AllLayers : boolean); virtual; abstract;
  end;

  TPlan = class (TProjection)
    CosH, SinH, Rotation_Horizontale : double;
    PMin, PMax : TPoint3D;
    Deja_Vu : boolean;
    constructor Create;
    procedure Dessine(Pixel_X, Pixel_Y : longint; Resolution, Rapport : double;
                      Canevas : TImage; Etat : smallInt); override;
    procedure Imprime(Page_X, Page_Y, Sup_X, Sup_Y : integer; Etat : smallInt;
                              Fond : TCanvas; Reduc : double); override;
    procedure Gabarit(var Pixel_Hori, Pixel_Verti  : longint; Resolution, Rapport : double; Canevas : TImage); override;
    procedure Gabarit_Imprimante(var Page_X, Page_Y : integer; MargeX, MargeY : integer); override;
    function Trouve(X, Y : longint;var N_Fiche, N_Galerie, N_Visee : integer) : double; override;
    procedure Isoler(X, Y, Rayon2 : longint); override;
    function Met_En_Relief(Nouveau : boolean; N_Fiche, N_Galerie, N_Visee : integer; Etat : smallInt) : string; override;
    function Essai(Hori : double) : string;
    procedure Definitif(Hori : double);
    procedure Export_Svg(Fichier_SVG : TFileName; Illustrator, AllLayers : boolean); override;
  end;


implementation

uses
  Unit1, GrapheCo, GrapheRe, GrapheTa, svgUtils;

function Tronc(X : double) : integer;
begin
  if X<0 then Tronc:=trunc(X)-1
  else Tronc:=trunc(X);
end;

(************************ TGraphique **********************)

function TGraphique.Contrainte_Maximum(var Les_Fiches : TList) : single;
var
  I, J : integer;
  Contrainte, Xi : single;
begin
  Contrainte:=0;
  for I:=0 to les_Fiches.Count-1 do with TFiche(Les_Fiches[I]) do begin
    for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
      if Approximation then begin
        if self is TPlan then Xi:=Contrainte_Plan;
        if En_coupe and (self is TCoupe) then Xi:=Contrainte_Z;
        if (self is TPerspective) or (self is TTableau) then Xi:=sqrt(sqr(Contrainte_Plan)+sqr(Contrainte_Z));
        if Xi>Contrainte then Contrainte:=Xi;
      end;
    end;
  end;
  if Contrainte=0 then Contrainte_Maximum:=1 else Contrainte_Maximum:=Contrainte;
end;

function TGraphique.Mauvais_Rebouclage(var N_Fiche, N_Galerie, N_Visee : integer) : boolean;
var
  Contrainte_Max, Xi : double;
  I, J, La_Fiche, La_Galerie : integer;
begin
  Contrainte_Max:=0;
  for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
    for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
      if Approximation then begin
        if self is TPlan then Xi:=Contrainte_Plan;
        if En_coupe and (self is TCoupe) then Xi:=Contrainte_Z;
        if self is TPerspective then Xi:=sqrt(sqr(Contrainte_Plan)+sqr(Contrainte_Z));
        if Xi>Contrainte_Max then begin
          Contrainte_Max:=Xi;
          La_Fiche:=I;
          La_Galerie:=J;
        end;
      end;
    end;
  end;
  if Contrainte_Max=0 then Mauvais_Rebouclage:=false
  else begin
    Mauvais_Rebouclage:=true;
    N_Fiche:=La_Fiche;
    N_Galerie:=La_Galerie;
    N_Visee:=1;
  end;
end;

(************************ TProjection **********************)

procedure TProjection.MoveToRel(X, Y : double);
var
  Facteur, SensX, SensY : double;
begin
  Facteur:=1000/Echelle*La_Resolution;
  SensX:=(X-Xmin)*Facteur-PixelX;
  if abs(SensX)<10000 then begin
    SensY:=(YMax-Y)*Facteur-PixelY;
    if abs(SensY)<10000 then
      Le_Caneva.MoveTo(round(SensX),round(SensY));
  end;
end;

procedure TProjection.LineToRel(X, Y : double);
var
  Facteur, SensX, SensY : double;
begin
  Facteur:=1000/Echelle*La_Resolution;
  SensX:=(X-Xmin)*Facteur-PixelX;
  if abs(SensX)<10000 then begin
    SensY:=(YMax-Y)*Facteur-PixelY;
    if abs(SensY)<10000 then
      Le_Caneva.LineTo(round(SensX),round(SensY));
  end;
end;

procedure TProjection.TextRel(X, Y : double;S : string);
begin
  if (abs((X-Xmin)*1000/Echelle*La_Resolution-PixelX)<10000) and
     (abs((Ymax-Y)*1000/Echelle*La_Resolution-PixelY)<10000) then
  Le_Caneva.TextOut(round((X-Xmin)*1000/Echelle*La_Resolution-PixelX),
                          round((Ymax-Y)*1000/Echelle*La_Resolution-PixelY),S);
end;

procedure TProjection.Cercle(X_Centre, Y_Centre : double; Diametre : integer);
begin
  if (abs((X_Centre-Xmin)*1000/Echelle*La_Resolution-PixelX)<10000) and
     (abs((Ymax-Y_Centre)*1000/Echelle*La_Resolution-PixelY)<10000) then
  Le_Caneva.Ellipse(round(((X_Centre-Xmin)*1000/Echelle-Diametre/20)*La_Resolution-PixelX),
                          round(((Ymax-Y_Centre)*1000/Echelle-Diametre/20)*La_Resolution-PixelY),
                          round(((X_Centre-Xmin)*1000/Echelle+Diametre/20)*La_Resolution-PixelX),
                          round(((Ymax-Y_Centre)*1000/Echelle+Diametre/20)*La_Resolution-PixelY));
end;

procedure TProjection.Carre(X_Centre, Y_Centre : double);
begin
  if (abs((X_Centre-Xmin)*1000/Echelle*La_Resolution-PixelX)<10000) and
     (abs((Ymax-Y_Centre)*1000/Echelle*La_Resolution-PixelY)<10000) then
  Le_Caneva.Rectangle(round((X_Centre-Xmin)*1000/Echelle*La_Resolution-PixelX)-4,
                          round((Ymax-Y_Centre)*1000/Echelle*La_Resolution-PixelY)-4,
                          round((X_Centre-Xmin)*1000/Echelle*La_Resolution-PixelX)+4,
                          round((Ymax-Y_Centre)*1000/Echelle*La_Resolution-PixelY)+4);
end;

procedure TProjection.MoveToImp(X, Y : double);
begin
  Support.MoveTo(round(((Supl_X+(X-Xmin)*1000/Echelle)*Resolution_Imprimante-
        (Printer.PageWidth-MX*Resolution_Imprimante)*(PageX-Marge_X))*Correction),
                        round(((Supl_Y+(Ymax-Y)*1000/Echelle)*Resolution_Imprimante-
        (Printer.PageHeight-MY*Resolution_Imprimante)*(PageY-Marge_Y))*Correction));
end;

procedure TProjection.LineToImp(X, Y : double);
begin
  Support.LineTo(round(((Supl_X+(X-Xmin)*1000/Echelle)*Resolution_Imprimante-
      (Printer.PageWidth-MX*Resolution_Imprimante)*(PageX-Marge_X))*Correction),
                        round(((Supl_Y+(Ymax-Y)*1000/Echelle)*Resolution_Imprimante-
      (Printer.PageHeight-MY*Resolution_Imprimante)*(PageY-Marge_Y))*Correction));
end;

procedure TProjection.Cercle_Imp(X_Centre, Y_Centre : double; Diametre : integer);
begin
  Support.Ellipse(round(((Supl_X+(X_Centre-Xmin)*1000/Echelle-Diametre/20)*Resolution_Imprimante-
      (Printer.PageWidth-MX*Resolution_Imprimante)*(PageX-Marge_X))*Correction),
                        round(((Supl_Y+(Ymax-Y_Centre)*1000/Echelle-Diametre/20)*Resolution_Imprimante-
      (Printer.PageHeight-MY*Resolution_Imprimante)*(PageY-Marge_Y))*Correction),
               round(((Supl_X+(X_Centre-Xmin)*1000/Echelle+Diametre/20)*Resolution_Imprimante-
      (Printer.PageWidth-MX*Resolution_Imprimante)*(PageX-Marge_X))*Correction),
                        round(((Supl_Y+(Ymax-Y_Centre)*1000/Echelle+Diametre/20)*Resolution_Imprimante-
      (Printer.PageHeight-MY*Resolution_Imprimante)*(PageY-Marge_Y))*Correction));

end;

procedure TProjection.TextImp(X, Y : double;S : string);
begin
  Support.TextOut(round(((Supl_X+(X-Xmin)*1000/Echelle)*Resolution_Imprimante-
      (Printer.PageWidth-MX*Resolution_Imprimante)*(PageX-Marge_X))*Correction),
                        round(((Supl_Y+(Ymax-Y)*1000/Echelle)*Resolution_Imprimante-
      (Printer.PageHeight-MY*Resolution_Imprimante)*(PageY-Marge_Y))*Correction),S);
end;

function TProjection.Bulle(N_Fiche, N_Galerie, N_Visee : integer) : string;
var
  S, SP : string;
  Pourcent : single;
begin
  with TFiche(Form1.Massif.Liste_Courante[N_Fiche]) do
   with TGalerie(Liste_Galerie[N_Galerie]) do begin
    S:='';
    if Form1.Bulle.Coordonnees then begin
      if N_Visee=1 then begin
        S:=S+'('+
          FloatTostrF(Depart.X,ffNumber,8,2)+', '+
          FloatTostrF(Depart.Y,ffNumber,8,2)+', '+
          FloatTostrF(Depart.Z,ffNumber,8,2)+')-(';
      end
      else begin
        S:=S+'('+
          FloatTostrF(Coordonnees^[N_Visee-1].X,ffNumber,8,2)+', '+
          FloatTostrF(Coordonnees^[N_Visee-1].Y,ffNumber,8,2)+', '+
          FloatTostrF(Coordonnees^[N_Visee-1].Z,ffNumber,8,2)+')-(';
      end;
      S:=S+
        FloatTostrF(Coordonnees^[N_Visee].X,ffNumber,8,2)+', '+
        FloatTostrF(Coordonnees^[N_Visee].Y,ffNumber,8,2)+', '+
        FloatTostrF(Coordonnees^[N_Visee].Z,ffNumber,8,2)+')';
    end;
    if Form1.Bulle.Info then begin
      Visees[N_Visee].Commentaires.lit(SP);
      S:=S+SP;
    end;
    if Form1.Bulle.Visee then begin
      S:=S+'('+
        FloatToStrF(Visees[N_Visee].Longueur,ffGeneral,3,1)+tuDistance[Visees[N_Visee].ULong]+','+
        FloatTostrF(Visees[N_Visee].Azimuth,ffGeneral,3,1)+tuAngle[Visees[N_Visee].UAzimuth]+','+
        FloatToStrF(Visees[N_Visee].Pente,ffGeneral,3,1)+tuAngle[Visees[N_Visee].UPente]+')';
    end;
    if Form1.Bulle.Largeur then begin
      S:=S+'(l='+Visees[N_Visee].LargeurGauche;
      if Visees[N_Visee].LargeurDroite<>'' then S:=S+'/'+Visees[N_Visee].LargeurDroite;
      S:=S+')';
    end;
    if Form1.Bulle.Hauteur then begin
      S:=S+'(h='+Visees[N_Visee].HauteurSup;
      if Visees[N_Visee].HauteurInf<>'' then S:=S+'/'+Visees[N_Visee].HauteurInf;
      S:=S+')';
    end;
    if Form1.Bulle.Deformation then begin
      if (N_Visee=1) and (Visees[1].Longueur<1e-2) then inc(N_Visee);
      if N_Visee=1 then begin
        if self is TPlan then begin
          Pourcent:=sqr((Coordonnees^[1].X-Depart.X)-Visees[1].X);
          Pourcent:=Pourcent+sqr((Coordonnees^[1].Y-Depart.Y)-Visees[1].Y);
          Pourcent:=sqrt(Pourcent)/Visees[1].Longueur;
          if Visees[1].ULong=m then Pourcent:=Pourcent*100 else Pourcent:=Pourcent*10000;
        end;
        if self is TCoupe then begin
          Pourcent:=sqr((Coordonnees^[1].Z-Depart.Z)-Visees[1].Z);
          Pourcent:=sqrt(Pourcent)/Visees[1].Longueur;
          if Visees[1].ULong=m then Pourcent:=Pourcent*100 else Pourcent:=Pourcent*10000;
        end;
        if self is TPerspective then begin
          Pourcent:=sqr((Coordonnees^[1].X-Depart.X)-Visees[1].X);
          Pourcent:=Pourcent+sqr((Coordonnees^[1].Y-Depart.Y)-Visees[1].Y);
          Pourcent:=Pourcent+sqr((Coordonnees^[1].Z-Depart.Z)-Visees[1].Z);
          Pourcent:=sqrt(Pourcent)/Visees[1].Longueur;
          if Visees[1].ULong=m then Pourcent:=Pourcent*100 else Pourcent:=Pourcent*10000;
        end;
      end
      else if N_Visee<=Nb_Visee then begin
        if self is TPlan then begin
          Pourcent:=sqr((Coordonnees^[N_Visee].X-Coordonnees^[N_Visee-1].X)-(Visees[N_Visee].X-Visees[N_Visee-1].X));
          Pourcent:=Pourcent+
            sqr((Coordonnees^[N_Visee].Y-Coordonnees^[N_Visee-1].Y)-(Visees[N_Visee].Y-Visees[N_Visee-1].Y));
          Pourcent:=sqrt(Pourcent)/Visees[N_Visee].Longueur;
          if Visees[N_Visee].ULong=m then Pourcent:=Pourcent*100 else Pourcent:=Pourcent*10000;
        end;
        if self is TCoupe then begin
          Pourcent:=sqr((Coordonnees^[N_Visee].Z-Coordonnees^[N_Visee-1].Z)-(Visees[N_Visee].Z-Visees[N_Visee-1].Z));
          Pourcent:=sqrt(Pourcent)/Visees[N_Visee].Longueur;
          if Visees[N_Visee].ULong=m then Pourcent:=Pourcent*100 else Pourcent:=Pourcent*10000;
        end;
        if self is TPerspective then begin
          Pourcent:=sqr((Coordonnees^[N_Visee].X-Coordonnees^[N_Visee-1].X)-(Visees[N_Visee].X-Visees[N_Visee-1].X));
          Pourcent:=Pourcent+
            sqr((Coordonnees^[N_Visee].Y-Coordonnees^[N_Visee-1].Y)-(Visees[N_Visee].Y-Visees[N_Visee-1].Y));
          Pourcent:=Pourcent+
            sqr((Coordonnees^[N_Visee].Z-Coordonnees^[N_Visee-1].Z)-(Visees[N_Visee].Z-Visees[N_Visee-1].Z));
          Pourcent:=sqrt(Pourcent)/Visees[N_Visee].Longueur;
          if Visees[N_Visee].ULong=m then Pourcent:=Pourcent*100 else Pourcent:=Pourcent*10000;
        end;
      end;
      S:=S+' '+FloatToStrF(Pourcent,ffFixed,3,1)+'%';
    end;
    Bulle:=S;
  end;
end;

(************************ TPlan **********************)

constructor TPlan.Create;
begin
  inherited Create;
  Rotation_Horizontale:=0;
  SinH:=0; CosH:=1;
  Echelle:=100;
end;

procedure TPlan.Dessine(Pixel_X, Pixel_Y : longint; Resolution, Rapport : double; Canevas: TImage; Etat : smallInt);
var
  I, J, K, Erreur : integer;
  X0, Y0, X1, Y1 : double;
  L_Autre : TGalerie;
  Xi, Yi, X_Precedent, Y_Precedent, Gauche_Precedent, Droite_Precedent, Gauche_Suivant, Droite_Suivant : double;
  Contrainte_Max, Poids_1, Poids_2, Delta, UX, UY : double;
  S, S2 : string;
  Hors_Plan : TBitMap;
begin
  PixelX:=Pixel_X; PixelY:=Pixel_Y;

  Hors_Plan:=TBitMap.Create;
  Hors_Plan.Width:=Canevas.Width;
  Hors_Plan.Height:=Canevas.Height;
  Le_Caneva:=Hors_Plan.Canvas;

  Hors_Plan.Canvas.Pen.Color:=clWhite; Hors_Plan.Canvas.Pen.Style:=psSolid; Hors_Plan.Canvas.Pen.Mode:=pmCopy;
  Hors_Plan.Canvas.Brush.Color:=clWhite; Hors_Plan.Canvas.Brush.Style:=bsSolid;
  Hors_Plan.Canvas.Rectangle(0,0,Hors_Plan.Width,Hors_Plan.Height);
  La_Resolution:=Resolution*Rapport;
  if Affichable and (Form1.Les_Options_Plan<>nil) then begin
    Poids_1:=Form1.Les_Options_Plan.Pourcent/200+0.5;
    Poids_2:=1-Poids_1;
    Hors_Plan.Canvas.Pen.Color:=clBlack; Hors_Plan.Canvas.Brush.Color:=clBlack;
    Hors_Plan.Canvas.Pen.Width:=round(Form1.Les_Options_Plan.Visee_Epaisseur/10*Resolution*Rapport);
    if Form1.Les_Options_Plan.Carrefour>0 then begin
      Hors_Plan.Canvas.Font:=Form1.Les_Options_Plan.Station_police;
      Hors_Plan.Canvas.Font.Height:=round(Hors_Plan.Canvas.Font.Height*Rapport);
      Delta:=Hors_Plan.Canvas.TextHeight('O')/Resolution/2000*Echelle/Rapport;
    end;
    if Etat=etContrainte then Contrainte_Max:=Contrainte_Maximum(Form1.Massif.Liste_Courante);
    for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
      for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
        if Approximation then begin
          case Etat of
            etContrainte : Hors_Plan.Canvas.Pen.Color:=Degrade[round(Contrainte_Plan/Contrainte_Max*60)];
            etNoir, etRotation : Hors_Plan.Canvas.Pen.Color:=clBlack;
            etCouleur :  Hors_Plan.Canvas.Pen.Color:=Couleur;
          end;
          if Form1.Les_Options_Plan.Habit_Affiche or (Form1.Les_Options_Plan.Type_Largeur<>0) then begin
            Les_Largeurs(J,0,Gauche_Precedent, Droite_Precedent);
          end;
          X0:=Depart.X; Y0:=Depart.Y;
          X_Precedent:=X0*cosH+Y0*sinH; Y_Precedent:=Y0*cosH-X0*sinH;
          if Form1.Les_Options_Plan.Diametre<>0 then begin
            Hors_Plan.Canvas.Brush.Color:=clBlack;
            Cercle(X_Precedent,Y_Precedent,Form1.Les_Options_Plan.Diametre);
          end;
          if (Form1.Les_Options_Plan.Carrefour>0) and not Depart.Terminus then begin
            Hors_Plan.Canvas.Brush.Color:=clWhite; Hors_Plan.Canvas.Brush.Style:=bsClear;
            Depart.Nom.lit(S);
            if Form1.Les_Options_Plan.Carrefour=2 then begin
              S:=S+'('+IntTostr(Depart.Indice)+')';
            end;
            TextRel(X_Precedent+Delta,Y_Precedent+Delta,S);
          end;
          for K:=1 to Nb_Visee do begin
            X1:=Coordonnees^[K].X;
            Y1:=Coordonnees^[K].Y;
            Xi:=X1*cosH+Y1*sinH; Yi:=Y1*cosH-X1*sinH;
            MoveToRel(X_Precedent*Poids_1+Xi*Poids_2,
                      Y_Precedent*Poids_1+Yi*Poids_2);
            LineToRel(X_Precedent*Poids_2+Xi*Poids_1,
                      Y_Precedent*Poids_2+Yi*Poids_1);
            if Form1.Les_Options_Plan.Habit_Affiche or (Form1.Les_Options_Plan.Type_Largeur<>0) then begin
              if (Xi<>X_Precedent) and (Yi<>Y_Precedent) then begin
                UX:=(Xi-X_Precedent)/sqrt(sqr(Xi-X_Precedent)+sqr(Yi-Y_Precedent));
                UY:=(Yi-Y_Precedent)/sqrt(sqr(Xi-X_Precedent)+sqr(Yi-Y_Precedent));
              end;
              Les_Largeurs(J,K,Gauche_Suivant, Droite_Suivant);
              if Form1.Les_Options_Plan.Habit_Affiche and (Visees[K].Longueur<>0) then begin
                Hors_Plan.Canvas.Pen.Width:=round(Form1.Les_Options_Plan.Habit_Epaisseur/10*Resolution*Rapport);
                Hors_Plan.Canvas.Brush.Color:=clBlack; Hors_Plan.Canvas.Brush.Style:=bsSolid;
                Hors_Plan.Canvas.Pen.Width:=round(Form1.Les_Options_Plan.Habit_Epaisseur/10*Resolution*Rapport);
                if (Gauche_Precedent>-0.5) and (Gauche_Suivant>-0.5) then begin
                  MoveToRel(X_Precedent-UY*Gauche_Precedent,Y_Precedent+UX*Gauche_Precedent);
                  LineToRel(Xi-UY*Gauche_Suivant,Yi+UX*Gauche_Suivant);
                end;
                if (Droite_Precedent>-0.5) and (Droite_Suivant>-0.5) then begin
                  MoveToRel(X_Precedent+UY*Droite_Precedent,Y_Precedent-UX*Droite_Precedent);
                  LineToRel(Xi+UY*Droite_Suivant,Yi-UX*Droite_Suivant);
                end;
              end;
              Hors_Plan.Canvas.Pen.Width:=round(Form1.Les_Options_Plan.Epaisseur_Largeur/10*Resolution*Rapport);
              case Form1.Les_Options_Plan.Type_Largeur of
                1 : if Visees[K].Longueur<>0 then begin
                  if Form1.Les_Options_Plan.Debut then begin
                    if Gauche_Precedent>-0.5 then begin
                      MoveToRel(X_Precedent,Y_Precedent);
                      LineToRel(X_Precedent-UY*Gauche_Precedent,
                                Y_Precedent+UX*Gauche_Precedent);
                    end;
                    if Droite_Precedent>-0.5 then begin
                      MoveToRel(X_Precedent,Y_Precedent);
                      LineToRel(X_Precedent+UY*Droite_Precedent,
                                Y_Precedent-UX*Droite_Precedent);
                    end;
                  end;
                  if Form1.Les_Options_Plan.Fin then begin
                    if Gauche_Suivant>-0.5 then begin
                      MoveToRel(Xi,Yi);
                      LineToRel(Xi-UY*Gauche_Suivant,
                                Yi+UX*Gauche_Suivant);
                    end;
                    if Droite_Suivant>-0.5 then begin
                      MoveToRel(Xi,Yi);
                      LineToRel(Xi+UY*Droite_Suivant,
                                Yi-UX*Droite_Suivant);
                    end;
                  end;
                end;
                2 : if Visees[K].Longueur<>0 then begin
                  if Form1.Les_Options_Plan.Debut then begin
                    if Gauche_Precedent>-0.5 then
                      Cercle(X_Precedent-UY*Gauche_Precedent,
                                Y_Precedent+UX*Gauche_Precedent,
                         Form1.Les_Options_Plan.Epaisseur_Largeur div 2);
                    if Droite_Precedent>-0.5 then
                      Cercle(X_Precedent+UY*Droite_Precedent,
                                Y_Precedent-UX*Droite_Precedent,
                         Form1.Les_Options_Plan.Epaisseur_Largeur div 2);
                  end;
                  if Form1.Les_Options_Plan.Fin then begin
                    if Gauche_Suivant>-0.5 then
                      Cercle(Xi-UY*Gauche_Suivant,
                                Yi+UX*Gauche_Suivant,
                         Form1.Les_Options_Plan.Epaisseur_Largeur div 2);
                    if Droite_Suivant>-0.5 then
                      Cercle(Xi+UY*Droite_Suivant,
                                Yi-UX*Droite_Suivant,
                         Form1.Les_Options_Plan.Epaisseur_Largeur div 2);
                  end;
                end;
              end;
              Hors_Plan.Canvas.Pen.Width:=round(Form1.Les_Options_Plan.Visee_Epaisseur/10*Resolution*Rapport);
            end;

            Gauche_Precedent:=Gauche_Suivant;
            Droite_Precedent:=Droite_Suivant;
            if Form1.Les_Options_Plan.Diametre<>0 then
                                  Cercle(Xi,Yi,Form1.Les_Options_Plan.Diametre);
            X_Precedent:=Xi; Y_Precedent:=Yi;
          end;
          if (Form1.Les_Options_Plan.Carrefour>0) and not Arrivee.Terminus then begin
            Hors_Plan.Canvas.Brush.Color:=clWhite; Hors_Plan.Canvas.Brush.Style:=bsClear;
            Arrivee.Nom.lit(S);
            if Form1.Les_Options_Plan.Carrefour=2 then begin
              S:=S+'('+IntTostr(Arrivee.Indice)+')';
            end;
            TextRel(X_Precedent+Delta,Y_Precedent+Delta,S);
          end;
        end;
      end;
    end;
    if Form1.Les_Options_Plan.Nord_Affiche then with Form1.Les_Options_Plan do begin
      Hors_Plan.Canvas.Pen.Width:=round(Nord_Epaisseur/10*Resolution);
      Hors_Plan.Canvas.Pen.Color:=clBlack;
      X0:=Nord_X*1000; Y0:=Nord_Y*1000;
      Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
      MoveToRel(Xi,Yi);
      if NG then begin
        Y0:=Y0+Nord_Longueur/1000*Echelle*cos(Form1.Massif.Angle_Geodesique/180*PI);
        X0:=X0-Nord_Longueur/1000*Echelle*sin(Form1.Massif.Angle_Geodesique/180*PI);
        Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
        LineToRel(Xi,Yi);
        Y0:=Y0-Nord_Longueur/1000*Echelle*
         (cos(Form1.Massif.Angle_Geodesique/180*PI)/2-sin(Form1.Massif.Angle_Geodesique/180*PI)/10);
        X0:=X0+Nord_Longueur/1000*Echelle*
         (sin(Form1.Massif.Angle_Geodesique/180*PI)/2+cos(Form1.Massif.Angle_Geodesique/180*PI)/10);
        Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
        LineToRel(Xi,Yi);
        Hors_Plan.Canvas.Font:=Nord_police;
        Hors_Plan.Canvas.Font.Height:=round(Hors_Plan.Canvas.Font.Height*Rapport);
        Hors_Plan.Canvas.Brush.Color:=clWhite; Hors_Plan.Canvas.Brush.Style:=bsClear;
        TextRel(Xi,Yi,Texte_Nord);
      end
      else begin
        Y0:=Y0+Nord_Longueur/1000*Echelle;
        Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
        LineToRel(Xi,Yi);
        Y0:=Y0-Nord_Longueur/1000*Echelle/2; X0:=X0+Nord_Longueur/1000*Echelle/10;
        Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
        LineToRel(Xi,Yi);
        Hors_Plan.Canvas.Font:=Nord_police;
        Hors_Plan.Canvas.Font.Height:=round(Hors_Plan.Canvas.Font.Height*Rapport);
        Hors_Plan.Canvas.Brush.Color:=clWhite; Hors_Plan.Canvas.Brush.Style:=bsClear;
        TextRel(Xi,Yi,Texte_Nord);
      end;
    end;
    if Form1.Les_Options_Plan.Echelle_Affiche then with Form1.Les_Options_Plan do begin
      Hors_Plan.Canvas.Pen.Width:=round(Echelle_Epaisseur/10*Resolution);
      Hors_Plan.Canvas.Pen.Color:=clBlack;
      X0:=Echelle_X*1000; Y0:=Echelle_Y*1000;
      Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
      MoveToRel(Xi,Yi);
      LineToRel(Xi+Echelle_Longueur,Yi);
      LineToRel(Xi+Echelle_Longueur,Yi+Echelle_Largeur/1000*Echelle);
      MoveToRel(Xi,Yi);
      LineToRel(Xi,Yi+Echelle_Largeur/1000*Echelle);
      if Echelle_Sub>1 then for I:=1 to Echelle_Sub-1 do begin
        MoveToRel(Xi+Echelle_Longueur/Echelle_Sub*I,Yi);
        LineToRel(Xi+Echelle_Longueur/Echelle_Sub*I,Yi+Echelle_Largeur/1000*Echelle);
      end;
      Hors_Plan.Canvas.Font:=Echelle_police;
      Hors_Plan.Canvas.Font.Height:=round(Hors_Plan.Canvas.Font.Height*Rapport);
      Hors_Plan.Canvas.Brush.Color:=clWhite; Hors_Plan.Canvas.Brush.Style:=bsClear;
      X0:=Hors_Plan.Canvas.TextWidth('0')/Resolution/2;
      Y0:=Hors_Plan.Canvas.TextHeight('0')/Resolution;
      TextRel(Xi-X0/1000*Echelle,Yi+(Echelle_Largeur+Y0)/1000*Echelle,'0');
      S:=IntTostr(Echelle_Longueur)+tuDistance[m];
      X0:=Hors_Plan.Canvas.TextWidth(S)/Resolution/2;
      TextRel(Xi+Echelle_Longueur-X0/1000*Echelle,Yi+(Echelle_Largeur+Y0)/1000*Echelle,S);
    end;
    if Form1.Les_Options_Plan.Quadri_Affiche then with Form1.Les_Options_Plan do begin
      Hors_Plan.Canvas.Pen.Width:=round(Quadri_Epaisseur/10*Resolution);
      Hors_Plan.Canvas.Pen.Color:=clBlack;
      X0:=Quadri_Longueur/1000*Echelle;
      Xi:=X0*cosH; Yi:=-X0*sinH;
      X0:=tronc(PMin.X/Quadri_Pas)*Quadri_Pas*cosH+tronc(PMin.Y/Quadri_Pas)*Quadri_Pas*sinH;
      Y0:=-tronc(PMin.X/Quadri_Pas)*Quadri_Pas*sinH+tronc(PMin.Y/Quadri_Pas)*Quadri_Pas*cosH;
      X1:=Quadri_Pas*cosH; Y1:=-Quadri_Pas*sinH;
      for I:=0 to tronc(PMax.X/Quadri_Pas)-tronc(PMin.X/Quadri_Pas)+1 do begin
        for J:=0 to tronc(PMax.Y/Quadri_Pas)-tronc(PMin.Y/Quadri_Pas)+1 do begin
          MoveToRel(X0+I*X1-J*Y1-Xi,Y0+I*Y1+J*X1-Yi);
          LineToRel(X0+I*X1-J*Y1+Xi,Y0+I*Y1+J*X1+Yi);
          MoveToRel(X0+I*X1-J*Y1-Yi,Y0+I*Y1+J*X1+Xi);
          LineToRel(X0+I*X1-J*Y1+Yi,Y0+I*Y1+J*X1-Xi);
        end;
      end;
    end;
  end;

  Le_Caneva:=Canevas.Canvas;
  Le_Caneva.CopyMode:=cmSrcCopy;
  Le_Caneva.Draw(0,0,Hors_Plan);
  Hors_Plan.Free;

end;


procedure TPlan.Imprime(Page_X, Page_Y, Sup_X, Sup_Y : integer; Etat : smallInt; Fond : TCanvas; Reduc : double);
var
  I, J, K, Page_Horizontale, Page_Verticale, Erreur : integer;
  X0, Y0, X1, Y1, Contrainte_Max : double;
  L_Autre : TGalerie;
  Xi, Yi, X_Precedent, Y_Precedent, Gauche_Precedent, Droite_Precedent,
  Gauche_Suivant, Droite_Suivant, Poids_1, Poids_2, Delta, UX, UY : double;
  S, S2 : string;
  Restitue : integer;
begin

  Supl_X:=Sup_X; Supl_Y:=Sup_Y;

  Support:=Fond; Correction:=Reduc;
  if Form1.Les_Options_Plan.Blocage_Origine then with Form1.Les_Options_Plan do begin
    Xi:=Origine_X*cosH+Origine_Y*sinH;
    Yi:=-Origine_X*sinH+Origine_Y*cosH;
    Page_Horizontale:=trunc((XMax-Xi-0.01*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MX))+1+
            trunc((Xi-XMin-0.01*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MX))+1;
    Page_Verticale:=trunc((YMax-Yi-0.01*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MY))+1+
            trunc((Yi-YMin-0.01*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MY))+1;
    { Marge_X et Marge_Y sont les marges globales en fraction de page }
    Marge_X:=1-frac((Xi-XMin)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MX));
    Marge_Y:=1-frac((YMax-Yi)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MY));
  end
  else begin
    Page_Horizontale:=trunc((XMax-XMin-0.02*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MX))+1;
    Page_Verticale:=trunc((YMax-YMin-0.02*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MY))+1;
    Marge_X:=(Page_Horizontale-(XMax-XMin)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MX))/2;
    Marge_Y:=(Page_Verticale-(YMax-YMin)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MY))/2;
  end;

//  Restitue:=colorinfo(printer.Handle,$FF,0);
  PageX:=Page_X; PageY:=Page_Y;
  Resolution_Imprimante:=Printer.PageWidth/GetDeviceCaps(Printer.Handle,HorzSize);
  if Affichable then begin
    Poids_1:=Form1.Les_Options_Plan.Pourcent/200+0.5;
    Poids_2:=1-Poids_1;
    Support.Pen.Color:=clBlack;
    Support.Brush.Color:=clBlack; Support.Brush.Style:=bsSolid;
    Support.Pen.Width:=round(Form1.Les_Options_Plan.Visee_Epaisseur/10*Resolution_Imprimante*Correction);
    if Form1.Les_Options_Plan.Carrefour>0 then begin
      Support.Font:=Form1.Les_Options_Plan.Station_police;
      Printer.Canvas.Font:=Form1.Les_Options_Plan.Station_police;
      Delta:=Printer.Canvas.TextHeight('O')/Resolution_Imprimante/2000*Echelle;
      if Correction<>1 then Support.Font.Height:=round(Support.Font.Height*Correction*
           Resolution_Imprimante/Form1.LePlan.PixelsPerInch*25.4);
    end;
    if Etat=etContrainte then Contrainte_Max:=Contrainte_Maximum(Form1.Massif.Liste_Courante);
    for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
      for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
        if Approximation then begin
          case Etat of
            etContrainte : Support.Pen.Color:=Degrade[round(Contrainte_Plan/Contrainte_Max*60)];
            etNoir, etRotation : Support.Pen.Color:=clBlack;
            etCouleur :  Support.Pen.Color:=Couleur;
          end;
          Les_Largeurs(J,0,Gauche_Suivant, Droite_Suivant);
          X0:=Depart.X; Y0:=Depart.Y;
          X_Precedent:=X0*cosH+Y0*sinH; Y_Precedent:=Y0*cosH-X0*sinH;
          if Form1.Les_Options_Plan.Diametre<>0 then
                       Cercle_Imp(X_Precedent,Y_Precedent,Form1.Les_Options_Plan.Diametre);
          if (Form1.Les_Options_Plan.Carrefour>0) and not Depart.Terminus then begin
            Support.Brush.Color:=clWhite; Support.Brush.Style:=bsClear;
            Depart.Nom.lit(S);
            if Form1.Les_Options_Plan.Carrefour=2 then begin
              S:=S+'('+IntTostr(Depart.Indice)+')';
            end;
            TextImp(X_Precedent+Delta,Y_Precedent+Delta,S);
          end;
          for K:=1 to Nb_Visee do begin
            X1:=Coordonnees^[K].X;
            Y1:=Coordonnees^[K].Y;
            Xi:=X1*cosH+Y1*sinH; Yi:=Y1*cosH-X1*sinH;
            MoveToImp(X_Precedent*Poids_1+Xi*Poids_2,
                      Y_Precedent*Poids_1+Yi*Poids_2);
            LineToImp(X_Precedent*Poids_2+Xi*Poids_1,
                      Y_Precedent*Poids_2+Yi*Poids_1);
            Support.Pen.Width:=round(Form1.Les_Options_Plan.Habit_Epaisseur/10*Resolution_Imprimante*Correction);
            Support.Brush.Color:=clBlack; Support.Brush.Style:=bsSolid;
            if (Xi<>X_Precedent) and (Yi<>Y_Precedent) then begin
              UX:=(Xi-X_Precedent)/sqrt(sqr(Xi-X_Precedent)+sqr(Yi-Y_Precedent));
              UY:=(Yi-Y_Precedent)/sqrt(sqr(Xi-X_Precedent)+sqr(Yi-Y_Precedent));
            end;
            Les_Largeurs(J,K,Gauche_Suivant, Droite_Suivant);
            if (Visees[K].Longueur<>0) and Form1.Les_Options_Plan.Habit_Affiche then begin
              Support.Pen.Width:=round(Form1.Les_Options_Plan.Habit_Epaisseur/10*Resolution_Imprimante*Correction);
              if (Gauche_Precedent>-0.5) and (Gauche_Suivant>-0.5) then begin
                MoveToImp(X_Precedent-UY*Gauche_Precedent,Y_Precedent+UX*Gauche_Precedent);
                LineToImp(Xi-UY*Gauche_Suivant,Yi+UX*Gauche_Suivant);
              end;
              if (Droite_Precedent>-0.5) and (Droite_Suivant>-0.5) then begin
                MoveToImp(X_Precedent+UY*Droite_Precedent,Y_Precedent-UX*Droite_Precedent);
                LineToImp(Xi+UY*Droite_Suivant,Yi-UX*Droite_Suivant);
              end;
            end;
            Support.Pen.Width:=round(Form1.Les_Options_Plan.Epaisseur_Largeur/10*Resolution_Imprimante*Correction);
            case Form1.Les_Options_Plan.Type_Largeur of
              1 : if Visees[K].Longueur<>0 then begin
                if Form1.Les_Options_Plan.Debut then begin
                  if Gauche_Precedent>-0.5 then begin
                    MoveToImp(X_Precedent,Y_Precedent);
                    LineToImp(X_Precedent-UY*Gauche_Precedent,
                              Y_Precedent+UX*Gauche_Precedent);
                  end;
                  if Droite_Precedent>-0.5 then begin
                    MoveToImp(X_Precedent,Y_Precedent);
                    LineToImp(X_Precedent+UY*Droite_Precedent,
                              Y_Precedent-UX*Droite_Precedent);
                  end;
                end;
                if Form1.Les_Options_Plan.Fin then begin
                  if Gauche_Suivant>-0.5 then begin
                    MoveToImp(Xi,Yi);
                    LineToImp(Xi-UY*Gauche_Suivant,
                              Yi+UX*Gauche_Suivant);
                  end;
                  if Droite_Suivant>-0.5 then begin
                    MoveToImp(Xi,Yi);
                    LineToImp(Xi+UY*Droite_Suivant,
                              Yi-UX*Droite_Suivant);
                  end;
                end;
              end;
              2 : if Visees[K].Longueur<>0 then begin
                if Form1.Les_Options_Plan.Debut then begin
                  if Gauche_Precedent>-0.5 then
                    Cercle_Imp(X_Precedent-UY*Gauche_Precedent,
                              Y_Precedent+UX*Gauche_Precedent,
                       Form1.Les_Options_Plan.Epaisseur_Largeur div 2);
                  if Droite_Precedent>-0.5 then
                    Cercle_Imp(X_Precedent+UY*Droite_Precedent,
                              Y_Precedent-UX*Droite_Precedent,
                       Form1.Les_Options_Plan.Epaisseur_Largeur div 2);
                end;
                if Form1.Les_Options_Plan.Fin then begin
                  if Gauche_Suivant>-0.5 then
                    Cercle_Imp(Xi-UY*Gauche_Suivant,
                              Yi+UX*Gauche_Suivant,
                       Form1.Les_Options_Plan.Epaisseur_Largeur div 2);
                  if Droite_Suivant>-0.5 then
                    Cercle_Imp(Xi+UY*Droite_Suivant,
                              Yi-UX*Droite_Suivant,
                       Form1.Les_Options_Plan.Epaisseur_Largeur div 2);
                end;
              end;
            end;

            Support.Pen.Width:=round(Form1.Les_Options_Plan.Visee_Epaisseur/10*Resolution_Imprimante*Correction);
            Gauche_Precedent:=Gauche_Suivant;
            Droite_Precedent:=Droite_Suivant;
            if Form1.Les_Options_Plan.Diametre<>0 then
                                  Cercle_Imp(Xi,Yi,Form1.Les_Options_Plan.Diametre);
            X_Precedent:=Xi; Y_Precedent:=Yi;
          end;
          if (Form1.Les_Options_Plan.Carrefour>0) and not Arrivee.Terminus then begin
            Support.Brush.Color:=clWhite; Support.Brush.Style:=bsClear;
            Arrivee.Nom.lit(S);
            if Form1.Les_Options_Plan.Carrefour=2 then begin
              S:=S+'('+IntTostr(Arrivee.Indice)+')';
            end;
            TextImp(X_Precedent+Delta,Y_Precedent+Delta,S);
          end;
        end;
      end;
    end;
    if Form1.Les_Options_Plan.Nord_Affiche then with Form1.Les_Options_Plan do begin
      Support.Pen.Width:=round(Nord_Epaisseur/10*Resolution_Imprimante*Correction);
      Support.Pen.Color:=clBlack;
      X0:=Nord_X*1000; Y0:=Nord_Y*1000;
      Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
      MoveToImp(Xi,Yi);
      if NG then begin
        Y0:=Y0+Nord_Longueur/1000*Echelle*cos(Form1.Massif.Angle_Geodesique/180*PI);
        X0:=X0-Nord_Longueur/1000*Echelle*sin(Form1.Massif.Angle_Geodesique/180*PI);
        Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
        LineToImp(Xi,Yi);
        Y0:=Y0-Nord_Longueur/1000*Echelle*
         (cos(Form1.Massif.Angle_Geodesique/180*PI)/2-sin(Form1.Massif.Angle_Geodesique/180*PI)/10);
        X0:=X0+Nord_Longueur/1000*Echelle*
         (sin(Form1.Massif.Angle_Geodesique/180*PI)/2+cos(Form1.Massif.Angle_Geodesique/180*PI)/10);
        Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
        LineToImp(Xi,Yi);
        Support.Font:=Nord_police;
        if Correction<>1 then Support.Font.Height:=round(Support.Font.Height*Correction*
             Resolution_Imprimante/Form1.LePlan.PixelsPerInch*25.4);
        Support.Brush.Color:=clWhite; Support.Brush.Style:=bsClear;
        TextImp(Xi,Yi,Texte_Nord);
      end
      else begin
        Y0:=Y0+Nord_Longueur/1000*Echelle;
        Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
        LineToImp(Xi,Yi);
        Y0:=Y0-Nord_Longueur/1000*Echelle/2; X0:=X0+Nord_Longueur/1000*Echelle/10;
        Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
        LineToImp(Xi,Yi);
        Support.Font:=Nord_police;
        if Correction<>1 then Support.Font.Height:=round(Support.Font.Height*Correction*
             Resolution_Imprimante/Form1.LePlan.PixelsPerInch*25.4);
        Support.Brush.Color:=clWhite; Support.Brush.Style:=bsClear;
        TextImp(Xi,Yi,Texte_Nord);
      end;
    end;
    if Form1.Les_Options_Plan.Echelle_Affiche then with Form1.Les_Options_Plan do begin
      Support.Pen.Width:=round(Echelle_Epaisseur/10*Resolution_Imprimante*Correction);
      Support.Pen.Color:=clBlack;
      X0:=Echelle_X*1000; Y0:=Echelle_Y*1000;
      Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
      MoveToImp(Xi,Yi);
      LineToImp(Xi+Echelle_Longueur,Yi);
      LineToImp(Xi+Echelle_Longueur,Yi+Echelle_Largeur/1000*Echelle);
      MoveToImp(Xi,Yi);
      LineToImp(Xi,Yi+Echelle_Largeur/1000*Echelle);
      if Echelle_Sub>1 then for I:=1 to Echelle_Sub-1 do begin
        MoveToImp(Xi+Echelle_Longueur/Echelle_Sub*I,Yi);
        LineToImp(Xi+Echelle_Longueur/Echelle_Sub*I,Yi+Echelle_Largeur/1000*Echelle);
      end;
      Support.Font:=Echelle_police; Printer.Canvas.Font:=Echelle_police;
      Support.Brush.Color:=clWhite; Support.Brush.Style:=bsClear;
      X0:=Printer.Canvas.TextWidth('0')/Resolution_Imprimante/2;
      Y0:=Printer.Canvas.TextHeight('0')/Resolution_Imprimante;
      if Correction<>1 then Support.Font.Height:=round(Support.Font.Height*Correction*
           Resolution_Imprimante/Form1.LePlan.PixelsPerInch*25.4);
      TextImp(Xi-X0/1000*Echelle,Yi+(Echelle_Largeur+Y0)/1000*Echelle,'0');
      S:=IntTostr(Echelle_Longueur)+tuDistance[m];
      Support.Font:=Echelle_police; Printer.Canvas.Font:=Echelle_police;
      X0:=Printer.Canvas.TextWidth(S)/Resolution_Imprimante/2;
      if Correction<>1 then Support.Font.Height:=round(Support.Font.Height*Correction*
           Resolution_Imprimante/Form1.LePlan.PixelsPerInch*25.4);
      TextImp(Xi+Echelle_Longueur-X0/1000*Echelle,Yi+(Echelle_Largeur+Y0)/1000*Echelle,S);
    end;
    if Form1.Les_Options_Plan.Quadri_Affiche then with Form1.Les_Options_Plan do begin
      Support.Pen.Width:=round(Quadri_Epaisseur/10*Resolution_Imprimante*Correction);
      Support.Pen.Color:=clBlack;
      X0:=Quadri_Longueur/1000*Echelle;
      Xi:=X0*cosH; Yi:=-X0*sinH;
      X0:=tronc(PMin.X/Quadri_Pas)*Quadri_Pas*cosH+tronc(PMin.Y/Quadri_Pas)*Quadri_Pas*sinH;
      Y0:=-tronc(PMin.X/Quadri_Pas)*Quadri_Pas*sinH+tronc(PMin.Y/Quadri_Pas)*Quadri_Pas*cosH;
      X1:=Quadri_Pas*cosH; Y1:=-Quadri_Pas*sinH;
      for I:=0 to tronc(PMax.X/Quadri_Pas)-tronc(PMin.X/Quadri_Pas)+1 do begin
        for J:=0 to tronc(PMax.Y/Quadri_Pas)-tronc(PMin.Y/Quadri_Pas)+1 do begin
          MoveToImp(X0+I*X1-J*Y1-Xi,Y0+I*Y1+J*X1-Yi);
          LineToImp(X0+I*X1-J*Y1+Xi,Y0+I*Y1+J*X1+Yi);
          MoveToImp(X0+I*X1-J*Y1-Yi,Y0+I*Y1+J*X1+Xi);
          LineToImp(X0+I*X1-J*Y1+Yi,Y0+I*Y1+J*X1-Xi);
        end;
      end;
    end;
  end;
  Support.Brush.Color:=clWhite; Support.Brush.Style:=bsClear;
end;

procedure TPlan.Gabarit(var Pixel_Hori, Pixel_Verti : longint; Resolution, Rapport : double; Canevas : TImage);
var
  Les_Angles : array[1..4] of TPoint3D;
  I : integer;
  Xi, Yi : double;
begin
  Affichable:=Calibre(PMin, PMax,Form1.Massif.Liste_Courante);
  if Affichable then begin
    Les_Angles[1].X:=PMin.X; Les_Angles[1].Y:=PMin.Y;
    Les_Angles[2].X:=PMin.X; Les_Angles[2].Y:=PMax.Y;
    Les_Angles[3].X:=PMax.X; Les_Angles[3].Y:=PMin.Y;
    Les_Angles[4].X:=PMax.X; Les_Angles[4].Y:=PMax.Y;
    for I:=1 to 4 do with Les_Angles[I] do begin
      Xi:=X*cosH+Y*sinH;
      Yi:=-X*sinH+Y*cosH;
      X:=Xi; Y:=Yi;
    end;
    XMin:=Les_Angles[1].X; XMax:=XMin;
    YMin:=Les_Angles[1].Y; YMax:=YMin;
    for I:=2 to 4 do with Les_Angles[I] do begin
      if X<XMin then XMin:=X;
      if X>XMax then XMax:=X;
      if Y<YMin then YMin:=Y;
      if Y>YMax then YMax:=Y;
    end;
    Xmin:=XMin-0.01*Echelle; Xmax:=XMax+0.01*Echelle;
    Ymin:=YMin-0.01*Echelle; Ymax:=YMax+0.01*Echelle;
    Pixel_Hori:=round((XMax-XMin)/Echelle*1000*Resolution*Rapport);
    Pixel_Verti:=round((YMax-YMin)/Echelle*1000*Resolution*Rapport);
  end
  else begin
    Pixel_Hori:=1;
    Pixel_Verti:=1;
  end;
end;

procedure TPlan.Gabarit_Imprimante(var Page_X, Page_Y : integer; MargeX, MargeY : integer);
var
  Xi, Yi : double;
begin
  if Form1.Les_Options_Plan.Blocage_Origine then with Form1.Les_Options_Plan do begin
    Xi:=Origine_X*cosH+Origine_Y*sinH;
    Yi:=-Origine_X*sinH+Origine_Y*cosH;
    Page_X:=trunc((XMax-Xi-0.01*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MargeX))+1+
            trunc((Xi-XMin-0.01*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MargeX))+1;
    Page_Y:=trunc((YMax-Yi-0.01*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MargeY))+1+
            trunc((Yi-YMin-0.01*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MargeY))+1;
  end
  else begin
    Page_X:=trunc((XMax-XMin-0.02*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,HorzSize)-MargeX))+1;
    Page_Y:=trunc((YMax-YMin-0.02*Echelle)/Echelle*1000/(GetDeviceCaps(Printer.Handle,VertSize)-MargeY))+1;
  end;
  MX:=MargeX;
  MY:=MargeY;
end;

function TPlan.Essai(Hori : double):string;
begin
  sinH:=sin((Rotation_Horizontale+Hori)/180*PI);
  cosH:=cos((Rotation_Horizontale+Hori)/180*PI);
  Essai:=FloatTostrF(Rotation_Horizontale+Hori,ffGeneral,3,0)+' ';
end;

procedure TPlan.Definitif(Hori : double);
begin
  Rotation_Horizontale:=Rotation_Horizontale+Hori;
  if Rotation_Horizontale>360 then Rotation_Horizontale:=Rotation_Horizontale-360;
  if Rotation_Horizontale<0 then Rotation_Horizontale:=Rotation_Horizontale+360;
  sinH:=sin(Rotation_Horizontale/180*PI);
  cosH:=cos(Rotation_Horizontale/180*PI);
end;

function TPlan.Trouve(X, Y : longint;var N_Fiche, N_Galerie, N_Visee : integer) : double;
var
  I, J, K : integer;
  X0, Y0, X1, Y1 : double;
  Xi, Yi, X_Abs, Y_Abs, Xd, Yd, DMin, Zone : double;
  Repere : boolean;
begin
  X_Abs:=X/1000*Echelle/La_Resolution+XMin;
  Y_Abs:=YMax-Y/1000*Echelle/La_Resolution;
  Repere:=false;
  if Affichable then begin
    for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
      for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
        if Approximation then begin
          X0:=Depart.X; Y0:=Depart.Y;
          Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
          Xd:=X0; Yd:=Y0;
          if not Repere then begin
            Repere:=true;
            Dmin:=sqr((Xi-X_Abs))+sqr((Yi-Y_Abs));
            N_Fiche:=I; N_Galerie:=J; N_Visee:=1;
          end
          else if sqr((Xi-X_Abs))+sqr((Yi-Y_Abs))<Dmin then begin
            Dmin:=sqr((Xi-X_Abs))+sqr((Yi-Y_Abs));
            N_Fiche:=I; N_Galerie:=J; N_Visee:=1;
          end;
          for K:=1 to Nb_Visee do begin
            X1:=Coordonnees^[K].X;
            Y1:=Coordonnees^[K].Y;
            X0:=X1*cosH+Y1*sinH; Y0:=Y1*cosH-X1*sinH;
            Zone:=(X_Abs-Xi)*(X0-Xi)+(Y_Abs-Yi)*(Y0-Yi);
            if Zone>sqr(X0-Xi)+sqr(Y0-Yi) then begin
              if sqr((X0-X_Abs))+sqr((Y0-Y_Abs))<Dmin then begin
                Dmin:=sqr((X0-X_Abs))+sqr((Y0-Y_Abs));
                N_Fiche:=I; N_Galerie:=J; N_Visee:=K;
              end;
            end
            else if (Zone>0) and ((Xi<>X0) or (Yi<>Y0)) then begin
              Zone:=(X0-Xi)*(Y_Abs-Yi)-(Y0-Yi)*(X_Abs-Xi);
              Zone:=sqr(Zone)/(sqr(X0-Xi)+sqr(Y0-Yi));
              if Zone<Dmin then begin
                Dmin:=Zone;
                N_Fiche:=I; N_Galerie:=J; N_Visee:=K;
              end;
            end;
            Xi:=X0; Yi:=Y0;
          end;
        end;
      end;
    end;
  end
  else Dmin:=1e6;
  Trouve:=sqrt(Dmin)*1000/Echelle*La_Resolution;;
end;

procedure TPlan.Isoler(X, Y, Rayon2 : longint);
var
  I, J, K : integer;
  X0, Y0 : double;
  Nouvelle_Selection : TList;
  Xi, Yi, X_Abs, Y_Abs : double;
  Repere : boolean;
begin
  X_Abs:=X/1000*Echelle/La_Resolution+XMin;
  Y_Abs:=YMax-Y/1000*Echelle/La_Resolution;
  Nouvelle_Selection:=TList.Create;
  if Affichable then begin
    for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
      Repere:=false;
      for J:=0 to Liste_Galerie.Count-1 do if not Repere then with TGalerie(Liste_Galerie[J]) do begin
        if Approximation then begin
          X0:=Depart.X; Y0:=Depart.Y;
          Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
          if sqr((Xi-X_Abs))+sqr((Yi-Y_Abs))<Rayon2*sqr(Echelle/La_Resolution/1000) then Repere:=true;
          for K:=1 to Nb_Visee do begin
            Xi:=Coordonnees^[K].X;
            Yi:=Coordonnees^[K].Y;
            X0:=Xi*cosH+Yi*sinH; Y0:=Yi*cosH-Xi*sinH;
            if sqr((X0-X_Abs))+sqr((Y0-Y_Abs))<Rayon2*sqr(Echelle/La_Resolution/1000) then Repere:=true;
          end;
        end;
      end;
      if Repere then Nouvelle_Selection.Add(Form1.Massif.Liste_Courante[I]);
    end;
  end;
  if Form1.Massif.Existe_Selection then Form1.Massif.Selection.Free;
  Form1.Massif.Selection:=Nouvelle_Selection;
  Form1.Massif.Liste_Courante:=Nouvelle_Selection;
  Form1.Massif.Existe_Selection:=true;
end;

function TPlan.Met_En_Relief(Nouveau : boolean; N_Fiche, N_Galerie, N_Visee : integer;Etat : smallInt) : string;
var
  K : integer;
  X0, Y0, X1, Y1 : double;
  Xi, Yi, X_Precedent, Y_Precedent : double;
  Resultat, S : string;
begin
  with TFiche(Form1.Massif.Liste_Courante[N_Fiche]) do
   with TGalerie(Liste_Galerie[N_Galerie]) do begin
    Le_Caneva.Pen.Style:=psSolid; Le_Caneva.Pen.Mode:=pmCopy;

  { cration de la chaine rsum }
    if Nouveau then begin
      Systeme.Lit(S); Resultat:=S;
      Cavite.Lit(S); Resultat:=Resultat+'/'+S;
      Reseau.Lit(S); Resultat:=Resultat+'/'+S;
      Lieu.Lit(S); Resultat:=Resultat+'/'+S;
      if Depart.Terminus then Resultat:=Resultat+' Terminus - '
      else begin
        Depart.Nom.Lit(S); Resultat:=Resultat+' '+S+' - ';
      end;
      if Arrivee.Terminus then Resultat:=Resultat+'Terminus'
      else begin
        Arrivee.Nom.Lit(S); Resultat:=Resultat+S;
      end;
      Met_En_Relief:=Resultat;
    end
    else Met_En_Relief:='';
    Le_Caneva.Pen.Width:=round(Form1.Les_Options_Plan.Visee_Epaisseur/10*La_Resolution);
    if Approximation then begin
      X0:=Depart.X; Y0:=Depart.Y;
      X_Precedent:=X0*cosH+Y0*sinH; Y_Precedent:=Y0*cosH-X0*sinH;
      for K:=1 to Nb_Visee do begin
        X1:=Coordonnees^[K].X;
        Y1:=Coordonnees^[K].Y;
        Xi:=X1*cosH+Y1*sinH; Yi:=Y1*cosH-X1*sinH;
        if K=N_Visee then begin
          Le_Caneva.Pen.Mode:=pmNot;
          if Nouveau then begin
            Le_Caneva.Pen.Color:=clBlack;
            Le_Caneva.Brush.Color:=clBlack;
          end
          else begin
            Le_Caneva.Pen.Color:=clWhite;
            Le_Caneva.Brush.Color:=clWhite;
          end;
          Carre(X_Precedent,Y_Precedent);
          Carre(Xi,Yi);
          Le_Caneva.Pen.Mode:=pmCopy;
          if Etat=etNoir then begin
            if Nouveau then Le_Caneva.Pen.Color:=clRed
            else Le_Caneva.Pen.Color:=clBlack;
            MoveToRel(X_Precedent*(0.5+Form1.Les_Options_Plan.Pourcent/200)+
                     Xi*(0.5-Form1.Les_Options_Plan.Pourcent/200),
                     Y_Precedent*(0.5+Form1.Les_Options_Plan.Pourcent/200)+
                     Yi*(0.5-Form1.Les_Options_Plan.Pourcent/200));
            LineToRel(X_Precedent*(0.5-Form1.Les_Options_Plan.Pourcent/200)+
                     Xi*(0.5+Form1.Les_Options_Plan.Pourcent/200),
                     Y_Precedent*(0.5-Form1.Les_Options_Plan.Pourcent/200)+
                     Yi*(0.5+Form1.Les_Options_Plan.Pourcent/200));
          end;
        end;
        X_Precedent:=Xi; Y_Precedent:=Yi;
      end;
    end;
  end;
end;

procedure TPlan.Export_Svg(Fichier_SVG: TFileName; Illustrator, AllLayers : boolean);
const
  coVisee = 0;
  coParois = 1;
  coLargeur = 2;
  coStation = 3;
  coCarrefour = 4;
var
  F : TextFile;
  I, J, K, Couche : integer;
  Poids_1, Poids_2, Delta : double;
  X0, Y0, X1, Y1, Xi, Yi, X_Precedent, Y_Precedent : double;
  UX, UY, Gauche_Suivant, Droite_Suivant, Gauche_Precedent, Droite_Precedent : double;
  S, sLayer : string;
begin
  inherited;
  AssignFile(F,Fichier_SVG);
  try
    rewrite(F);
    EnteteSvg(F,round((XMax-XMin)*1000/Echelle),round((YMax-YMin)*1000/Echelle),Illustrator);
    writeln(F,Format('<title>%s</title>',[ExtractFileName(Form1.Massif.Fichier)]));
    writeln(F,'<desc>Plan</desc>');

    if Affichable and (Form1.Les_Options_Plan<>nil) then begin
      Poids_1:=Form1.Les_Options_Plan.Pourcent/200+0.5;
      Poids_2:=1-Poids_1;
      Delta:=Form1.Les_Options_Plan.Station_police.Size*25.4/72/2;
      if Illustrator then
        sLayer:=' i:layer="yes"'
      else
        sLayer:='';
      for Couche:=0 to 4 do
      begin
        case Couche of
          coVisee : writeln(F,Format('<g id="visees"%s style="stroke-width:%fmm; stroke:black;">',
                                     [sLayer,Form1.Les_Options_Plan.Visee_Epaisseur/10]));
          coParois : writeln(F,Format('<g id="parois"%s style="stroke-width:%fmm; stroke:black;">',
                                     [sLayer,Form1.Les_Options_Plan.Habit_Epaisseur/10]));
          coLargeur : writeln(F,Format('<g id="largeurs"%s style="stroke-width:%fmm; stroke:black; fill:black;">',
                                     [sLayer,Form1.Les_Options_Plan.Epaisseur_Largeur/10]));
          coStation : writeln(F,Format('<g id="stations"%s style="stroke:black; fill:black;">',
                                     [sLayer]));
          coCarrefour : writeln(F,Format('<g id="carrefours"%s style="font-size:%dpt; font-family:%s;">',
                                     [sLayer,
                                     Form1.Les_Options_Plan.Station_police.Size,
                                     Form1.Les_Options_Plan.Station_police.Name]));
        end;
        for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
          writeln(F,'<g>'); // un groupe par fiche
          for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
            if Approximation then begin
              writeln(F,'<g>'); // un groupe par galerie
              Les_Largeurs(J,0,Gauche_Precedent, Droite_Precedent);
              X0:=Depart.X; Y0:=Depart.Y;
              X_Precedent:=X0*cosH+Y0*sinH; Y_Precedent:=Y0*cosH-X0*sinH;
              if (Couche=coStation) and ((Form1.Les_Options_Plan.Diametre<>0) or AllLayers) then begin
                writeln(F,Format('<circle cx="%fmm" cy="%fmm" r="%fmm"/>',
                                  [(X_Precedent-XMin)/Echelle*1000,
                                   (YMax-Y_Precedent)/Echelle*1000,
                                   Form1.Les_Options_Plan.Diametre/20]));
              end;
              if Couche=coCarrefour then
              if ((Form1.Les_Options_Plan.Carrefour>0) or AllLayers) and not Depart.Terminus then begin
                Depart.Nom.lit(S);
                if Form1.Les_Options_Plan.Carrefour=2 then begin
                  S:=S+'('+IntTostr(Depart.Indice)+')';
                end;
                writeln(F,Format('<text x="%fmm" y="%fmm">%s</text>',
                                 [(X_Precedent-XMin)/Echelle*1000+Delta,(Ymax-Y_Precedent)/Echelle*1000+Delta,S]));
              end;
              for K:=1 to Nb_Visee do begin
                X1:=Coordonnees^[K].X;
                Y1:=Coordonnees^[K].Y;
                Xi:=X1*cosH+Y1*sinH; Yi:=Y1*cosH-X1*sinH;
                if Couche=coVisee then
                  writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                                   [(X_Precedent*Poids_1+Xi*Poids_2-XMin)/Echelle*1000,
                                    (YMax-(Y_Precedent*Poids_1+Yi*Poids_2))/Echelle*1000,
                                    (X_Precedent*Poids_2+Xi*Poids_1-XMin)/Echelle*1000,
                                    (YMax-(Y_Precedent*Poids_2+Yi*Poids_1))/Echelle*1000]));
                if Form1.Les_Options_Plan.Habit_Affiche or (Form1.Les_Options_Plan.Type_Largeur<>0) or AllLayers then begin
                  if (Xi<>X_Precedent) and (Yi<>Y_Precedent) then
                  begin
                    UX:=(Xi-X_Precedent)/sqrt(sqr(Xi-X_Precedent)+sqr(Yi-Y_Precedent));
                    UY:=(Yi-Y_Precedent)/sqrt(sqr(Xi-X_Precedent)+sqr(Yi-Y_Precedent));
                  end;
                  Les_Largeurs(J,K,Gauche_Suivant, Droite_Suivant);
                  if (Couche=coParois) and
                     (Form1.Les_Options_Plan.Habit_Affiche or AllLayers) and
                     (Visees[K].Longueur<>0) then
                  begin
                    if (Gauche_Precedent>-0.5) and (Gauche_Suivant>-0.5) then
                      writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                                       [(X_Precedent-UY*Gauche_Precedent-XMin)/Echelle*1000,
                                        (YMax-(Y_Precedent+UX*Gauche_Precedent))/Echelle*1000,
                                        (Xi-UY*Gauche_Suivant-XMin)/Echelle*1000,
                                        (YMax-(Yi+UX*Gauche_Suivant))/Echelle*1000]));
                    if (Droite_Precedent>-0.5) and (Droite_Suivant>-0.5) then
                      writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                                       [(X_Precedent+UY*Droite_Precedent-XMin)/Echelle*1000,
                                        (YMax-(Y_Precedent-UX*Droite_Precedent))/Echelle*1000,
                                        (Xi+UY*Droite_Suivant-XMin)/Echelle*1000,
                                        (YMax-(Yi-UX*Droite_Suivant))/Echelle*1000]));
                  end;
                  if Couche=coLargeur then
                  begin
                    if (Form1.Les_Options_Plan.Type_Largeur=1) or AllLayers then
                    begin
                      if Visees[K].Longueur<>0 then begin
                        if Form1.Les_Options_Plan.Debut or AllLayers then begin
                          if Gauche_Precedent>-0.5 then
                            writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                                             [(X_Precedent-XMin)/Echelle*1000,
                                              (YMax-Y_Precedent)/Echelle*1000,
                                              (X_Precedent-UY*Gauche_Precedent-XMin)/Echelle*1000,
                                              (YMax-(Y_Precedent+UX*Gauche_Precedent))/Echelle*1000]));
                          if Droite_Precedent>-0.5 then
                            writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                                             [(X_Precedent-XMin)/Echelle*1000,
                                              (YMax-Y_Precedent)/Echelle*1000,
                                              (X_Precedent+UY*Droite_Precedent-XMin)/Echelle*1000,
                                              (YMax-(Y_Precedent-UX*Droite_Precedent))/Echelle*1000]));
                        end;
                        if Form1.Les_Options_Plan.Fin or AllLayers then begin
                          if Gauche_Suivant>-0.5 then
                            writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                                             [(Xi-XMin)/Echelle*1000,
                                              (YMax-Yi)/Echelle*1000,
                                              (Xi-UY*Gauche_Suivant-XMin)/Echelle*1000,
                                              (YMax-(Yi+UX*Gauche_Suivant))/Echelle*1000]));
                          if Droite_Suivant>-0.5 then
                            writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                                             [(Xi-XMin)/Echelle*1000,
                                              (YMax-Yi)/Echelle*1000,
                                              (Xi+UY*Droite_Suivant-XMin)/Echelle*1000,
                                              (YMax-(Yi-UX*Droite_Suivant))/Echelle*1000]));
                        end;
                      end;
                    end;
                    if (Form1.Les_Options_Plan.Type_Largeur=2) and not  AllLayers then
                    begin
                      if Visees[K].Longueur<>0 then begin
                        if Form1.Les_Options_Plan.Debut then begin
                          if Gauche_Precedent>-0.5 then
                            writeln(F,Format('<circle cx="%fmm" cy="%fmm" r="%fmm"/>',
                                              [(X_Precedent-UY*Gauche_Precedent-XMin)/Echelle*1000,
                                              (YMax-(Y_Precedent+UX*Gauche_Precedent))/Echelle*1000,
                                              Form1.Les_Options_Plan.Epaisseur_Largeur/20]));
                          if Droite_Precedent>-0.5 then
                            writeln(F,Format('<circle cx="%fmm" cy="%fmm" r="%fmm"/>',
                                              [(X_Precedent+UY*Droite_Precedent-XMin)/Echelle*1000,
                                              (YMax-(Y_Precedent-UX*Droite_Precedent))/Echelle*1000,
                                              Form1.Les_Options_Plan.Epaisseur_Largeur/20]));
                        end;
                        if Form1.Les_Options_Plan.Fin then begin
                          if Gauche_Suivant>-0.5 then
                            writeln(F,Format('<circle cx="%fmm" cy="%fmm" r="%fmm"/>',
                                              [(Xi-UY*Gauche_Suivant-XMin)/Echelle*1000,
                                              (YMax-(Yi+UX*Gauche_Suivant))/Echelle*1000,
                                              Form1.Les_Options_Plan.Epaisseur_Largeur/20]));
                          if Droite_Suivant>-0.5 then
                            writeln(F,Format('<circle cx="%fmm" cy="%fmm" r="%fmm"/>',
                                              [(Xi+UY*Droite_Suivant-XMin)/Echelle*1000,
                                              (YMax-(Yi-UX*Droite_Suivant))/Echelle*1000,
                                              Form1.Les_Options_Plan.Epaisseur_Largeur/20]));
                        end;
                      end;
                    end;
                  end;
                end;
                Gauche_Precedent:=Gauche_Suivant;
                Droite_Precedent:=Droite_Suivant;
                if ((Form1.Les_Options_Plan.Diametre<>0) or AllLayers) and (Couche=coStation) then
                  writeln(F,Format('<circle cx="%fmm" cy="%fmm" r="%fmm"/>',
                                    [(Xi-XMin)/Echelle*1000,
                                     (YMax-Yi)/Echelle*1000,
                                     Form1.Les_Options_Plan.Diametre/20]));
                X_Precedent:=Xi; Y_Precedent:=Yi;
              end;
              if Couche=coCarrefour then
              if ((Form1.Les_Options_Plan.Carrefour>0) or AllLayers) and not Arrivee.Terminus then begin
                Arrivee.Nom.lit(S);
                if Form1.Les_Options_Plan.Carrefour=2 then begin
                  S:=S+'('+IntTostr(Arrivee.Indice)+')';
                end;
                writeln(F,Format('<text x="%fmm" y="%fmm">%s</text>',
                                 [(X_Precedent-XMin)/Echelle*1000+Delta,
                                  (Ymax-Y_Precedent)/Echelle*1000+Delta,S]));
              end;
              writeln(F,'</g>'); // un groupe par galerie
            end;
          end;
          writeln(F,'</g>'); // un groupe par fiche
        end;
        writeln(F,'</g>');
      end;
      if Form1.Les_Options_Plan.Nord_Affiche or AllLayers then
      with Form1.Les_Options_Plan do begin
        writeln(F,Format('<g id="nord"%s>',[sLayer]));
        X0:=Nord_X*1000; Y0:=Nord_Y*1000;
        Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
        writeln(F,Format('<g style="stroke-width:%fmm; stroke:black;">',[Nord_Epaisseur/10])); // on fait un groupe pour runir les 2 trais de la flche
        write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Yi)/Echelle*1000]));
        if NG then begin
          Y0:=Y0+Nord_Longueur/1000*Echelle*cos(Form1.Massif.Angle_Geodesique/180*PI);
          X0:=X0-Nord_Longueur/1000*Echelle*sin(Form1.Massif.Angle_Geodesique/180*PI);
          Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
          writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Yi)/Echelle*1000]));
          write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Yi)/Echelle*1000]));
          Y0:=Y0-Nord_Longueur/1000*Echelle*
           (cos(Form1.Massif.Angle_Geodesique/180*PI)/2-sin(Form1.Massif.Angle_Geodesique/180*PI)/10);
          X0:=X0+Nord_Longueur/1000*Echelle*
           (sin(Form1.Massif.Angle_Geodesique/180*PI)/2+cos(Form1.Massif.Angle_Geodesique/180*PI)/10);
          Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
          writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Yi)/Echelle*1000]));
        end
        else begin
          Y0:=Y0+Nord_Longueur/1000*Echelle;
          Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
          writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Yi)/Echelle*1000]));
          write(F,Format('<line x1="%fmm" y1="%fmm" ',[(Xi-XMin)/Echelle*1000,(YMax-Yi)/Echelle*1000]));
          Y0:=Y0-Nord_Longueur/1000*Echelle/2; X0:=X0+Nord_Longueur/1000*Echelle/10;
          Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
          writeln(F,Format('x2="%fmm" y2="%fmm"/>',[(Xi-XMin)/Echelle*1000,(YMax-Yi)/Echelle*1000]));
        end;
        writeln(F,'</g>');
        writeln(F,Format('<g style="font-size:%dpt; font-family:%s;">',
                         [Nord_police.Size, Nord_police.Name])); // et un groupe pour le texte du nord
        Delta:=Form1.Les_Options_Plan.Nord_police.Size*25.4/72/2;
        writeln(F,Format('<text x="%fmm" y="%fmm">%s</text>',
                         [(Xi-XMin)/Echelle*1000+Delta,
                          (Ymax-Yi)/Echelle*1000+Delta,Texte_Nord]));
        writeln(F,'</g>');
        writeln(F,'</g>');
      end;
      if Form1.Les_Options_Plan.Echelle_Affiche or AllLayers then
      with Form1.Les_Options_Plan do begin
        writeln(F,Format('<g id="echelle"%s>',[sLayer]));
        X0:=Echelle_X*1000; Y0:=Echelle_Y*1000;
        Xi:=X0*cosH+Y0*sinH; Yi:=Y0*cosH-X0*sinH;
        writeln(F,Format('<g style="stroke-width:%fmm; stroke:black;">',[Echelle_Epaisseur/10])); // on fait un groupe pour runir les trais de l'chelle
        writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                         [(Xi-XMin)/Echelle*1000,
                          (YMax-Yi)/Echelle*1000,
                          (Xi+Echelle_Longueur-XMin)/Echelle*1000,
                          (YMax-Yi)/Echelle*1000]));
        writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                         [(Xi+Echelle_Longueur-XMin)/Echelle*1000,
                          (YMax-Yi)/Echelle*1000,
                          (Xi+Echelle_Longueur-XMin)/Echelle*1000,
                          (YMax-Yi)/Echelle*1000-Echelle_Largeur]));
        writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                         [(Xi-XMin)/Echelle*1000,
                          (YMax-Yi)/Echelle*1000,
                          (Xi-XMin)/Echelle*1000,
                          (YMax-Yi)/Echelle*1000-Echelle_Largeur]));
        if Echelle_Sub>1 then for I:=1 to Echelle_Sub-1 do begin
          writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                           [(Xi+Echelle_Longueur/Echelle_Sub*I-XMin)/Echelle*1000,
                            (YMax-Yi)/Echelle*1000,
                            (Xi+Echelle_Longueur/Echelle_Sub*I-XMin)/Echelle*1000,
                            (YMax-Yi)/Echelle*1000-Echelle_Largeur]));
        end;
        writeln(F,'</g>');
        writeln(F,Format('<g style="font-size:%dpt; font-family:%s; text-anchor: middle">',
                         [Echelle_police.Size, Echelle_police.Name]));// et un groupe pour le texte
        Delta:=Form1.Les_Options_Plan.Echelle_police.Size*25.4/72/2;
        writeln(F,Format('<text x="%fmm" y="%fmm">0</text>',
                         [(Xi-XMin)/Echelle*1000,
                          (Ymax-Yi)/Echelle*1000-Echelle_Largeur-Delta]));
        S:=IntTostr(Echelle_Longueur)+tuDistance[m];
        writeln(F,Format('<text x="%fmm" y="%fmm">%s</text>',
                         [(Xi+Echelle_Longueur-XMin)/Echelle*1000,
                          (Ymax-Yi)/Echelle*1000-Echelle_Largeur-Delta,S]));
        writeln(F,'</g>');
        writeln(F,'</g>');
      end;
      if Form1.Les_Options_Plan.Quadri_Affiche or AllLayers then
      with Form1.Les_Options_Plan do begin
        writeln(F,Format('<g id="quadrillage"%s style="stroke-width:%fmm; stroke:black;">',
                         [sLayer,Quadri_Epaisseur/10]));
        try
          X0:=Quadri_Longueur/1000*Echelle;
          Xi:=X0*cosH; Yi:=-X0*sinH;
          X0:=tronc(PMin.X/Quadri_Pas)*Quadri_Pas*cosH+tronc(PMin.Y/Quadri_Pas)*Quadri_Pas*sinH;
          Y0:=-tronc(PMin.X/Quadri_Pas)*Quadri_Pas*sinH+tronc(PMin.Y/Quadri_Pas)*Quadri_Pas*cosH;
          X1:=Quadri_Pas*cosH; Y1:=-Quadri_Pas*sinH;
          for I:=0 to tronc(PMax.X/Quadri_Pas)-tronc(PMin.X/Quadri_Pas)+1 do begin
            for J:=0 to tronc(PMax.Y/Quadri_Pas)-tronc(PMin.Y/Quadri_Pas)+1 do begin
              writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                               [(X0+I*X1-J*Y1-Xi-XMin)/Echelle*1000,
                                (YMax-(Y0+I*Y1+J*X1-Yi))/Echelle*1000,
                                (X0+I*X1-J*Y1+Xi-XMin)/Echelle*1000,
                                (YMax-(Y0+I*Y1+J*X1+Yi))/Echelle*1000]));
              writeln(F,Format('<line x1="%fmm" y1="%fmm" x2="%fmm" y2="%fmm"/>',
                               [(X0+I*X1-J*Y1-Yi-XMin)/Echelle*1000,
                                (YMax-(Y0+I*Y1+J*X1+Xi))/Echelle*1000,
                                (X0+I*X1-J*Y1+Yi-XMin)/Echelle*1000,
                                (YMax-(Y0+I*Y1+J*X1-Xi))/Echelle*1000]));
            end;
          end;
        except
        end;
        writeln(F,'</g>');
      end;
    end;
    writeln(F,'</svg>');
  finally
    CloseFile(F);
  end;
end;

end.

