{$N-}
unit Prefecra;

interface

uses SysUtils;

const

  coUnite   = 128;  // +128 pour les colonnes unit
  coDef     = 64;// +64 pour les colonnes O/N

  coPoint   = 1;
  coLong    = 2;
  coULong   = 3+coUnite;
  coAz      = 4;
  coUAz     = 5+coUnite;
  coPente   = 6;
  coUPente  = 7+coUnite;
  coLarg    = 8;
  coLargG   = 9;
  coLargD   = 10;
  coULarg   = 11+coUnite;
  coHaut    = 12;
  coHautH   = 13;
  coHautB   = 14;
  coUHaut   = 15+coUnite;
  coSchema  = 16;
  coComment = 17;
  coDefCor  = 18+coDef;
  coCorH    = 19;
  coDefDlt  = 20+coDef;
  coCorDlt  = 21;

type

  TSimple = record
    Taille, Taille_Unite, Ordre : SmallInt;
    Unite_Defaut : string[10];
  end;

  TComplique = record
    Taille, Taille_Unite, Ordre : SmallInt;
    Unite_Defaut : string[10];
    Separateur : char;
    Deux_Colonne : boolean;
  end;

  THabillage = record
    Taille, Ordre : SmallInt;
  end;

  THorizontale = record
    TailleDef, Taille, Ordre :SmallInt;
    Existe : boolean;
  end;
  Etat = record
    Intitule, Pixel : SmallInt;
    En_tete, Remplissage : string[25];
  end;


  TPreference_Ecran = class(TObject)
    Longueur, Azimuth, Pente : TSimple;
    Largeur, Hauteur : TComplique;
    Schema, Commentaire, Point : THabillage;
    Correction_Horizontale, Correction_Dlt : THorizontale;
    Nb_Colonne : SmallInt;
    Visuel : array[1..19] of Etat;
    Fichier : TFileName;
  public
    procedure Defaut;
    procedure Compile;
    procedure Enregistrer;
    procedure Recuperer; 
  end;


implementation

uses FicheTpo;

procedure TPreference_Ecran.Defaut;
begin
  with Point do begin
    Taille:=60;
    Ordre:=0;
  end;
  with Longueur do begin
    Taille:=50;
    Taille_Unite:=20;
    Ordre:=1;
    Unite_Defaut:=tuDistance[m];
  end;
  with Azimuth do begin
    Taille:=50;
    Taille_Unite:=20;
    Ordre:=2;
    Unite_Defaut:=tuAngle[deg];
  end;
  with Pente do begin
    Taille:=50;
    Taille_Unite:=20;
    Ordre:=3;
    Unite_Defaut:=tuAngle[deg];
  end;
  with Largeur do begin
    Taille:=50;
    Taille_Unite:=20;
    Ordre:=4;
    Unite_Defaut:=tuDistance[m];
    Separateur:='/';
    Deux_Colonne:=true;
  end;
  with Hauteur do begin
    Taille:=50;
    Taille_Unite:=20;
    Ordre:=5;
    Unite_Defaut:=tuDistance[m];
    Separateur:='/';
    Deux_Colonne:=false;
  end;
  with Schema do begin
    Taille:=15;
    Ordre:=6;
  end;
  with Commentaire do begin
    Taille:=100;
    Ordre:=7;
  end;
  with Correction_Horizontale do begin
    TailleDef:=20;
    Taille:=50;
    Ordre:=8;
    Existe:=false;
  end;
  with Correction_Dlt do begin
    TailleDef:=20;
    Taille:=50;
    Ordre:=9;
    Existe:=false;
  end;
end;

procedure TPreference_Ecran.Compile;
var
  I : SmallInt;
begin
  Nb_Colonne:=1;
  with Visuel[1] do begin
    Pixel:=Point.Taille;
    Intitule:=coPoint;
    En_Tete:='Point';
    Remplissage:='';
  end;
  for I:=1 to 9 do begin
    if Longueur.Ordre=I then begin
      with Visuel[Nb_Colonne+1] do begin
        Pixel:=Longueur.Taille;
        Intitule:=coLong;
        En_Tete:='Longueur';
        Remplissage:='';
      end;
      with Visuel[Nb_Colonne+2] do begin
        Pixel:=Longueur.Taille_Unite;
        Intitule:=coULONG;
        En_Tete:='';
        Remplissage:=Longueur.Unite_Defaut;
      end;
      inc(Nb_Colonne,2);
    end;
    if Azimuth.Ordre=I then begin
      with Visuel[Nb_Colonne+1] do begin
        Pixel:=Azimuth.Taille;
        Intitule:=coAZ;
        En_Tete:='Azimut';
        Remplissage:='';
      end;
      with Visuel[Nb_Colonne+2] do begin
        Pixel:=Azimuth.Taille_Unite;
        Intitule:=coUAZ;
        En_Tete:='';
        Remplissage:=Azimuth.Unite_Defaut;
      end;
      inc(Nb_Colonne,2);
    end;
    if Pente.Ordre=I then begin
      with Visuel[Nb_Colonne+1] do begin
        Pixel:=Pente.Taille;
        Intitule:=coPENTE;
        En_Tete:='Pente';
        Remplissage:='';
      end;
      with Visuel[Nb_Colonne+2] do begin
        Pixel:=Pente.Taille_Unite;
        Intitule:=coUPENTE;
        En_Tete:='';
        Remplissage:=Pente.Unite_Defaut;
      end;
      inc(Nb_Colonne,2);
    end;
    if Largeur.Ordre=I then begin
      if Largeur.Deux_Colonne then begin
        with Visuel[Nb_Colonne+1] do begin
          Pixel:=Largeur.Taille;
          Intitule:=coLARGG;
          En_Tete:='Largeur G';
          Remplissage:='';
        end;
        with Visuel[Nb_Colonne+2] do begin
          Pixel:=Largeur.Taille;
          Intitule:=coLARGD;
          En_Tete:='Largeur D';
          Remplissage:='';
        end;
        inc(Nb_Colonne,3);
      end
      else begin
        with Visuel[Nb_Colonne+1] do begin
          Pixel:=Largeur.Taille;
          Intitule:=coLARG;
          En_Tete:='Largeur';
          Remplissage:='';
        end;
        inc(Nb_Colonne,2);
      end;
      with Visuel[Nb_Colonne] do begin
        Pixel:=Largeur.Taille_Unite;
        Intitule:=coULARG;
        En_Tete:='';
        Remplissage:=Largeur.Unite_Defaut;
      end;
    end;
    if Hauteur.Ordre=I then begin
      if Hauteur.Deux_Colonne then begin
        with Visuel[Nb_Colonne+1] do begin
          Pixel:=Hauteur.Taille;
          Intitule:=coHAUTH;
          En_Tete:='Haut suprieure';
          Remplissage:='';
        end;
        with Visuel[Nb_Colonne+2] do begin
          Pixel:=Hauteur.Taille;
          Intitule:=coHAUTB;
          En_Tete:='Haut infrieure';
          Remplissage:='';
        end;
        inc(Nb_Colonne,3);
      end
      else begin
        with Visuel[Nb_Colonne+1] do begin
          Pixel:=Hauteur.Taille;
          Intitule:=coHAUT;
          En_Tete:='Hauteur';
          Remplissage:='';
        end;
        inc(Nb_Colonne,2);
      end;
      with Visuel[Nb_Colonne] do begin
        Pixel:=Hauteur.Taille_Unite;
        Intitule:=coUHAUT;
        En_Tete:='';
        Remplissage:=Hauteur.Unite_Defaut;
      end;
    end;
    if Schema.Ordre=I then begin
      inc(Nb_Colonne);
      with Visuel[Nb_Colonne] do begin
        Pixel:=Schema.Taille;
        Intitule:=coSCHEMA;
        En_Tete:='Croquis';
        Remplissage:='';
      end;
    end;
    if Commentaire.Ordre=I then begin
      inc(Nb_Colonne);
      with Visuel[Nb_Colonne] do begin
        Pixel:=Commentaire.Taille;
        Intitule:=coCOMMENT;
        En_Tete:='Observations';
        Remplissage:='';
      end;
    end;
    if (Correction_Horizontale.Ordre=I) and Correction_Horizontale.Existe then begin
      with Visuel[Nb_Colonne+1] do begin
        Pixel:=Correction_Horizontale.TailleDef;
        Intitule:=coDEFCOR;
        En_Tete:='';
        Remplissage:='N';
      end;
      with Visuel[Nb_Colonne+2] do begin
        Pixel:=Correction_Horizontale.Taille;
        Intitule:=coCORH;
        En_Tete:='Corr. Horizontale';
        Remplissage:='';
      end;
      inc(Nb_Colonne,2);
    end;
    if (Correction_Dlt.Ordre=I) and Correction_Dlt.Existe then begin
      with Visuel[Nb_Colonne+1] do begin
        Pixel:=Correction_Dlt.TailleDef;
        Intitule:=coDEFDLT;
        En_Tete:='';
        Remplissage:='N';
      end;
      with Visuel[Nb_Colonne+2] do begin
        Pixel:=Correction_Dlt.Taille;
        Intitule:=coCORDLT;
        En_Tete:='Dv. manuel';
        Remplissage:='';
      end;
      inc(Nb_Colonne,2);
    end;
  end;
end;

procedure TPreference_Ecran.Enregistrer;
var
  F : file;
begin
  assignFile(F,Fichier);
  Rewrite(F,1);
  BlockWrite(F,Longueur,SizeOf(Longueur));
  BlockWrite(F,Azimuth,SizeOf(Azimuth));
  BlockWrite(F,Pente,SizeOf(Pente));
  BlockWrite(F,Largeur,SizeOf(Largeur));
  BlockWrite(F,Hauteur,SizeOf(Hauteur));
  BlockWrite(F,Schema,SizeOf(Schema));
  BlockWrite(F,Commentaire,SizeOf(Commentaire));
  BlockWrite(F,Correction_Horizontale,SizeOf(Correction_Horizontale));
  BlockWrite(F,Correction_Dlt,SizeOf(Correction_Dlt));
  BlockWrite(F,Point,SizeOf(Point));
  CloseFile(F);
end;

procedure TPreference_Ecran.Recuperer;
var
  F : file;
begin
  if FileExists(Fichier) then begin
    assignFile(F,Fichier);
    Reset(F,1);
    BlockRead(F,Longueur,SizeOf(Longueur));
    BlockRead(F,Azimuth,SizeOf(Azimuth));
    BlockRead(F,Pente,SizeOf(Pente));
    BlockRead(F,Largeur,SizeOf(Largeur));
    BlockRead(F,Hauteur,SizeOf(Hauteur));
    BlockRead(F,Schema,SizeOf(Schema));
    BlockRead(F,Commentaire,SizeOf(Commentaire));
    BlockRead(F,Correction_Horizontale,SizeOf(Correction_Horizontale));
    BlockRead(F,Correction_Dlt,SizeOf(Correction_Dlt));
    BlockRead(F,Point,SizeOf(Point));
    CloseFile(F);

    // suite  un bug dans les version <=2.1.0.1, il peut tre ncessaire de dcaler tous les indices
    // vers le bas sauf celui du point

    // pour dtecter si on a une ancienne version du fichier d'option,
    // on regarde s'il existe une colonne  d' ordre=1
    if (Longueur.Ordre<>1) and (Azimuth.Ordre<>1) and (Pente.Ordre<>1) and
       (Largeur.Ordre<>1) and  (Hauteur.Ordre<>1) and (Schema.Ordre<>1) and
       (Commentaire.Ordre<>1) and  (Correction_Horizontale.Ordre<>1) and (Correction_Dlt.Ordre<>1) then
    begin // il faut corriger
      dec(Longueur.Ordre);
      dec(Azimuth.Ordre);
      dec(Pente.Ordre);
      dec(Largeur.Ordre);
      dec(Hauteur.Ordre);
      dec(Schema.Ordre);
      dec(Commentaire.Ordre);
      dec(Correction_Horizontale.Ordre);
      dec(Correction_Dlt.Ordre);
    end;
  end;
end;


end.
