unit Visuel;

interface

uses Windows, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, Fichetpo;

type
  TApercu = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Image1: TImage;
    OKBtn: TButton;
    CancelBtn: TButton;
    procedure Image1Click(Sender: TObject);
    procedure Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure Image1DblClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    XMin, XMax, YMin, YMax : double;
    Numero_Fiche, Numero_Galerie : smallInt;
    SX, SY : integer;
    Trouve, Surligne, Coupe : boolean;
    Les_Fiches : TList;
    PMin, PMax : TPoint3D;
    function Remplissage(La_Liste : TList):boolean;
    procedure MoveToRel(X, Y : double);
    procedure LineToRel(X, Y : double);
    procedure Dessine;
    procedure Propre;
    procedure Met_En_Relief(Ancien : boolean);
  end;

var
  Apercu: TApercu;

implementation

{$R *.DFM}

function TApercu.Remplissage(La_Liste : TList) : boolean;
begin
  Les_Fiches:=La_Liste;
  Trouve:=Calibre(PMin, PMax,La_Liste);
  Remplissage:=Trouve;
  XMin:=PMin.X; XMax:=PMax.X;
  YMin:=PMin.Y; YMax:=PMax.Y;
end;

procedure TApercu.MoveToRel(X, Y : double);
begin
  Image1.Canvas.MoveTo(round((X-Xmin)/(XMax-Xmin)*Image1.Width),
                                    round((Ymax-Y)/(YMax-Ymin)*Image1.Height));
end;

procedure TApercu.LineToRel(X, Y : double);
begin
  Image1.Canvas.LineTo(round((X-Xmin)/(XMax-Xmin)*Image1.Width),
                                    round((YMax-Y)/(YMax-Ymin)*Image1.Height));
end;

procedure TApercu.Dessine;
var
  I, J, K : integer;
  ContrainteX, ContrainteY, L, X0, Y0, X1, Y1, Delta, Ratio : double;
begin
  Panel1.Caption:='';
  Surligne:=false;
  OkBtn.Enabled:=false;
  with Image1.Canvas do begin
    Pen.Color:=clWhite;
    Brush.Color:=clWhite;
    Rectangle(0,0,Image1.Width,Image1.Height);
  end;
  XMin:=PMin.X; XMax:=PMax.X;
  YMin:=PMin.Y; YMax:=PMax.Y;
  Ratio:=(XMax-XMin)/(YMax-YMin)/Image1.Width*Image1.Height;
  if Ratio>1 then begin
    Delta:=YMax-YMin; Delta:=Delta*(Ratio-1)/2;
    YMin:=YMin-Delta; YMax:=YMax+Delta;
  end
  else begin
    Delta:=XMax-XMin; Delta:=Delta*(1/Ratio-1)/2;
    XMin:=XMin-Delta; XMax:=XMax+Delta;
  end;
  Delta:=YMax-YMin; Delta:=Delta*0.05;
  YMin:=YMin-Delta; YMax:=YMax+Delta;
  Delta:=XMax-XMin; Delta:=Delta*0.05;
  XMin:=XMin-Delta; XMax:=XMax+Delta;
  for I:=0 to les_Fiches.Count-1 do with TFiche(Les_Fiches[I]) do begin
    for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
      if Coupe and En_Coupe then Image1.Canvas.Pen.Color:=clSilver
      else Image1.Canvas.Pen.Color:=clBlack;
      if not Approximation then ContrainteX:=0;
      if Approximation then begin
        if Depart.Terminus or Arrivee.Terminus then begin
          ContrainteX:=0;
          ContrainteY:=0;
        end
        else begin
          ContrainteX:=((Arrivee.X-Depart.X)-DeltaX)/SigmaL;
          ContrainteY:=((Arrivee.Y-Depart.Y)-DeltaY)/SigmaL;
        end;
        L:=0;
        if Depart.Terminus then begin
          X0:=Arrivee.X-DeltaX; Y0:=Arrivee.Y-DeltaY;
        end
        else begin
          X0:=Depart.X; Y0:=Depart.Y;
        end;
        MoveToRel(X0,Y0);
        for K:=1 to Nb_Visee do begin
          if Visees[K].ULong=cm then L:=L+Visees[K].Longueur/100 else L:=L+Visees[K].Longueur;
          X1:=X0+Visees[K].X+ContrainteX*L;
          Y1:=Y0+Visees[K].Y+ContrainteY*L;
          LineToRel(X1, Y1);
        end;
      end;
    end;
  end;
end;


procedure TApercu.Propre;
begin
end;

procedure TApercu.Image1Click(Sender: TObject);
var
  I, J, K : integer;
  X0, Y0, X1, Y1, X2, Y2, DMin, Zone, RX, RY : double;
  Une_Fiche : TFiche;
  Une_Galerie : TGalerie;
begin
  if Surligne then Met_En_Relief(true);
  Rx:=(SX*1.0/Image1.Width)*(Xmax-XMin)+XMin;
  Ry:=(SY*1.0/Image1.Height)*(Ymin-Ymax)+YMax;
  Trouve:=false;
  for I:=0 to Les_Fiches.Count-1 do begin
    Une_Fiche:=Les_Fiches[I];
    for J:=0 to Une_Fiche.Liste_Galerie.Count-1 do begin
      Une_Galerie:=Une_Fiche.Liste_Galerie[J];
      if Une_Galerie.Approximation and not(Coupe and Une_Galerie.En_Coupe) then with Une_Galerie do begin
        X0:=Depart.X; Y0:=Depart.Y;
        if not Trouve then begin
          Trouve:=true;
          Dmin:=sqr((X0-Rx))+sqr((Y0-Ry));
          Numero_Fiche:=I; Numero_Galerie:=J;
        end
        else if sqr((X0-Rx))+sqr((Y0-Ry))<Dmin then begin
          Dmin:=sqr((X0-Rx))+sqr((Y0-Ry));
          Numero_Fiche:=I; Numero_Galerie:=J;
        end;
        X1:=X0; Y1:=Y0;
        for K:=1 to Nb_Visee do begin
          X2:=Coordonnees^[K].X;
          Y2:=Coordonnees^[K].Y;
          Zone:=(Rx-X1)*(X2-X1)+(Ry-Y1)*(Y2-Y1);
          if Zone>sqr(X2-X1)+sqr(Y2-Y1) then begin
            if sqr((X2-Rx))+sqr((Y2-Ry))<Dmin then begin
              Dmin:=sqr((X2-Rx))+sqr((Y2-Ry));
              Numero_Fiche:=I; Numero_Galerie:=J;
            end;
          end
          else if (Zone>0) and ((X1<>X2) or (Y1<>Y2)) then begin
            Zone:=(X2-X1)*(Ry-Y1)-(Y2-Y1)*(Rx-X1);
            Zone:=sqr(Zone)/(sqr(X2-X1)+sqr(Y2-Y1));
            if Zone<Dmin then begin
              Dmin:=Zone;
              Numero_Fiche:=I; Numero_Galerie:=J;
            end;
          end;
          X1:=X2; Y1:=Y2;
        end;
      end;
    end;
  end;
  Surligne:=true;
  Met_En_Relief(false);
  OkBtn.Enabled:=true;
  OkBtn.SetFocus;
end;

procedure TApercu.Met_En_Relief(Ancien : boolean);
var
  K : integer;
  ContrainteX, ContrainteY, L, X0, Y0, X1, Y1 : double;
  Une_Fiche : TFiche;
  Une_Galerie : TGalerie;
  S, S2 : string;
begin
  Une_Fiche:=Les_Fiches[Numero_Fiche];
  Une_Galerie:=Une_Fiche.Liste_Galerie[Numero_Galerie];
  if not Ancien then begin
    Image1.Canvas.Pen.Color:=clRed;
    Une_Fiche.Systeme.lit(S);
    Une_Fiche.Cavite.lit(S2); S:=S+'/'+S2;
    Une_Fiche.Reseau.lit(S2); S:=S+'/'+S2;
    Une_Fiche.Lieu.lit(S2); S:=S+'/'+S2;
    if Une_Galerie.Depart.Terminus then S2:='Terminus'
                                   else Une_Galerie.Depart.Nom.lit(S2);
    S:=S+'  '+S2+' -> ';
    if Une_Galerie.Arrivee.Terminus then S2:='Terminus'
                                    else Une_Galerie.Arrivee.Nom.lit(S2);
    S:=S+S2;
    Panel1.Caption:=S;
  end;
  if not Une_Galerie.Approximation then begin
    ContrainteX:=0;
  end;
  if Une_Galerie.Approximation then with Une_Galerie do begin
    if Depart.Terminus or Arrivee.Terminus then begin
      ContrainteX:=0;
      ContrainteY:=0;
    end
    else begin
      ContrainteX:=((Arrivee.X-Depart.X)-DeltaX)/SigmaL;
      ContrainteY:=((Arrivee.Y-Depart.Y)-DeltaY)/SigmaL;
    end;
    L:=0;
    if Depart.Terminus then begin
      X0:=Arrivee.X-DeltaX; Y0:=Arrivee.Y-DeltaY;
    end
    else begin
      X0:=Depart.X; Y0:=Depart.Y;
    end;
    MoveToRel(X0,Y0);
    for K:=1 to Nb_Visee do begin
      if Visees[K].ULong=cm then L:=L+Visees[K].Longueur/100 else L:=L+Visees[K].Longueur;
      X1:=X0+Visees[K].X+ContrainteX*L;
      Y1:=Y0+Visees[K].Y+ContrainteY*L;
      LineToRel(X1, Y1);
    end;
  end;
  Image1.Canvas.Pen.Color:=clBlack;
end;

procedure TApercu.Image1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  SX:=X; SY:=Y;
end;

procedure TApercu.Image1DblClick(Sender: TObject);
begin
  ModalResult:=mrOk;
end;

procedure TApercu.FormCreate(Sender: TObject);
var
  BitMap : TBitMap;
begin
  BitMap:=TBitMap.Create;
  BitMap.Width:=Image1.Width; BitMap.Height:=Image1.Height;
  Image1.Picture.Graphic := Bitmap;
end;

procedure TApercu.FormResize(Sender: TObject);
begin
  Image1.Picture.Graphic.Width:=Image1.Width;
  Image1.Picture.Graphic.Height:=Image1.Height;
  Dessine;
end;

end.
