unit sendkey;

{$X+,D-}

{Cette unit doit tre incluse dans l'application que vous voulez utiliser
}

interface

uses Windows, Classes, SysUtils, Forms, KeyDefs;

type
  { Return values for SendKeys function }
  TSendKeyError = (sk_None, sk_InvalidToken, sk_UnknownError);
  { exceptions }
  ESendKeyError = class(Exception);
  EInvalidToken = class(ESendKeyError);

var
  { variables global to the DLL }
  AltPressed, ControlPressed, ShiftPressed: Boolean;
  AltPrtScr: Boolean;

//  NextSpecialKey: TKeyString;

// procedure Keybd_Event; far; external 'USER32' name 'KEYBD_EVENT';

function SendKeys(S: String): TSendKeyError;
procedure SimulateKeyPresses(VKeyCode: Word);


implementation


function MakeWord(L, H: Byte): Word;
{ macro creates a word from low and high bytes }
begin
  MakeWord := (H shl 8) + L;
end;

procedure PostVKey(bVirtKey: Byte; Up: Boolean);
var
  dwFlags : DWORD;
  bScan   : Byte;
begin
  if Up
  then dwFlags := KEYEVENTF_KEYUP
  else dwFlags := 0;
  if AltPrtScr
  then bScan := 1
  else bScan := 0;
  Keybd_Event(bVirtKey, bScan, dwFlags, 0);
end;

procedure KeyDown(vKey: byte);
begin
  PostVKey(vKey, False);
end;

procedure KeyUp(vKey: byte);
begin
  PostVKey(vKey, True);
end;

procedure SimulateKeyPresses(VKeyCode: Word);
{ This function simulates keypresses for the given key, taking into }
{ account the current state of Alt, Control, and Shift keys }

begin
  { press Alt key if flag has been set }
  if AltPressed then
    KeyDown(vk_Menu);

  { press Control key if flag has been set }
  if ControlPressed then
    KeyDown(vk_Control);

  { if shift is pressed, or shifted key and control is not pressed... }
  if (((Hi(VKeyCode) and 1) <> 0) and (not ControlPressed)) or ShiftPressed
then
    KeyDown(vk_Shift);    { ...press shift }

  KeyDown(Lo(VKeyCode));  { press key down }
  KeyUp(Lo(VKeyCode));    { release key }

  { if shift is pressed, or shifted key and control is not pressed... }
  if (((Hi(VKeyCode) and 1) <> 0) and (not ControlPressed)) or ShiftPressed
then
    KeyUp(vk_Shift);      { ...release shift }

  { if shift flag is set, reset flag }
  if ShiftPressed then
  begin
    ShiftPressed := False;
  end;

  { Release Control key if flag has been set, reset flag }
  if ControlPressed then
  begin
    KeyUp(vk_Control);
    ControlPressed := False;
  end;

  { Release Alt key if flag has been set, reset flag }
  if AltPressed then
   begin
    KeyUp(vk_Menu);
    AltPressed := False;
  end;
  // attend que la touche soit execute avant de poursuivre
  Application.ProcessMessages;
end;

procedure ProcessKey(S: String);
{ This function parses each character in the string to create the message
list }
var
  KeyCode: word;
  Key: byte;
  index: integer;
  Token: TKeyString;
begin
  index := 1;
  repeat
    case S[index] of

      KeyGroupOpen :
        begin
          { It's the beginning of a special token! }
          Token := '';
          inc(index);
          while S[index] <> KeyGroupClose do
          begin
            { add to Token until the end token symbol is encountered }
            Token := Token + S[index];
            inc(index);
            { check to make sure the token's not too long }
            if (Length(Token) = 7) and (S[index] <> KeyGroupClose) then
              raise EInvalidToken.Create('Acolade fermante absente "}"');
          end; // while

          { look for token in array, Key parameter will }
          { contain vk code if successful }
          if not FindKeyInArray(UpperCASE(Token), Key) then
            raise EInvalidToken.Create('Token invalide');
          // Special Print Screen
          AltPrtScr := Token = 'PRTSCR';
          { simulate keypress sequence }
          if Key <> 0
          then SimulateKeyPresses(MakeWord(Key, 0))
          else Application.ProcessMessages;
        end;

      AltKey :
        begin
          { set Alt flag }
          AltPressed := True;
        end;

      ControlKey :
        begin
          { set Control flag }
          ControlPressed := True;
        end;

      ShiftKey :
        begin
          { set Shift flag }
          ShiftPressed := True;
        end;

      else
        begin
          { A normal character was pressed }
          KeyCode := vkKeyScan(Char(Byte(S[index])));
          { simulate keypress sequence }
          SimulateKeyPresses(KeyCode);
        end;
    end;
    inc(index);
  until index > Length(S);
end;

function SendKeys(S: String): TSendKeyError; export;
{ This is the one entry point.  Based on the string passed in the S  }
{ parameter, this function creates a list of keyup/keydown messages, }
begin
  Result := sk_None;                   { assume success }
  try
    ProcessKey(S);
  except
    { if an exception occurs, return an error code, and clean up }
    on E:ESendKeyError do
      begin
        if E is EInvalidToken then
          Result := sk_InvalidToken;
      end
      else
        { Catch-all exception handler ensures than an exception }
        { doesn't walk up into application stack }
        Result := sk_UnknownError;
  end; // except
end;


end.

