unit toporobot;

interface

uses SysUtils, Classes, Dialogs;

procedure Importer_TopoRobot(fichier : TfileName);
procedure Exporter_TopoRobot(fichier : TfileName);

implementation

uses Fichetpo, Unit1;

type

  TExpe = record
    Date : string[8];
    SpeleoMetre, Speleographe, Commentaire : string;
    Declinaison : single;
  end;

  TCode = record
    Horizontale, Verticale : integer;
  end;

  TSerie = class
    Les_Visees : TList;
    Info_Expe : array[0..998] of integer;
    Nom : string;
    S_Depart, V_Depart, S_Arrivee, V_Arrivee : integer;
    Vide : boolean;
    constructor Create;
    destructor Destroy; override;
  end;

  TRobotVisee = record
    Expe, Code : integer;
    Longueur, Azimut,Pente, Lgauche, LDroite, Haut, Bas : single;
    Commentaire : TEco;
  end;

var

  F : file;
//  Expe_Fictive : TExpe;
  Code_Fictive : TCode;
  N_Expe, N_Code : integer;

constructor TSerie.Create;
begin
  inherited Create;
  Les_Visees:=TList.Create;
  Vide:=true;
end;

destructor TSerie.Destroy;
begin
  Les_Visees.Free;
  inherited Destroy;
end;

function Lit_Ligne(var N : integer;var Contenu : string) : boolean;
var
  C : char;
begin
  N:=0;
  if Eof(F) then Lit_Ligne:=false
  else begin
    Contenu:='';
    repeat
      blockRead(F,C,sizeOf(C));
      if C=#9 then inc(N);
      case C of
        #136 : C:='';
        #142 : C:='';
        #143 : C:='';
        #144 : C:='';
        #137 : C:='';
        #145 : C:='';
        #148 : C:='';
        #149 : C:='';
        #157 : C:='';
        #159 : C:='';
        #141 : C:='';
      end;
      if (C<>#13) and (C<>#10) then Contenu:=Contenu+C;
    until Eof(F) or (C=#13) or (C=#10);
    Lit_Ligne:=true;
    Contenu:=Contenu+#0;
  end;
end;

function Colonne_Suivante(var Contenu : string) : string;
var
  Position : integer;
begin
  Position:=1;
  while (Position<length(Contenu)) and (Contenu[Position]<>#9) do inc(Position);
  Colonne_Suivante:=Copy(Contenu,1,Position-1);
  Contenu:=Copy(Contenu,Position+1,length(Contenu)-Position);
end;

function Lineaire(S : string):double;
var
  Somme, Ajout : reel;
  SP : string[2];
  Erreur : integer;
begin
  SP:=S[7]+S[8];
  val(SP,Somme,Erreur);
  SP:=S[4]+S[5];
  val(SP,Ajout,Erreur);
  if Erreur=0 then Somme:=Somme+(Ajout-1)/12;
  SP:=S[1]+S[2];
  val(SP,Ajout,Erreur);
  if Erreur=0 then Somme:=Somme+(Ajout-1)/365;
  Lineaire:=Somme;
end;

procedure Importer_TopoRobot(fichier : TfileName);
var
  N, I, J, K, L, Erreur, Nb_Visee, LeCode, Nb_Correspondance, Expe_En_Cours : integer;
  S, Ptt_Bout : string;
  Cavites : TStringList;
  Entree : TPointTopo;
  Ptt_Reel : double;
  Fin : boolean;

  Les_Expes : array[1..998] of TExpe;
  Les_Codes : array[1..998] of TCode;
  Correspondance : array[1..10000,1..4] of integer; { Srie, Station, Prcdent, Indice }
  Les_Series, Intermediaire : TList;
  Une_Serie : TSerie;
  Une_Visee : TVisee;
  Une_Fiche : TFiche;
  Une_Galerie : Tgalerie;
  Topog : TEco;

  function Nouvelle_Correspondance(Ser, Vis : integer) : integer;
  // attribue un numro au point Ser(ie) Vis(ee)
  // si le point existe dj, fourni l'ancien numro
  // sinon, en cre un nouveau
  var
    I : integer;
  begin
    I:=1;
    while (I<=Nb_Correspondance) and ((Correspondance[I,1]<>Ser) or
      (Correspondance[I,2]<>Vis)) do inc(I);
    if I>Nb_Correspondance then begin
      inc(Nb_Correspondance);
      Correspondance[Nb_Correspondance,1]:=Ser;
      Correspondance[Nb_Correspondance,2]:=Vis;
    end;
    Nouvelle_Correspondance:=I;
  end;

  function Numero_Correspondance(Ser, Vis : integer) : integer;
  // indique le numro globale attribu  Ser/Vis
  // les entres multiples sont au dbut de la liste
  var
    I : integer;
  begin
    repeat
      I:=1;
      while (I<=Nb_Correspondance) and ((Correspondance[I,1]<>Ser) or
        (Correspondance[I,2]<>Vis)) do inc(I);
      if Correspondance[I,3]<>0 then begin
        Ser:=Correspondance[Correspondance[I,3],1];
        Vis:=Correspondance[Correspondance[I,3],2];
      end;
    until Correspondance[I,3]=0;
    Numero_Correspondance:=I;
  end;

  procedure Supprime_Terminus;
  var
    Compteur : array[0..10000] of integer;
    LesPoints : TList;
    I, J, K : integer;
    Echec : boolean;
  begin
    LesPoints:=TList.Create;
    for I:=0 to Intermediaire.Count-1 do with TFiche(Intermediaire[I]) do begin
      for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
        if not Depart.Terminus then begin
          K:=0;
          while (K<LesPoints.Count) and not TPointTopo(LesPoints[K]).Identique(Depart) do inc(K);
          if K<LesPoints.Count then inc(Compteur[K])
          else begin
            Compteur[K]:=1;
            LesPoints.Add(Depart);
          end;
        end;
        if not Arrivee.Terminus then begin
          K:=0;
          while (K<LesPoints.Count) and not TPointTopo(LesPoints[K]).Identique(Arrivee) do inc(K);
          if K<LesPoints.Count then inc(Compteur[K])
          else begin
            Compteur[K]:=1;
            LesPoints.Add(Arrivee);
          end;
        end;
      end;
    end;

    for I:=0 to LesPoints.Count-1 do if Compteur[I]=1 then with TPointTopo(LesPoints[I]) do begin
      Echec:=false;
      for J:=0 to Form1.Massif.Liste_Reference.Count-1 do
         if Identique(TPointTopo(Form1.Massif.Liste_Reference[J])) then Echec:=true;
      if not Echec then begin
        Terminus:=true;
        Nom.Ecrit('');
      end;
    end;
    LesPoints.Free;
  end;

  procedure Nouvelle_Entree;
  var
    I : integer;
  begin
    if Form1.Massif.Liste_Reference.Count<>0 then begin
      I:=0;
      repeat
        if TPointTopo(Form1.Massif.Liste_Reference[I]).Identique(Entree) then begin
          inc(Entree.Indice); I:=-1;
        end;
        inc(I);
      until I>=Form1.Massif.Liste_Reference.Count;
    end;
    Form1.Massif.Liste_Reference.Add(Entree);
  end;


begin

  for I:=1 to 10000 do for N:=1 to 4 do Correspondance[I,N]:=0;
  Nb_Correspondance:=0;

  assignFile(F,fichier);
  Reset(F,1);

(****** Cavite ligne -6 *********)
  Cavites:=TStringList.Create;
  Lit_Ligne(N,S);
  while Colonne_Suivante(S)='-6' do begin
    Colonne_Suivante(S);
    Ptt_Bout:=Colonne_Suivante(S);
    if Ptt_Bout='' then
      Ptt_Bout:=IntToStr(Cavites.Count+1);
    Cavites.Add(Ptt_Bout);
    Lit_Ligne(N,S);
  end;

(********* Point d'entre ligne -5 *****************)
  S:='-5'+#9+S;
  while Colonne_Suivante(S)='-5' do begin
    Entree:=TPointTopo.Create;
    with Entree do begin
      Terminus:=false;
      Ptt_Bout:=Colonne_Suivante(S);

      val(Colonne_Suivante(S),X,Erreur); val(Colonne_Suivante(S),Y,Erreur); val(Colonne_Suivante(S),Z,Erreur);
      X:=X/1000; Y:=Y/1000;
      Nom.Ecrit('Entre '+Cavites[StrToInt(Ptt_Bout)-1]);
      Nouvelle_Entree;
      inc(Nb_Correspondance);
      Correspondance[Nb_Correspondance,1]:=StrToINt(Colonne_Suivante(S));
      Correspondance[Nb_Correspondance,2]:=StrToINt(Colonne_Suivante(S));
      Indice:=1;
    end;
    Lit_Ligne(N,S);
  end;

(************** Exps *********************)
  N_Expe:=0;
  S:='-2'+#9+S;
  while Colonne_Suivante(S)='-2' do begin
    N_Expe:=StrToInt(Colonne_Suivante(S));
    if N_Expe=999 then begin
{      with Expe_Fictive do begin
      end; }
    end
    else begin
      with Les_Expes[N_Expe] do begin
        Ptt_Bout:=Colonne_Suivante(S);
        if length(Ptt_Bout)<2 then Date:='0'+Ptt_Bout else Date:=Ptt_Bout;
        Ptt_Bout:=Colonne_Suivante(S);
        if length(Ptt_Bout)<2 then Date:=Date+'/0'+Ptt_Bout else Date:=Date+'/'+Ptt_Bout;
        Ptt_Bout:=Colonne_Suivante(S);
        if length(Ptt_Bout)<2 then Date:=Date+'/0'+Ptt_Bout else Date:=Date+'/'+Ptt_Bout;
        Speleometre:=Colonne_Suivante(S);
        Speleographe:=Colonne_Suivante(S);
        if Colonne_Suivante(S)='1' then begin // calcul auto de la dclinaison
          Declinaison:=Lineaire(Date)-Lineaire(Form1.Massif.Date_Decli);
          if Lineaire(Date)<50 then Declinaison:=Declinaison+100;
          Declinaison:=Form1.Massif.Origine_Decli-Form1.Massif.Variation_Decli*Declinaison;
          Declinaison:=Declinaison/0.9;
        end
        else Declinaison:=0;
        val(Colonne_Suivante(S),Ptt_Reel,Erreur);
        Declinaison:=Declinaison+Ptt_Reel - Form1.Massif.Angle_Geodesique;
        Colonne_Suivante(S); Colonne_Suivante(S);
        Commentaire:=Colonne_Suivante(S);
      end;
    end;
    Lit_Ligne(N,S);
  end;


(*************** Codes *******************)
  N_Code:=0;
  S:='-1'+#9+S;
  while Colonne_Suivante(S)='-1' do begin
    N_Code:=StrToInt(Colonne_Suivante(S));
    if N_Code=999 then begin
      with Code_Fictive do begin
        val(Colonne_Suivante(S),Ptt_Reel,Erreur);
        Horizontale:=round(Ptt_Reel);
        val(Colonne_Suivante(S),Ptt_Reel,Erreur);
        Verticale:=round(Ptt_Reel);
      end;
    end
    else begin
      with Les_Codes[N_Code] do begin
        val(Colonne_Suivante(S),Ptt_Reel,Erreur);
        Horizontale:=round(Ptt_Reel);
        val(Colonne_Suivante(S),Ptt_Reel,Erreur);
        Verticale:=round(Ptt_Reel);
      end;
    end;
    Lit_Ligne(N,S);
  end;

(****************** Sries *******************)
  Les_Series:=TList.Create;
  repeat
    Form1.StatusBar.Panels[0].Text:=
      Format('Lecture de la %d srie',[Les_Series.Count+1]);
    Lit_Ligne(N,S);  //on lit la premire ligne
    I:=StrToInt(Colonne_Suivante(S)); //on rcupre le numro de srie
    repeat // on remplit avec des sries vides s'il y des trous dans la liste des sries
      Une_Serie:=TSerie.Create;
      Les_Series.Add(Une_Serie);
    until Les_Series.Count=I;
    with Une_Serie do begin
      Vide:=false;
      Colonne_Suivante(S);
      S_Depart:=StrToInt(Colonne_Suivante(S)); V_Depart:=StrToInt(Colonne_Suivante(S));
      S_Arrivee:=StrToInt(Colonne_Suivante(S)); V_Arrivee:=StrToInt(Colonne_Suivante(S));
      Nb_Visee:=StrToInt(Colonne_Suivante(S));
      Colonne_Suivante(S); Colonne_Suivante(S);
      Nom:=Colonne_Suivante(S);

      for I:=0 to Nb_Visee do begin
        Lit_Ligne(N,S);
        Une_Visee:=TVisee.Create;
        Colonne_Suivante(S); Colonne_Suivante(S);
        LeCode:=StrToInt(Colonne_Suivante(S));
        Info_Expe[I]:=StrToInt(Colonne_Suivante(S));
        if (LeCode=999) and (Info_Expe[I]=999) then begin
          if I<>0 then begin  (**** vise fictive *******)
            Entree:=TPointTopo.Create;  // on cre une nouvelle entre
            with Entree do begin
              Terminus:=false;
              Nom.Ecrit(IntToStr(Les_Series.Count)+'_'+IntToStr(I));
              Indice:=Form1.Massif.Nouveau_Point(IntToStr(Les_Series.Count)+'_'+IntToStr(I),0);
              inc(Nb_Correspondance);
              Correspondance[Nb_Correspondance,1]:=Les_Series.Count;
              Correspondance[Nb_Correspondance,2]:=I;
              Correspondance[Nb_Correspondance,4]:=Indice;

              Colonne_Suivante(S); Colonne_Suivante(S); Colonne_Suivante(S);
              Colonne_Suivante(S); Colonne_Suivante(S); Colonne_Suivante(S);
              Colonne_Suivante(S); Ptt_Bout:=Colonne_Suivante(S);
              val(copy(Ptt_Bout,1,pos(' ',Ptt_Bout)-1),X,Erreur);
              Ptt_Bout:=copy(Ptt_Bout,pos(' ',Ptt_Bout)+1,length(Ptt_Bout));
              while Ptt_Bout[1]=' ' do Ptt_Bout:=copy(Ptt_Bout,2,length(Ptt_Bout));
              val(copy(Ptt_Bout,1,pos(' ',Ptt_Bout)-1),Y,Erreur);
              Ptt_Bout:=copy(Ptt_Bout,pos(' ',Ptt_Bout)+1,length(Ptt_Bout));
              val(Ptt_Bout,Z,Erreur);
              X:=X/1000; Y:=Y/1000;
            end;
            Nouvelle_Entree;
            if I=1 then  // parfois, la vise 0 qui prcde la vise fictive n'est pas fictive
              Info_Expe[0]:=999;  // alors on force la vise 0  fictive
          end;
        end
        else with Une_Visee do begin
          val(Colonne_Suivante(S),Longueur,Erreur);
          if Les_Codes[LeCode].Horizontale mod 10 = 8 then Longueur:=Longueur*0.3048;
          if Les_Codes[LeCode].Horizontale mod 10 = 7 then begin
            Longueur:=int(Longueur)*0.3048 + frac(Longueur)*100*0.0254;
          end;
          ULong:=m;
          val(Colonne_Suivante(S),Azimuth,Erreur);
          case Les_Codes[LeCode].Horizontale of
            360, 359, 358, 357 : UAzimuth:=deg;
            350, 349, 348, 347 : UAzimuth:=ndeg;
            400, 399, 398, 397 : UAzimuth:=gr;
            390, 389, 388, 387 : UAzimuth:=ngr;
            else begin
              ShowMessage('Code azimut inconnu : '+IntToStr(Les_Codes[LeCode].Horizontale));
            end;
          end;
          val(Colonne_Suivante(S),Pente,Erreur);
          case Les_Codes[LeCode].Verticale of
            360 : UPente:=deg;
            400 : UPente:=gr;
            380 : UPente:=mp;
            370 : begin
              Pente:=Pente*Longueur/100;
              UPente:=mp;
            end;
            401 : begin
              Pente:=100-Pente;
              UPente:=gr;
            end;
            361 : UPente:=VDeg;
            399 : begin
              Pente:=Pente-100;
              UPente:=gr;
            end;
            359 : begin
              Pente:=Pente-90;
              UPente:=deg;
            end;
            else begin
              ShowMessage('Code pente inconnu : '+IntToStr(Les_Codes[LeCode].Verticale));
            end;
          end;
          LargeurGauche:=Colonne_Suivante(S); LargeurDroite:=Colonne_Suivante(S);
          ULarg:=m;
          HauteurSup:=Colonne_Suivante(S); HauteurInf:=Colonne_Suivante(S);
          UHauteur:=m;
          Ptt_Bout:=Colonne_Suivante(S);
          if (length(Ptt_Bout)>2) and (Ptt_Bout[1]='{') then Ptt_Bout:=Colonne_Suivante(S);
          Commentaires.Ecrit(Ptt_Bout);
          Schema:=0;
          Correction_Horizontale:=false; Correction_Dlt:=false; Topofil:=false;
        end;
        Les_Visees.Add(Une_Visee);
      end;
    end;
    Lit_Ligne(N,S);
  until Eof(F);
  CloseFile(F);

(*************** Etablissement des correspondances **************************)

  for I:=0 to Les_Series.Count-1 do with TSerie(Les_Series[I]) do begin
    if (I+1=S_Depart) and (V_Depart=0) then
    begin // la srie commence avec sa propre numrotation, c'est un point topo en soi
      Nouvelle_Correspondance(I+1,0);
    end
    else begin // la srie par depuis une autre srie
      N:=Nouvelle_Correspondance(S_Depart,V_Depart); // on rcupre le numro dj attribu au point correspondant
      Nouvelle_Correspondance(I+1,0);  // on cre
      Correspondance[Nb_Correspondance,3]:=N;
    end;
    if (I+1=S_Arrivee) and (V_Arrivee=Les_Visees.Count-1) then
    begin // la srie fini avec sa propre numrotation
      Nouvelle_Correspondance(I+1,Les_Visees.Count-1);
    end
    else begin // la srie rejoint une autre srie
      if S_Arrivee<I+1 then begin // une srie antrieure
        N:=Nouvelle_Correspondance(S_Arrivee,V_Arrivee);
        Nouvelle_Correspondance(I+1,Les_Visees.Count-1);
        Correspondance[Nb_Correspondance,3]:=N;
      end
      else begin // rejoint un point futur encore inconnu
        N:=Nouvelle_Correspondance(I+1,Les_Visees.Count-1);
        Nouvelle_Correspondance(S_Arrivee,V_Arrivee);
        Correspondance[Nb_Correspondance,3]:=N;
      end;
    end;
  end;

(******** Dtermination des indices *****************)

  Form1.StatusBar.Panels[0].Text:='Etablissement des correspondances';
  for I:=2 to Nb_Correspondance do
   if (Correspondance[I,3]=0) and (Correspondance[I,4]=0) then
    Correspondance[I,4]:=Form1.Massif.Nouveau_Point(IntToStr(Correspondance[I,1])+'_'+IntToStr(Correspondance[I,2]),0);

(********** Cration des fiches et des galerie *************)


  Form1.StatusBar.Panels[0].Text:='Cration des fiches';
  Intermediaire:=TList.Create;
  Une_Fiche:=TFiche.Create;
  I:=0;
  repeat // on avance jusqu' ce qu'on trouve un code d'exp non-fictive
    with TSerie(Les_Series[I]) do begin
      N:=0;
      while (N<Les_Visees.Count) and (Info_Expe[N]=999) do inc(N);
      if N=Les_Visees.Count then begin
        inc(I);
        N:=0;
      end;
    end;
  until TSerie(Les_Series[I]).Info_Expe[N]<>999;
  Expe_En_Cours:=TSerie(Les_Series[I]).Info_Expe[N];

  with Les_Expes[Expe_En_Cours] do begin
    Une_Fiche.Unite_Decli:=gr;
    Une_Fiche.Date:=Date;
    Une_Fiche.Declinaison:=Declinaison;
    Topog:=TEco.Create;
    Topog.Ecrit(SpeleoMetre);
    Une_Fiche.Topographes.Add(Topog);
    Topog:=TEco.Create;
    Topog.Ecrit(SpeleoGraphe);
    Une_Fiche.Topographes.Add(Topog);
    Une_Fiche.Cavite.Ecrit(Cavites[0]);
  end;
  for I:=0 to Les_Series.Count-1 do with TSerie(Les_Series[I]) do begin

    N:=0;
    repeat
      while (N<Les_Visees.Count) and (Info_Expe[N]=999) do inc(N);
      if N<Les_Visees.Count then begin
        if Info_Expe[N]<>Expe_En_Cours then begin // Changement de fiche
          Intermediaire.Add(Une_Fiche);
          Une_Fiche:=TFiche.Create;
          Expe_En_Cours:=Info_Expe[N];
          with Les_Expes[Expe_En_Cours] do begin
            Une_Fiche.Unite_Decli:=gr;
            Une_Fiche.Date:=Date;
            Une_Fiche.Declinaison:=Declinaison;
            Topog:=TEco.Create;
            Topog.Ecrit(SpeleoMetre);
            Une_Fiche.Topographes.Add(Topog);
            Topog:=TEco.Create;
            Topog.Ecrit(SpeleoGraphe);
            Une_Fiche.Topographes.Add(Topog);
            Une_Fiche.Cavite.Ecrit(Cavites[0]);
          end;
        end;

// Creation d'une galerie
        Une_Galerie:=TGalerie.Create;
        with Une_Galerie do begin
          Information.Ecrit(Nom); Nom:='';
          Precision:=1; Correction_Topofil:=1; Taille_Coord:=0;
          Approximation:=false;
          if N=0 then Erreur:=Numero_Correspondance(I+1,0)
          else Erreur:=Numero_Correspondance(I+1,N-1);
          Depart.Terminus:=false;
          if Erreur<=Cavites.Count then begin
            Depart.Nom.Ecrit('Entre '+Cavites[Erreur-1]);
            Depart.Indice:=1;
          end
          else begin
            Depart.Nom.Ecrit(IntToStr(Correspondance[Erreur,1])+'_'+IntToStr(Correspondance[Erreur,2]));
            Depart.Indice:=Correspondance[Erreur,4];
          end;
        end;

// Avanver jusqu' rupture : fin srie, carrefour, chgt explo, visee fictive
        Une_Galerie.Nb_Visee:=0;
        repeat
          inc(Une_Galerie.Nb_Visee);
          Une_Galerie.Visees[Une_Galerie.Nb_Visee]:=Les_Visees[N];
          Fin:=(Numero_Correspondance(I+1,N)<=Nb_Correspondance) or
            (Info_Expe[N+1]<>Expe_En_Cours) or (Info_Expe[N+1]=999);
          if N=0 then Fin:=false;
          if not Fin then inc(N);
        until {(N=>Les_Visees.Count) or} Fin;
        Erreur:=Numero_Correspondance(I+1,N);
        Une_Galerie.Arrivee.Terminus:=false;
        if Erreur<=Cavites.Count then begin
          Une_Galerie.Arrivee.Nom.Ecrit('Entre '+Cavites[Erreur-1]);
          Une_Galerie.Arrivee.Indice:=1;
        end
        else begin
          if Erreur>Nb_Correspondance then Nouvelle_Correspondance(I+1,N);
          Une_Galerie.Arrivee.Nom.Ecrit(IntToStr(Correspondance[Erreur,1])+'_'+IntToStr(Correspondance[Erreur,2]));
          Une_Galerie.Arrivee.Indice:=Correspondance[Erreur,4];
        end;
        inc(N);
        Une_Fiche.Liste_Galerie.Add(Une_Galerie);
      end;
    until N>=Les_Visees.Count;
  end;
  Intermediaire.Add(Une_Fiche);


  Form1.StatusBar.Panels[0].Text:='Nettoyage';
(************* Nettoyer les vises fictives  et des sries **********************)
  for I:=Les_Series.Count-1 downto 0 do with TSerie(Les_Series[I]) do begin
    for N:=Les_Visees.Count-1 downto 0 do if Info_Expe[N]=999 then TVisee(Les_Visees[N]).Free;
    TSerie(Les_Series[I]).Free;
  end;
  Cavites.Free;

(*************** suppression des vises nulles **********************)
  for I:=0 to Intermediaire.Count-1 do begin
    Une_Fiche:=Intermediaire[I];
    for J:=Une_Fiche.Liste_Galerie.Count-1 downto 0 do begin
      Une_Galerie:=Une_Fiche.Liste_Galerie[J];
      K:=Une_Galerie.Nb_Visee;
      while (K>1) do begin
        if (Une_Galerie.Visees[K].Longueur<1e-4) then begin
          Une_Galerie.Visees[K].Free;
          for L:=K to Une_Galerie.Nb_Visee-1 do Une_Galerie.Visees[L]:=Une_Galerie.Visees[L+1];
          Une_Galerie.Visees[Une_Galerie.Nb_Visee]:=nil;
          dec(Une_Galerie.Nb_Visee);
        end;
        dec(K);
      end;
//      if (Une_Galerie.Nb_Visee=1) and (Une_Galerie.Visees[1].Longueur<1e-4) then Une_Galerie.Visees[1].Longueur:=1;
      if (Une_Galerie.Nb_Visee=1) and (Une_Galerie.Visees[1].Longueur<1e-4) then begin
        for K:=0 to Intermediaire.Count-1 do with TFiche(Intermediaire[K]) do begin
          for N:=0 to Liste_Galerie.Count-1 do if (K<>I) or (J<>N) then with TGalerie(Liste_Galerie[N]) do begin
            if Une_Galerie.Arrivee.Identique(Depart) then begin
              Depart.Duplique(Une_Galerie.Depart);
            end;
            if Arrivee.Identique(Une_Galerie.Arrivee) then begin
              Arrivee.Duplique(Une_Galerie.Depart);
            end;
          end;
        end;
        Une_Fiche.Liste_Galerie.Remove(Une_Galerie);
        Une_Fiche.Liste_Galerie.Pack;
      end;
    end;
  end;

  Supprime_Terminus;
  
  for I:=0 to Intermediaire.Count-1 do begin
    Une_Fiche:=Intermediaire[I];
    if Une_Fiche.Liste_Galerie.Count<>0 then begin
      Une_Fiche.Compile(Form1.Massif.Angle_Geodesique);
      Form1.Massif.Ajouter(Une_Fiche,Form1.Massif.Existe_Selection);
    end
    else Une_Fiche.Free;
  end;

end;

procedure Exporter_TopoRobot(fichier : TfileName);
var
  F : Textfile;
  S, S1 : string;
  I, J, K, N_Serie, N_Visee, Nb_Points, Erreur : integer;
  Correspondance : array[0..10000,1..2] of integer;
  Les_Points : TList;
  Une_Serie : array[0..999] of TRobotVisee;
  Precedent, Debut : TPointTopo;
  Connu, Stock : boolean;
  DX, DY, DZ : double;

  function LaSerie(Un_Point : TPointTopo) : integer;
  var
    I : integer;
  begin
    I:=Nb_Points-1;
    while (I>=0) and not TPointTopo(Les_Points[I]).Identique(Un_Point) do dec(I);
    if I>=0 then LaSerie:=Correspondance[I,1]
    else LaSerie:=-1;
  end;

  function LaVisee(Un_Point : TPointTopo) : integer;
  var
    I : integer;
  begin
    I:=Nb_Points-1;
    while (I>=0) and not TPointTopo(Les_Points[I]).Identique(Un_Point) do dec(I);
    if I>=0 then LaVisee:=Correspondance[I,2]
    else LaVisee:=-1;
  end;

  procedure Accent(var S : string);
  // on remplace les accents Windows par des accents Mac dans les commentaires
  var
    I : integer;
  begin
    for I:=1 to length(S) do begin
      case S[I] of
        '' : S[I]:=#136;
        '' : S[I]:=#142;
        '' : S[I]:=#143;
        '' : S[I]:=#144;
        '' : S[I]:=#137;
        '' : S[I]:=#145;
        '' : S[I]:=#148;
        '' : S[I]:=#149;
        '' : S[I]:=#157;
        '' : S[I]:=#159;
        '' : S[I]:=#141;
      end;
    end;
    // et pendant qu'on y est
    if pos('(',S)=1 then // s'il y a une parenthse en dbut de commentaire
      S:='.'+S; // a gne toporobot, alors on ajoute un point
  end;

procedure Purification(var S : string);
// supprime les tabulations prsentent dans S
begin
  assert(pos(#13,S)=0,'CR dans le nom d''explorateur');
  assert(pos(#10,S)=0,'LF dans le nom d''explorateur');
  while Pos(#9,S)<>0 do delete(S,pos(#9,S),1);
end;

begin
  AssignFile(F,fichier);
  rewrite(F);
  Les_Points:=TList.Create;


{---------- En tete ------------------------}

{ligne -6}
  S:=ExtractFileName(Form1.Massif.Fichier); S:=copy(S,1,pos('.',S)-1);
  Accent(S);
  write(F,'-6'+#9+'1'+#9+S+#13);

{ligne -5}
  write(F,'-5'+#9+'1'+#9);
  with TPointTopo(Form1.massif.Liste_Reference[0]) do begin
    str(X*1000:6:2,S); write(F,S+#9);
    str(Y*1000:6:2,S); write(F,S+#9);
    str(Z:4:2,S); write(F,S+#9);
    write(F,'1'+#9+'0'+#13);
  end;
  Les_Points.Add(Form1.massif.Liste_Reference[0]);
  Correspondance[0,1]:=1; Correspondance[0,2]:=0;
  Nb_Points:=1;
  N_Serie:=1;

{lignes -2}
  for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
    write(F,'-2',#9,IntToStr(I+1),#9);
    write(F,Date[1]+Date[2]+#9+Date[4]+Date[5]+#9+Date[7]+Date[8]+#9);
    if Topographes.Count>0 then begin
      TEco(Topographes[0]).Lit(S);
      Accent(S);
      Purification(S); // suppression des tabulations parasites
      write(F,S+#9);
      if Topographes.Count>1 then begin
        TEco(Topographes[1]).Lit(S);
        Accent(S);
        Purification(S); // suppression des tabulations parasites
        write(F,S+#9);
      end
      else write(F,' '+#9);
    end
    else write(F,' '+#9+' '+#9);
    write(F,'0'+#9);
    // signe oppos pour les dclinaisons dans Toporobot et CyberTopo
    if Unite_Decli=deg then str((Declinaison+Form1.Massif.Angle_Geodesique)/180*200:1:2,S)
    else str((Declinaison+Form1.Massif.Angle_Geodesique/180*200):1:2,S);
    write(F,S+#9);
    write(F,'0'+#9+'1'+#13);
  end;
  write(F,'-2'+#9+'999'+#9+'4'+#9+'1'+#9+'99'+#9+' '+#9+' '+#9+'0'+#9+'0.00'+#9+'0'+#9+'0'+#9+'Point fixe'+#13);


{lignes -1}
  write(F,'-1'+#9+'1'+#9+'360.00'+#9+'360.00'+#9+'0.10'+#9+'1.00'+#9+'1.00'+#9+'100.00'+#9+'0.00'+#13);
  write(F,'-1'+#9+'2'+#9+'360.00'+#9+'400.00'+#9+'0.10'+#9+'1.00'+#9+'1.00'+#9+'100.00'+#9+'0.00'+#13);
  write(F,'-1'+#9+'3'+#9+'360.00'+#9+'380.00'+#9+'0.10'+#9+'1.00'+#9+'1.00'+#9+'100.00'+#9+'0.00'+#13);
  write(F,'-1'+#9+'4'+#9+'360.00'+#9+'361.00'+#9+'0.10'+#9+'1.00'+#9+'1.00'+#9+'100.00'+#9+'0.00'+#13);
  write(F,'-1'+#9+'5'+#9+'400.00'+#9+'360.00'+#9+'0.10'+#9+'1.00'+#9+'1.00'+#9+'100.00'+#9+'0.00'+#13);
  write(F,'-1'+#9+'6'+#9+'400.00'+#9+'400.00'+#9+'0.10'+#9+'1.00'+#9+'1.00'+#9+'100.00'+#9+'0.00'+#13);
  write(F,'-1'+#9+'7'+#9+'400.00'+#9+'380.00'+#9+'0.10'+#9+'1.00'+#9+'1.00'+#9+'100.00'+#9+'0.00'+#13);
  write(F,'-1'+#9+'8'+#9+'400.00'+#9+'361.00'+#9+'0.10'+#9+'1.00'+#9+'1.00'+#9+'100.00'+#9+'0.00'+#13);

  write(F,'-1'+#9+'999'+#9+'360.00'+#9+'360.00'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9+'100.00'+#9+'-100.00'+#13);
  write(F,#13);

{ les sries }
  {points de rfrence}
  if Form1.Massif.Liste_Reference.Count<=1 then begin
    if not TPointTopo(Form1.Massif.Liste_Reference[0]).Identique(
         TGalerie(TFiche(Form1.Massif.Liste_Fiche[0]).Liste_Galerie[0]).Depart)
    then begin
      write(F,'1'+#9+'-1'+#9+'1'+#9+'0'+#9+'1'+#9+'1'+#9+'1'+#9+'0'+#9+'0'+#9+#13);
      write(F,'1'+#9+'0'+#9+'999'+#9+'999'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9+#13);
      write(F,'1'+#9+'1'+#9+'999'+#9+'999'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9+'0 0 0'#13);
      write(F,#13);
      inc(N_Serie);
    end;
  end
  else begin
    for I:=1 to Form1.Massif.Liste_Reference.Count-1 do with TPointTopo(Form1.Massif.Liste_Reference[I]) do begin
      write(F,IntToStr(I)+#9+'-1'+#9+'1'+#9+'0'+#9+IntToStr(I)+#9+'1'+#9+'1'+#9+'0'+#9+'0'+#9+#13);
      write(F,IntToStr(I)+#9+'0'+#9+'999'+#9+'999'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9+#13);
      write(F,IntToStr(I)+#9+'1'+#9+'999'+#9+'999'+#9);

      DX:=(X-TPointTopo(Form1.Massif.Liste_Reference[0]).X)*1000;
      DY:=(Y-TPointTopo(Form1.Massif.Liste_Reference[0]).Y)*1000;
      DZ:=(Z-TPointTopo(Form1.Massif.Liste_Reference[0]).Z);
      str(sqrt(sqr(DX)+sqr(DY)+sqr(DZ)):4:2,S); write(F,S+#9);
      if abs(DX)<1e-3 then begin
        if DY>0 then S:='0.00' else S:='180.00';
      end
      else begin
        if DX>0 then str(90-arctan(DY/DX)/PI*180:3:2,S)
        else str((270-arctan(DY/DX)/PI*180):3:2,S);
      end;
      write(F,S+#9);
      if sqrt(sqr(DX)+sqr(DY))>1e-3 then str(arctan(DZ/sqrt(sqr(DX)+sqr(DY)))/PI*180:3:2,S)
      else if DZ>0 then S:='90.00' else S:='-90.00';
      write(F,S+#9);

      write(F,'0.00'+#9+'0.00'+#9+'0.00'+#9+'0.00'+#9);
      str(X*1000:4:2,S); write(F,S+'       ');
      str(Y*1000:4:2,S); write(F,S+'       ');
      str(Z:4:2,S); write(F,S+#13);
      write(F,#13);
      Les_Points.Add(Form1.massif.Liste_Reference[I]);
      Correspondance[Nb_Points,1]:=I; Correspondance[Nb_Points,2]:=1;
      inc(Nb_Points); inc(N_Serie);
    end;
  end;

  N_Visee:=0;
  for I:=0 to Form1.Massif.Liste_Courante.Count-1 do with TFiche(Form1.Massif.Liste_Courante[I]) do begin
    for J:=0 to Liste_Galerie.Count-1 do with TGalerie(Liste_Galerie[J]) do begin
      if N_Visee<>0 then begin  {changement de srie ? }
        if not Precedent.Identique(Depart) or Connu then begin {oui}
          write(F,IntToStr(N_Serie)+#9+'-1'+#9);
          if Debut.Terminus then write(F,IntToStr(N_Serie)+#9+'0'+#9)
          else begin
            write(F,IntToStr(LaSerie(Debut))+#9);
            write(F,IntToStr(LaVisee(Debut))+#9);
          end;
          if Precedent.Terminus then write(F,IntToStr(N_Serie)+#9+IntToStr(N_Visee-1)+#9)
          else begin
            write(F,IntToStr(LaSerie(Precedent))+#9);
            write(F,IntToStr(LaVisee(Precedent))+#9);
          end;
          Une_Serie[0].Commentaire.Lit(S);
          Accent(S);
          write(F,IntToStr(N_Visee-1)+#9+'0'+#9+'0'+#9+#9+S+#13);
          for K:=0 to N_Visee-1 do begin
            write(F,IntToStr(N_Serie)+#9+IntToStr(K)+#9);
            write(F,IntToStr(Une_Serie[K].Code)+#9);
            write(F,IntToStr(Une_Serie[K].Expe)+#9);
            str(Une_Serie[K].Longueur:2:2,S); write(F,S+#9);
            str(Une_Serie[K].Azimut:3:2,S); write(F,S+#9);
            str(Une_Serie[K].Pente:2:2,S); write(F,S+#9);
            str(Une_Serie[K].LGauche:2:2,S); write(F,S+#9);
            str(Une_Serie[K].LDroite:2:2,S); write(F,S+#9);
            str(Une_Serie[K].Haut:2:2,S); write(F,S+#9);
            str(abs(Une_Serie[K].Bas):2:2,S); write(F,S+#9);
            if K<>0 then begin
              Une_Serie[K].Commentaire.Lit(S);
              Accent(S);
            end
            else S:='';
            write(F,S+#13);
          end;
          write(F,#13);
          inc(N_Serie); N_Visee:=0;
        end;
      end;
      Stock:=false;
      if N_Visee=0 then begin {faire la premire vise}
        Debut:=Depart;
        if Visees[1].Longueur>1e-4 then begin
          with Une_Serie[0] do begin
            Expe:=I+1; Code:=1; Longueur:=0; Azimut:=0; Pente:=0;
            LDroite:=0; LGauche:=0; Haut:=0; Bas:=0;
            Commentaire:=Information;
          end;
          inc(N_Visee);
        end;
      end
      else Stock:=true;
      {faire les quivalences pour le depart et l'arrivee }
      if not Depart.Terminus and (LaSerie(Depart)=-1) then begin
        Les_Points.Add(Depart);
        Correspondance[Nb_Points,1]:=N_Serie;
        if N_Visee=0 then Correspondance[Nb_Points,2]:=0
        else Correspondance[Nb_Points,2]:=N_Visee-1;
        inc(Nb_Points);
      end;
      for K:=1 to Nb_Visee do with Visees[K] do begin
        Une_Serie[N_Visee].Expe:=I+1;
        case UPente of
          deg, ndeg : Une_Serie[N_Visee].Code:=1;
          gr, ngr  : Une_Serie[N_Visee].Code:=2;
          mp, cmp, nmp, ncmp : Une_Serie[N_Visee].code:=3;
          vdeg,nvdeg: Une_Serie[N_Visee].Code:=4;
        end;
        if UAzimuth in [gr,ngr] then Une_Serie[N_Visee].Code:=Une_Serie[N_Visee].Code+4;
        if ULong=m then Une_Serie[N_Visee].Longueur:=Longueur
        else Une_Serie[N_Visee].Longueur:=Longueur/100;
        case UAzimuth of
          deg, gr : Une_Serie[N_Visee].Azimut:=Azimuth;
          ndeg : if Azimuth<180 then Une_Serie[N_Visee].Azimut:=Azimuth+180
                                else Une_Serie[N_Visee].Azimut:=Azimuth-180;
          ngr : if Azimuth<200 then Une_Serie[N_Visee].Azimut:=Azimuth+200
                                else Une_Serie[N_Visee].Azimut:=Azimuth-200;
        end;
        case UPente of
          deg, gr, mp, vdeg : Une_Serie[N_Visee].Pente:=Pente;
          cmp : Une_Serie[N_Visee].Pente:=Pente/100;
          nmp, ndeg, ngr : Une_Serie[N_Visee].Pente:=-Pente;
          ncmp : Une_Serie[N_Visee].Pente:=-Pente/100;
          nvdeg : Une_Serie[N_Visee].Pente:=180-Pente;
        end;

        S:=LargeurGauche;
        val(S,Une_Serie[N_Visee].LGauche,Erreur);
        if Erreur<>0 then begin
          S:=copy(S,1,length(S)-1);
          val(S,Une_Serie[N_Visee].LGauche,Erreur);
        end;
        if ULarg=cm then Une_Serie[N_Visee].LGauche:=Une_Serie[N_Visee].LGauche/100;
        S:=LargeurDroite;
        if (Erreur=0) and (S='') then begin
          Une_Serie[N_Visee].LGauche:=Une_Serie[N_Visee].LGauche/2;
          Une_Serie[N_Visee].LDroite:=Une_Serie[N_Visee].LGauche;
        end
        else begin
          val(S,Une_Serie[N_Visee].LDroite,Erreur);
          if Erreur<>0 then begin
            S:=copy(S,1,length(S)-1);
            val(S,Une_Serie[N_Visee].LDroite,Erreur);
          end;
          if ULarg=cm then Une_Serie[N_Visee].LDroite:=Une_Serie[N_Visee].LDroite/100;
        end;

        S:=HauteurSup;
        val(S,Une_Serie[N_Visee].Haut,Erreur);
        if Erreur<>0 then begin
          S:=copy(S,1,length(S)-1);
          val(S,Une_Serie[N_Visee].Haut,Erreur);
        end;
        if UHauteur=cm then Une_Serie[N_Visee].Haut:=Une_Serie[N_Visee].Haut/100;
        S:=HauteurInf;
        val(S,Une_Serie[N_Visee].Bas,Erreur);
        if Erreur<>0 then begin
          S:=copy(S,1,length(S)-1);
          val(S,Une_Serie[N_Visee].Bas,Erreur);
        end;
        if UHauteur=cm then Une_Serie[N_Visee].Bas:=Une_Serie[N_Visee].Bas/100;

        if Stock then begin
          Information.Lit(S);
          if S='' then Une_Serie[N_Visee].Commentaire:=Commentaires
          else begin
            Commentaires.Lit(S);
            if S='' then Une_Serie[N_Visee].Commentaire:=Information
            else begin
              Information.Lit(S1);
              Une_Serie[N_Visee].Commentaire:=TEco.Create;
              Une_Serie[N_Visee].Commentaire.Ecrit(S1+' '+S);
            end;
          end;
        end
        else Une_Serie[N_Visee].Commentaire:=Commentaires;
        Stock:=false;
        inc(N_Visee);
      end;
      if not Arrivee.Terminus and (LaSerie(Arrivee)=-1) then begin
        Les_Points.Add(Arrivee);
        Correspondance[Nb_Points,1]:=N_Serie; Correspondance[Nb_Points,2]:=N_Visee-1;
        inc(Nb_Points);
        Connu:=false;
      end
      else Connu:=true;

      Precedent:=Arrivee;
    end;
  end;

  write(F,IntToStr(N_Serie)+#9+'-1'+#9);
  if Debut.Terminus then write(F,IntToStr(N_Serie)+#9+'0'+#9)
  else begin
    write(F,IntToStr(LaSerie(Debut))+#9);
    write(F,IntToStr(LaVisee(Debut))+#9);
  end;
  if Precedent.Terminus then write(F,IntToStr(N_Serie)+#9+IntToStr(N_Visee-1)+#9)
  else begin
    write(F,IntToStr(LaSerie(Precedent))+#9);
    write(F,IntToStr(LaVisee(Precedent))+#9);
  end;
  Une_Serie[0].Commentaire.Lit(S);
  Accent(S);
  write(F,IntToStr(N_Visee-1)+#9+'0'+#9+'0'+#9+S);
  for K:=0 to N_Visee-1 do begin
    write(F,#13);
    write(F,IntToStr(N_Serie)+#9+IntToStr(K)+#9);
    write(F,IntToStr(Une_Serie[K].Code)+#9);
    write(F,IntToStr(Une_Serie[K].Expe)+#9);
    str(Une_Serie[K].Longueur:2:2,S); write(F,S+#9);
    str(Une_Serie[K].Azimut:3:2,S); write(F,S+#9);
    str(Une_Serie[K].Pente:2:2,S); write(F,S+#9);
    str(Une_Serie[K].LGauche:2:2,S); write(F,S+#9);
    str(Une_Serie[K].LDroite:2:2,S); write(F,S+#9);
    str(Une_Serie[K].Haut:2:2,S); write(F,S+#9);
    str(Une_Serie[K].Bas:2:2,S); write(F,S+#9);
    if K<>0 then begin
      Une_Serie[K].Commentaire.Lit(S);
      Accent(S);
    end
    else S:='';
    write(F,S);
  end;
  Les_Points.Free;
  closeFile(F);
end;

end.
