unit visualtopo;

interface

uses SysUtils, Classes, Dialogs;

procedure Importer_VisualTopo(fichier : TfileName);

implementation

uses Fichetpo, Unit1, Graphics;

type

  TOutil_Pente = (Clino, Vulcain, Hauteur, Profondeur);
  TUnite_Angle = (Deg_dec, Deg_min, Grade, Pourcent);

  TSortie = record
    Topofil : boolean;
    Cal_Topofil : real;
    Unite_Az : TUnite_Angle;
    Declinaison : real;
    Az_Direct : boolean;
    Outil_Pente : TOutil_Pente;
    Unite_Pente : TUnite_Angle;
    Pente_Direct : boolean;
    Largeur_Direct : boolean;
    Couleur : TColor;
  end;

  TInfo_Point = record
    Nom : string;
    Indice : integer;
    Carrefour, Impasse : boolean;
  end;


procedure Importer_VisualTopo(fichier : TfileName);
// on procde en deux passes
// dans la premire, on recherche les stations qui correspondent  des carrefours
// dans la seconde, on contruit les fiches elles-mme
type
  TEquivalent = record
    Ancien, Nouveau : integer;
  end;
var
  F : textFile;
  Ligne, Prec_Point, Depart, Arrivee, Morceau : string;
  Les_Points : array of TInfo_Point;
  Num, I, J, K, Deb, Fin : integer;
  Xe, Ye, Ze : real;

  Cavite : string;
  Entree : TPointTopo;
  Sortie : TSortie;
  Une_Fiche : TFiche;
  Une_Galerie : Tgalerie;
  Nouveautes : TList;
  Visee_Nulle : boolean;
  Commentaire_Fiche : string;
  Equivalents : array of TEquivalent;
  S_Depart, S_Arrivee : string;
  Prof_Prec : real;
  Warning_Prof : boolean;



  function Numero_Point (Nom : string) : integer;
  // renvoie la position du point dans le tableau de points
  begin
    result:=high(Les_Points);
    while (result>=0) and (Les_Points[result].Nom<>Nom) do
      dec(result);
  end;

  function Ajouter_Point (Nvx_Nom : string) : integer;
  // ajoute un point et renvoie sa position dans le tableau de points
  begin
    result:=high(Les_Points)+1;
    setLength(Les_Points,result+1);
    with Les_Points[result] do
    begin
      Nom:=Nvx_Nom;
      Carrefour:=false;
      Impasse:=true;
    end;
  end;

begin
  assignFile(F,fichier);
  reset(F);
  Warning_Prof:=false;
  try
  (***************************)
  // premire passe
  (***************************)
    // lecture de l'en-tte
    repeat
      readln(F,Ligne);
      if pos('Entree',Ligne)=1 then
      begin
        delete(Ligne,1,7);
        with Les_Points[Ajouter_Point(Ligne)] do
        begin
          Carrefour:=true;
          Impasse:=false;
        end;
      end;

    until pos('Param',Ligne)<>0;

    // on travaille par paquet de vises entre deux lignes de paramtres
    repeat
      Prec_Point:='';
      readln(F,Ligne); // lecture de la ligne blanche
      readln(F,Ligne); // lecture de la premire vise
      repeat
        Depart:=copy(Ligne,1,pos(' ',Ligne)-1);
        Delete(Ligne,1,11);
        Arrivee:=copy(Ligne,1,pos(' ',Ligne)-1);

        if Depart=Prec_Point then
        begin // on est dans la continuit
          Les_Points[Numero_Point(Depart)].Impasse:=false;
        end
        else begin // il y a discontinuit
          Num:=Numero_Point(Depart);
          if Num=-1 then // nouveau point
            Ajouter_Point(Depart)
          else // point dj connu
            with Les_Points[Num] do
            begin
              Carrefour:=true;
              Impasse:=false;
            end;
        end;

        Num:=Numero_Point(Arrivee);
        if Num=-1 then // nouveau point
          Ajouter_Point(Arrivee)
        else // point dj connu, il faut le convertir en carrefour
          with Les_Points[Num] do
          begin
            Carrefour:=true;
            Impasse:=false;
          end;

        Prec_Point:=Arrivee;
        readln(F,Ligne); // lecture de la ligne suivante
      until eof(F) or (Ligne='');
      readln(F,Ligne); // lecture de la ligne de paramtres

    until eof(F) or (pos('[Configuration',Ligne)<>0);

    // dtermination des indices des points
    for I:=0 to high(Les_Points) do
     with Les_Points[I] do
      Indice:=Form1.Massif.Nouveau_Point(Nom,0);

  (***************************)
  // seconde passe
  (***************************)
    reset(F);
    // lecture de l'en-tte
    Xe:=0; Ye:=0; Ze:=0;
    Prof_Prec:=0;
    repeat
      readln(F,Ligne);
      if pos('Trou',Ligne)=1 then
      begin
        delete(Ligne,1,5);
        Cavite:=copy(Ligne,1,pos(',',Ligne)-1);
        Delete(Ligne,1,pos(',',Ligne));
        Xe:=StrToFloat(copy(Ligne,1,pos(',',Ligne)-1));
        Delete(Ligne,1,pos(',',Ligne));
        Ye:=StrToFloat(copy(Ligne,1,pos(',',Ligne)-1));
        Delete(Ligne,1,pos(',',Ligne));
        Ze:=StrToFloat(copy(Ligne,1,pos(',',Ligne)-1));
      end;
      if pos('Entree',Ligne)=1 then
      begin
        delete(Ligne,1,7);
        Entree:=TPointTopo.Create;
        with Entree do
        begin
          Terminus:=false;
          Nom.Ecrit(Ligne);
          for I:=0 to high(Les_Points) do
           if Les_Points[I].Nom=Ligne then
            Indice:=Les_Points[I].Indice;
          X:=Xe;
          Y:=Ye;
          Z:=Ze;
        end;
        form1.Massif.Ecrire_Point(Entree);
      end;

    until pos('Param',Ligne)<>0;

    Nouveautes:=TList.Create;
    try
      // on travaille par paquet de vises entre deux lignes de paramtres
      repeat
        // analyse des paramtres du paquet de vises
        delete(Ligne,1,pos(' ',Ligne));
        with Sortie do
        begin
          Topofil:= (copy(Ligne,1,pos(' ',Ligne)-1)='Topof');
          delete(Ligne,1,pos(' ',Ligne));
          if Topofil then
          begin
            Cal_Topofil:=StrToFloat(copy(Ligne,1,pos(' ',Ligne)-1));
            delete(Ligne,1,pos(' ',Ligne));
          end;
          Morceau:=copy(Ligne,1,pos(' ',Ligne)-1);
          delete(Ligne,1,pos(' ',Ligne));
          if Morceau='Degd' then
            Unite_Az:=Deg_dec;
          if Morceau='Deg' then
            Unite_Az:=Deg_min;
          if Morceau='Gra' then
            Unite_Az:=Grade;

          Morceau:=copy(Ligne,1,pos(' ',Ligne)-1);
          delete(Ligne,1,pos(' ',Ligne));
          if Morceau='Clino' then
            Outil_Pente:=Clino;
          if Morceau='Vulc' then
            Outil_Pente:=Vulcain;
          if Morceau='Prof' then
          begin
            Outil_Pente:=Profondeur;
            Warning_Prof:=true;
          end;
          if Morceau='Deniv' then
            Outil_Pente:=Hauteur;
          if Outil_Pente in [Clino,Vulcain] then
          begin
            Morceau:=copy(Ligne,1,pos(' ',Ligne)-1);
            delete(Ligne,1,pos(' ',Ligne));
            if Morceau='Degd' then
              Unite_Pente:=Deg_dec;
            if Morceau='Deg' then
              Unite_Pente:=Deg_min;
            if Morceau='Gra' then
              Unite_Pente:=Grade;
            if Morceau='%' then
              Unite_Pente:=Pourcent;
          end;

          Declinaison:=StrToFloat(copy(Ligne,1,pos(' ',Ligne)-1));
          Declinaison:=int(Declinaison)+frac(Declinaison)/0.6;   // conversion de deg/min vers deg dcimal
          delete(Ligne,1,pos(' ',Ligne));

          Az_Direct:=(Ligne[1]='D');
          Pente_Direct:=(Ligne[5]='D');
          Largeur_Direct:=(Ligne[9]='D');

          delete(Ligne,1,pos(' ',Ligne));
          if pos(';',Ligne)<>0 then
            Morceau:=copy(Ligne,1,pos(' ',Ligne)-1) // on coupe avant le commentaire
          else
            Morceau:=Ligne; // sinon, on prend tout
          if Morceau<>'Std' then
          begin
            Couleur:=StrToInt(copy(Morceau,1,pos(',',Morceau)-1));
            delete(Morceau,1,pos(',',Morceau));
            inc(Couleur,StrToInt(copy(Morceau,1,pos(',',Morceau)-1))*256);
            delete(Morceau,1,pos(',',Morceau));
            inc(Couleur,StrToInt(Morceau)*65536);
          end
          else
            Couleur:=0;

          if pos(';',Ligne)<>0 then
          begin // il y a un commentaire gnral
            delete(Ligne,1,pos(';',Ligne));
            Commentaire_Fiche:=copy(Ligne,1,pos(';',Ligne)-1);
          end;

        end;

        Une_Fiche:=TFiche.Create;

        Une_Fiche.Unite_Decli:=deg;
        Une_Fiche.Declinaison:=-Sortie.Declinaison;
        Une_Fiche.Premiere:=0;
        Une_Fiche.Date:='  /  /  ';
        Une_Fiche.Cavite.Ecrit(Cavite);

        readln(F,Ligne); // lecture de la ligne blanche
        readln(F,Ligne); // lecture de la premire vise
        Visee_Nulle:=false;
        repeat
          Depart:=copy(Ligne,1,pos(' ',Ligne)-1);
          Delete(Ligne,1,11);
          Arrivee:=copy(Ligne,1,pos(' ',Ligne)-1);
          Delete(Ligne,1,11);
          if (Les_Points[Numero_Point(Depart)].Carrefour) and not Visee_Nulle then
          begin
            Une_Galerie:=TGalerie.Create;
            Une_Galerie.Precision:=1; Une_Galerie.Taille_Coord:=0;
            Une_Galerie.Couleur:=Sortie.Couleur;
            Une_Galerie.Approximation:=false;
            Une_Galerie.Depart.Terminus:=false;
            Une_Galerie.Depart.Nom.Ecrit(Depart);
            Une_Galerie.Depart.Indice:=Les_Points[Numero_Point(Depart)].Indice;
            if Sortie.Topofil then
              Une_Galerie.Correction_Topofil:=Sortie.Cal_Topofil
            else
              Une_Galerie.Correction_Topofil:=1;
            Une_Galerie.Information.Ecrit(Commentaire_Fiche);
            Commentaire_Fiche:='';
            Une_Galerie.Nb_Visee:=0;
          end;

          if (Les_Points[Numero_Point(Depart)].Impasse) and not Visee_Nulle then
          begin
            Une_Galerie:=TGalerie.Create;
            Une_Galerie.Precision:=1; Une_Galerie.Taille_Coord:=0;
            Une_Galerie.Approximation:=false;
            Une_Galerie.Couleur:=Sortie.Couleur;
            Une_Galerie.Depart.Terminus:=true;
            if Sortie.Topofil then
              Une_Galerie.Correction_Topofil:=Sortie.Cal_Topofil
            else
              Une_Galerie.Correction_Topofil:=1;
            Une_Galerie.Information.Ecrit(Commentaire_Fiche);
            Commentaire_Fiche:='';
            Une_Galerie.Nb_Visee:=0;
          end;

          inc(Une_Galerie.Nb_Visee);
          with Une_Galerie do
          begin
            Visees[Nb_Visee]:=TVisee.Create;

            Visees[Nb_Visee].Schema:=0;
            Visees[Nb_Visee].Correction_Horizontale:=false;
            Visees[Nb_Visee].Correction_Dlt:=false; 

            if Sortie.Topofil then
            begin
              Visees[Nb_Visee].Topofil:=true;
              Visees[Nb_Visee].Longueur:=(StrToFloat(copy(Ligne,11,9))-StrToFloat(copy(Ligne,1,9)))*Sortie.Cal_Topofil;
              Visees[Nb_Visee].Debut_Topofil:=StrToFloat(copy(Ligne,11,9));
              Visees[Nb_Visee].ULong:=cm;
            end
            else begin
              Visees[Nb_Visee].Topofil:=false;
              Visees[Nb_Visee].Longueur:=StrToFloat(copy(Ligne,11,9));
              Visees[Nb_Visee].ULong:=m;
            end;

            delete(Ligne,1,20);
            Visees[Nb_Visee].Azimuth:=StrToFloat(copy(Ligne,1,7));
            case Sortie.Unite_Az of
              deg_dec :
                if Sortie.Az_Direct then
                  Visees[Nb_Visee].UAzimuth:=deg
                else
                  Visees[Nb_Visee].UAzimuth:=ndeg;
              deg_min : begin
                if Sortie.Az_Direct then
                  Visees[Nb_Visee].UAzimuth:=deg
                else
                  Visees[Nb_Visee].UAzimuth:=ndeg;
                Visees[Nb_Visee].Azimuth:=int(Visees[Nb_Visee].Azimuth)+frac(Visees[Nb_Visee].Azimuth)/0.6;
              end;
              grade :
                if Sortie.Az_Direct then
                  Visees[Nb_Visee].UAzimuth:=gr
                else
                  Visees[Nb_Visee].UAzimuth:=ngr;
            end;


            delete(Ligne,1,8);
            Visees[Nb_Visee].Pente:=StrToFloat(copy(Ligne,1,7));
            case Sortie.Outil_Pente of
              Clino : begin
                case Sortie.Unite_Pente of
                  deg_dec :
                    if Sortie.Pente_Direct then
                      Visees[Nb_Visee].UPente:=deg
                     else
                      Visees[Nb_Visee].UPente:=ndeg;
                  deg_min : begin
                    if Sortie.Pente_Direct then
                      Visees[Nb_Visee].UPente:=deg
                     else
                      Visees[Nb_Visee].UPente:=ndeg;
                    Visees[Nb_Visee].Pente:=int(Visees[Nb_Visee].Pente)+frac(Visees[Nb_Visee].Pente)/0.6;
                  end;
                  grade :
                    if Sortie.Pente_Direct then
                      Visees[Nb_Visee].UPente:=gr
                     else
                      Visees[Nb_Visee].UPente:=ngr;
                  pourcent : begin
                    if Sortie.Pente_Direct then
                      Visees[Nb_Visee].UPente:=deg
                     else
                      Visees[Nb_Visee].UPente:=ndeg;
                    Visees[Nb_Visee].Pente:=arctan(Visees[Nb_Visee].Pente/100)/Pi*180;
                  end;
                end;
              end;
              Vulcain : begin
                case Sortie.Unite_Pente of
                  deg_dec :
                    if Sortie.Pente_Direct then
                      Visees[Nb_Visee].UPente:=vdeg
                     else
                      Visees[Nb_Visee].UPente:=nvdeg;
                  deg_min : begin
                    if Sortie.Pente_Direct then
                      Visees[Nb_Visee].UPente:=vdeg
                     else
                      Visees[Nb_Visee].UPente:=nvdeg;
                    Visees[Nb_Visee].Pente:=int(Visees[Nb_Visee].Pente)+frac(Visees[Nb_Visee].Pente)/0.6;
                  end;
                  grade : begin // les grades ne sont pas prvus dans Cybertopo avec le botier Vulcain
                    if Sortie.Pente_Direct then
                      Visees[Nb_Visee].UPente:=vdeg
                     else
                      Visees[Nb_Visee].UPente:=nvdeg;
                    Visees[Nb_Visee].Pente:=Visees[Nb_Visee].Pente/200*180;
                  end;
                  pourcent : begin
                    if Sortie.Pente_Direct then
                      Visees[Nb_Visee].UPente:=deg
                     else
                      Visees[Nb_Visee].UPente:=ndeg;
                    Visees[Nb_Visee].Pente:=90-arctan(Visees[Nb_Visee].Pente/100)/Pi*180;
                  end;
                end;
              end;
              Hauteur : if Sortie.Pente_Direct then
                Visees[Nb_Visee].UPente:=mp
               else
                Visees[Nb_Visee].UPente:=nmp;
              Profondeur :
              begin
                if Sortie.Pente_Direct then
                  Visees[Nb_Visee].UPente:=mp
                else
                  Visees[Nb_Visee].UPente:=nmp;
                Visees[Nb_Visee].Pente:=Visees[Nb_Visee].Pente-Prof_Prec;
                Prof_Prec:=StrToFloat(copy(Ligne,1,7));
              end;
            end;

            delete(Ligne,1,5);
            Morceau:=copy(Ligne,1,7);
            while pos(' ',Morceau)<>0 do delete(morceau,pos(' ',Morceau),1);
            if Morceau='*' then
              Morceau:='';
            if Sortie.Largeur_Direct then
              Visees[Nb_Visee].LargeurGauche:=Morceau
            else
              Visees[Nb_Visee].LargeurDroite:=Morceau;
            delete(Ligne,1,7);
            Morceau:=copy(Ligne,1,7);
            while pos(' ',Morceau)<>0 do delete(morceau,pos(' ',Morceau),1);
            if Morceau='*' then
              Morceau:='';
            if Sortie.Largeur_Direct then
              Visees[Nb_Visee].LargeurDroite:=Morceau
            else
              Visees[Nb_Visee].LargeurGauche:=Morceau;

            delete(Ligne,1,7);
            Morceau:=copy(Ligne,1,7);
            while pos(' ',Morceau)<>0 do delete(morceau,pos(' ',Morceau),1);
            if Morceau='*' then
              Visees[Nb_Visee].HauteurSup:=''
            else
              Visees[Nb_Visee].HauteurSup:=Morceau;
            delete(Ligne,1,7);
            Morceau:=copy(Ligne,1,7);
            while pos(' ',Morceau)<>0 do delete(morceau,pos(' ',Morceau),1);
            if Morceau='*' then
              Visees[Nb_Visee].HauteurInf:=''
            else
              Visees[Nb_Visee].HauteurInf:=Morceau;

          end;
          if pos(';',Ligne)<>0 then
          begin // il y a un commentaire
            delete(Ligne,1,pos(';',Ligne));
            Une_Galerie.Visees[Une_Galerie.Nb_Visee].Commentaires.Ecrit(Format('[%s->%s] %s',[Depart,Arrivee,copy(Ligne,1,pos(';',Ligne)-1)]));
          end
          else
            Une_Galerie.Visees[Une_Galerie.Nb_Visee].Commentaires.Ecrit(Format('[%s->%s]',[Depart,Arrivee]));


          Visee_Nulle:=(Depart=Arrivee);

          if (Une_Galerie.Visees[Une_Galerie.Nb_Visee].Longueur<1e-4) and not Visee_Nulle then
          begin // vise fictive (longueur=0) reliant un point avec deux noms
            dec(Une_Galerie.Nb_Visee);
            if (Les_Points[Numero_Point(Arrivee)].Carrefour) and
               (Les_Points[Numero_Point(Depart)].Carrefour) then
            begin // mmorisation de l'quivalence des points de dpart et d'arrive
              Deb:=Numero_Point(Depart);
              Fin:=Numero_Point(Arrivee);
              // on commence par regarder s'il n'est pas dj impliqu dans des quivalences
              for I:=0 to high(Equivalents) do
              begin
                if Equivalents[I].Nouveau=Deb then
                  Deb:=Equivalents[I].Ancien; // dj impliqu par le dbut, on met le numro de dpart de l'autre quivalence
                if Equivalents[I].Nouveau=Fin then
                  Fin:=Equivalents[I].Ancien; // dj impliqu par la fin , on met le numro de dpart de l'autre quivalence
              end;
              if Deb<>fin then // Deb=Fin => quivalence dj connue en sens inverse
              begin
                for I:=0 to high(Equivalents) do
                begin
                  if Equivalents[I].Ancien=Fin then // point d'arrive commun avec le dpart d'une autre quivalence
                  begin // il faut tre invers pour avoir les mmes dparts
                    Fin:=Deb;
                    Deb:=Equivalents[I].Ancien;
                  end;
                end;
                I:=high(Equivalents);
                while (I>=0) and
                      ((Equivalents[I].Ancien<>Deb) or (Equivalents[I].Nouveau<>Fin)) and
                      ((Equivalents[I].Ancien<>Fin) or (Equivalents[I].Nouveau<>Deb)) do
                  dec(I);
                if I=-1 then
                begin // l'quivalence n'existe pas  l'tat actuel
                  SetLength(Equivalents,high(Equivalents)+2);
                  with Equivalents[high(Equivalents)] do
                  begin
                    Ancien:=Deb;
                    Nouveau:=Fin;
                  end;
                end;
              end;
            end;
          end;

          if (Les_Points[Numero_Point(Arrivee)].Carrefour) and not Visee_Nulle then
          begin
            Une_Galerie.Arrivee.Terminus:=false;
            Une_Galerie.Arrivee.Nom.Ecrit(Arrivee);
            Une_Galerie.Arrivee.Indice:=Les_Points[Numero_Point(Arrivee)].Indice;
            if Une_Galerie.Nb_Visee<>0 then
              Une_Fiche.Liste_Galerie.Add(Une_Galerie)
            else
              Une_Galerie.Free;
          end;

          if (Les_Points[Numero_Point(Arrivee)].Impasse) and not Visee_Nulle then
          begin
            Une_Galerie.Arrivee.Terminus:=true;
            if Une_Galerie.Nb_Visee<>0 then
              Une_Fiche.Liste_Galerie.Add(Une_Galerie)
            else
              Une_Galerie.Free;
          end;



          readln(F,Ligne); // lecture de la ligne suivante
        until eof(F) or (Ligne='');
        if Visee_Nulle then // on sort d'une fiche avec une vise de dbut de galerie, on jte la galerie correspondante
          Une_Galerie.Free;
        if Une_Fiche.Liste_Galerie.Count>0 then
        begin
          Une_Fiche.Compile(Form1.Massif.Angle_Geodesique);
          Form1.Massif.Liste_Fiche.Add(Une_Fiche);
          Nouveautes.Add(Une_Fiche);
        end
        else
          Une_Fiche.Free;
        readln(F,Ligne); // lecture de la ligne de paramtres

      until eof(F) or (pos('[Configuration',Ligne)<>0);

      // analyse complmentaire pour tenir compte des vise nulles = points identiques
      for I:=0 to Nouveautes.Count-1 do
      begin
        for J:=0 to TFiche(Nouveautes[I]).Liste_Galerie.Count-1 do
         with TGalerie(TFiche(Nouveautes[I]).Liste_Galerie[J]) do
        begin
          Depart.Nom.Lit(S_Depart);
          Arrivee.Nom.Lit(S_Arrivee);
          for K:=0 to high(Equivalents) do
          begin
            if (S_Depart=Les_Points[Equivalents[K].Nouveau].Nom) and
               (Depart.Indice=Les_Points[Equivalents[K].Nouveau].Indice) then
            begin
              Depart.Nom.Ecrit(Les_Points[Equivalents[K].Ancien].Nom);
              Depart.Indice:=Les_Points[Equivalents[K].Ancien].Indice;
            end;
            if (S_Arrivee=Les_Points[Equivalents[K].Nouveau].Nom) and
               (Arrivee.Indice=Les_Points[Equivalents[K].Nouveau].Indice) then
            begin
              Arrivee.Nom.Ecrit(Les_Points[Equivalents[K].Ancien].Nom);
              Arrivee.Indice:=Les_Points[Equivalents[K].Ancien].Indice;
            end;
          end;
        end;
      end;

    finally
      Nouveautes.Free;
    end;
    if Warning_Prof then
      MessageDlg('Le fichier comporte des mesures de pente de type "Profondeur". Vrifiez que les donnes sont correctement importes',
                 mtWarning,[mbOk],0);
  finally
    closeFile(F);
  end;
end;

end.
